import QtQuick 2.1
import QtQuick.Controls 1.1

Item {
    id: root    
    state: "expanded"
    property var collapsedWidth: 0
    property var collapsedHeight: 0
    property var transitionEnabled: true
    property var transitionDuration: 300
    
    property bool expandedX: (state == "expanded" || state == "collapsedY") && !inXTransition
    property bool expandedY: (state == "expanded" || state == "collapsedX") && !inYTransition
    property bool collapsedX: (state == "collapsedX" || state == "collapsedXY") && !inXTransition
    property bool collapsedY: (state == "collapsedY" || state == "collapsedXY") && !inYTransition
    property bool inXTransition: false
    property bool inYTransition: false

    function collapseX() {
        if(state == "expanded") {
            if(transitionEnabled) inXTransition = true
            state = "collapsedX"
        }
        else if(state == "collapsedY") {
            if(transitionEnabled) inXTransition = true
            state = "collapsedXY"
        }
    }
    
    function collapseY() {
        if(state == "expanded") {
            if(transitionEnabled) inYTransition = true
            state = "collapsedY"
        }
        else if(state == "collapsedX"){
            if(transitionEnabled) inYTransition = true
            state = "collapsedXY"
        }
    }    
    
    function expandX() {
        if(state == "collapsedX") {
            if(transitionEnabled) inXTransition = true
            state = "expanded"
        }
        else if(state == "collapsedXY"){
            if(transitionEnabled) inXTransition = true
            state = "collapsedY"
        }
    }
    
    function expandY() {
        if(state == "collapsedY") {
            if(transitionEnabled) inYTransition = true
            state = "expanded"
        }
        else if(state == "collapsedXY"){
            if(transitionEnabled) inYTransition = true
            state = "collapsedX"
        }
    }    
    
    function flipXState() {
        if(collapsedX) expandX() 
        else if(expandedX) collapseX()
    }

    function flipYState() {
        if(collapsedY) expandY() 
        else if(expandedY) collapseY()
    }    
    
    states: [
        State {
            name: "expanded"          
        },
        State {
            name: "collapsedX"
            PropertyChanges {
                target: root
                width: collapsedWidth
            }
        },   
        State {
            name: "collapsedY"
            PropertyChanges {
                target: root
                height: collapsedHeight
            }
        },           
        State {
            name: "collapsedXY"
            PropertyChanges {
                target: root
                width: collapsedWidth
                height: collapsedHeight
            }
        }		
    ]

    transitions: [
        Transition {
            enabled: root.transitionEnabled
            from: "expanded"; to: "collapsedX"
            reversible: true
            NumberAnimation {target: root; property: "width"; duration: root.transitionDuration}
            onRunningChanged: inXTransition = running
        },
        Transition {
            enabled: root.transitionEnabled
            from: "collapsedXY"; to: "collapsedY"
            reversible: true
            NumberAnimation {target: root; property: "width"; duration: root.transitionDuration}
            onRunningChanged: inXTransition = running
        },
        Transition {
            enabled: root.transitionEnabled
            from: "expanded"; to: "collapsedY"
            reversible: true
            NumberAnimation {target: root; property: "height"; duration: root.transitionDuration}
            onRunningChanged: inYTransition = running
        },
        Transition {
            enabled: root.transitionEnabled
            from: "collapsedXY"; to: "collapsedX"
            reversible: true
            NumberAnimation {target: root; property: "height"; duration: root.transitionDuration}
            onRunningChanged: inYTransition = running
        },        
        Transition {
            from: "expanded"; to: "collapsedXY"
            enabled: root.transitionEnabled
            NumberAnimation {target: root; property: "width"; duration: root.transitionDuration}
            NumberAnimation {target: root; property: "height"; duration: root.transitionDuration}
            onRunningChanged: {
                inXTransition = running
                inYTransition = running
            }
        }        
        
    ]	
}