import QtQuick 2.1
import QtQuick.Controls 1.1
import QtQuick.Controls.Private 1.0
import QtQuick.Layouts 1.0

Item {
    id: root
    default property Item content
    property Component style: {
        var styleName = Settings.style.toString().split('/').pop()
        var componentName = styleName == "ECGStyle" ? "CollapsiblePanelStyleECG.qml" : "CollapsiblePanelStyle.qml"
        return Qt.createComponent(componentName, root)
    }    
    property CollapsiblePanelStyle _style: styleLoader.item
    
    property string title
    property real topSpacing: spacing
    property real bottomSpacing: spacing
    property real leftSpacing: spacing
    property real rightSpacing: spacing
    property real spacing: 2
    property bool expandOnClicking: true
    
    property Item headerControl
    property list<Item> headerButtons
    
    implicitHeight: contentContainer.y + contentContainer.height
    implicitWidth: content ? (contentContainer.x + content.implicitWidth + leftSpacing + rightSpacing) : contentContainer.x + leftSpacing + rightSpacing
    state: "collapsed"
    
    Loader {
        id: styleLoader
        sourceComponent: root.style
    }
    
    QtObject {
        id: styleData
        property string title: root.title
        property bool expanded: root.state == "expanded"
        property bool pressed: headerMouseArea.pressed
        property bool hovered: headerMouseArea.containsMouse
    }
   
    function toggleExpanded() {
        if (root.state == "collapsed") {
            root.state = "expanded"
        }
        else {
            root.state = "collapsed"
        }    
    }
    
    Item {
        id: headerItem
        width: root.width
        height: headerBackgroundLoader.implicitHeight
        RowLayout {
            anchors.fill: parent
            spacing: _style.headerButtonSpacing       
            Item {
                Layout.fillWidth: true
                Layout.preferredHeight: parent.height
                Loader {
                    id: headerBackgroundLoader
                    property Item control: root
                    property QtObject styleData: styleData
                    anchors.fill: parent
                    anchors.topMargin: headerMouseArea.pressed ? _style.headerVerticalOffsetOnPressing : 0
                    anchors.bottomMargin: headerMouseArea.pressed ? -_style.headerVerticalOffsetOnPressing : 0
                    sourceComponent: _style.headerBackground
                }
                MouseArea{
                    id: headerMouseArea
                    anchors.fill: parent
                    enabled: root.expandOnClicking
                    hoverEnabled: true
                    onClicked: toggleExpanded()
                }
                RowLayout {
                    id: headerLayout
                    anchors.fill: parent
                    anchors.topMargin: headerMouseArea.pressed ? _style.headerVerticalOffsetOnPressing : 0
                    anchors.bottomMargin: headerMouseArea.pressed ? -_style.headerVerticalOffsetOnPressing : 0                    
                    spacing: _style.headerItemSpacing
                    
                    Item {
                        implicitWidth: 1
                    }
                    
                    Loader {
                        id: expanderLoader
                        property Item control: root
                        property QtObject styleData: styleData                
                        sourceComponent: _style.expander
                    }
                    
                    Item {
                        visible: headerControl != null
                        Layout.preferredHeight: parent.height
                        Layout.preferredWidth: headerControlContainer.implicitWidth                    
                        Item {
                            id: headerControlContainer
                            implicitWidth: headerControl ? headerControl.implicitWidth : 0
                            implicitHeight: headerControl ? headerControl.implicitHeight : 0
                            anchors.centerIn: parent
                            data: root.headerControl
                        }
                    }
                    
                    Loader {
                        id: headerTitleLoader
                        property Item control: root
                        property QtObject styleData: styleData
                        Layout.fillWidth: true
                        sourceComponent: _style.headerTitle
                    }
                }   
            }
            
            RowLayout {
                visible: children.length > 0
                spacing: 2
                readonly property Component _collapsiblePanelStyle: root.style
                data: root.headerButtons
            }
        }
    }
    
    Item {
        id: contentContainer
        y:  headerItem.y + headerItem.height
        width: root.width
        clip: true
        
        Item {
            id: contentContainerInner
            anchors.fill: parent
            anchors.leftMargin: leftSpacing
            anchors.rightMargin: rightSpacing
            anchors.bottomMargin: bottomSpacing
            anchors.topMargin: topSpacing			
            data: content
        }
        
        Binding {
            target: content
            property: "width"
            value: contentContainerInner.width
        }
    }
    
    states: [
        State {
            name: "expanded"
            PropertyChanges {
                target: contentContainer
                opacity: 1
            }
            PropertyChanges {
                target: contentContainer
                height: content ? (topSpacing + content.height+ bottomSpacing) : topSpacing + bottomSpacing
            }	
        },
        State {
            name: "collapsed"
            PropertyChanges {
                target: contentContainer
                opacity: 0
            }
            PropertyChanges {
                target: contentContainer
                height: 0			
            }
        }		
    ]

    transitions: [
        Transition {
            NumberAnimation {target: contentContainer; property: "height"; duration: 300}
            NumberAnimation {target: contentContainer; property: "opacity"; duration: 200}
        }
    ]
}