import QtQuick 2.1
import QtQuick.Controls 1.1
import Guicontrols 1.1

Item
{
	id: root
	property CollapsiblePanelStyle style: CollapsiblePanelStyle {}
	
	implicitWidth: Math.max(titleArrow.implicitWidth, titleArrow.implicitHeight)
	implicitHeight: Math.max(titleArrow.implicitWidth, titleArrow.implicitHeight)
	Image {
		id: titleArrow
		anchors.centerIn: parent
		transformOrigin: Item.Center
		source: style.expanderImageUrl
	}
	
	property bool expanded: state == "expanded" && !inTransition
    property bool collapsed: state == "collapsed" && !inTransition
    property bool inTransition: false
	property var transitionEnabled: true

    function collapse() {
        if(collapsed) return
        if(transitionEnabled) inTransition = true
        state = "collapsed"
    }
    
    function expand() {
        if(expanded) return
        if(transitionEnabled) inTransition = true
        state = "expanded"
    }
    
    function flipState()
    {
        if(collapsed) expand() 
        else if(expanded) collapse()
    }
	
	state: "collapsed"
	
	states: [
        State {
            name: "expanded"
            PropertyChanges {
                target: titleArrow
                rotation: 90
            }
        },
        State {
            name: "collapsed"
            PropertyChanges {
                target: titleArrow
                rotation: 0
            }
        }		
    ]

    transitions: [
        Transition {
            NumberAnimation {target: titleArrow; property: "rotation"; duration: 200}
			onRunningChanged: inTransition = running
        }
    ]
	
	MouseArea 
	{
		id: collapsingButtonArea
		anchors.fill: parent
		onClicked: root.flipState()
	}
}