import QtQuick 2.1
import QtQuick.Controls 1.1

Item {
    id: root
	default property Item contentItem
	property var circleRadius: 7
	height: width	
	implicitHeight: width
	
	property Item circle: 
		Rectangle
		{
			id: circle
			width: circleRadius * 2
			height: width
			color: "transparent"
			border.width: 2
			radius: width
			x: parent.width / 2 - circleRadius
			y: parent.height / 2 - circleRadius
			Rectangle
			{
				anchors.fill:parent
				radius: width
				color: "transparent"
				border.color: "white"
			}
			/*onXChanged: root.update()
			onYChanged: root.update()*/
		}
	
	property real minAngle: 0.0
	property real maxAngle: 1.0
	
	property real minValue: 0.0
	property real maxValue: 1.0
	
	property real angle: 0
	property real value: 0
	
	onValueChanged: updateCirclePosition()
	onAngleChanged: updateCirclePosition()
	onWidthChanged: updateCirclePosition()
	onHeightChanged: updateCirclePosition()
	
	function updateCirclePosition()
	{		
		var __angle = 0;
		if (maxAngle - minAngle != 0)
		{
			__angle = 2 * (angle - minAngle) / (maxAngle - minAngle)
		}
		var __value = 0;
		if (maxValue - minValue != 0)
		{
			__value = (value - minValue) / (maxValue - minValue)
			__value = __value * (circleRect.width)
		}

		var _y = -Math.cos(__angle * Math.PI) * __value				
		var _x = Math.sqrt(__value*__value - _y*_y)
		if (__angle > 1)
		{
			_x = -_x
		}
			
		_x = _x + (circleRect.width / 2) - circleRadius
		_y = _y + (circleRect.height / 2) - circleRadius;
		if (Math.abs(circle.x - _x) > 2)
			circle.x = _x
		if (Math.abs(circle.y - _y) > 2)
			circle.y = _y
	}
	
	function setNewAngle(__angle)
	{
		if (__angle != angle)
			angle = __angle;
	}
	
	function updateAngle(_x, _y)
	{		
		var x = _x - (circleRect.width / 2)
		var y = _y - (circleRect.height / 2)
		if (x == 0 )
		{
			setNewAngle(0);
			return 0;
		}
		var angle = (Math.atan(y / x) + Math.PI/2) / Math.PI;
		if (x < 0)
		{
			angle += 1.0;
		}
		setNewAngle(minAngle + (angle / 2.0) * (maxAngle - minAngle));			
	}

	function setNewValue(__value)
	{
		if (!__value)
			return
		if (root.value != __value)
			root.value = __value;		
	}
	
	function updateValue(_x, _y)
	{		
		var x = _x - (circleRect.width / 2)
		var y = _y - (circleRect.height / 2)
		var r = (x*x + y*y);
		setNewValue(minValue + (Math.sqrt(r) / circleRect.width) * (maxValue - minValue));
	}
	
	function update(_x, _y)
	{
		updateAngle(_x, _y)
		updateValue(_x, _y)
	}
	
	onCircleRadiusChanged: root.update(circle.x, circle.y);
	
	Rectangle
	{
		id: circleRect
		anchors.fill: parent
		anchors.margins: circleRadius
		color: "transparent"
		height: width
		radius: width
		Item
		{
			anchors.fill: parent
			data: [
				root.contentItem,
				circle
			]
		}
		
		MouseArea
		{
			anchors.fill: parent
			property var 
			onClicked: changeCirclePosition(mouse)
			onPositionChanged: changeCirclePosition(mouse)			
			function changeCirclePosition(mouse)
			{
				var x = mouse.x - (width / 2);
				var y = mouse.y - (height / 2);
				var r = (x*x + y*y);
				
				if (r > width*width / 4)
				{
					mouse.accepted = false;					
					var scale = Math.sqrt(width*width / (4*r))
					var _x = x*scale + (width / 2)
					var _y = y*scale + (height / 2)
					root.update(_x, _y)
					return;
				}								
				root.update(mouse.x, mouse.y)
			}
		}
	}
}