import QtQuick 2.2
import QtQuick.Controls 2.4
import QtQuick.Controls 1.4 as QC1
import QtQuick.Window 2.11
import MixerGuiControls 1.0

ComboBox {
    id: control
    readonly property var theme: MixerThemeManager.theme
    property alias color: label.color
    property alias clickable: label.clickable
    property alias tooltip: label.tooltip
    property alias hoveredTextColor: label.hoveredTextColor
    property alias underlineOnHovering: label.underlineOnHovering
    readonly property real textHorizontalOffset: label.x
    property int menuPadding: theme.controlPadding

    baselineOffset: contentItem.y + contentItem.baselineOffset

    font.family:    theme.fontFamily
    font.pixelSize: theme.menuFontSize

    Binding {
        target: contentItem
        property: "opacity"
        value:  control.editable && control.activeFocus ? 1.0 : 0.0
    }

    onAccepted:  focus = false
    onActivated: focus = false

    background : null
    indicator: null /*
    MixerIcon {
        anchors.verticalCenter: control.verticalCenter
        x: control.width - width - control.padding
        width:  theme.propertyHeaderHeight
        source: "icons/ArrowDown.png"
        noMouseArea: true
        color: {
            if (!control.enabled) return theme.colTextDisabled
            if (control.pressed)  return theme.colCommon
            if (control.hovered)  return theme.colNeutralLight
            return theme.colNeutral
        }

    }
*/
    Component {
        id: editComp
        QC1.TextField {
            text: control.editText
            readOnly: control.down
            validator: control.validator
            inputMethodHints: control.inputMethodHints
            onTextChanged: control.editText = text
            textColor: theme.colNeutral
/*
            Binding {
                target: __panel
                property: "visible"
                value: activeFocus
            }
            Binding {
                target: children[3]
                property: "opacity"
                value: activeFocus ? 1.0 : 0.0
            }
*/

        }
    }
    Component {
        id: selComp
        MixerLabel { }
    }

    contentItem: Loader {
        sourceComponent: control.editable ? editComp : selComp
    }

    delegate: ItemDelegate  {
        width: control.width
        padding: 0
        implicitHeight: theme.menuHeight
        //color: control.highlightedIndex === index ?  MixerThemeManager.theme.colCommon : MixerThemeManager.theme.colNeutral
        text: control.textRole ? (Array.isArray(control.model) ? modelData[control.textRole] : model[control.textRole]) : modelData
        contentItem: Text {
            text: parent.text
            font: control.font
            elide: Text.ElideRight
            verticalAlignment: Text.AlignVCenter
            color: highlighted ? theme.colCommon : theme.colNeutral
        }
        background: null
        highlighted: control.highlightedIndex === index
        hoverEnabled: control.hoverEnabled
    }


    popup: Popup {
        x: control.x - menuPadding
        y: control.height + menuPadding
        width: control.width + menuPadding
        height: Math.min(contentItem.implicitHeight, control.Window.height - topMargin - bottomMargin)
        leftPadding: menuPadding
        topPadding : 0
        bottomPadding: topPadding

        background: Rectangle {
            color: theme.colNeutralDark
            border.color: theme.colNeutral
            border.width: 0
        }

        contentItem: ListView {
            clip: true
            implicitHeight: contentHeight
            spacing: 0
            model: control.delegateModel
            currentIndex: control.highlightedIndex
            ScrollIndicator.vertical: ScrollIndicator {}
        }
     }

    HyperLinkLabel {
        id: label
        visible: !control.editable || !control.activeFocus
        text: control.currentText
        width: control.width
        //property var item: control.children[3]
        //anchors.left: item.left
        anchors.verticalCenter: parent.verticalCenter
        clickable: false
    }
}
