import QtQuick 2.1
import QtQuick.Controls 1.0
import QtQuick.Controls.Styles 1.1

SpinBox {
    id: root
    property color hoveredTextColor: "white"
    property bool underlineOnHovering: false
    property int  digits: 0
    property string compactText: textItem.text
    readonly property Item backgroundItem: __panel
    readonly property Item textItem: children[5]

    readonly property real textHorizontalOffset: textItem.x
    signal wheel()
/*
    Binding on implicitWidth {
        value: label.implicitWidth
        when: label.visible
    }
*/
    Component.onCompleted: update()
    onValueChanged: update()

    function update() {
        if (!digits) return;
        var decimals = digits - Math.floor(Math.log(Math.abs(value)+0.0001)/Math.LN10);
        root.decimals = decimals = Math.max(0,Math.min(decimals,digits));
        var step = 1/Math.pow(10,decimals); if (root.stepSize<step) stepSize = step;
    }

    horizontalAlignment: Qt.AlignLeft
    Binding {
        target: backgroundItem
        property: "visible"
        value: activeFocus
    }
    Binding {
        target: textItem
        property: "visible"
        value: activeFocus
    }
    Label {
        id: label
        visible: !root.activeFocus
        anchors.fill: root.textItem
        textFormat: Text.RichText
        property color textColor: hoveredLink == "label" ? root.hoveredTextColor : color
        property string textDecoration: hoveredLink == "label" && root.underlineOnHovering ? "underline" : "none"
        text: "<a href='label' style='text-decoration: " + textDecoration + "; color:" + textColor + "'>" + root.compactText + "</a>"
        verticalAlignment : Text.AlignVCenter
    }

    MouseArea {
        id: wheelArea
        property bool focused: root.activeFocus
        anchors.centerIn: parent
        width: focused ? 1000 : parent.width
        height: focused ? 1000 : parent.height
        acceptedButtons: Qt.NoButton
        onWheel: {
            if (wheel.angleDelta.y > 0) {
                if(root.value >= root.maximumValue) return
                root.__increment()
            }
            else {
                if(root.value <= root.minimumValue) return
                root.__decrement()
            }
            wheel.accepted = true
            root.wheel()
        }
    }
    onEditingFinished: focus = false
}
