import QtQuick 2.0
import QtQuick.Layouts 1.1
import QtQuick.Controls 1.1
import GuiControls 1.0
import "."

Item {
    id: root
    implicitWidth: layout.implicitWidth
    implicitHeight: layout.implicitHeight
    property bool showAlphaChannel: false
    property color color: "black"
    property color currentColor
    
    property bool __recalculation: false

    onCurrentColorChanged: setCurrentColorToControls()
    onVisibleChanged: if(visible) setCurrentColorToControls()
    
    function setCurrentColorToControls() {
        __recalculation = true
        var hsla = rgbaToHsla(currentColor)
        if(hsla.b == 0 || hsla.g == 0) {
            hsla.r = hueSlider.value
            hsla.g = slPicker.saturation
        }
        if(hsla.r == 0 && hueSlider.value == 1) hsla.r = 1
        hueSpinBox.value = hsla.r * 255
        saturationSpinBox.value = hsla.g * 255
        lightnessSpinBox.value = hsla.b * 255
        slPicker.hue = hsla.r
        slPicker.saturation = hsla.g
        slPicker.lightness = hsla.b
        hueSlider.value = hsla.r
        alphaSlider.value = currentColor.a       
        redSpinBox.value = currentColor.r * 255
        greenSpinBox.value = currentColor.g * 255
        blueSpinBox.value = currentColor.b * 255
        alphaSpinBox.value = currentColor.a * 255        
        __recalculation = false
    }
    
    function setFocusToControls() {
        if(!saturationSpinBox.activeFocus && !lightnessSpinBox.activeFocus && !alphaSpinBox.activeFocus && !redSpinBox.activeFocus && !greenSpinBox.activeFocus && !blueSpinBox.activeFocus)
            hueSpinBox.forceActiveFocus()
    }
    
    ColumnLayout {
        id: layout
        anchors.fill: parent
        RowLayout {
            spacing: 4
            Layout.alignment: Qt.AlignHCenter
            SLPicker {
                id: slPicker
                Layout.preferredWidth: 300
                Layout.preferredHeight: 300
                hue: hueSlider.value
                onSaturationChanged: if(!__recalculation) currentColor = Qt.hsla(hueSlider.value, slPicker.saturation, slPicker.lightness, alphaSlider.value)
                onLightnessChanged: if(!__recalculation) currentColor = Qt.hsla(hueSlider.value, slPicker.saturation, slPicker.lightness, alphaSlider.value)
            }
            
            HueSlider {
                id: hueSlider
                Layout.preferredHeight: 300
                onValueChanged: if(!__recalculation) currentColor = Qt.hsla(hueSlider.value, slPicker.saturation, slPicker.lightness, alphaSlider.value)
            }
            Item {
                implicitWidth: 1
            }  
            AlphaSlider {
                visible: showAlphaChannel
                id: alphaSlider
                Layout.preferredHeight: 300
                onValueChanged: if(!__recalculation) currentColor = Qt.hsla(hueSlider.value, slPicker.saturation, slPicker.lightness, alphaSlider.value)
            }           
            
        }
         Item {
            implicitHeight: 10
        }        
        RowLayout {
            Layout.alignment: Qt.AlignHCenter
            GridLayout {
                Layout.alignment: Qt.AlignTop
                columns: 2
                rowSpacing: 0
                Label {
                    //: Hue (HSL)
                    text: qsTr("H:")
                }
                CompactSpinBox {
                    id: hueSpinBox
                    maximumValue: 255
                    onValueChanged: hueSlider.value = value / 255
                }
                Label {
                    //: Saturation (HSL)
                    text: qsTr("S:")
                }            
                 CompactSpinBox {
                    id: saturationSpinBox
                    maximumValue: 255
                    onValueChanged: if(!__recalculation) currentColor = Qt.hsla(hueSpinBox.value / 255, saturationSpinBox.value / 255, lightnessSpinBox.value / 255, alphaSlider.value)
                }
                Label {
                    //: Lightness (Intensity) (HSL)
                    text: qsTr("L:")
                }            
                CompactSpinBox {
                    id: lightnessSpinBox
                    maximumValue: 255
                    onValueChanged: if(!__recalculation) currentColor = Qt.hsla(hueSpinBox.value / 255, saturationSpinBox.value / 255, lightnessSpinBox.value / 255, alphaSlider.value)
                }
            }
            Item {
                implicitWidth: 10
            }                
            GridLayout {
                Layout.alignment: Qt.AlignTop
                columns: 2
                rowSpacing: 0
                Label {
                    //: Red (RGB)
                    text: qsTr("R:")
                }
                CompactSpinBox {
                    id: redSpinBox
                    maximumValue: 255
                    onValueChanged: if(!__recalculation) currentColor = Qt.rgba(redSpinBox.value / 255, greenSpinBox.value / 255, blueSpinBox.value / 255, alphaSlider.value)
                }
                Label {
                    //: Green (RGB)
                    text: qsTr("G:")
                }            
                 CompactSpinBox {
                    id: greenSpinBox
                    maximumValue: 255
                    onValueChanged: if(!__recalculation) currentColor = Qt.rgba(redSpinBox.value / 255, greenSpinBox.value / 255, blueSpinBox.value / 255, alphaSlider.value)
                }
                Label {
                    //: Blue (RGB)
                    text: qsTr("B:")
                }            
                CompactSpinBox {
                    id: blueSpinBox
                    maximumValue: 255
                    onValueChanged: if(!__recalculation) currentColor = Qt.rgba(redSpinBox.value / 255, greenSpinBox.value / 255, blueSpinBox.value / 255, alphaSlider.value)
                }
                Label {
                    visible: showAlphaChannel
                    //: Alpha/opacity (RGBA) 
                    text: qsTr("A:")
                }            
                CompactSpinBox {
                    visible: showAlphaChannel
                    id: alphaSpinBox
                    maximumValue: 255
                    onValueChanged: if(!__recalculation) currentColor = Qt.rgba(redSpinBox.value / 255, greenSpinBox.value / 255, blueSpinBox.value / 255, alphaSpinBox.value / 255)
                }                
            }  
            Item {
                implicitWidth: 4
            }
            RowLayout {
                Layout.alignment: Qt.AlignTop
                spacing: 3
                ColumnLayout {
                    spacing: 3
                    Label {
                        Layout.alignment: Qt.AlignHCenter
                        text: qsTr("current")
                    }
                    ChessBoard {
                        Layout.alignment: Qt.AlignHCenter
                        Layout.preferredWidth: 50
                        Layout.preferredHeight: 36
                        checkSize: 6
                        colorBlack: "gray"
                        colorWhite: "white"
                        Rectangle {
                            anchors.fill: parent
                            color: root.color
                            border.color: Qt.rgba(40/255, 40/255, 40/255)
                        }
                    }
                }
                ColumnLayout {
                    spacing: 3
                    Label {
                        Layout.alignment: Qt.AlignHCenter
                        text: qsTr("new")
                    }
                    ChessBoard {
                        Layout.alignment: Qt.AlignHCenter
                        Layout.preferredWidth: 50
                        Layout.preferredHeight: 36
                        checkSize: 6
                        colorBlack: "gray"
                        colorWhite: "white"
                        Rectangle {
                            anchors.fill: parent
                            color: root.currentColor
                            border.color: Qt.rgba(40/255, 40/255, 40/255)
                        }
                    }
                }          
            }
        }
    }
    
    function rgbaToHsla(color){
        var max = Math.max(color.r, color.g, color.b), min = Math.min(color.r, color.g, color.b);
        var h, s, l = (max + min) / 2;

        if(max == min){
            h = s = 0;
        }else{
            var d = max - min;
            s = l > 0.5 ? d / (2 - max - min) : d / (max + min);
            switch(max){
                case color.r: h = (color.g - color.b) / d + (color.g < color.b ? 6 : 0); break;
                case color.g: h = (color.b - color.r) / d + 2; break;
                case color.b: h = (color.r - color.g) / d + 4; break;
            }
            h /= 6;
        }
        return Qt.rgba(h, s, l, color.a);
    }
}