import QtQuick 2.0

Item {
    id: root
    property real hue : 0.5
    property real saturation : 0.5    
    property real lightness : 0.5

    implicitWidth: 300
    implicitHeight: 300
    clip: true

    property bool externalChange: false
    
    onSaturationChanged: {
        externalChange = true
        pickerCursor.x = saturation * root.width
        externalChange = false    
    }
    onLightnessChanged: {
        externalChange = true
        pickerCursor.y = (1 - lightness) * root.height
        externalChange = false
    }    
    
    ShaderEffect {
        anchors.fill: parent
        layer.enabled: true
        layer.smooth: true
        
        property real hue: root.hue
        fragmentShader: "
        varying mediump vec2 qt_TexCoord0;
        uniform highp float qt_Opacity;
        uniform highp float hue;

        highp float hueToIntensity(highp float v1, highp float v2, highp float h) {
            h = fract(h);
            if (h < 1.0 / 6.0)
                return v1 + (v2 - v1) * 6.0 * h;
            else if (h < 1.0 / 2.0)
                return v2;
            else if (h < 2.0 / 3.0)
                return v1 + (v2 - v1) * 6.0 * (2.0 / 3.0 - h);

            return v1;
        }

        highp vec3 HSLtoRGB(highp float h, highp float l, highp float s) {
            if (s < 1.0 / 256.0)
                return vec3(l, l, l);

            highp float v1;
            highp float v2;
            if (l < 0.5)
                v2 = l * (1.0 + s);
            else
                v2 = (l + s) - (s * l);

            v1 = 2.0 * l - v2;

            highp float d = 1.0 / 3.0;
            highp float r = hueToIntensity(v1, v2, h + d);
            highp float g = hueToIntensity(v1, v2, h);
            highp float b = hueToIntensity(v1, v2, h - d);
            return vec3(r, g, b);
        }

        void main() {
            lowp vec4 c = vec4(1.0);
            c.rgb = HSLtoRGB(hue, 1.0 - qt_TexCoord0.t, qt_TexCoord0.s);
            gl_FragColor = c * qt_Opacity;
        }
        "        
    }
    
    Item {
        id: pickerCursor
        property int r : 8
        Rectangle {
            x: -parent.r; y: -parent.r
            width: parent.r*2; height: parent.r*2
            radius: parent.r
            border.color: "black"
            color: "transparent"
            Rectangle {
                anchors.fill: parent
                anchors.margins: 1
                border.color: "white"
                radius: width/2
                color: "transparent"
                Rectangle {
                    anchors.fill: parent
                    anchors.margins: 1
                    border.color: "black"
                    radius: width/2
                    color: "transparent"
                }
            }
        }
        
        onXChanged: if(!externalChange) saturation = pickerCursor.x / root.width
        onYChanged: if(!externalChange) lightness = 1 - pickerCursor.y / root.height       
    }
    MouseArea {
        anchors.fill: parent
        function handleMouse(mouse) {
            if (mouse.buttons & Qt.LeftButton) {
                pickerCursor.x = Math.max(0, Math.min(width,  mouse.x));
                pickerCursor.y = Math.max(0, Math.min(height, mouse.y));
            }
        }
        onPositionChanged: handleMouse(mouse)
        onPressed: handleMouse(mouse)
    }
    Rectangle {
        color: "transparent"
        border.color: Qt.rgba(40/255, 40/255, 40/255)
        anchors.fill: parent
    }
}

