import QtQuick 2.0
import QtQuick.Controls 1.0
import QtQuick.Window 2.2
import MixerGuiControls 1.0
    
ApplicationWindow {
    id: root
    flags: Qt.FramelessWindowHint | Qt.WindowSystemMenuHint | Qt.WindowMinMaxButtonsHint | Qt.Window 

    property int headerHeight: 30
    property int borderSize: 5
    property int cornerSize: 10
    property var margins: [0, 0, 0, 0]
    property bool resizeEnabled: true
    property bool maximizeOnHeaderDoubleClick: true

    default property alias data: content.data    

    MouseArea {
        anchors.top: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        height: root.headerHeight
        anchors.leftMargin: root.margins[0]
        anchors.topMargin: root.margins[1]
        anchors.rightMargin: root.margins[2]
        anchors.bottomMargin: root.margins[3]         
        
        property point clickPos
        onPressed: {
            clickPos  = Qt.point(mouse.x, mouse.y)
        }   
        onPositionChanged: {
            if(root.visibility == Window.Maximized) return
            root.x += mouse.x - clickPos.x
            root.y += mouse.y - clickPos.y
        }
        onDoubleClicked: if(maximizeOnHeaderDoubleClick) root.visibility = (root.visibility == Window.Maximized ? Window.Windowed : Window.Maximized)
    }
    
    WindowResizeControl {
        window: root
        anchors.fill: parent
        anchors.leftMargin: root.margins[0]
        anchors.topMargin: root.margins[1]
        anchors.rightMargin: root.margins[2]
        anchors.bottomMargin: root.margins[3]         
        borderSize: root.borderSize
        cornerSize: root.cornerSize
        enabled: root.resizeEnabled && root.visibility != Window.Maximized
    }    
    
    Item {
        id: content
        anchors.fill: parent
        anchors.leftMargin: root.margins[0]
        anchors.topMargin: root.margins[1]
        anchors.rightMargin: root.margins[2]
        anchors.bottomMargin: root.margins[3] 
    }
    
    Timer {             // Обход бага, окно плохо перерисовывается при ресайзе
        id: windowUpdateTimer
        interval: 1
        onTriggered: root.update()
    }
    onWidthChanged: windowUpdateTimer.restart()    
    onHeightChanged: windowUpdateTimer.restart() 

}