import QtQuick 2.0
import QtQuick.Window 2.2
import QtQuick.Layouts 1.1

Item {
    id: root
    property Window window
    property int borderSize: 5
    property int cornerSize: 10

    function setWindowWidth(value) {
        window.width = Math.max(window.minimumWidth, Math.min(value, window.maximumWidth))
    }
    
    function setWindowHeight(value) {
        window.height = Math.max(window.minimumHeight, Math.min(value, window.maximumHeight))
    }
    
    ResizeControlElement {
        width: root.width - root.borderSize * 2
        height: root.borderSize
        anchors.horizontalCenter: root.horizontalCenter
        cursorShape: Qt.SizeVerCursor
        onDragged: if(window) {
            var prevWindowHeight = window.height
            root.setWindowHeight(window.height - delta.y)
            window.y += (prevWindowHeight - window.height)
        }              
    }        
    
    ResizeControlElement {
        width: root.borderSize
        height: root.height - root.borderSize * 2
        anchors.verticalCenter: root.verticalCenter
        cursorShape: Qt.SizeHorCursor
        onDragged: if(window) {
            var prevWindowWidth = window.width
            root.setWindowWidth(window.width - delta.x)
            window.x += (prevWindowWidth - window.width)
        }            
    } 

    ResizeControlElement {
        width: root.borderSize
        height: root.height - root.borderSize * 2
        anchors.verticalCenter: root.verticalCenter
        anchors.right: root.right
        cursorShape: Qt.SizeHorCursor
        onDragged: if(window) {
            root.setWindowWidth(window.width + delta.x)
        }
    }          
    

    ResizeControlElement {
        width: root.width - root.borderSize * 2
        height: root.borderSize
        anchors.horizontalCenter: root.horizontalCenter
        anchors.bottom: root.bottom
        cursorShape: Qt.SizeVerCursor
        onDragged: if(window) {
            root.setWindowHeight(window.height + delta.y)
        }            
    }         
    
    ResizeControlElement { 
        width: root.cornerSize
        height: root.cornerSize
        cursorShape: Qt.SizeFDiagCursor
        onDragged: if(window) {
            var prevWindowHeight = window.height
            root.setWindowHeight(window.height - delta.y)
            window.y += (prevWindowHeight - window.height)
            var prevWindowWidth = window.width
            root.setWindowWidth(window.width - delta.x)
            window.x += (prevWindowWidth - window.width)                
        }            
    }    
    
    ResizeControlElement {
        width: root.cornerSize
        height: root.cornerSize
        anchors.right: parent.right
        cursorShape: Qt.SizeBDiagCursor
        onDragged: if(window) {
            root.setWindowWidth(window.width + delta.x)
            var prevWindowHeight = window.height
            root.setWindowHeight(window.height - delta.y)
            window.y += (prevWindowHeight - window.height)              
        }              
    }      
    
    ResizeControlElement {
        width: root.cornerSize
        height: root.cornerSize
        cursorShape: Qt.SizeBDiagCursor
        anchors.bottom: parent.bottom
        onDragged: if(window) {
            var prevWindowWidth = window.width
            root.setWindowWidth(window.width - delta.x)
            window.x += (prevWindowWidth - window.width)          
            root.setWindowHeight(window.height + delta.y)
        }              
    }    
    
    ResizeControlElement {
        width: root.cornerSize
        height: root.cornerSize
        anchors.right: parent.right
        anchors.bottom: parent.bottom
        cursorShape: Qt.SizeFDiagCursor
        onDragged: if(window) {
            root.setWindowWidth(window.width + delta.x)
            root.setWindowHeight(window.height + delta.y)
        }
    }
}