import QtQml 2.2
import QtQuick 2.1

QtObject {
    id: root
    property QtObject target
    property var originalState
    property string type: "BaseAnimation"
    property string name: qsTr("Base Animation")
    property int duration: 1000
    property int pauseBefore: 0
    property int pauseAfter: 0
    property bool startWithPrevious: false
    property var easingType: Easing.Linear
    property real easingAmplitude: 1
    property real easingOvershoot: 1.70158
    property real easingPeriod: 0.3

    property real t
    property real progress
    property bool running
    property bool seeking
    property int startTime
    property QtObject startAfter: null
    readonly property alias seekAnimation: seekAnimationInstantiator.object
    readonly property AnimationController seekAnimationController: seekAnimation ? seekAnimation.animationController : null
    
    signal finished()

    property Animation animation: SequentialAnimation {
        id: animation
        property bool _stopping: false
        PauseAnimation { 
            duration: root.pauseBefore
        }
        ParallelAnimation {
            NumberAnimation {
                target: root
                property: "t"
                duration: Math.max(1, root.duration)
                easing.type: root.easingType
                easing.amplitude: root.easingAmplitude
                easing.overshoot: root.easingOvershoot
                easing.period: root.easingPeriod             
                from: 0
                to: 1
            }
            NumberAnimation {
                target: root
                property: "progress"
                duration: Math.max(1, root.duration)
                from: 0
                to: 1
            }            
        }
        PauseAnimation { 
            duration: root.pauseAfter
        }        
        onStopped: {
            root.running = false
            root.t = 1
            root.progress = 1
            if(!_stopping) root.finished()
        }
    }

    property QtObject seekAnimationInstantiator: Instantiator {
        id: seekAnimationInstantiator
        active: root.seeking
        NumberAnimation {
            id: seekAnimation
            property AnimationController animationController: AnimationController {
                animation: seekAnimation
                function setProgress(p) {
                    if(progress == 0) progress = 1
                    else progress = progress * 0.5
                    progress = p
                    root.progress = progress
                }
            }
            target: root
            property: "t"
            duration: Math.max(1, root.duration)
            easing.type: root.easingType
            easing.amplitude: root.easingAmplitude
            easing.overshoot: root.easingOvershoot
            easing.period: root.easingPeriod             
            from: 0
            to: 1
        }
    }    
    
    function start() {
        t = 0
        progress = 0
        running = true
        animation.restart()
    }
    
    function beforeStart() {}   // override in child
    
    function stop() {
        if(animation.running) {
            animation._stopping = true
            animation.stop()
            t = 0
            progress = 0            
            animation._stopping = false
        }
    }
    
    function beginSeek() {
        seeking = true
    }
    
    function seek(time) {
        if(!seeking) return
        var progress = (time - pauseBefore) / duration
        progress = Math.max(0, Math.min(progress, 1))
        if(time >= 0 || !startAfter) beforeStart()
        if(time >= 0) seekAnimationController.setProgress(progress)
    }
    
    function endSeek() {
        if(!seeking) return
        seeking = false
        t = 0
        progress = 0
    }
    
    function baseAnimation_save() {
        return {
            type: type,
            duration: duration,
            pauseBefore: pauseBefore,
            pauseAfter: pauseAfter,
            startWithPrevious: startWithPrevious,
            easingType: easingType,
            easingAmplitude: easingAmplitude,
            easingOvershoot: easingOvershoot,
            easingPeriod: easingPeriod          
        }
    }
    
    function save() {
        return baseAnimation_save()
    }
    
    function baseAnimation_load(data) {
        type = data.type
        duration = data.duration
        pauseBefore = data.pauseBefore
        pauseAfter = data.pauseAfter
        startWithPrevious = data.startWithPrevious
        easingType = data.easingType
        easingAmplitude = data.easingAmplitude
        easingOvershoot = data.easingOvershoot
        easingPeriod = data.easingPeriod          
    }    
    
    function load(data) {
        baseAnimation_load(data)
    }
}