import QtQuick 2.3
import GuiControls 1.0
import QtQml 2.2

Item
{
	id: root
	property GraphicalScene scene
	property Component objectComponent
		
	property GraphicalObject currentObject: null
	
	property SelectionManager selectionManager
	
	property bool __shiftPressed: false
	property bool __controlPressed: false
	
	readonly property bool shiftPressed: __shiftPressed
	readonly property bool controlPressed: __controlPressed

	readonly property bool creationEnabled: ((!__controlPressed) || (ma.drag.active))
	signal pressed(var mouse)
	
    focus: visible
    onFocusChanged:
	{
		if (!focus)
		{
			__shiftPressed = false
			__controlPressed = false
		}
    }
	MouseArea
	{
		id: ma
		visible: root.objectComponent && ((!__controlPressed) || (ma.drag.active))
		cursorShape: Qt.CrossCursor
		anchors.centerIn: parent
		width: 100000
		height: 100000		
		hoverEnabled: true
		property var startX: 0
		property var startY: 0
		
		property var endX: 0
		property var endY: 0
		
		property bool keepAspect: false
		property real aspect: root.currentObject ? (root.currentObject.aspect ? root.currentObject.aspect : 1.0) : 1.0
		readonly property rect selectedRect: keepAspect ? aspectedRect : mouseRect
		
		
		readonly property real aspectedWidth: (mouseRect.width > (mouseRect.height * aspect)) ? mouseRect.height * aspect : mouseRect.width
		readonly property real aspectedHeight: ((mouseRect.width / aspect) > mouseRect.height) ? (mouseRect.height) : (mouseRect.width / aspect)
		readonly property rect aspectedRect:
			Qt.rect( 
				mouseRect.x + ((ma.startX > ma.endX) ? (mouseRect.width - aspectedWidth) : 0),
				mouseRect.y + ((ma.startY > ma.endY) ? (mouseRect.height - aspectedHeight) : 0),
				aspectedWidth,
				aspectedHeight)
				
		readonly property rect mouseRect: 
			Qt.rect( 
				Math.min(ma.startX, ma.endX),
				Math.min(ma.startY, ma.endY),
				Math.abs(ma.startX - ma.endX),
				Math.abs(ma.startY - ma.endY))
		
		onPressed:
        {
            //root.focus = true
            if (root.__shiftPressed != (mouse.modifiers & Qt.ShiftModifier)) {
                root.__shiftPressed = (mouse.modifiers & Qt.ShiftModifier);
            }
            if (root.__controlPressed != (mouse.modifiers & Qt.ControlModifier)) {
                root.__controlPressed = (mouse.modifiers & Qt.ControlModifier);
            }

			if (mouse.buttons & Qt.LeftButton)
			{
				startX = mouse.x - 50000 + parent.width /2
				startY = mouse.y - 50000 + parent.height /2
				endX = startX
				endY = endY				
			}
			root.pressed(mouse)
		}
		
		onPositionChanged:
        {
            if (root.__shiftPressed != (mouse.modifiers & Qt.ShiftModifier)) {
                root.__shiftPressed = (mouse.modifiers & Qt.ShiftModifier);
            }

            if (root.__controlPressed != (mouse.modifiers & Qt.ControlModifier)) {
                root.__controlPressed = (mouse.modifiers & Qt.ControlModifier);
            }
			if (mouse.buttons & Qt.LeftButton)
			{
				endX = mouse.x - 50000 + parent.width /2
				endY = mouse.y - 50000 + parent.height /2
				if (!currentObject)
				{
					currentObject = createObject()
					if (currentObject)
					{
						scene.add(currentObject)
						currentObject.hovered = true
						if (selectionManager)
							selectionManager.selectItem(currentObject, true)
					}
				}				
			}
			keepAspect = mouse.modifiers & Qt.ShiftModifier
		}
		
		onReleased:
		{		
            if (root.__shiftPressed != (mouse.modifiers & Qt.ShiftModifier)) {
                root.__shiftPressed = (mouse.modifiers & Qt.ShiftModifier);
            }

            if (root.__controlPressed != (mouse.modifiers & Qt.ControlModifier)) {
                root.__controlPressed = (mouse.modifiers & Qt.ControlModifier);
            }
            if(currentObject && scene && scene.undoManager) scene.undoManager.pushNewState()
			currentObject = null
		}

		function createObject()
		{			
			if (!root.objectComponent) return

			var object = root.objectComponent.createObject(root,
				{
					"scene": root.scene,
					"x": selectedRect.x,
					"y": selectedRect.y, 
					"width": selectedRect.width,
					"height": selectedRect.height
				});

			if (object) object.name = scene.getUniqueName(object.name)	
			return object	
		}
		
		Binding
		{
			target: currentObject
			property: "x"
			value: ma.selectedRect.x
		}
		Binding
		{
			target: currentObject
			property: "y"
			value: ma.selectedRect.y
		}		
		Binding
		{
			target: currentObject
			property: "width"
			value: ma.selectedRect.width
		}
		Binding
		{
			target: currentObject
			property: "height"
			value: ma.selectedRect.height
		}
	}

	Keys.onPressed:
	{
		if (event.modifiers & Qt.ShiftModifier)
		{
			root.__shiftPressed = true
		}
		if (event.modifiers & Qt.ControlModifier)
		{
			root.__controlPressed = true
		}
		event.accepted = false
	}
	Keys.onReleased:
	{
		if (!(event.modifiers & Qt.ShiftModifier))
		{
			root.__shiftPressed = false
		}
		if (!(event.modifiers & Qt.ControlModifier))
		{
			root.__controlPressed = false
		}
		event.accepted = false
	}

}
