import QtQml 2.0
import QtQuick 2.4
import "mixer_media"
import "graphical_objects"

QtObject
{	
	id: root
	property var objects: []
    property var parentScene: null
	property GroupNode node: GroupNode {
        objParent: root 
        scene: root
    }
    property string name: "Composition"
    property int curNumber: 1
	property real width: 1920
	property real height: 1080
    property real aspect: 1./1.   //16./9.
	property EditorSettings editorSettings: EditorSettings{}
    property int id: 1
    property var usingObjects: []
    property bool cacheEnabled: true
    property real texturesQuality: 1.0
    
	property UndoRedoSceneManager undoManager: 
		UndoRedoSceneManager
		{
			scene: root
		}
        
    property TextTable textTable: TextTable { }
    property var mainTables: []

    property var allTextTables: {
        var res = []
        res.push(textTable) //default table always added
        for (var t in mainTables) {
            var tab = mainTables[t]
            if (tab.columnCount)
                res.push(tab)
        }
        for (var i in objects) {
            var obj = objects[i]
            if (obj.nestedScene!=undefined && obj.nestedScene.textTable.columnCount)
               res.push(obj.nestedScene.textTable)
        }
        return res
    }
    property int controlTablesCount: {
        var count=0
        for (var i in allTextTables) {
            var tab = allTextTables[i]
            if (tab.columnCount>0 && tab.controlInMixer)
                count++
        }
        return count
    }

    readonly property Component tableComp: TextTable {}

    function findMainTable(type, name) {
        if ((!type || textTable.type==type) && (!name || textTable.name==name))
            return textTable
        for (var t in mainTables) {
            var tab = mainTables[t]
            if ((!type || tab.type==type) && (!name || tab.name==name))
                return tab
        }
        return null
    }
    function getMainTable(type, name) {
        var res = findMainTable(type, name)
        if (!res) res = addMainTable(type, name)
        return res
    }
    function addMainTable(type, name) {
        var table = tableComp.createObject(root)
        if (type) table.type = type
        if (name) table.name = name
        table.onNoUsers.connect(removeMainTable.bind(root, table))
        mainTables.push(table)
        mainTables = mainTables
        //console.warn("Table added", table, type)
        return table
    }
    function removeMainTable(table) {
        for (var i in mainTables)
            if (mainTables[i]==table) {
                mainTables.splice(i, 1)
                console.warn("Table removed", table)
                table.destroy()
                mainTables = mainTables
                return
            }
    }


    property bool playMedia: true
    property bool loadMedia: true
    property bool animationEnabled: true
    
    property MixerMediaManager mixerMediaManager: EmptyMixerMediaManager{}
    
    readonly property GraphicalSceneAnimator animator: GraphicalSceneAnimator {
        animationEnabled: root.animationEnabled
        objectAnimators: {
            var result = []
            if(!objects) return result
            for(var i = 0; i < objects.length; i++) {
                result.push(objects[i].animator)
                var v = objects[i]
                if (!(v=v.nestedScene    )) continue
                if (!(v=v.animator       )) continue 
                if (!(v=v.objectAnimators)) continue
                for (var j in v) result.push(v[j])
            }
            return result
        }
    }    

    signal updated()
    onUpdated: {
        objects = node.getChildrenRecurs();
    }
    
    function add(object) {
        return node.add(object);
    }
    function deleteObject(object) {
        if (!object) return;
        var objectParent = object.getParent();
        if (objectParent === this) {
            return node.deleteObject(object);
        }
        else {
            return objectParent.deleteObject(object);
        }
    } 
    function deleteObjects(objects) {
        if (objects.length === 0) return;
        var objectsParent = objects[0].getParent();
        if (objectsParent === this) {
            return node.deleteObjects(objects);
        }
        else {
            return objectsParent.deleteObjects(objects);
        }
    }
    function deleteAllObjects(objects) {
        return node.deleteAllObjects(objects)
    }
    function groupObjects(objects) {
        if (objects.length === 0) return null;
        var objectsParent = objects[0].getParent();
        if (objectsParent === this) {
            return node.groupObjects(objects);
        }
        else {
            return objectsParent.groupObjects(objects);
        }
    }
    function mergeInGroup(objectList) {
        return node.mergeInGroup(objectList);
    }
    function getChildren(reverse) {
        return node.getChildren(reverse);
    }
    function getChildrenRecurs(reverse) {
        return node.getChildrenRecurs(reverse);
    }
    function getGlobalCoord() {
        return Qt.vector2d(0, 0);
    }
    function getParent() {
        return this;
    }
    function bringLevelUp(items) {
        return node.bringLevelUp(items);
    }    
    function bringLevelDown(items) {
        return node.bringLevelDown(items);
    }   
    function bringLevelTop(items) {
        return node.bringLevelTop(items);
    }
    function getGlobalAngle() {
        return 0;
    }
    function isOnTopLevel(items) {
        return node.isOnTopLevel(items);
    }
    function isOnDownLevel(items) {
        return node.isOnDownLevel(items);
    }
    function bringLevelBottom(items) {
        return node.bringLevelBottom(items);
    }
    function getNextObject(object) {
        return node.getNextObject(object);
    }
    function getPrevObject(object) {
        return node.getPrevObject(object);
    }
    function addGroup() {
        return node.addGroup();
    }
    function swapObjectsToParent(listObjects, newParent) {
        return node.swapObjectsToParent(listObjects, newParent);
    }
    function swapToParent(object, newParent) {
        return node.swapToParent(object, newParent);
    }
    function getIndex(object) {
        return node.getIndex(object);
    }
    function setObjectsPosition(objects, position) {
        return node.setObjectsPosition(objects, position);
    }
    function getUniqueName(name) {
        var names = []
        for (var i in objects)  names.push(objects[i].name)
        if (names.indexOf(name)==-1)
            return name
        var s = name.replace(/\d+$/,'')
        var n =+name.replace(s,'');

        for (var i=1; i<=names.length; i++) {
            var res = s+(n+i);
            if (names.indexOf(res)==-1)
                return res
        }
        return name
    }
	function save()
	{		
        animator.stop()
		var objectsData = []
        var children = getChildren();
        for (var index in children) {
			var object = children[index]			
			objectsData.push(object.save())
		}
		var data = {
			name: root.name,
			width: root.width,
			height: root.height,
			aspect: root.aspect,
			objects: objectsData,
			editorSettings: editorSettings.save(),
            textTable: textTable.save(),
            id: root.id
		}
		return data
	}
	
	function saveObjects(objectList)
	{		
        animator.stop()
		if (!objectList)
			return []
		var objectsData = []
		var origin = calcOrigin(objectList)		
		for (var index in objectList)
		{
			var object = objectList[index]
			var data = object.save()
			data.x -= origin.x
			data.y -= origin.y
			objectsData.push(data)
        }
		return objectsData
	}
	
	function calcOrigin(objectList)
	{		
		if (!objectList)
			return Qt.vector2d(0,0)
		var objectsData = []
		if (objectList.length == 0)
			return Qt.vector2d(0,0)
		var x = objectList[0].x
		var y = objectList[0].y
		for (var index in objectList)
		{
			var object = objectList[index]
			x = Math.min(x, object.x)
			y = Math.min(y, object.y)
		}
		return Qt.vector2d(x,y)
	}
    function calcMaxPoint(objectList)
    {
        if (!objectList)
            return Qt.vector2d(0,0)
        var objectsData = []
        if (objectList.length == 0)
            return Qt.vector2d(0,0)
        var x = objectList[0].x + objectList[0].width
        var y = objectList[0].y + objectList[0].height
        for (var index in objectList)
        {
            var object = objectList[index]
            x = Math.max(x, object.x + object.width)
            y = Math.max(y, object.y + object.height)
        }
        return Qt.vector2d(x,y)
    }

	function load(config)
	{
		root.width = config.width
		root.height = config.height
		root.aspect = config.aspect
		root.name = config.name
		root.editorSettings.load(config.editorSettings)
		root.textTable.load(config.textTable)
        root.id = config.id || 10000
		loadObjects(config.objects)		
	}
    function addObjects(data, x, y, uniqName)
	{
		var newObjects = []
        var prevCacheEnabled = cacheEnabled
        cacheEnabled = false
		for (var index in data)
		{
            var newObj = createGraphicalObject(data[index])
			if (newObj)
			{
				newObj.x += x
                newObj.y += y
                if (uniqName)
                    newObj.name = getUniqueName(newObj.name)
                add(newObj)
                newObj.resetId()
				newObjects.push(newObj)
			}
		}
        undoManager.pushNewState()
        cacheEnabled = prevCacheEnabled
		return newObjects
	}
	
	function loadObjects(data)
	{
        usingObjects = []
        var newObjects = []
		for (var index in data)
		{            
            var newObj = null
            if (data[index].id)
            {
                newObj = findObjectById(data[index].id)
                if (newObj)
                {
                    if (data[index].typeName == newObj.typeName)
                    {
                        newObj.load(data[index])
                        root.id = Math.max(root.id, newObj.id)
                    }
                    else
                    {
                        newObj = null
                    }
                }
            }
            if (!newObj)
            {
                newObj = createGraphicalObject(data[index])
            }
            if (newObj)
            {
                newObjects.push(newObj)
                usingObjects.push(newObj)
            }
		}
        deleteAllObjects(usingObjects)

        for (var index in newObjects)
        {
            newObjects[index].setParent(root)
            node.children.push(newObjects[index]);
        }
        node.children = node.children
        updated();
	}
	
	function createGraphicalObject(data)
	{				
		var objectComponent = Qt.createComponent("graphical_objects/" + data.typeName + ".qml", root);
		if (objectComponent.status === Component.Ready)
        {			
            data["scene"] = root
			var obj = objectComponent.createObject(root, data);
			obj.load(data)
			if (obj)
			{
				return obj
			}
		}
		return null
	}

    function findObjectById(id)
    {
        if (!cacheEnabled)
            return null
        if (id == 0)
            return null
        for (var index in objects)
        {
            if (objects[index].id == id)
                return objects[index]
        }
        return null
    }
    function findObjectByName(name)
    {
        if (name)
            for (var i in objects)
                if (objects[i].name == name)
                    return objects[i]
        return null
    }
}
