import QtQuick 2.3
import QtQuick.Controls 1.0
import GuiControls 1.2
import MixerGuiControls 1.0
import Serialization 1.0
import Clipboard 1.0

FocusScope
{
	id: root
	property GraphicalScene scene
	property Component objectComponent

    property var   composition
    property UndoRedoSceneManager undoManager: scene ? scene.undoManager : null
	property real zoom: 1.0
	property var selectedItems: selectionManager ? selectionManager.selectedItems : []
	property vector2d pastePosition: Qt.vector2d(0,0)
	property ShortcutManager shortcutManager: null	
	
	property SelectionManager selectionManager:
		SelectionManager
		{		
			exclusiveSelection: true
			items: scene.objects
		}
		
    property var buffer : 
	{
		if (!Clipboard.mimeTypes)
			return null
		var index = Clipboard.mimeTypes.indexOf("application/gfxobjects") // TODO: make global MIME type
		if (index < 0)
			return null
		return Clipboard.getData("application/gfxobjects");
	}    

	signal pressed(var mouse)
	
	function saveSelectedItems()
	{
		if (!root.selectionManager) return undefined
		if (!root.scene) return undefined
        var objectsData = root.scene.saveObjects(selectionManager.selectedItems)
        if(!objectsData) return undefined
		return Serialization.writeToMemory(objectsData)
	}
	
	function saveObjectsToClipboard()
	{
		Clipboard.setData("application/gfxobjects", saveSelectedItems());
	}	
	
	function pasteObjects(x,y)
	{
		var newObjects = []
		if (buffer && root.scene)
		{
            var objectsData = Serialization.readFromMemory(root.buffer) || []
            newObjects = root.scene.addObjects(objectsData, x, y, true)
			if (selectionManager)
				selectionManager.selectItems(newObjects)
			saveObjectsToClipboard();
		}
	}
	
	function pushNewState()
	{
		if (scene && scene.undoManager)
			scene.undoManager.pushNewState()
	}
	function setProjectDirty()
	{
		if (scene && scene.undoManager)
			scene.undoManager.dirtyProject  = true
	}

	focus: true
	MouseArea
	{
		id: ma
		acceptedButtons: Qt.LeftButton | Qt.RightButton
		anchors.centerIn: parent
		width: 100000
		height: 100000
		propagateComposedEvents: true
		hoverEnabled: true
		property var startX: 0
		property var startY: 0
		
		property var endX: 0
		property var endY: 0
		
		property rect selectedRect: 
			Qt.rect( 
				Math.min(ma.startX, ma.endX),
				Math.min(ma.startY, ma.endY),
				Math.abs(ma.startX - ma.endX),
				Math.abs(ma.startY - ma.endY))
		
		property bool selecting: false
		onPressed:
		{
            if (objectCreator.__shiftPressed != (mouse.modifiers & Qt.ShiftModifier)) {
                objectCreator.__shiftPressed = (mouse.modifiers & Qt.ShiftModifier);
            }
            if (objectCreator.__controlPressed != (mouse.modifiers & Qt.ControlModifier)) {
                objectCreator.__controlPressed = (mouse.modifiers & Qt.ControlModifier);
            }
			root.pressed(mouse)
			if (mouse.buttons & Qt.LeftButton)
			{
				startX = mouse.x - 50000 + parent.width /2
				startY = mouse.y - 50000 + parent.height /2
				endX = startX
				endY = startY
				selecting = true
			}
		}
        onPressAndHold: {
          if (mouse.button === Qt.LeftButton) {
              root.pastePosition = Qt.vector2d(
              mouse.x - 50000 + parent.width /2,
              mouse.y - 50000 + parent.height /2);
              menuPopup.popup(this, mouseX, mouseY)
          }
        }
		onClicked:
        {
			if (mouse.button === Qt.RightButton)
			{
				root.pastePosition = Qt.vector2d(
                mouse.x - 50000 + parent.width /2,
                mouse.y - 50000 + parent.height /2);
				menuPopup.popup(this, mouseX, mouseY)
			}
		}
		onPositionChanged:
		{
            if (objectCreator.__shiftPressed != (mouse.modifiers & Qt.ShiftModifier)) {
                objectCreator.__shiftPressed = (mouse.modifiers & Qt.ShiftModifier);
            }
            if (objectCreator.__controlPressed != (mouse.modifiers & Qt.ControlModifier)) {
                objectCreator.__controlPressed = (mouse.modifiers & Qt.ControlModifier);
            }
			if (mouse.buttons & Qt.LeftButton)
			{								
				endX = mouse.x - 50000 + parent.width /2
				endY = mouse.y - 50000 + parent.height /2
				selectionManager.hoverArea(selectedRect)
			}
			mouse.accepted = false
		}
        onReleased:
		{			
            if (objectCreator.__shiftPressed != (mouse.modifiers & Qt.ShiftModifier)) {
                objectCreator.__shiftPressed = (mouse.modifiers & Qt.ShiftModifier);
            }
            if (objectCreator.__controlPressed != (mouse.modifiers & Qt.ControlModifier)) {
                objectCreator.__controlPressed = (mouse.modifiers & Qt.ControlModifier);
            }

			selecting = false
			selectionManager.selectArea(selectedRect, !(mouse.modifiers & Qt.ShiftModifier))
			selectionManager.hoverArea(Qt.rect(0,0,0,0))
		}
	}

	Keys.onPressed:
	{
		if (event.modifiers & Qt.ShiftModifier)
		{
			objectCreator.__shiftPressed = true
		}
		if (event.modifiers & Qt.ControlModifier)
		{
			objectCreator.__controlPressed = true
		}
		event.accepted = false
	}
	Keys.onReleased:
	{
		if (!(event.modifiers & Qt.ShiftModifier))
		{
			objectCreator.__shiftPressed = false
		}
		if (!(event.modifiers & Qt.ControlModifier))
		{
			objectCreator.__controlPressed = false
		}
		event.accepted = false
    }
	GraphicalObjectCreator
	{
		id: objectCreator
		anchors.fill: parent
		scene: root.scene
		visible: root.objectComponent		
		objectComponent: root.objectComponent
		selectionManager: root.selectionManager
		onPressed:
		{
			root.focus = true
			singleSelect.focus = true
			root.pressed(mouse)
		}
	}

	SingleSelection
	{
		focus: true
        //bug fix
        /*onFocusChanged:	{
			if (!focus)
			{
                print("objectCreator.focus")
				objectCreator.focus = false
				objectCreator.__shiftPressed = false
				objectCreator.__controlPressed = false
			}
        }*/
		id: singleSelect
		scene: root.scene
		selectionManager: root.selectionManager		
		zoom: root.zoom
        menuPopup: menuPopupObjects
		enableAddSelection: !objectCreator.creationEnabled || !objectCreator.visible
        onPressed:
		{
			root.focus = true
			root.pressed(mouse)
		}
	}

	MultiSelectionDrag
	{
		id: multiSelect
		selectionManager: root.selectionManager		
		zoom: root.zoom
		onPressed:
		{
			root.focus = true
			root.pressed(mouse)
		}
		menuPopup: menuPopupObjects
	}
		
	Rectangle
	{
		x: ma.selectedRect.x
		y: ma.selectedRect.y
		width: ma.selectedRect.width
		height: ma.selectedRect.height
		color: "black"
		opacity: 0.2
		id: rect
		visible: ma.selecting
	}
	Frame
	{		
		borderWidth: 1. / root.zoom
		showAnchors: false
		anchors.fill: rect
		visible: ma.selecting
		anchors.margins: -borderWidth/2			
	}

    //===================================================================
    // Actions (not Very Hot!)
    //===================================================================
    property Action actionUndo:
    Action
    {
        text: qsTr("&Undo")
        shortcut: StandardKey.Undo
        onTriggered:
        {
            if (undoManager)
                undoManager.undo()
        }
        enabled: undoManager && undoManager.canUndo
    }
    property Action actionRedo:
    Action
    {
        text: qsTr("&Redo")
        shortcut: StandardKey.Redo
        onTriggered:
        {
            if (undoManager)
                undoManager.redo()
        }
        enabled: undoManager && undoManager.canRedo
    }
    //--------------------------------------------------------------------
    property ActionsList actionsList: ActionsList {
        copyFunction: (root.selectedItems && root.scene && root.selectedItems.length > 0) ?
			function () { saveObjectsToClipboard(); } :null
		
        pasteFunction: (root.buffer) ? 
			function (source) {
				if (root.buffer) {
					var origin = Qt.vector2d(0, 0)
					if (source)
					{
						origin = pastePosition
					}
					if (root.selectedItems && root.scene && root.selectedItems.length > 0)
					{
						origin = root.scene.calcOrigin(root.selectedItems)
						origin.x += 25
						origin.y += 25
					}
					pasteObjects(origin.x, origin.y)
					root.pushNewState()
				}
			} : null
        cutFunction: (root.selectedItems && root.scene && root.selectedItems.length > 0) ? 
			function () {	
				if(selectedItems.length == 0) return
				saveObjectsToClipboard()
				scene.deleteObjects(selectionManager.selectedItems)
                selectionManager.updateSelectedItems()
				root.pushNewState()
			} : null 
        deleteFunction: (root.selectedItems && root.scene && root.selectedItems.length > 0) ?
			function () {
				if(selectedItems.length == 0) return
				scene.deleteObjects(selectedItems)
                selectionManager.updateSelectedItems()
				root.pushNewState()
			} : null
    }
    property Action actionCut: shortcutManager.cutAction
    property Action actionCopy: shortcutManager.copyAction
    property Action actionPaste: shortcutManager.pasteAction
    property Action actionDelete: shortcutManager.deleteAction
    //--------------------------------------------------------------------
    property Action actionSelectAll:
    Action
    {
        text: qsTr("Select &All Objects")
        shortcut: StandardKey.SelectAll
        enabled: scene.objects.length>0
        onTriggered:    {    selectionManager.selectAll()     }
    }
    property Action actionSelectNext:
    Action
    {
        text: qsTr("Select &Next Object")
        shortcut: "Tab"
        onTriggered:    {  
            var nextObject = null;
            if (selectedItems.length === 0) {
                nextObject = root.scene.getNextObject(null);
            }
            else {
                var objectParent = selectedItems[0].getParent();
                nextObject = objectParent.getNextObject(selectedItems[0]);
            }
            if (nextObject) {
                root.selectionManager.selectItem(nextObject);
            }
        }
    }
    property Action actionSelectPrev:
    Action
    {
        text: qsTr("Select &Previous Object")
        shortcut: "Ctrl+Tab"
        onTriggered:
        {
            var prevObject = null;
            if (selectedItems.length === 0) {
                prevObject = root.scene.getPrevObject(null);
            }
            else {
                var objectParent = selectedItems[0].getParent();
                prevObject = objectParent.getPrevObject(selectedItems[0]);
            }
            if (prevObject) {
                root.selectionManager.selectItem(prevObject);
            }
        }
    }
    //--------------------------------------------------------------------
    readonly property var _selParent: selectedItems && selectedItems[0] ? selectedItems[0].getParent() : null
    property bool isOnTopLevel:   _selParent ? !_selParent.isOnTopLevel(selectedItems)  : false
    property bool isOnDownLevel:  _selParent ? !_selParent.isOnDownLevel(selectedItems) : false
    //--------------------------------------------------------------------
    property Action actionOrderUp:
    Action
    {
        text: qsTr("Move Level &Up")
        shortcut: "Ctrl+PageUp"
        enabled: isOnTopLevel
        onTriggered:
        {
            if (selectedItems[0].haveSameParentWithObjects(selectedItems)) {
                var objectsParent = selectedItems[0].getParent()
                objectsParent.bringLevelUp(selectedItems)
                root.pushNewState()
            }
        }
    }
    property Action actionOrderDown:
    Action
    {
        text: qsTr("Move Level &Down")
        shortcut: "Ctrl+PageDown"
        enabled: isOnDownLevel
        onTriggered:
        {
            if (selectedItems[0].haveSameParentWithObjects(selectedItems)) {
                var objectsParent = selectedItems[0].getParent()
                objectsParent.bringLevelDown(selectedItems)
                root.pushNewState()
            }
        }
    }
    property Action actionOrderTop:
    Action
    {
        text: qsTr("Move to &Front")
        shortcut: "Ctrl+Home"
        enabled: isOnTopLevel
        onTriggered:
        {
            if (selectedItems[0].haveSameParentWithObjects(selectedItems)) {
                var objectsParent = selectedItems[0].getParent()
                objectsParent.bringLevelTop(selectedItems)
                root.pushNewState()
            }
        }
    }
    property Action actionOrderBottom:
    Action
    {
        text: qsTr("Move to &Back")
        shortcut: "Ctrl+End"
        enabled: isOnDownLevel
        onTriggered:
        {
            if (selectedItems[0].haveSameParentWithObjects(selectedItems)) {
                var objectsParent = selectedItems[0].getParent()
                objectsParent.bringLevelBottom(selectedItems)
                root.pushNewState()
            }
        }
    }
    //--------------------------------------------------------------------------
    property Action actionMoveHCenter:
    Action
    {
        text: qsTr("Center &Horizontally")
        shortcut: "Ctrl+Shift+H"
        enabled: (selectedItems.length > 0) ? (selectedItems[0].haveSameParentWithObjects(selectedItems)) : false
        onTriggered:
        {
            var firstItem = selectedItems[0]
            var left = firstItem.x
            var right = firstItem.x + firstItem.width
            for (var index in selectedItems)
            {
                var currentItem = selectedItems[index]
                if (left > currentItem.x)
                {
                    left = currentItem.x
                }
                if (right < currentItem.x + currentItem.width)
                {
                    right = currentItem.x + currentItem.width
                }
            }
            var center = (right + left) / 2
            var objectParent = selectedItems[0].getParent()
            var shift = objectParent.width / 2 - center

            var selectedItemsSave = selectedItems
            selectionManager.resetSelection()
            for (var index in selectedItemsSave)
            {
                var currentItem = selectedItemsSave[index]
                if (!currentItem.freezed)
                    currentItem.x = currentItem.x + shift
            }
            selectionManager.restoreSelection(selectedItemsSave)
            root.pushNewState()
        }
    }
    property Action actionMoveVCenter:
    Action
    {
        text: qsTr("Center &Vertically")
        shortcut: "Ctrl+Shift+V"
        enabled: (selectedItems.length > 0) ? (selectedItems[0].haveSameParentWithObjects(selectedItems)) : false
        onTriggered:
        {
            var firstItem = selectedItems[0]
            var top = firstItem.y
            var bottom = firstItem.y + firstItem.height
            for (var index in selectedItems)
            {
                var currentItem = selectedItems[index]
                if (top > currentItem.y)
                {
                    top = currentItem.y
                }
                if (bottom < currentItem.y + currentItem.height)
                {
                    bottom = currentItem.y + currentItem.height
                }
            }
            var center = (top + bottom) / 2
            var objectParent = selectedItems[0].getParent()
            var shift = objectParent.height / 2 - center
            var selectedItemsSave = selectedItems
            selectionManager.resetSelection()
            for (var index in selectedItemsSave)
            {
                var currentItem = selectedItemsSave[index]
                if (!currentItem.freezed)
                    currentItem.y = currentItem.y + shift
            }
            selectionManager.restoreSelection(selectedItemsSave)
            root.pushNewState()
        }
    }
    //--------------------------------------------------------------------------
    property Action actionAlignLeft:
    Action
    {
        text: qsTr("Align &Lefts")
        shortcut: "Ctrl+Left"
        enabled: selectedItems.length > 1
        onTriggered:
        {
            var leftItem = selectedItems[0]
            for (var index in selectedItems)
            {
                var currentItem = selectedItems[index]
                if (leftItem.x > currentItem.x)
                {
                    leftItem = currentItem
                }
            }
            var leftX = leftItem.x
            var selectedItemsSave = selectedItems
            selectionManager.resetSelection()
            for (var index in selectedItemsSave)
            {
                var currentItem = selectedItemsSave[index]
                if (!currentItem.freezed)
                    currentItem.x = leftX
            }
            selectionManager.restoreSelection(selectedItemsSave)
            root.pushNewState()
        }
    }
    property Action actionAlignHCenter:
    Action
    {
        text: qsTr("Align &Horizontal Centers")
        shortcut: "Ctrl+H"
        enabled: selectedItems.length > 1
        onTriggered:
        {
            var firstItem = selectedItems[0]
            var left = firstItem.x
            var right = firstItem.x + firstItem.width
            for (var index in selectedItems)
            {
                var currentItem = selectedItems[index]
                if (left > currentItem.x)
                {
                    left = currentItem.x
                }
                if (right < currentItem.x + currentItem.width)
                {
                    right = currentItem.x + currentItem.width
                }
            }
            var center =  (right + left) / 2
            var selectedItemsSave = selectedItems
            selectionManager.resetSelection()
            for (var index in selectedItemsSave)
            {
                var currentItem = selectedItemsSave[index]
                if (!currentItem.freezed)
                    currentItem.x = center - currentItem.width / 2
            }
            selectionManager.restoreSelection(selectedItemsSave)
            root.pushNewState()
        }
    }
    property Action actionAlignRight:
    Action
    {
        text: qsTr("Align &Rights")
        shortcut: "Ctrl+Right"
        enabled: selectedItems.length > 1
        onTriggered:
        {
            var rightItem = selectedItems[0]
            var right = rightItem.x + rightItem.width
            for (var index in selectedItems)
            {
                var currentItem = selectedItems[index]
                if (right < currentItem.x + currentItem.width)
                {
                    right = currentItem.x + currentItem.width
                }
            }
            var selectedItemsSave = selectedItems
            root.selectionManager.resetSelection()
            for (var index in selectedItemsSave)
            {
                var currentItem = selectedItemsSave[index]
                if (!currentItem.freezed)
                    currentItem.x = right - currentItem.width
            }
            root.selectionManager.restoreSelection(selectedItemsSave)
            root.pushNewState()
        }
    }
    property Action actionAlignTop:
    Action
    {
        text: qsTr("Align &Tops")
        shortcut: "Ctrl+Up"
        enabled: selectedItems.length > 1
        onTriggered:
        {
            var topItem = selectedItems[0]
            for (var index in selectedItems)
            {
                var currentItem = selectedItems[index]
                if (topItem.y > currentItem.y)
                {
                    topItem = currentItem
                }
            }
            var topY = topItem.y
            var selectedItemsSave = selectedItems
            root.selectionManager.resetSelection()
            for (var index in selectedItemsSave)
            {
                var currentItem = selectedItemsSave[index]
                if (!currentItem.freezed)
                    currentItem.y = topY
            }
            root.selectionManager.restoreSelection(selectedItemsSave)
            root.pushNewState()
        }
    }
    property Action actionAlignVCenter:
    Action
    {
        text: qsTr("Align &Vertical Centers")
        //shortcut: "Ctrl+V"
        enabled: selectedItems.length > 1
        onTriggered:
        {
            var firstItem = selectedItems[0]
            var top = firstItem.y
            var bottom = firstItem.y + firstItem.height
            for (var index in selectedItems)
            {
                var currentItem = selectedItems[index]
                if (top > currentItem.y)
                {
                    top = currentItem.y
                }
                if (bottom < currentItem.y + currentItem.height)
                {
                    bottom = currentItem.y + currentItem.height
                }
            }
            var center =  (top + bottom) / 2
            var selectedItemsSave = selectedItems
            root.selectionManager.resetSelection()
            for (var index in selectedItemsSave)
            {
                var currentItem = selectedItemsSave[index]
                if (!currentItem.freezed)
                    currentItem.y = center - currentItem.height / 2
            }
            root.selectionManager.restoreSelection(selectedItemsSave)
            root.pushNewState()
        }
    }
    property Action actionAlignBottom:
    Action
    {
        text: qsTr("Align &Bottoms")
        shortcut: "Ctrl+Down"
        enabled: selectedItems.length > 1
        onTriggered:
        {
            var bottomItem = selectedItems[0]
            var bottom = bottomItem.y + bottomItem.height
            for (var index in selectedItems)
            {
                var currentItem = selectedItems[index]
                if (bottom < currentItem.y + currentItem.height)
                {
                    bottom = currentItem.y + currentItem.height
                }
            }
            var selectedItemsSave = selectedItems
            root.selectionManager.resetSelection()
            for (var index in selectedItemsSave)
            {
                var currentItem = selectedItemsSave[index]
                if (!currentItem.freezed)
                    currentItem.y = bottom - currentItem.height
            }
            root.selectionManager.restoreSelection(selectedItemsSave)
            root.pushNewState()
        }
    }
    property Action actionGroupObjects: Action {
        text: qsTr("Group")
        enabled: (selectedItems.length > 1) ? (selectedItems[0].haveSameParentWithObjects(selectedItems)) : false
        onTriggered: {
            var group = root.scene.groupObjects(selectedItems);
            if (group) {
                root.selectionManager.selectItem(group);
                root.pushNewState();
            }
        }
    }
    property Action actionUngroupObjects: Action {
        text: qsTr("Ungroup")
        enabled: (selectedItems.length === 1) && (selectedItems[0].ungroupEnabled)
        onTriggered: {
            var list = selectedItems[0].ungroupObjects();
            if (list.length !== 0) {
                root.selectionManager.selectItems(list, false, false);
            }
            root.pushNewState();
        }
    }
    property Action actionMergeInGroup: Action {
        text: qsTr("Merge In Group")
        enabled: (selectedItems.length > 1) ? (selectedItems[0].haveSameParentWithObjects(selectedItems)) : false
        onTriggered: {
            var itemsParent = selectedItems[0].getParent();
            var group = itemsParent.mergeInGroup(selectedItems);
            if (group) {
                root.selectionManager.selectItem(group);
                root.pushNewState();
            }
        }
    }
    property Action actionAddGroup: Action {
        text: qsTr("Add New Group")
        onTriggered: {
            if (selectedItems.length === 0) {
                root.scene.addGroup();
            }
            else {
                var itemsParent = selectedItems[0].getParent();
                itemsParent.addGroup();
            }
            root.pushNewState();
        }
    }
    property Action actionRenameObject: Action {
        text: qsTr("Rename")
        enabled: (selectedItems.length === 1)
        onTriggered: {
            renameObjectDialog.text = selectedItems[0].name;
            renameObjectDialog.open();
        }
    }
    MixerStringInputDialog {
        id: renameObjectDialog
        title: qsTr("Rename Object")
        caption: qsTr("Name: ")
        onEntered: {
            root.selectionManager.selectedItems[0].renameObject(value);
            root.pushNewState();
        }
    }
    //=======================================================================
    // Menus (not from Restourants!!!)
    //=======================================================================
    property MixerMenu menuPopup: MixerMenu
	{
        MixerMenuItem {  action1: actionUndo }
        MixerMenuItem {  action1: actionRedo }
        MixerMenuItem {  action1: actionPaste }
        MixerMenuSeparator{}
        MixerMenuItem {  action1: actionSelectAll }
        MixerMenuSeparator{}
        MixerMenu
		{
			title: qsTr("&View")
            MixerMenuItem {  action1: composition.actionZoomIn        }
            MixerMenuItem {  action1: composition.actionZoomOut		}
            MixerMenuItem {  action1: composition.actionZoomActual    }
            MixerMenuSeparator{}
            MixerMenuItem {  action1: composition.actionZoomFit       }
            MixerMenuSeparator{}
            MixerMenuItem {  action1: composition.actionShowHiddenObjects  }
            MixerMenuItem {  action1: composition.actionShowSafeFrame  }
		}
        MixerMenuSeparator{}
        MixerMenuItem {
			text: qsTr("&Preview Mode")
			enabled: false
		}
        MixerMenuSeparator{}
        MixerMenuItem {
			text: qsTr("Enable &Snapping")
			enabled: false
		}
        MixerMenuSeparator{}
        MixerMenuItem {
			text: qsTr("&Options...")
			enabled: false
		}
	}
    //-------------------------------------------------------------------
    //on objects
    property MixerMenu menuPopupObjects: MixerMenu 
	{
        MixerMenuItem {  action1: actionRenameObject }
        MixerMenuSeparator{}
        MixerMenuItem {  action1: actionCut }
        MixerMenuItem {  action1: actionCopy }
        MixerMenuItem {  action1: actionPaste }
        MixerMenuItem {  action1: actionDelete }
        MixerMenuSeparator{}
        MixerMenuItem {   text: qsTr("&Link");      enabled: false     }
        MixerMenuItem {   text: qsTr("&Unlink");    enabled: false     }
        MixerMenuSeparator{}
        MixerMenu
        {
            title: qsTr("&Select")
            MixerMenuItem { action1: actionSelectNext; text: qsTr("&Next Object");      }
            MixerMenuItem { action1: actionSelectPrev; text: qsTr("&Previous Object");  }
            MixerMenuItem { action1: actionSelectAll;  text: qsTr("&All Objects")       }
        }
        MixerMenu {
            title: qsTr("&Group")
            MixerMenuItem { action1: actionGroupObjects }
            MixerMenuItem { action1: actionUngroupObjects }
            MixerMenuItem { action1: actionMergeInGroup }
            enabled: (actionGroupObjects.enabled || actionUngroupObjects.enabled || actionMergeInGroup)
        }
        MixerMenuSeparator{}
        MixerMenu   {
            title: qsTr("&Font Size");
            enabled: false
        }
        Menu   {
            title: qsTr("&Text Alignment");
            enabled: false
        }
        MixerMenuSeparator{}
        MixerMenu   {
            title: qsTr("&Order");
            MixerMenuItem { action1: actionOrderUp     }
            MixerMenuItem { action1: actionOrderDown   }
            MixerMenuItem { action1: actionOrderTop    }
            MixerMenuItem { action1: actionOrderBottom  }
        }
        MixerMenu   {
            title: qsTr("&Position");
            MixerMenuItem { action1: actionMoveHCenter   }
            MixerMenuItem { action1: actionMoveVCenter   }
        }
        MixerMenu   {
            title: qsTr("&Align");
            MixerMenuItem { action1: actionAlignLeft;     text: qsTr("&Lefts")  }
            MixerMenuItem { action1: actionAlignHCenter;  text: qsTr("&Horizontal Centers")  }
            MixerMenuItem { action1: actionAlignRight;    text: qsTr("&Rights")  }
            MixerMenuSeparator{}
            MixerMenuItem { action1: actionAlignTop;      text: qsTr("&Tops")  }
            MixerMenuItem { action1: actionAlignVCenter;  text: qsTr("&Vertical Centers")  }
            MixerMenuItem { action1: actionAlignBottom;   text: qsTr("&Bottoms")  }
        }
    }
    //=======================================================================
}





