import QtQuick 2.0
import GuiControls 1.2
import QtQuick.Controls 1.2
import MixerGuiControls 1.0

Item{
	id: root
	property real zoom: 1.0
    property bool guard: false
	property var selectedItems: selectionManager ? selectionManager.selectedItems : []
	property SelectionManager selectionManager
	signal pressed(var mouse)
	
	focus: (root.selectedItems.length > 1)
    property var menuPopup
	Repeater
	{
		id: multiSelector
		
		model: root.selectedItems.length > 1 ? root.selectedItems : 0
        delegate: ResizebleFrame {
            id: frame
            property GraphicalObject object: root.selectedItems[index]
            property var prevX: 0
            property var prevY: 0
            width: (object && object.getParent()) ? object.getParent().width : 0
            height: (object && object.getParent()) ? object.getParent().height : 0
            x: (object && object.getParent()) ? object.getParent().getGlobalCoord().x : 0
            y: (object && object.getParent()) ? object.getParent().getGlobalCoord().y : 0
            rotation: (object && object.getParent() && (object.getParent() !== scene)) ? object.getParent().getGlobalAngle() : 0
            selected: true
            clickEnabled: object && object.typeName !== "GroupGraphicalObject"
            dragEnabled: object && !object.freezed
            target: object
            borderWidth: 1.2 / root.zoom
            borderColor: {
                 if (object) {
                     if (object.hidden)  return MixerThemeManager.theme.colCommon
                     if (object.freezed) return MixerThemeManager.theme.colProgram
                 }
                 return MixerThemeManager.theme.colNeutralLight
             }
            visible: object && object.visible
            resizeEnabled: false

            onClicked: {
                  if (menuPopup && mouse.button === Qt.RightButton)
                      menuPopup.popup(item, mouse.x, mouse.x)
            }
            onPressAndHold: {
                if (menuPopup && mouse.button === Qt.LeftButton)
                    menuPopup.popup(item, mouse.x, mouse.x)
            }

            Connections {
                target: frame.object
                onXChanged: {
                    var delta = frame.object.x - frame.prevX
                    frame.prevX = frame.object.x
                    if (root.guard)
                        return
                    root.shift(delta, "x", frame.object)
                }
                onYChanged: {
                    var delta = frame.object.y - frame.prevY
                    frame.prevY = frame.object.y
                    if (root.guard)
                        return
                    root.shift(delta, "y", frame.object)
                }
            }
            property rect frameRectBeforeDrag
            onActiveChanged: {
                if(active) frameRectBeforeDrag = frame.targetRect
                if(!active && frameRectBeforeDrag != frame.targetRect && scene && scene.undoManager) scene.undoManager.pushNewState()
            }         
            Component.onCompleted:
            {
                if (!frame.object)
                    return
                frame.prevX = frame.object.x
                frame.prevY = frame.object.y
            }
        }
	}	
	
    function shift(delta, property, exception) {
        root.guard = true
        for (var i in root.selectedItems)
        {
            var currentObj = root.selectedItems[i]
            if (currentObj!=exception && !currentObj.freezed)
               currentObj[property] += delta
       }
        root.guard = false
    }

	Keys.onLeftPressed:
	{
		event.accepted = false
        var delta = -1
		if (event.modifiers & Qt.ShiftModifier)
		{
            delta = -10
		}
        root.shift(delta, "x")
	}
    Keys.onRightPressed:
	{
		event.accepted = false
		var delta = 1
		if (event.modifiers & Qt.ShiftModifier)
		{
			delta = 10
		}
        root.shift(delta, "x")
    }
	Keys.onUpPressed:
	{
		event.accepted = false
        var delta = -1
		if (event.modifiers & Qt.ShiftModifier)
		{
            delta = -10
		}
        root.shift(delta, "y")
	}
	Keys.onDownPressed:
	{
		event.accepted = false
		var delta = 1
		if (event.modifiers & Qt.ShiftModifier)
		{
			delta = 10
		}
        root.shift(delta, "y")
	}
    Keys.onEscapePressed: selectionManager.resetSelection()
}
