import QtQuick 2.0
import GuiControls 1.2
import QtQuick.Controls 1.1
import MixerGuiControls 1.0

FocusScope{
	id: root
	property real zoom: 1.0
	property var selectedItems: selectionManager ? selectionManager.selectedItems : []
	property SelectionManager selectionManager
	property GraphicalScene scene
	signal pressed(var mouse)
	
	property bool enableAddSelection: true
    property var  menuPopup
    property int itemsLevel: selectionManager ? selectionManager.selectionLevel : 0
    property bool enableChangeLevel: true
	property bool keepAspect: false

    Keys.onPressed: {
		keepAspect = event.modifiers & Qt.ShiftModifier
    }
    Keys.onReleased: {
		keepAspect = event.modifiers & Qt.ShiftModifier
    }

	onSelectedItemsChanged:
	{
		focus = (root.selectedItems.length == 1)
	}
	Repeater
	{
		model: root.scene ? root.scene.objects : null
		delegate: 
			ResizebleFrame 
			{
                id: frame
                property GraphicalObject object: scene.objects[index]
                width: (object && object.getParent()) ? object.getParent().width : 0
                height: (object && object.getParent()) ? object.getParent().height : 0
                x: (object && object.getParent()) ? object.getParent().getGlobalCoord().x : 0
                y: (object && object.getParent()) ? object.getParent().getGlobalCoord().y : 0
                rotation: (object && object.getParent() && (object.getParent() !== scene)) ? object.getParent().getGlobalAngle() : 0
                z: (object.selected && (root.selectedItems.length == 1) && (object.typeName !== "GroupGraphicalObject")) ? 1 : 0
				selected: object.selected && root.selectedItems.length == 1
                clickEnabled: (object.typeName !== "GroupGraphicalObject") //&& !object.freezed
                dragEnabled: (root.selectedItems.length == 1) && (!object.freezed)
				target: object
				keepAspect: root.keepAspect
				borderWidth: 1.2 / root.zoom
               borderColor: {
                    if (object) {
                        if (object.hidden)  return MixerThemeManager.theme.colCommon
                        if (object.freezed) return MixerThemeManager.theme.colProgram
                    }
                    return MixerThemeManager.theme.colNeutralLight
                }
                anchorsColor: object && object.hidden ? MixerThemeManager.theme.colCommon : MixerThemeManager.theme.colNeutral
				Connections
				{
					target: root
					onSelectedItemsChanged:
					{
						if (root.selectedItems.length == 1)
						{
							frame.focus = (root.selectedItems[0] == object)
						}
					}
				}
                visible: {
                    return (root.enableAddSelection || object.selected || object.isEmpty) && object.visible
                }
                resizeEnabled: root.selectionManager ? (root.selectionManager.isInSelectedItems(object)) && !object.freezed : false
                onPressed:
				{
                    root.pressed(mouse)
                    var nextItemInSelectionLine = object.getNextItemInSelectionLine();
                    var nextItemLevel = nextItemInSelectionLine.getParentCount();
                    if (nextItemLevel <= root.itemsLevel) {
                        if (!selected) {
                            if (root.selectionManager) {
                                root.selectionManager.selectItem(object, !(mouse.modifiers & Qt.ShiftModifier))
                            }
                        }
                        enableChangeLevel = false;
                    }
                    frame.focus = true
				}
				onClicked:
				{
                    if (mouse.button === Qt.LeftButton) {
                        if (enableChangeLevel) {
                            var nextItemInSelectionLine = object.getNextItemInSelectionLine();
                            var nextItemLevel = nextItemInSelectionLine.getParentCount();
                            if (nextItemLevel > root.itemsLevel) {
                                if (root.selectionManager) {
                                    root.selectionManager.selectItem(object, !(mouse.modifiers & Qt.ShiftModifier))
                                }
                            }
                        }
                        else {
                            enableChangeLevel = true;
                        }
                    }
                    else {
                        if (menuPopup) {
                            menuPopup.popup(item, mouse.x, mouse.x)
                        }
                    }
                }
                onPressAndHold: {
                    if (menuPopup && mouse.button === Qt.LeftButton)
                        menuPopup.popup(item, mouse.x, mouse.x)
                }
				onDoubleClicked: {
					if (loader.item && loader.item.doubleClicked)
					{
						loader.item.doubleClicked(mouse);
					}
				}
                minimumWidth: 2
				minimumHeight: 2
				
                property rect frameRectBeforeDrag
                onActiveChanged: {
                    if(active) frameRectBeforeDrag = frame.targetRect
                    if(!active && frameRectBeforeDrag != frame.targetRect && scene && scene.undoManager) scene.undoManager.pushNewState()
                }
                
                Keys.onEscapePressed: selectionManager.resetSelection()
				Loader {
					id: loader
					source: "graphical_objects_empty_views/" + object.typeName + "EmptyView.qml"
					x: object.x
					y: object.y
                    width: object.width
					height: object.height
					rotation: object.rotation
                    visible: object.isEmpty
					Binding {
						target: loader.item
						property: "object"
						value: frame.object
					}
					Binding {
						target: loader.item
						property: "zoom"
						value: root.zoom
					}
				}
                onMoveActiveChanged: {
                    if (!object) return;
                    if (!moveActive) {
                        object.updated();
                    }
                }
			}
	}
	property var selectedItem: null
	
	property Action actionSelect:
		Action
		{
			text: qsTr("&Select")
			onTriggered: 
			{
				if (root.selectionManager)
					root.selectionManager.selectItem(root.selectedItem, false)
			}
		}
		
    MixerMenu
	{
		id: menuPopupSelect
        MixerMenuItem {  action1: actionSelect}
	}
}
