import QtQuick 2.3
import QtQml 2.0

QtObject {
    id: root
	property string name: "New Style Collection"
    //: Default preview text for style collection
    property string textPreview: qsTr("Text")
	property var styles: []
    readonly property Component styleSettingsComponent: Component{ StyleSettings{} }
    
    function save() {
        var data = {};
        data["name"] = name;
        data["textPreview"] = textPreview;
        data["styles"] = saveStyles();
        return data;
    }
    function saveStyles() {
        var data = [];
        for (var i = 0; i < styles.length; ++i) {
            var elem = styles[i].save();
            data.push(elem);
        }
        return data;
    }
    function load (data) {
        name = data.name;
        textPreview = data.textPreview;
        styles = loadStyles(data.styles);
    }
    function loadStyles(data) {
        var list = [];
        for (var i = 0; i < data.length; ++i) {
            var obj = styleSettingsComponent.createObject(root, {});
            obj.load(data[i]);
            list.push(obj);
        }
        return list;
    }
    function get(index) {
        if (index < 0) return;
        if (index >= styles.length) return;
        return styles[index];
    }
    function add(newStyle) {
        if (!newStyle) return;
        styles.push(newStyle);
        styles = styles;
    }
    function remove(index) {
        if (index < 0) return;
        if (index >= styles.length) return;
        styles.splice(index, 1);
        styles = styles;
    }
    function removeItems(indexArray) {
        if (!indexArray) return;
        if (indexArray.length < 0) return;
        if (indexArray.length > styles.length) return;
        var list = [];
        for (var i = 0; i < styles.length; ++i) {
            var isFind = false;
            for (var j = 0; j < indexArray.length; ++j) {
                if (parseInt(i) === parseInt(indexArray[j])) {
                    isFind = true;
                    break;
                }
            }
            if (!isFind) {
                list.push(styles[i]);
            }
        }
        styles = list;
    }
}