import QtQml 2.0
import QtQuick 2.3

QtObject
{
    id: root
    readonly property string name: "FillColor"
    property bool perLetter: true
    property int type: 0
    property color color: "white"
    property string useSysColor: "" //"program", "preset", "common", "from_slot"
    property int slotIndex: 1
    property int repeat: 0
    property real angle: 0
    property real opacity: 100
    property bool visible: true
    property Gradient gradient: Gradient {
        GradientStop { position: 0.0; color: "#FF00FFFF" }
        GradientStop { position: 1.0; color: "#FF0000FF" }
    }
    
    function saveGradient()
    {
        var gradientStopList = [];
        for (var i = 0; i < gradient.stops.length; ++i)
        {
            var gradientStopColor = gradient.stops[i].color;
            var gradientStopPosition = gradient.stops[i].position;
            var gradientStop = {};
            gradientStop["color"] = gradientStopColor;
            gradientStop["position"] = gradientStopPosition;
            gradientStopList.push(gradientStop);
        }
        return gradientStopList;
    }
    
    function save()
    {
        var settings = {};
        settings["name"] = name;
        settings["perLetter"] = perLetter;
        settings["type"] = type;
        settings["color"] = color;
        settings["repeat"] = repeat;
        settings["angle"] = angle;
        settings["opacity"] = opacity;
        settings["gradientStopList"] = saveGradient();
        settings["visible"] = visible;
        settings["useSysColor"] = useSysColor;
        settings["slotIndex"] = slotIndex;
        return settings;
    }
    
    function loadGradient(data)
    {
        var list = [];
        for (var i = 0; i < data.length; ++i)
        {
            var stop = gradientStopComponent.createObject( root.gradient, { "position": data[i].position, "color": data[i].color } );
            list.push(stop);
        }
        return list;
    }
    
    readonly property Component gradientStopComponent: Component { GradientStop{} }
    function load(data)
    {
        perLetter = data.perLetter;
        type = data.type;
        color = data.color;
        repeat = data.repeat;
        angle = data.angle;
        opacity = data.opacity;
        gradient.stops = loadGradient(data.gradientStopList);
        visible = data.visible;
        useSysColor = data.useSysColor || "";
        slotIndex = data.slotIndex || 1;
    }
}
