import QtQuick 2.0
import MixerGuiControls 1.0
import ".."

ShapeShaderEffect {
    id: root
    property FillColorEffectSettings settings
    property Item colorTexture
    property rect colorTextureRect
    margins: antialiasingSize
    Rectangle {
        id: gradientTexture
        width: 1
        height: 512
        gradient: settings.gradient
        layer.enabled: true
        layer.smooth: true
        layer.wrapMode: ShaderEffectSource.Repeat
        visible: false
    }
    readonly property bool isOnProgram: settings.useSysColor=="program" || (settings.useSysColor=="from_slot" && mixer && mixer.mixerController && mixer.mixerController.programSlotIndex==(settings.slotIndex-1))
    readonly property bool isOnPreview: settings.useSysColor=="preset"  || (settings.useSysColor=="from_slot" && mixer && mixer.mixerController && mixer.mixerController.previewSlotIndex==(settings.slotIndex-1))
    readonly property color color: {
        if (isOnProgram) return MixerThemeManager.theme.colProgram
        if (isOnPreview) return MixerThemeManager.theme.colPreset
        var col = MixerThemeManager.theme.tagToSysColor[settings.useSysColor]
        return col!=undefined ? col : settings.color
    }
    readonly property Item gradientTexture: (settings.type == 1 || settings.type == 2) ? gradientTexture : null
    readonly property real texcoordScale: (settings.type == 1) ? settings.repeat + 1.0 : 1.0
    readonly property real invGradientTextureSize: 1.0 / gradientTexture.height
    readonly property real maxInnerDistance: shape.maxInnerDistance
    readonly property real angle: settings.angle * Math.PI / 180
    readonly property bool perLetter: settings.perLetter && shape.hasOwnProperty("letterSize")

    readonly property string linearGradientTexcoordFunction: "
        uniform bool perLetter;
        uniform float angle;
        float getTexcoord(ShapeInfo shapeInfo, vec2 pos)
        {
            vec2 texcoord = perLetter ? shapeInfo.tc : (pos - shapeContentRect.xy) / shapeContentRect.zw;
            float mid = 0.5;
            return cos(angle) * (texcoord.y - mid) - sin(angle) * (texcoord.x - mid) + mid;                
        }
    "
    
    readonly property string radialGradientTexcoordFunction: "
        uniform float maxInnerDistance;
        float getTexcoord(ShapeInfo shapeInfo, vec2 pos)
        {
            return max(0.0, -shapeInfo.d / maxInnerDistance);
        }    
    "
    
    readonly property string gradientColorFunction: "
        uniform sampler2D gradientTexture;
        uniform float invGradientTextureSize;
        uniform float texcoordScale;
        "
        + (settings.type == 1 ? linearGradientTexcoordFunction : radialGradientTexcoordFunction) +
        "
        vec4 getColor(ShapeInfo shapeInfo, vec2 pos) 
        {
            float texcoord = getTexcoord(shapeInfo, pos) * texcoordScale;
            texcoord = clamp(texcoord, invGradientTextureSize, texcoordScale - invGradientTextureSize);
            return texture2D(gradientTexture, vec2(0.5, texcoord));        
        }
    "

    readonly property string solidColorFunction: "
        uniform vec4 color;
        vec4 getColor(ShapeInfo shapeInfo, vec2 pos) 
        {
            return color;
        }        
    "
    
    readonly property string textureColorFunction: "
        uniform sampler2D colorTexture;
        uniform vec4 colorTextureRect;
        vec4 getTextureColor(vec2 pos) 
        {
            vec2 tc = (pos - colorTextureRect.xy) / colorTextureRect.zw;
            return texture2D(colorTexture, tc);
        }
    "
    readonly property string emptyTextureColorFunction: "
        vec4 getTextureColor(vec2 pos) 
        {
            return vec4(1.0, 1.0, 1.0, 1.0);
        }
    "    
    
    shaderFunction: (settings.type == 0 ? solidColorFunction : gradientColorFunction) + 
                    (root.colorTexture == null ? emptyTextureColorFunction : textureColorFunction) +
    "
        vec4 getResult(vec2 pos) 
        {
            ShapeInfo shapeInfo = getShapeInfo(pos);
            return getColor(shapeInfo, pos) * getTextureColor(pos) * shapeInfo.mask;
        }
    "  
}
