import QtQuick 2.0
import ".."

ShaderEffect {
    id: root
    property Shape shape
    property real antialiasingSize
    property real margins
    property string shaderFunction
    
    readonly property Item shapeTexture: shape.texture
    readonly property Item shapeTexture2: shape.texture2
    readonly property bool hasShapeTexture2: shape.texture2 != null
    readonly property size shapeSize: shape.size
    readonly property rect shapeContentRect: shape.contentRect
    readonly property rect shapeTextureRect: shape.textureRect
    readonly property vector4d shapeParameters: shape.shaderParameters
    readonly property vector4d shapeParameters2: shape.shaderParameters2
    readonly property vector4d tcToPosition: {
        var sw = shape.size.width
        var sh = shape.size.height
        var w = sw + margins * 2.0
        var h = sh + margins * 2.0
        var mx = margins / w
        var my = margins / h
        var ax = sw / (1 - 2 * mx)
        var ay = sh / (1 - 2 * my)
        var bx = -mx * ax
        var by = -my * ay
        return Qt.vector4d(ax, ay, bx, by)
    }
    x: -margins
    y: -margins
    implicitWidth: shape.size.width + margins * 2.0
    implicitHeight: shape.size.height + margins * 2.0
    visible: root.settings && root.settings.hasOwnProperty("visible") ? root.settings.visible : true
    opacity: root.settings && root.settings.hasOwnProperty("opacity") ? root.settings.opacity/100 : 1
    fragmentShader: "
            precision highp float;
            varying vec2 qt_TexCoord0;
            uniform float qt_Opacity;
            uniform vec2 shapeSize;
            uniform vec4 shapeContentRect;
            uniform vec4 shapeTextureRect;
            uniform sampler2D shapeTexture;
            uniform sampler2D shapeTexture2;
            uniform bool hasShapeTexture2;
            uniform vec4 shapeParameters;
            uniform vec4 shapeParameters2;
            uniform float antialiasingSize;
            uniform vec4 tcToPosition;            
            
            struct ShapeInfo {
                vec2 tc;
                float d;
                float mask;
            };
    "
    +
            shape.shaderFunction
    +
            shaderFunction
    +
    "
            void main()
            {
                vec2 pos = qt_TexCoord0 * tcToPosition.xy + tcToPosition.zw;
                vec4 result = getResult(pos);
                gl_FragColor = result * qt_Opacity;
            }            
    "
}