import QtQuick 2.0
import ".."

ShapeShaderEffect {
    property SheenEffectSettings settings
    readonly property color color: settings.color
    readonly property real angle: settings.angle * Math.PI / 180
    readonly property bool perLetter: settings.perLetter && shape.hasOwnProperty("letterSize")
    property real offset: Math.max(Math.min(0.5, settings.offset / 200), -0.5)	
    property real size: Math.max(Math.min(1.0, settings.size / 100), 0.0)
	property real leftMargin: 0.5 + offset - size / 2
	property real rightMargin: 0.5 + offset + size / 2
    
    margins: antialiasingSize
    shaderFunction: "
        uniform vec4 color;
        uniform bool perLetter;
        uniform float angle;
        uniform float leftMargin;
        uniform float rightMargin;
        
        float getTexcoord(ShapeInfo shapeInfo, vec2 pos)
        {
            vec2 texcoord = perLetter ? shapeInfo.tc : (pos - shapeContentRect.xy) / shapeContentRect.zw;
            float mid = 0.5;
            return cos(angle) * (texcoord.y - mid) - sin(angle) * (texcoord.x - mid) + mid;                
        }
        
        float getSheen(float tc) {
            float result = (tc - leftMargin) / (rightMargin - leftMargin);
            result = clamp(result, 0.0, 1.0);
			result = 0.5 - abs(result - 0.5);
            result = result / 0.5;
			result = pow(result, 2.0);
            return result;
        }
        
        vec4 getResult(vec2 pos) 
        {
            ShapeInfo shapeInfo = getShapeInfo(pos);
            float tc = getTexcoord(shapeInfo, pos);
            float sheen = getSheen(tc);
            return color * sheen * shapeInfo.mask;
        }
    "
}