import QtQuick 2.0
import MixerGuiControls 1.0
import ".."

ShapeShaderEffect {
    property StrokeEffectSettings settings
    readonly property bool isOnProgram: settings.useSysColor=="program" || (settings.useSysColor=="from_slot" && mixer && mixer.mixerController && mixer.mixerController.programSlotIndex==(settings.slotIndex-1))
    readonly property bool isOnPreview: settings.useSysColor=="preset"  || (settings.useSysColor=="from_slot" && mixer && mixer.mixerController && mixer.mixerController.previewSlotIndex==(settings.slotIndex-1))
    readonly property color color: {
        if (isOnProgram) return MixerThemeManager.theme.colProgram
        if (isOnPreview) return MixerThemeManager.theme.colPreset
        var col = MixerThemeManager.theme.tagToSysColor[settings.useSysColor]
        return col!=undefined ? col : settings.color
    }
    readonly property real borderWidth: settings.outerWidth + settings.innerWidth
    readonly property real thinLineOpacity: Math.min(1.0, borderWidth / antialiasingSize / 2)
    readonly property vector4d intervals: {
        var center = (settings.outerWidth - settings.innerWidth) / 2.0
        if(settings.innerWidth + settings.outerWidth < antialiasingSize * 2.0 - 0.001) center = 0.0
        else if(settings.innerWidth < antialiasingSize / 4.0) center -= antialiasingSize / 4.0
        else if(settings.outerWidth < antialiasingSize / 4.0) center += antialiasingSize / 4.0
        var b0 = center - borderWidth/2
        var b1 = center + borderWidth/2
        var i0 = b0 - antialiasingSize
        var i1 = b0 + antialiasingSize
        var i2 = b1 - antialiasingSize
        var i3 = b1 + antialiasingSize
        if(i1 > center) i1 = center
        if(i2 < center) i2 = center
        return Qt.vector4d(i0, i1, i2, i3)
    }

    visible: settings.visible && borderWidth > 0.001
    margins: settings.outerWidth + antialiasingSize
    shaderFunction: "
        uniform vec4 color;
        uniform float thinLineOpacity;
        uniform vec4 intervals;
        vec4 getResult(vec2 pos) 
        {
            ShapeInfo shapeInfo = getShapeInfo(pos);
            float border = 0.0;
            if(shapeInfo.d < intervals[1]) border = smoothstep(intervals[0], intervals[1], shapeInfo.d);
            else if(shapeInfo.d < intervals[2]) border = 1.0;
            else border = 1.0 - smoothstep(intervals[2], intervals[3], shapeInfo.d);
            return border * color * thinLineOpacity;
        }
    "
}
