import QtQuick 2.3
import ".."

ShapeShaderEffect {
    property TextureMappingEffectSettings settings
    margins: antialiasingSize
    Item {
        id: texture
        visible: false
        layer.enabled: true
        layer.smooth: true
        layer.mipmap: true
        layer.wrapMode: ShaderEffectSource.Repeat
        implicitWidth: childrenRect.width
        implicitHeight: childrenRect.height
        Image {
            source: settings.texture
        }
    }
    readonly property Item source: texture
    readonly property vector2d texcoordOffset: Qt.vector2d(-settings.imageX, -settings.imageY)
    readonly property vector2d texcoordScale: Qt.vector2d(1.0 / settings.xScale * 100, 1.0 / settings.yScale * 100)
    readonly property size invTextureSize: Qt.size(1.0 / texture.width, 1.0 / texture.height)
    readonly property bool perLetter: settings.perLetter && shape.hasOwnProperty("letterSize")
    readonly property size letterSize: shape.letterSize ? shape.letterSize : Qt.size(0, 0)
    shaderFunction: "
        uniform sampler2D source;
        uniform vec2 texcoordOffset;
        uniform vec2 texcoordScale;
        uniform vec2 invTextureSize;
        uniform bool perLetter;
        uniform vec2 letterSize;
        
        vec2 getTexCoord(ShapeInfo shapeInfo, vec2 pos)
        {
            return perLetter ? shapeInfo.tc * letterSize : pos - shapeContentRect.xy;
        } 
        
        vec4 getResult(vec2 pos)
        {
            ShapeInfo shapeInfo = getShapeInfo(pos);
            vec2 texcoord = (getTexCoord(shapeInfo, pos) * texcoordScale + texcoordOffset) * invTextureSize;
            vec4 c = texture2D(source, texcoord);
            return c * shapeInfo.mask;
        }
    "
}