import QtQuick 2.0
import GuiControls 1.0
import ImageEffects 1.0
import ".."

QtObject {
    id: root
    property size objectSize
    property size mediaSize
    property AspectSettings aspectSettings
    property CropSettings cropSettings
    property rect shapeRect: {
        var c = root.cropSettings
        var cr0 = aspectItem.croppedContentRect
        if(!c.enabled || !c.keepOriginalSize) return cr0
        var r0 = aspectItem.contentRect
        var x1 = r0.x + r0.width * (c.left * 0.01)
        var x2 = r0.x + r0.width - r0.width * (c.right * 0.01)
        var y1 = r0.y + r0.height * (c.top * 0.01)
        var y2 = r0.y + r0.height - r0.height * (c.bottom * 0.01)
        x1 = Math.max(x1, cr0.x)
        y1 = Math.max(y1, cr0.y)
        x2 = Math.min(x2, cr0.x + cr0.width)
        y2 = Math.min(y2, cr0.y + cr0.height)            
        return Qt.rect(x1, y1, x2 - x1, y2 - y1)
    }
    readonly property rect textureRect: {
        var r0 = aspectItem.contentRect
        var c = root.cropSettings
        if(!c.enabled || c.keepOriginalSize) return Qt.rect(r0.x - shapeRect.x, r0.y - shapeRect.y, r0.width, r0.height)
        var x = Math.min(0, r0.x) - (croppedMediaRect.x / croppedMediaRect.width) * r0.width
        var y = Math.min(0, r0.y) - (croppedMediaRect.y / croppedMediaRect.height) * r0.height
        var w = (mediaSize.width / croppedMediaRect.width) * r0.width
        var h = (mediaSize.height / croppedMediaRect.height) * r0.height
        return Qt.rect(x, y, w, h)
    }
    readonly property rect uncroppedRect: aspectItem.contentRect
    
    readonly property rect croppedMediaRect: {
        var s0 = root.mediaSize
        var c = root.cropSettings
        if(!c.enabled || c.keepOriginalSize) return Qt.rect(0, 0, s0.width, s0.height)
        var x1 = s0.width * (c.left * 0.01)
        var x2 = s0.width - s0.width * (c.right * 0.01)
        var y1 = s0.height * (c.top * 0.01)
        var y2 = s0.height - s0.height * (c.bottom * 0.01)   
        return Qt.rect(x1, y1, x2 - x1, y2 - y1)    
    } 
    readonly property AspectItem aspectItem: AspectItem {
        id: aspectItem
        width: root.objectSize.width
        height: root.objectSize.height
        settings: root.aspectSettings
        cropSettings: root.cropSettings
        defaultAspect: root.mediaSize
    }
}