import QtQuick 2.3
import QtQuick.Layouts 1.2
import MixerGuiControls 1.0
import "../mixer_media"

MediaGraphicalObject {
    id: root
    name: "AudioLED"
    typeName: "GraphicalObjectAudioLED"
    typeNameUI: qsTr("Audio LED")
    typeIconName: "AudioLED"
    isEmpty: false
    property real dbLinearEnd: -20
    property real  ledMaxValue: 1.02

    property int   ledWidth:  20
    property int   ledThickness: 5
    property int   ledSpacing: 2
    property bool  separatedLeds: true

    property bool showDigits: true
    property bool digitsAtLeft: false
    property int  fontSize: MixerThemeManager.theme.promptFontSize
    property string fontFamily: MixerThemeManager.theme.fontFamily
    property bool  boldFont: false
    property color digitsColor: MixerThemeManager.theme.colNeutral

    property color colRed:    MixerThemeManager.theme.colProgram
    property color colYellow: MixerThemeManager.theme.colHSL(50, 90, 100)
    property color colGreen:  MixerThemeManager.theme.colPreset

    property bool  showBgnd:    false
    property color colBgnd:     MixerThemeManager.theme.colWAlpha(MixerThemeManager.theme.colNeutralDark,  0.5)
    property int   bgndMargins: 0

    readonly property bool isMixer: typeof mixer!='undefined' && mixer

    readonly property Loader mixerMediaSourceLoader: Loader {
        sourceComponent: (scene && scene.mixerMediaManager) ? scene.mixerMediaManager.mixerMediaSourceComponent : null
    }
    readonly property MixerMediaSource mixerMediaSource: mixerMediaSourceLoader.item
    readonly property string srcName: mixerMediaSource ? mixerMediaSource.name : ''
    readonly property bool isslot: srcName.indexOf('Slot:///')===0
    readonly property var  slot:  isslot && isMixer ? mixer.inputSlots[index] : null
    readonly property var  input: isslot ? (slot ? slot.input : null) : isMixer ? mixer.outputs[index].input : null
    readonly property int  index: isslot ? parseInt(srcName.split(':///')[1])-1 : srcName=='Program' ? 1:0

    function getAudioChannel(param) {
      var channel = slot || (isMixer  && mixer.audioMixer && mixer.audioMixer.outputChannels && mixer.audioMixer.outputChannels[index] && mixer.audioMixer.outputChannels[index])
      if (!channel) return [0, 0]
      return channel[param] || [0, 0]
    }

    media: ShaderEffectSource {
        recursive: true
        sourceItem: Item {
         width: root.width
         height: root.height
         Rectangle {
            anchors.fill: parent
            color: colBgnd
            visible: showBgnd
         }
         RowLayout {
            anchors.fill: parent
            anchors.margins: showBgnd ? bgndMargins : 0
            Layout.alignment: Qt.AlignHCenter
            spacing: 2
            layoutDirection: digitsAtLeft ? Qt.RightToLeft : Qt.LeftToRight
            AudioLevelControlBig {
              id: contr
              Layout.fillHeight: true
              dbLinearEnd: root.dbLinearEnd
              handleVisible: false
              peakVisible: true
              flatColorStyle: true
              showBgnd: false
              ledWidth: root.ledWidth
              ledThickness: root.ledThickness
              ledSpacing: root.ledSpacing
              ledStyle:   root.separatedLeds
              colRed:     root.colRed
              colYellow:  root.colYellow
              colGreen:   root.colGreen
              level:     getAudioChannel(slot ? 'audioLevels':'levels')
              peakLevel: getAudioChannel(slot ? 'audioLevelsIntegral':'levelsIntegral')
              redLevel:  getAudioChannel(slot ? 'audioLevelsRed':'levelsRed')
              ledMaxValue: root.ledMaxValue
            }
            AudioScale {
                Layout.fillHeight: true
                Layout.alignment: Qt.AlignRight
                visible: showDigits
                dbLinearEnd: root.dbLinearEnd
                fontSize: root.fontSize
                fontFamily: root.fontFamily
                boldFont: root.boldFont
                numbersColor: root.digitsColor
                showSliderTicks: false
                maximumValue: root.ledMaxValue
            }
        }
        }
    }

    function save() {
        var data = mediaGraphicalObject_save();
        if (mixerMediaSource) data["mixerMediaSource"] = mixerMediaSource.save()
        data.ledWidth     = ledWidth
        data.ledThickness  = ledThickness
        data.ledSpacing   = ledSpacing
        data.separatedLeds= separatedLeds
        data.showDigits   = showDigits
        data.fontSize     = fontSize
        data.fontFamily   = fontFamily
        data.boldFont     = boldFont
        data.digitsColor  = digitsColor
        data.digitsAtLeft = digitsAtLeft
        data.colRed       = colRed
        data.colYellow    = colYellow
        data.colGreen     = colGreen
        data.showBgnd     = showBgnd
        data.colBgnd      = colBgnd
        data.bgndMargins  = bgndMargins
        return data;
    }

    function load(data) {
        mediaGraphicalObject_load(data);
        if (data.mixerMediaSource && mixerMediaSource) mixerMediaSource.load(data.mixerMediaSource)
        ledWidth    = data.ledWidth     || ledWidth
        ledThickness = data.ledThickness  || ledThickness
        ledSpacing  = data.ledSpacing   || ledSpacing
        separatedLeds= data.separatedLeds !=undefined ? data.separatedLeds : separatedLeds
        showDigits  = data.showDigits     !=undefined ? data.showDigits : showDigits
        fontSize    = data.fontSize     || fontSize
        fontFamily  = data.fontFamily   || fontFamily
        boldFont    = data.boldFont!=undefined      ? data.boldFont : boldFont
        digitsColor = data.digitsColor!=undefined   ? data.digitsColor : digitsColor
        digitsAtLeft= data.digitsAtLeft!=undefined  ? data.digitsAtLeft : digitsAtLeft
        colRed      = data.colRed!=undefined        ? data.colRed    : colRed
        colYellow   = data.colYellow!=undefined     ? data.colYellow : colYellow
        colGreen    = data.colGreen!=undefined      ? data.colGreen : colGreen
        showBgnd    = data.showBgnd !=undefined     ? data.showBgnd : showBgnd
        colBgnd     = data.colBgnd!=undefined       ? data.colBgnd  : colBgnd
        bgndMargins = data.bgndMargins!=undefined   ? data.bgndMargins: bgndMargins
    }
}
