import QtQuick 2.3
import Multimedia 1.0
import QtQuickUtils 1.0
import "../shapes"

MediaGraphicalObject {
    id: root
    property url source
    property int fps: 24
    typeName: "GraphicalObjectSequence"
    typeNameUI: qsTr("Sequence")
    typeIconName: "Animation"
    name: "Sequence"
    isEmpty: !source.toString()
    readonly property alias duration: sequence.duration
    readonly property ImageSequence sequence: ImageSequence {
        id: sequence
        folder: root.source
        enabled: scene && scene.loadMedia
        frameRate: Qt.size(root.fps, 1)
        loops: 0
        readonly property bool needPlay: scene && scene.playMedia
        onNeedPlayChanged: {
            if(needPlay) play()
            else {
                pause()
                position = 0
            }
        }
        onLoaded: {
            if(needPlay) play()
            else pause()
        }
    }
    
    media: ShaderEffectSource {
        implicitWidth: sourceItem.width
        implicitHeight: sourceItem.height
        sourceItem: TextureView {
            source: sequence
        }
        mipmap: true
    }
    mediaSize: sequence.frameSize
    
    function save() {
        var data = mediaGraphicalObject_save()
        data["source"] = root.source
        data["fps"] = root.fps
        return data
    }
    
    function load(data) {
        mediaGraphicalObject_load(data)
        root.source = data.source || ""
        root.fps = data.fps || 24
    }
    
}
