import QtQuick 2.3
import Multimedia 1.0
import Multimedia.FFmpeg 1.0
import Multimedia.SoftlabCodecs 1.0

import "../shapes"

MediaGraphicalObject {
    id: root
    property url source
    typeName: "GraphicalObjectVideo"
    typeNameUI: qsTr("Video")
    typeIconName: "Video"
    name: "Video"
    isEmpty: !source.toString()
    readonly property alias duration: mediaSession.duration
    readonly property alias videoSink: videoSink
    readonly property MediaSession mediaSession: MediaSession {
        id: mediaSession
        enabled: scene && scene.loadMedia
        readonly property bool needPlay: scene && scene.playMedia
        
        function updatePlaying() {
            if(!enabled) return
            if(needPlay) play()
            else {
                pause()
                position = 0
            }
        }
        
        onNeedPlayChanged: updatePlaying()
        onEnabledChanged: updatePlaying()
        onStatusChanged: if(status == MediaSession.Stopped) updatePlaying()
        
        FFmpegSource {
            id: sourceNode
            url: root.source
            loops: 0
        }
        
        QtQuickSink {
            id: videoSink
        }
        NodeConnection {
            from: sourceNode.videoOutput
            to: videoSink.input
        }
    }
    media: ShaderEffectSource {
        implicitWidth: sourceItem.width
        implicitHeight: sourceItem.height
        onImplicitWidthChanged: root.mediaSize.width = implicitWidth
        onImplicitHeightChanged: root.mediaSize.height = implicitHeight
        sourceItem: VideoView {
            source: root.videoSink
        }
        mipmap: true
    }
    mediaSize: Qt.size(0, 0)
    
    function save() {
        var data = mediaGraphicalObject_save()
        data["source"] = root.source
        return data
    }
    
    function load(data) {
        mediaGraphicalObject_load(data)
        root.source = data.source || ""
    }
    
}
