import QtQuick 2.0
import "../shapes"
import ".."

ShapeGraphicalObject {
    id: root
    isEmpty: text.length == 0
    property real aspect: 1.0
    property string text: ""
    property string textSourceType: "Static"
    property string textSourceTypeComponentName: "Static"
    property TextSource textSource: null
    property vector2d contentOffset: Qt.vector2d(0, 0)
    property bool fixedTextureWidth
    property bool fixedTextureHeight
    property rect contentRect: Qt.rect(0, 0, 0, 0)
    property rect nonOffsetedContentRect: Qt.rect(0, 0, 0, 0)
    typeName: "GraphicalText"   
    typeNameUI: qsTr("Text")
    typeIconName: "ToolText"
    name: "Text"   
    
    shape: TextShape {
        size: Qt.size(root.width, root.height)
        text: root.text
        textSettings: root.styleSettings.textSettings
        outerDistanceFieldRadius: root.styleSettings.outerDistanceFieldRadius
        innerDistanceFieldRadius: root.styleSettings.innerDistanceFieldRadius
        contentOffset: root.contentOffset
        fixedTextureWidth: root.fixedTextureWidth
        fixedTextureHeight: root.fixedTextureHeight
        texturesQuality: root.scene.texturesQuality
        onContentRectChanged: if(root.contentRect != contentRect) root.contentRect = contentRect
        onNonOffsetedContentRectChanged: if(root.nonOffsetedContentRect != nonOffsetedContentRect) root.nonOffsetedContentRect = nonOffsetedContentRect
    }
    
    property Loader textSourceLoader: Loader {
        source: textSourceTypeComponentName ? "../text_sources/" + textSourceTypeComponentName + "TextSource.qml" : ""
        onItemChanged: {
            if(textSource) textSource.onDeleted()
            textSource = item
            if(textSource) {
                textSource.owner = root
                textSourceType = textSource.type
                if(textSourceType=="Table") {
                    textSource.table = scene.getMainTable()
                }
                else
                if(textSourceType=="File") {
                    textSource.table = scene.getMainTable("files")
                    textSource.table.readonly = true
                }
                textSource.onCreated()
            }
        }
    }

    property Connections textSourceConnections: Connections {
        target: textSource
        onSignificantTextChanging: {
            if(!scene.playMedia || currentAnimatorCategory == "Hide" || (animator.hasCategory("Hide") && scene.animator.scheduledCategory == "Hide")) return
            animator.cancelScheduledStart()
            if(text != nextValue && (currentAnimatorCategory != "Change" || text == textFromSource)) animator.scheduleStart("Change")
        }
    }

    readonly property string textFromSource: textSource ? textSource.value : ""
    readonly property string currentAnimatorCategory: animator.currentCategory
    onTextFromSourceChanged: if(!scene.playMedia || !animator.running) updateDataFromSource()
    onCurrentAnimatorCategoryChanged: if(currentAnimatorCategory != "Change") updateDataFromSource()
    
    function updateDataFromSource() {
        text = textFromSource
    }

    onDeletedFromScene: if(textSource) textSource.onDeleted()
    
    function graphicalText_save() {
        var data = shapeGraphicalObject_save()
        data["aspect"] = aspect
        data["textSourceType"] = textSourceType
        data["textSourceTypeComponentName"] = textSourceTypeComponentName
        data["textSource"] = textSource ? textSource.save() : {}        
        return data
    }
    
    function graphicalText_load(data) {
        shapeGraphicalObject_load(data)
        aspect = data.aspect
        textSourceType = data.textSourceType || "Static"
        textSourceTypeComponentName = data.textSourceTypeComponentName || textSourceType
        if(textSource) {
            textSource.load(data.textSource)
            if (textSourceType=="Table") {
                var colName = textSource.columnName
                if (colName && colName!=root.name)
                    root.name = colName
            }
        }
    }
    
    function save() {
        return graphicalText_save()
    }
    
    function load(data) {
        graphicalText_load(data)
    }
}
