import QtQuick 2.4
import GuiControls 1.0
import ".."

GraphicalObject {
    id: root    
    property GroupNode node: GroupNode { 
        objParent: root 
        scene: root.scene
    } 
    property real prevWidth: width
    property real prevHeight: height
    property bool updatingOrigins: false
    property bool updatingGroup: false
    name: "Group"
    typeName: "GroupGraphicalObject"
    typeNameUI: qsTr("Group")
    typeIconName: "Link"
    ungroupEnabled: true
    
    readonly property real paddingSize: {
        var result = 0
        var objects = getChildren()
        for(var i = 0; i < objects.length; i++) {
            result = Math.max(result, objects[i].paddingSize || 0)
        }
        return result
    }
    boundingBox: {
        var children = getChildren();
        var boundingBox = getBoundingBox(children);

        var c = Math.cos((Math.PI * rotation) / 180);
        var s = Math.sin((Math.PI * rotation) / 180);
        var x1 = root.x;
        var y1 = root.y;
        var x2 = boundingBox.x;
        var y2 = boundingBox.y;
        var w = (boundingBox.z - root.width) / 2;
        var h = (boundingBox.w - root.height) / 2;
        var newX = c*(x2 + w) - s*(y2 + h) - w + x1;
        var newY = s*(x2 + w) + c*(y2 + h) - h + y1;

        return Qt.rect(newX, newY, boundingBox.z, boundingBox.w)
    }
    
    drawingDelegate: GroupView {
        x: root.x
        y: root.y
        visible: root.visible
        width: root.width
        height: root.height
        transform: Scale {
            xScale: root.scaleX / 100* (root.flipHorizontal ? -1 : 1)
            yScale: root.scaleY / 100 * (root.flipVertical ? -1 : 1)
            origin.x: root.width * 0.5
            origin.y: root.height * 0.5
        }             
        objects: getChildren()
    }
    onUpdated: {
        updateOrigins();
        if (objParent) {
            objParent.updated();
        }
    }

    function resetId() {
        id = root.scene.id++
        var children = getChildren()
        for (var i in children) {
            children[i].resetId()
        }
    }
    function add(object) {
        return node.add(object);
    }
    function deleteObject(object) {
        return node.deleteObject(object);
    }
    function deleteObjects(objects) {
        return node.deleteObjects(objects);
    }
    function deleteAllObjects() {
        return node.deleteAllObjects();
    }
    function groupObjects(objects) {
        return node.groupObjects(objects);
    }
    function mergeInGroup(objectList) {
        return node.mergeInGroup(objectList);
    }
    function ungroupObjects() {
        return node.ungroupObjects();
    }
    function getChildren(reverse) {
        return node.getChildren(reverse);
    }
    function getChildrenRecurs() {
        return node.getChildrenRecurs();
    }
    function clearChildrenList() {
        return node.clearChildrenList();
    }
    function bringLevelUp(items) {
        return node.bringLevelUp(items);
    }
    function bringLevelDown(items) {
        return node.bringLevelDown(items);
    }        
    function bringLevelTop(items) {
        return node.bringLevelTop(items);
    }
    function isOnTopLevel(items) {
        return node.isOnTopLevel(items);
    }
    function isOnDownLevel(items) {
        return node.isOnDownLevel(items);
    }
    function bringLevelBottom(items) {
        return node.bringLevelBottom(items);
    }
    function getNextObject(object) {
        return node.getNextObject(object);
    }
    function getPrevObject(object) {
        return node.getPrevObject(object);
    }
    function addGroup() {
        return node.addGroup();
    }
    function swapObjectsToParent(listObjects, newParent) {
        return node.swapObjectsToParent(listObjects, newParent);
    }
    function getIndex(object) {
        return node.getIndex(object);
    }
    function setObjectsPosition(objects, position) {
        return node.setObjectsPosition(objects, position);
    }
    function updateOrigins() {
        if (updatingOrigins)
            return
        updatingOrigins = true;        
        var children = getChildren();
        var boundingBox = getBoundingBox(children);
        var c = Math.cos((Math.PI * rotation) / 180);
        var s = Math.sin((Math.PI * rotation) / 180);
        var x1 = root.x;
        var y1 = root.y;
        var x2 = boundingBox.x;
        var y2 = boundingBox.y;
        var w = (boundingBox.z - root.width) / 2;
        var h = (boundingBox.w - root.height) / 2;
        var newX = c*(x2 + w) - s*(y2 + h) - w + x1;
        var newY = s*(x2 + w) + c*(y2 + h) - h + y1;
        for (var index in children) {
            var curObject = children[index];
            curObject.x += -x2 
            curObject.y += -y2
        }
        root.width = boundingBox.z
        root.height = boundingBox.w
        root.x = newX
        root.y = newY
        
        updatingOrigins = false;        
    }
    function updateChildrenWidth() {
        var k = (width - prevWidth) / prevWidth;
        var children = getChildren();
        for (var i in children) {
            children[i].width += (children[i].width * k);
            children[i].x += (children[i].x * k);
        }
    }
    onWidthChanged: {
        if (updatingOrigins)
        {
            prevWidth = width;
            return;
        }
        updatingGroup = true;
        
        updateChildrenWidth();
        prevWidth = width;
        
        updatingGroup = false;
    }
    function updateChildrenHeight() {
        var k = (height - prevHeight) / prevHeight;
        var children = getChildren();
        for (var i in children) {
            children[i].height += (children[i].height * k);
            children[i].y += (children[i].y * k);
        }
    } 
    onHeightChanged: {
        if (updatingOrigins)
        {
            prevHeight = height;
            return;
        }
        updatingGroup = true;
        
        updateChildrenHeight();
        prevHeight = height;
        
        updatingGroup = false;
    }
    
    function getBoundingBox(objectList)
    {
        if (!objectList)
			return Qt.vector2d(0,0)
		var objectsData = []
		if (objectList.length == 0)
			return Qt.vector2d(0,0)
		var xmin = objectList[0].x
		var ymin = objectList[0].y
        var xmax = objectList[0].x
		var ymax = objectList[0].y
		for (var index in objectList)
		{
			var object = objectList[index]
            xmin = Math.min(xmin, object.boundingBox.x)
            ymin = Math.min(ymin, object.boundingBox.y)
            xmax = Math.max(xmax, object.boundingBox.x + object.boundingBox.width)
            ymax = Math.max(ymax, object.boundingBox.y + object.boundingBox.height)
		}
		return Qt.vector4d(xmin, ymin, xmax - xmin, ymax - ymin)
    }
    
    function save(){
        var data = graphicalObject_save();
        data["objects"] = saveChildren();
        return data;
    }
    function saveChildren() {
        var data = [];
        var children = getChildren();
        for (var i in children) {
            var object = children[i];
            var childData = object.save();
            data.push(childData);
        }
        return data;
    }
    function load(data) {
        updatingOrigins = true
        loadChildren(data.objects);
        graphicalObject_load(data);
        updatingOrigins = false
    }
    function loadChildren(data) {
        var newObjects = []
        for (var index in data)
        {
            var newObj = null
            if (data[index].id && root.scene)
            {
                newObj = root.scene.findObjectById(data[index].id)
                if (newObj)
                {
                    if (data[index].typeName == newObj.typeName)
                    {
                        newObj.load(data[index])
                        root.id = Math.max(root.id, newObj.id)
                    }
                    else
                    {
                        newObj = null
                    }
                }
            }
            if (!newObj)
            {
                newObj = createGraphicalObject(data[index])
            }
            if (newObj)
            {
                newObjects.push(newObj)
                if (scene && scene.usingObjects)
                {
                    scene.usingObjects.push(newObj)
                }
            }
        }

        if (scene && scene.usingObjects)
        {
            node.deleteAllObjects(scene.usingObjects)
        }
        else
        {
            node.deleteAllObjects(newObjects)
        }
        for (var index in newObjects)
        {
            node.children.push(newObjects[index]);
            newObjects[index].setParent(node.objParent);
        }

        node.children = node.children;
    }
    function deleteFromScene(objects) {
        var children = getChildren();
        for (var i in children) {
            if (objects)
            {
                var index = objects.indexOf(children[i]);
                if (index !== -1) continue;
            }
           children[i].deleteFromScene(); 
        }
        deletedFromScene();
        destroy();
    }
    function deselect() {
        var children = getChildren();
        for (var i in children) {
            children[i].deselect();
        }
        selected = false;
    }
}
