import QtQml 2.0
import QtQuick 2.4

QtObject {	
	id: root
    property var children: []
    property var objParent: null
    property var scene: null
    
    function add(object) {      
        if (!object) return;
        var objectGlobalCoord = object.getGlobalCoord();
        objectGlobalCoord = Qt.vector2d(objectGlobalCoord.x + object.width / 2, objectGlobalCoord.y + object.height / 2);
        var result = objectGlobalCoord;
        if (objParent && objParent != scene) {
            var parentGlobalCoord = objParent.getTransformMatrixInvert();            
            objectGlobalCoord = objectGlobalCoord.toVector4d()
            objectGlobalCoord.w = 1.0
            result = objectGlobalCoord.times(parentGlobalCoord);
        }
        result = Qt.vector2d(result.x - object.width / 2, result.y - object.height / 2);
        object.x = result.x;
        object.y = result.y;
        object.rotation = object.getGlobalAngle() - objParent.getGlobalAngle();
        object.setParent(objParent);
        if (!object.id) {
            object.id = root.scene.id++
        }
        children.push(object);        
        children = children;
        if (objParent) {
            objParent.updated();
        }
    }
    function deleteObject(object) {
        if (!object) return;
        var index = children.indexOf(object);
        if (index !== -1) {
            children.splice(index, 1);
            object.deleteFromScene();
            children = children;
            if (objParent) {
                objParent.updated();
            }
        }
    }
    function deleteObjects(objects) {
        if (objects.length === 0) return;
        if (!objects[0].onTheSameLevelWithObjects(objects)) return;
        if (!objects[0].haveSameParentWithObjects(objects)) return;
        for (var i in objects) {
            var object = objects[i];
            var index = children.indexOf(object);
            if (index === -1) continue;
            children.splice(index, 1);
            object.deleteFromScene();
        }
        children = children;
        if (objParent) {
            objParent.updated();
        }
    }
    function deleteAllObjects(objects) {
        for (var i in children) {
            var child = children[i];
            if (objects)
            {
                var index = objects.indexOf(child);
                if (index !== -1) continue;
            }
            child.deleteFromScene(objects);
        }
        children = [];
        if (objParent) {
            objParent.updated();
        }
    }
    function swapToParent(object, newParent) {
        if (!object || !newParent) return;
        if (newParent === objParent) return;
        var index = children.indexOf(object);
        var objectX = object.getGlobalCoord().x;
        var objectY = object.getGlobalCoord().y;
        children.splice(index, 1);
        children = children;
        object.x = objectX;
        object.y = objectY;
        object.setParent(null);
        newParent.add(object);
    }
    function swapObjectsToParent(listObjects, newParent) {
        if ((listObjects.length === 0) || !newParent) return;
        if (newParent === objParent) return;
        var newObjects = [];
        var objectsCoord = [];
        for (var i in listObjects) {
            var object = listObjects[i];
            if (object.freezed) continue
            var index = children.indexOf(object);
            if (index !== -1) {
                var objectCoord = object.getGlobalCoord();
                objectsCoord.push(objectCoord);
                newObjects.push(children[index]);
            }
        }
        for (var indexObject in listObjects) {
            var object = listObjects[indexObject];
            if (object.freezed) continue
            var indexChild = children.indexOf(object);
            if (indexChild !== -1) {
                children.splice(indexChild, 1);
            }
        }
        children = children;
        for (var index in newObjects) {
            var object = newObjects[index];
            var objectX = objectsCoord[index].x;
            var objectY = objectsCoord[index].y;
            object.x = objectX;
            object.y = objectY;
            object.setParent(null);
            newParent.add(object);
        }
        newObjects = [];
        objectsCoord = [];
    }
    function swapAllObjectsToParent(newParent) {
        if (!newParent) return;
        if (newParent === objParent) return;
        var objectsCoord = [];
        for (var i in children) {
            var child = children[i];
            var objectCoord = child.getGlobalCoord();
            objectsCoord.push(objectCoord);
        }
        var newObjects = [];
        newObjects = newObjects.concat(children);
        children = [];
        for (var index in newObjects) {
            var object = newObjects[index];
            var objectX = objectsCoord[index].x;
            var objectY = objectsCoord[index].y;
            object.x = objectX;
            object.y = objectY;
            object.setParent(null);
            newParent.add(object);
        }
        newObjects = [];
        objectsCoord = [];
    }
    function getChildrenRecurs() {
        var list = [];
        for (var i in children) {
            var child = children[i];
            list.push(child);
            if (child.typeName === "GroupGraphicalObject") {
                var innerList = child.getChildrenRecurs();
                list = list.concat(innerList);
            }
        }
        return list;
    }
    
    function getChildren(reverse) {
        if ((reverse !== undefined) && (reverse === true)) {
            var reverseChildren = [];
            for (var i in children) {
                var child = children[i];
                reverseChildren.splice(0, 0, child);
            }
            return reverseChildren;
        }
        else {
            return children;
        }
    }
    function clearChildrenList() {
        children = [];
    }
    function groupObjects(objects) {
        if (objects.length === 0) return null;
        if (!objects[0].onTheSameLevelWithObjects(objects)) return null;
        if (!objects[0].haveSameParentWithObjects(objects)) return null;
        
        var group = createGroup();
        swapObjectsToParent(objects, group);
        add(group);
        return group;
    }
    function createGroup() {
        var component = Qt.createComponent("GroupGraphicalObject.qml", root);
        var result = null;
        if (component.status == Component.Ready) {
            result = component.createObject(root, { "scene": root.scene });
        }
        return result
    }
    function addGroup() {
        var group = createGroup();
        add(group);
    }
    function ungroupObjects() {
        var list = [];
        list = list.concat(children);
        
        if (objParent) {
            var grandParent = objParent.getParent();
            swapAllObjectsToParent(grandParent);
            grandParent.deleteObject(objParent);
        }
        
        return list;
    }
    function mergeInGroup(objectList) {
        if (objectList.length === 0) return;
        var list = [];
        for (var i in objectList) {
            var object = objectList[i];
            if (object.freezed) continue
            var index = children.indexOf(object);
            if (index === -1) continue;
            if (object.typeName === "GroupGraphicalObject") {
                var innerList = object.ungroupObjects();    
                list = list.concat(innerList);
            }
            else {
                list.push(object)
            }
        }
        var result = groupObjects(list);
        return result;
    }
	function bringLevelUp(items) {
        for (var i = items.length - 1; i >= 0; --i) {
            var item = items[i];
            var index = children.indexOf(item);
            if ((index === -1) || (index === children.length - 1)) continue;
            var nextObject = children[index + 1];
            if (items.indexOf(nextObject) !== -1) continue;
            children.splice(index, 1);
            ++index;
            children.splice(index, 0, item);
        }
        children = children;
	}
    function isOnTopLevel(items) {
        var topPosition = children.length - items.length;
        for (var i in items) {
            var item = items[i];
            if (children.indexOf(item) >= topPosition) continue;
            return false;
        }
        return true;
    }
    function bringLevelDown(items) {
        for (var i in items) {
            var item = items[i];
            var index = children.indexOf(item);
            if ((index === -1) || (index === 0)) continue;
            var nextObject = children[index - 1];
            if (items.indexOf(nextObject) !== -1) continue;
            children.splice(index, 1);
            --index;
            children.splice(index, 0, item);
        }
        children = children;
	}
    function isOnDownLevel(items) {
        var downPosition = items.length - 1;
        for (var i in items) {
            var item = items[i];
            if (children.indexOf(item) <= downPosition) continue;
            return false;
        }
        return true;
    }
	function bringLevelTop(items) {
        for (var i in items) {
            var item = items[i];
            var index = children.indexOf(item);
            if ((index === -1) || (index === children.length - 1)) continue;
            children.splice(index, 1);
            children.push(item);
        }
        children = children;
	}
    function bringLevelBottom(items) {
        for (var i = items.length - 1; i >= 0; --i) {
            var item = items[i];
            var index = children.indexOf(item);
            if ((index === -1) || (index === 0)) continue;
            children.splice(index, 1);
            children.splice(0, 0, item);
        }
        children = children;
	}
    function getNextObject(object) {
        if (children.length === 0) return null;
        var index = children.indexOf(object);
        if ((index === -1) || (index === (children.length - 1))) {
            index = 0;
        }
        else {
            ++index;
        }
        return children[index];
    }
    function getPrevObject(object) {
        if (children.length === 0) return null;
        var index = children.indexOf(object);
        if (index === -1) {
            index = 0;
        }
        else if (index === 0) {
            index = children.length - 1;
        }
        else {
            --index;
        }
        return children[index];
    }
    function setObjectPosition(object, position) {
        if (!object) return;
        var index = children.indexOf(object);
        if (index === position) return;
        if (index === -1) return;
        var objectOnPosition = children[position];
        children.splice(index, 1);
        var newPostion = children.indexOf(objectOnPosition);
        children.splice(newPostion + 1, 0, object);
        children = children;
    }
    function setObjectsPosition(objects, position) {
        var object = objects[0];
        var index = children.indexOf(object);
        for (var i in objects) {
            var object = objects[i];
            var newPostion = parseInt(position) + parseInt(i);
            setObjectPosition(object, newPostion);
        }
    }
    function getIndex(object) {
        return children.indexOf(object);
    }
}
