import QtQuick 2.3
import GuiControls 1.0
import ImageEffects 1.0
import "../shapes"
import ".."

GraphicalObject {
    id: root
    typeName: "MediaGraphicalObject"
    property Component media
    property size mediaSize: Qt.size(0, 0)
    property size croppedMediaSize: {
        if(!cropSettings.enabled || cropSettings.keepOriginalSize) return mediaSize
        var w = mediaSize.width * (1 - (cropSettings.left + cropSettings.right) * 0.01)
        var h = mediaSize.height * (1 - (cropSettings.top + cropSettings.bottom) * 0.01)
        return Qt.size(w, h)
    }
    property StyleSettings styleSettings: StyleSettings{}
    property AspectSettings aspectSettings: AspectSettings {}
    property CropSettings cropSettings: CropSettings {}

    function autoSize() {
        var aspect = aspectSettings.aspect
        if(cropSettings.enabled && !cropSettings.keepOriginalSize) {
            aspect = Qt.size(
                aspect.width * (100 - cropSettings.left - cropSettings.right), 
                aspect.height * (100 - cropSettings.top - cropSettings.bottom)
            )
        }
        width = aspect.height > 0 ? croppedMediaSize.height * aspect.width/aspect.height : croppedMediaSize.width
        height = croppedMediaSize.height
    }
    
    function mediaGraphicalObject_save() {
        var data = graphicalObject_save()
        data["styleSettings"] = styleSettings.save()
        data["aspectSettings"] = aspectSettings.save()
        data["cropSettings"] = cropSettings.save()
        return data
    }
    
    function mediaGraphicalObject_load(data) {
        graphicalObject_load(data)
        styleSettings.load(data.styleSettings)
        aspectSettings.load(data.aspectSettings)
        cropSettings.load(data.cropSettings)
    }
    
    function save() {
        return mediaGraphicalObject_save()
    }
    
    function load(data) {
        return mediaGraphicalObject_load(data)
    }
    
    drawingDelegate: Item {
        id: drawingDelegate
        x: root.x
        y: root.y
        visible: root.visible
        width: root.width
        height: root.height
        transform: Scale {
            xScale: root.scaleX / 100 * (root.flipHorizontal ? -1 : 1)
            yScale: root.scaleY / 100 * (root.flipVertical ? -1 : 1)
            origin.x: root.width * 0.5
            origin.y: root.height * 0.5
        }
        property Item media: mediaLoader.item
        Component {
            id: defaultMedia
            ShaderEffectSource {
                implicitWidth: sourceItem.width
                implicitHeight: sourceItem.height
                sourceItem: Rectangle {
                    width: 1
                    height: 1
                }
            }
        }   
        property Loader mediaLoader: Loader {
            sourceComponent: root.media || defaultMedia
        }
        CropCalculator {
            id: crop
            objectSize: Qt.size(root.width, root.height)
            mediaSize: Qt.size(media.width, media.height)
            cropSettings: root.cropSettings
            aspectSettings: root.aspectSettings
        }
        RectangleShape {
            id: shape
            size: Qt.size(crop.shapeRect.width, crop.shapeRect.height)
        }
        Item {
            anchors.fill: parent
            clip: true
            ShaderEffect {
                x: crop.uncroppedRect.x
                y: crop.uncroppedRect.y
                width: crop.uncroppedRect.width
                height: crop.uncroppedRect.height
                property var source: drawingDelegate.media
                visible: root.selected && root.cropSettings.enabled && root.cropSettings.keepOriginalSize
                opacity: 0.2
            }
        }
        ShapeWithStyleView {
            x: crop.shapeRect.x
            y: crop.shapeRect.y
            crop: root.crop
            shape: shape
            styleSettings: root.styleSettings
            texture: media
            textureRect: crop.textureRect
            texturesQuality: root.scene.texturesQuality
            visible: !root.isEmpty
        }
    }
}
