import QtQuick 2.3
import GuiControls 1.0
import ".."

GraphicalObject {
    id: root    
    typeName: "StyledGraphicalObject"
    property StyleSettings styleSettings: StyleSettings{}
    property Component shape
    property bool clip: false
    readonly property real paddingSize: styleSettings.distanceFieldRadius + 2
    
    drawingDelegate: Item {
        clip: root.clip
        visible: root.visible
        x: root.x
        y: root.y
        width: root.width
        height: root.height
        transform: Scale {
            xScale: root.scaleX / 100 * (root.flipHorizontal ? -1 : 1)
            yScale: root.scaleY / 100 * (root.flipVertical ? -1 : 1)
            origin.x: root.width * 0.5
            origin.y: root.height * 0.5
        }        
        ShapeWithStyleView {
            shape: shapeLoader.item
            styleSettings: root.styleSettings
            crop: root.crop
            texturesQuality: root.scene.texturesQuality
            property Loader shapeLoader: Loader {
                sourceComponent: root.shape
            }
        }
    }    
    
    function shapeGraphicalObject_save() {
        var data = graphicalObject_save();
        data["styleSettings"] = styleSettings.save();
        return data;
    }
    
    function shapeGraphicalObject_load(data) {
        graphicalObject_load(data);
        styleSettings.load(data.styleSettings);
    }
    
    function save() {
        return shapeGraphicalObject_save()
    }
    
    function load(data) {
        return shapeGraphicalObject_load(data)
    }    
}
