import QtQuick 2.0
import ".."

Shape {
    shaderFunction: "
        float distanceToEllipseImpl(vec2 e, vec2 y)
        {
            vec2 x;
            float sqrDistance;
            if (y[1] > 0.001)
            {
                if (y[0] > 0.001)
                {
                    // Bisect to compute the root of F(t) for t >= -e1*e1.
                    vec2 esqr = vec2(e[0]*e[0], e[1]*e[1]);
                    vec2 ey = vec2(e[0]*y[0], e[1]*y[1]);
                    float t0 = -esqr[1] + ey[1];
                    float t1 = -esqr[1] + sqrt(ey[0]*ey[0] + ey[1]*ey[1]);
                    float t = t0;
                    for (int i = 0; i < 30; ++i)
                    {
                        t = 0.5*(t0 + t1);
                        if (t == t0 || t == t1)
                        {
                            break;
                        }

                        vec2 r = vec2(ey[0]/(t + esqr[0]), ey[1]/(t + esqr[1]));
                        float f = r[0]*r[0] + r[1]*r[1] - 1.0;
                        if (f > 0.0)
                        {
                            t0 = t;
                        }
                        else if (f < 0.0)
                        {
                            t1 = t;
                        }
                        else
                        {
                            break;
                        }
                    }

                    x[0] = esqr[0]*y[0]/(t + esqr[0]);
                    x[1] = esqr[1]*y[1]/(t + esqr[1]);
                    vec2 d = vec2(x[0] - y[0], x[1] - y[1]);
                    sqrDistance = d[0]*d[0] + d[1]*d[1];
                }
                else  // y0 == 0
                {
                    x[0] = 0.0;
                    x[1] = e[1];
                    float diff = y[1] - e[1];
                    sqrDistance = diff*diff;
                }
            }
            else  // y1 == 0
            {
                float denom0 = e[0]*e[0] - e[1]*e[1];
                float e0y0 = e[0]*y[0];
                if (e0y0 < denom0)
                {
                    // y0 is inside the subinterval.
                    float x0de0 = e0y0/denom0;
                    float x0de0sqr = x0de0*x0de0;
                    x[0] = e[0]*x0de0;
                    x[1] = e[1]*sqrt(abs(1.0 - x0de0sqr));
                    float d0 = x[0] - y[0];
                    sqrDistance = d0*d0 + x[1]*x[1];
                }
                else
                {
                    // y0 is outside the subinterval.  The closest ellipse point has
                    // x1 == 0 and is on the domain-boundary interval (x0/e0)^2 = 1.
                    x[0] = e[0];
                    x[1] = 0.0;
                    float diff = y[0] - e[0];
                    sqrDistance = diff*diff;
                }
            }
            float sign = dot(y, y) < dot(x, x) ? -1.0 : 1.0;
            return sqrt(sqrDistance) * sign;
        }
        
        float distanceToEllipse(vec2 size, vec2 pos) 
        {
            vec2 e = size * 0.5;
            vec2 y = abs(pos - e);
            if(e.x < e.y)
            {
                e = e.yx;
                y = y.yx;
            }
            return distanceToEllipseImpl(e, y);       
        }

        ShapeInfo getShapeInfo(vec2 pos)
        {
            ShapeInfo result;
            result.tc = pos/shapeSize;
            result.d = distanceToEllipse(shapeSize, pos);
            result.mask = smoothstep(antialiasingSize, -antialiasingSize, result.d);
            return result;
        }    
    "
}