import QtQuick 2.0
import ".."

Shape {
    property real radius: 0.0
    readonly property real radiusInPixels: Math.min(size.width, size.height) / 2 * radius / 100.0
    shaderParameters: Qt.vector4d(radiusInPixels, 0.0, 0.0, 0.0)
    shaderFunction: "
        float distanceToRoundedRectangle(vec2 size, vec2 pos, float radius)
        {
            pos = abs(pos - size/2.0);
            vec2 circleCenter = size/2.0 - vec2(radius, radius);
            if(pos.x > circleCenter.x && pos.y > circleCenter.y) 
            {
                return length(circleCenter - pos) - radius;
            }
            else
            {
                vec2 d = pos - size/2.0;
                return min(max(d.x, d.y), 0.0) + length(max(d, 0.0));
            } 
        }        
        ShapeInfo getShapeInfo(vec2 pos)
        {
            float radius = shapeParameters[0];
            ShapeInfo result;
            result.tc = pos/shapeSize;
            result.d = distanceToRoundedRectangle(shapeSize, pos, radius);
            result.mask = smoothstep(antialiasingSize, -antialiasingSize, result.d);
            return result;
        }
    "
}