import QtQuick 2.0
import DistanceField 1.0
import QtQuickUtils 1.0
import ".."
import "TextShape"

Shape {
    id: root
    property string text
    property TextSettings textSettings: TextSettings{}
    property real outerDistanceFieldRadius
    property real innerDistanceFieldRadius
    property vector2d contentOffset
    property alias fixedTextureWidth: distanceFieldDrawer.fixedTextureWidth
    property alias fixedTextureHeight: distanceFieldDrawer.fixedTextureHeight
    property alias texturesQuality: distanceFieldDrawer.texturesQuality
    readonly property alias nonOffsetedContentRect: distanceFieldDrawer.nonOffsetedContentRect
    readonly property size letterSize: Qt.size(textSettings.pointSize, textSettings.pointSize * textSettings.xScale / 100)

    property alias font: distanceFieldDrawer.font
    readonly property TextDistanceFieldDrawer distanceFieldDrawer: TextDistanceFieldDrawer {
        id: distanceFieldDrawer
        text: root.text
        textSettings: root.textSettings
        size: root.size
        margins: root.outerDistanceFieldRadius
        contentOffset: root.contentOffset
    }

    property TextFontSizeMeashurer fontSizeMeashurer: TextFontSizeMeashurer {
        textItem: textSettings.fitText ? distanceFieldDrawer.textItem : null
        onFontPointSizeChanged: if(fontPointSize) textSettings.pointSize = fontPointSize
    }
    
    property DistanceField distanceField: DistanceField {
        source: texture
        implicitWidth: source.implicitWidth
        implicitHeight: source.implicitHeight
        maxDistance: Math.max(0.001, Math.max(root.outerDistanceFieldRadius, Math.min(root.innerDistanceFieldRadius, root.maxInnerDistance))) * root.texturesQuality
    }
    contentRect: distanceFieldDrawer.contentRect
    textureRect: distanceFieldDrawer.textureRect
    texture: distanceFieldDrawer.texture
    texture2: distanceField
    shaderParameters: {
        var x0 = Math.max(0, contentRect.x)
        var y0 = Math.max(0, contentRect.y)
        var x1 = Math.min(size.width, contentRect.x + contentRect.width)
        var y1 = Math.min(size.height, contentRect.y + contentRect.height)
        return Qt.vector4d(x0, y0, x1 - x0, y1 - y0)
    }
    shaderParameters2: Qt.vector4d(1/root.texturesQuality, 0, 0, 0)
    
    shaderFunction: "
        float distanceToRectangle(vec2 size, vec2 pos)
        {
            pos = abs(pos - size/2.0);
            vec2 d = pos - size/2.0;
            return min(max(d.x, d.y), 0.0) + length(max(d, 0.0));
        }
        
        ShapeInfo getShapeInfo(vec2 pos)
        {
            vec2 tc = (pos - shapeTextureRect.xy) / shapeTextureRect.zw;
            vec4 data = texture2D(shapeTexture, tc);
            vec4 clipRect = shapeParameters;
            float distanceScale = shapeParameters2.x;
            ShapeInfo result;
            result.tc = data.xy;
            result.d = max(texture2D(shapeTexture2, tc).r * distanceScale, distanceToRectangle(clipRect.zw, pos - clipRect.xy));
            result.mask = smoothstep(antialiasingSize, -antialiasingSize, result.d);
            return result;
        }
    "
    maxInnerDistance: font.pixelSize * 0.093
}