import QtQml 2.2
import "../"

TextSource {
    id: root
    type: "Link"
    property string linkName: ""
    property string textTemplate: "<Media>"  //"<Index>", "<Label>", "<Type>", "<Error>", "<Info>"
    //onLinkNameChanged: significantTextChanging(value || "")

    readonly property var  input:    mixer && mixer.mediaLibrary    ? mixer.mediaLibrary.getInputByName(linkName) : null
    readonly property var    inputSlots: mixer && mixer.inputSlots      ? mixer.inputSlots : null
    readonly property var    pipInputs:  mixer && mixer.transitionLogic ? mixer.transitionLogic.pipInputs : null

    readonly property bool isSlot: linkName.indexOf('Slot:///')===0
    readonly property bool isPip:  linkName.indexOf('Pip:///')===0
    readonly property bool isLogo: linkName.indexOf('Logo:///')===0
    readonly property bool isOutput: linkName=='Preset' || linkName=='Program'

    readonly property int  index: {
        if (linkName=="") return 0
        if (isOutput) return linkName=='Preset' ? 0 : 1
        return parseInt(linkName.split(':///')[1]) - 1
    }
    readonly property string typeName: {
        if (linkName=="") return ""
        if (linkName=='Preset') return qsTr("Preview")
        if (linkName=='Program') return qsTr("Program")
        if (isSlot) return qsTr("Slot")
        if (isPip)  return qsTr("PIP")
        if (isLogo) return qsTr("Logo")
        return linkName.split(':///')[0]
    }
    readonly property string mediaName: {
        if (linkName=="") return ""
        if (!input) return "<" + typeName + (isOutput ? "" : index+1) + ">"
        return srcInput ? srcInput.humanReadableName : qsTr("<empty>")
    }
    readonly property string label: {
        if (linkName=="") return ""
        if (isSlot && slot) return slot.label
        if (isOutput) return (linkName=='Preset') ? qsTr("PVW") : qsTr("PRG")
        return typeName + (isOutput ? "" : index+1)
    }

    readonly property var  slot:  isSlot && inputSlots ? inputSlots[index] : null
    readonly property var  pipInput: isPip && pipInputs ? pipInputs[index] : null
    readonly property var  srcInput: {
        if (pipInput) return pipInput
        if (slot) return slot.input
        if (isOutput && mixer && mixer.outputs) return  mixer.outputs[index].input
        return null
    }
    value: {
        var repl = textTemplate.replace("<Media>", mediaName)
        repl = repl.replace("<Index>", index+1)
        repl = repl.replace("<Label>", root.label)
        repl = repl.replace("<Type>", typeName)
        return repl
    }
    function onCreated() {
    }
    function onDeleted() {
    }
    function save() {
        return {
            linkName: linkName,
            textTemplate: textTemplate
        }
    }
    function load(data) {
        linkName = data.linkName || ""
        textTemplate = data.textTemplate || textTemplate
    }    
}
