import QtQml 2.2
import "../"

TextSource {
    id: root
    type: "Table"
    property TextTable table: null
    property string columnName: ""
    property var values: []
      
    property int tableCurrentRow: table ? table.currentRow : -1
    onTableCurrentRowChanged: significantTextChanging(values[tableCurrentRow] || "")
    value: values[tableCurrentRow] || ""

    Binding on columnName {
        value: owner.name
        when: !!owner
    }

    onColumnNameChanged: if (table) table.updateTable()

    property Timer updateDelayed: Timer {
        interval: 1
        onTriggered: {
            values = values
            table.updateTable()
        }
    }

    function onCreated() {
        if(!columnName)  columnName = "Source" + table.columnCount
        table.addSource(root)
    }
    function onDeleted() {
        table.removeSource(root)
    }
    
    function setValue(value, index) {
        if(index < 0 || index >= values.length) return
        if (index==tableCurrentRow && values[index] != value) significantTextChanging(value||"")
        values[index] = value
        updateDelayed.restart()
    }

    function updateValue(value, index) {
        setValue(value, index)
    }
    
    function addValue(value) {
        values.push(value)
        updateDelayed.restart()
    }
    
    function addValueAfter(value, index) {
        values.splice(index + 1, 0, value)
        updateDelayed.restart()
    }
    
    function removeValue(index) {
        values.splice(index, 1)
        updateDelayed.restart()
    }
    
    function swapValues(i1, i2) {
        if(i1 < 0 || i1 >= values.length) return
        if(i2 < 0 || i2 >= values.length) return
        var tmp = values[i1]
        values[i1] = values[i2]
        values[i2] = tmp
        updateDelayed.restart()
    }

    function save() {
        return {
            columnName: columnName,
            values: values
        }
    }
    function load(data) {
        columnName = data.columnName || ""
        values = data.values || []
    }
}
