import QtQuick 2.0

Item {
    id: root
    property list<QtObject> content
    default property alias __content: root.content
    property bool enabled: true
    property rect crop: Qt.rect(0.0, 0.0, 1.0, 1.0)
    property real texturesQuality: 1.0
    
    Item {
        visible: root.opacity != 0
        width: contentItem.visibleChildrenRect.width
        height: contentItem.visibleChildrenRect.height
        x: contentItem.visibleChildrenRect.x
        y: contentItem.visibleChildrenRect.y
        layer.enabled: root.enabled
        layer.smooth: true
        layer.mipmap: true
        layer.textureSize: Qt.size(Math.min(width * root.texturesQuality, 8000), Math.min(height * root.texturesQuality, 8000))
        layer.effect: ShaderEffect {
            property vector4d crop: Qt.vector4d(root.crop.x, root.crop.y, 1.0 - (root.crop.x + root.crop.width), 1.0 - (root.crop.y + root.crop.height))
            fragmentShader: "
                uniform lowp sampler2D source;
                uniform lowp float qt_Opacity;
                uniform lowp vec4 crop;
                varying highp vec2 qt_TexCoord0;
                void main() {
                    lowp vec4 p = texture2D(source, qt_TexCoord0);
                    p *= step(crop.xxxx, qt_TexCoord0.xxxx);
                    p *= step(crop.zzzz, vec4(1,1,1,1) - qt_TexCoord0.xxxx);
                    p *= step(crop.yyyy, qt_TexCoord0.yyyy);
                    p *= step(crop.wwww, vec4(1,1,1,1) - qt_TexCoord0.yyyy);
                    gl_FragColor = p * qt_Opacity;
                }"
        }
        Item {
            id: contentItem
            readonly property rect visibleChildrenRect: {
                var x0 = 100000
                var y0 = 100000
                var x1 = -100000
                var y1 = -100000
                var hasVisibleChildren = false
                for(var i = 0; i < children.length; i++) {
                    var child = children[i]
                    if(!child.visible) continue
                    x0 = Math.min(x0, child.x)
                    y0 = Math.min(y0, child.y)
                    x1 = Math.max(x1, child.x + child.width)
                    y1 = Math.max(y1, child.y + child.height)
                    hasVisibleChildren = true
                }
                return hasVisibleChildren ? Qt.rect(x0, y0, x1 - x0, y1 - y0) : Qt.rect(0, 0, 0, 0)
            }
            width: root.width
            height: root.height
            x: -visibleChildrenRect.x
            y: -visibleChildrenRect.y
            data: root.content
        }
    }
}
