import QtQuick 2.3
import QtQuick.Controls 1.0

Item
{
    id: root
    signal clicked(var mouse)
    signal doubleClicked(var mouse)
    property color hoveredTextColor: "white"
    property bool underlineOnHovering: false
    property string text
    property string tooltip
    property bool clickable: true
    property alias color: labelItem.color
    property alias pixelSize: labelItem.font.pixelSize
    implicitWidth: labelItem.implicitWidth
    implicitHeight: labelItem.implicitHeight
    
    Label
    {
        id: labelItem
        elide: Text.ElideRight
        verticalAlignment : Text.AlignVCenter
        text: root.text !== "" ? root.text : qsTr("<empty>")
        state: "default"
        maximumLineCount: 1
        width: root.width

        TooltipArea
        {
            anchors.fill: parent
            hoverEnabled: true
            tooltip: {
                if(root.tooltip) return root.tooltip
                else return labelItem.truncated ? root.text : ""
            }
            onClicked: 
            {
                if (!root.clickable)
                {
                    mouse.accepted = false;
                }
                else 
                {
                    root.clicked(mouse);
                }
            }
            onDoubleClicked: root.doubleClicked(mouse);
            onEntered: 
            {
                labelItem.state = "hovered";
            }
            onExited: 
            {
                labelItem.state = "default";
            }
            onPressed:
            {
                if (!root.clickable) mouse.accepted = false;
            }
        }
        states: 
        [
            State 
            {
                name: "hovered"
                PropertyChanges 
                { 
                    target: labelItem
                    color: root.hoveredTextColor 
                }
                PropertyChanges 
                { 
                    target: labelItem
                    font.underline: root.underlineOnHovering
                }
            },
            State 
            {
                name: "default"
            }
        ]
    } 
}