import QtQuick 2.4
import QtGraphicalEffects 1.0

Item {
    id: root
    property alias source: image.source
    property alias fillMode: image.fillMode
    
    property alias color: colorEffect.color
    property alias strokeColor: strokeEffect.color
    property alias glowColor: glowEffect.color
    property alias glowRadius: glowEffect.radius
    property alias glowSpread: glowEffect.spread    
    
    implicitWidth: image.implicitWidth
    implicitHeight: image.implicitHeight
    
    readonly property color transparentColor: "transparent"
    readonly property bool strokeEnabled: strokeColor != transparentColor
    readonly property bool glowEnabled: glowColor != transparentColor && glowRadius > 0

    Image {
        id: image
        anchors.fill: parent
        visible: false
    }

    onSourceChanged: {
        colorEffect.supportsAtlasTextures = false
        colorEffect.supportsAtlasTextures = true
    }
    
    ShaderEffect {
        id: colorEffect
        anchors.fill: parent
        visible: !strokeEnabled && !glowEnabled
        supportsAtlasTextures: true
        
        property Item source: image
        property color color: "black"
        
        fragmentShader: 
        "
            varying highp vec2 qt_TexCoord0;
            uniform highp float qt_Opacity;
            uniform lowp sampler2D source;
            uniform highp vec4 color;
        
            void main(void) {
                highp float alpha = texture2D(source, qt_TexCoord0.xy).a;
                gl_FragColor = color * color.a * alpha * qt_Opacity;
            }
        "
    }

    Glow {
        id: strokeEffect
        anchors.fill: parent
        visible: strokeEnabled && !glowEnabled
        
        radius: 1
        samples: 4
        spread: 0.9
        color: "transparent"
        source: colorEffect    
    }
    
    Glow {
        id: glowEffect
        anchors.fill: parent
        visible: glowEnabled
        
        radius: 3
        samples: 16
        spread: 0.5
        color: "transparent"
        source: strokeEnabled ? strokeEffect : colorEffect
    }
    
    
}