import QtQuick 2.4
import QtQuick.Controls 1.3
import QtQuick.Layouts 1.1
import GuiControls 1.3

Item {
    id: root
    //public
    property int cellWidth: 0
    property int cellHeight: 0
    property int spacing: 0
    property Component delegate: null
    property Component highlight: null
    property var model: null
    property alias count: grid.count
    property alias interactive: grid.interactive
    property alias background: scrollPanel.background
    property alias scrollStyle: scrollPanel.style
    signal appended(int count)
    signal removed(int index)
    property SelectionManager selectionManager: SelectionManager { 
        items: grid.model
        cover: false
    }
    readonly property var selectedItems: selectionManager ? selectionManager.selectedItems : []
    readonly property var selectedItem: (selectionManager.selectedCount === 1) ? selectionManager.selectedItems[0] : null	
    readonly property int selectedCount: selectionManager ? selectionManager.selectedCount : 0	
    readonly property var selectedItemIndexes: selectionManager ? selectionManager.getSelectedItemIndexes() : null
    function selectAll() {
        selectionManager.selectAll();
    }
    
    //private
    onAppended: grid.appended(count)
    onRemoved: grid.removed(index)
    readonly property Component wrapper: Component {
        Item {
            property var item: null
            property bool selected: false
            property bool hovered: false
            property rect boundingBox: Qt.rect(x,y,width,height)
        }
    }
    onModelChanged: {
        var list = [];
        for (var i = 0; i < model.length; ++i) {
            var obj = wrapper.createObject( root, { item: model[i] } );
            list.push(obj);
        }
        grid.model = list;
    }
    ScrollPanel {
        id: scrollPanel
        anchors.fill: parent
        Item {
            height: grid.contentHeight
            GridView {
                id: grid
                property int prevIndex: 0
                property int hoveredIndex: 0
                signal appended(int count)
                signal removed(int index)
                anchors.fill: parent
                clip: true
                interactive: false
                highlightFollowsCurrentItem: false
                keyNavigationWraps: true
                cellWidth: root.cellWidth + root.spacing
                cellHeight: root.cellHeight + root.spacing
                onAppended: {
                    var list = [];
                    for (var i = (grid.count - count); i < grid.count; ++i) {
                        var obj = model[i];
                        list.push(obj);
                    }
                    root.selectionManager.selectItems(list, true, false);
                }
                onRemoved: {
                    var lastOne = model[index];
                    if (lastOne) {
                        selectionManager.selectItem(lastOne);
                    }
                }
                delegate: Item {
                    id: rootDelegate
                    property var currentItem: modelData
                    property int currentItemIndex: index
                    width: grid.cellWidth
                    height: grid.cellHeight
                    
                    //highlight
                    Item {
                        id: highlightItem
                        property bool selected: (rootDelegate.currentItem.selected !== undefined) ? rootDelegate.currentItem.selected : false
                        property bool hovered: (rootDelegate.currentItem.hovered !== undefined) ? rootDelegate.currentItem.hovered : false
                        z: 2
                        anchors.fill: parent
                        visible: selected || hovered
                        anchors.margins: root.spacing / 2
                        Loader {
                            anchors.fill: parent
                            sourceComponent: root.highlight 
                        }
                    }
                    
                    Binding {
                        target: currentItem
                        property: "x"
                        value: rootDelegate.x + root.spacing / 2
                    }
                    Binding {
                        target: currentItem
                        property: "y"
                        value: rootDelegate.y + root.spacing / 2
                    }
                    Binding {
                        target: currentItem
                        property: "width"
                        value: innerdDelegate.width 
                    }
                    Binding {
                        target: currentItem
                        property: "height"
                        value: innerdDelegate.height
                    }
                    
                    //delegate
                    Item {
                        id: innerdDelegate
                        anchors.margins: root.spacing / 2
                        anchors.fill: parent                
                        Loader { 
                            property alias index: rootDelegate.currentItemIndex
                            property var modelData: rootDelegate.currentItem.item
                            property bool highlighted: highlightItem.visible
                            sourceComponent: root.delegate 
                        }
                        MouseArea {
                            enabled: !ma.active
                            anchors.fill: parent
                            propagateComposedEvents: true
                            acceptedButtons: Qt.LeftButton | Qt.RightButton
                            onClicked: {
                                root.forceActiveFocus();
                                grid.currentIndex = index;
                                if (mouse.button == Qt.LeftButton) {
                                    if (mouse.modifiers & Qt.ControlModifier) {
                                        root.selectionManager.selectItem(rootDelegate.currentItem, false);
                                        grid.prevIndex = grid.currentIndex;
                                    }
                                    else if (mouse.modifiers & Qt.ShiftModifier ) {
                                        var selectedItemIndexes = root.selectionManager.getSelectedItemIndexes();
                                        if (grid.currentIndex < selectedItemIndexes[0]) {
                                            grid.prevIndex = selectedItemIndexes[selectedItemIndexes.length - 1];
                                        }
                                        if (grid.currentIndex > selectedItemIndexes[selectedItemIndexes.length - 1]) {
                                            grid.prevIndex = selectedItemIndexes[0];
                                        }
                                        
                                        var list = [];
                                        for (var i = Math.min(grid.prevIndex, grid.currentIndex); i <= Math.max(grid.prevIndex, grid.currentIndex); ++i) {
                                            list.push(grid.model[i]);
                                        }
                                        root.selectionManager.selectItems(list, true, false);
                                    }
                                    else {
                                        root.selectionManager.selectItem(rootDelegate.currentItem, true);
                                        grid.prevIndex = grid.currentIndex;
                                    }
                                }
                                else {
                                    if (!currentItem.selected) {
                                        root.selectionManager.selectItem(currentItem, true, false);
                                    }
                                    grid.prevIndex = grid.currentIndex;
                                }
                                mouse.accepted = false;
                            }
                        }
                    }
                    
                    Keys.onLeftPressed: {
                        event.accepted = true;
                        grid.moveCurrentIndexLeft();
                        root.selectionManager.selectItem(currentItem, true);
                        grid.prevIndex = grid.currentIndex;
                    }
                    Keys.onRightPressed: {
                        event.accepted = true;
                        grid.moveCurrentIndexRight();
                        root.selectionManager.selectItem(currentItem, true);
                        grid.prevIndex = grid.currentIndex;
                    }
                    Keys.onUpPressed: {
                        event.accepted = true;
                        grid.moveCurrentIndexUp();
                        root.selectionManager.selectItem(currentItem, true);
                        grid.prevIndex = grid.currentIndex;
                    }
                    Keys.onDownPressed: {
                        event.accepted = true;
                        grid.moveCurrentIndexDown();
                        root.selectionManager.selectItem(currentItem, true);
                        grid.prevIndex = grid.currentIndex;
                    }
                }
            }
            
            Rectangle {
                x: ma.selectedRect.x
                y: ma.selectedRect.y
                width: ma.selectedRect.width
                height: ma.selectedRect.height
                color: "black"
                opacity: 0.2
                id: rect
                visible: ma.selecting
            }
            
            MouseArea {
                id: ma
                acceptedButtons: Qt.LeftButton | Qt.RightButton
                anchors.centerIn: parent
                width: 100000
                height: 100000
                propagateComposedEvents: true
                property bool active: false
                property var startX: 0
                property var startY: 0
                property var endX: 0
                property var endY: 0
                property rect selectedRect: Qt.rect( 
                        Math.min(ma.startX, ma.endX),
                        Math.min(ma.startY, ma.endY),
                        Math.abs(ma.startX - ma.endX),
                        Math.abs(ma.startY - ma.endY))
                property bool selecting: false
                onPressed: {
                    if (mouse.buttons & Qt.LeftButton) {
                        active = false
                        startX = mouse.x - 50000 + parent.width /2
                        startY = mouse.y - 50000 + parent.height /2
                        endX = startX 
                        endY = startY 
                        selecting = true
                    }
                    root.forceActiveFocus();
                }
                onPositionChanged: {
                    if (mouse.buttons & Qt.LeftButton) {	
                        endX = mouse.x - 50000 + parent.width /2
                        endY = mouse.y - 50000 + parent.height /2
                        selectionManager.hoverArea(selectedRect)

                        if ((selectedRect.width * selectedRect.height) !== 0) {
                            active = true
                        }
                    }
                    mouse.accepted = false
                }
                onReleased: {		
                    selecting = false
                    if (mouse.button == Qt.LeftButton) {
                        if (mouse.modifiers & Qt.ControlModifier) {
                            selectionManager.selectArea(selectedRect, false, true)
                            selectionManager.hoverArea(Qt.rect(0,0,0,0))
                        }
                        else if (mouse.modifiers & Qt.ShiftModifier ) {
                            selectionManager.selectArea(selectedRect, false, false)
                            selectionManager.hoverArea(Qt.rect(0,0,0,0))
                        }
                        else {
                            selectionManager.selectArea(selectedRect, true, false)
                            selectionManager.hoverArea(Qt.rect(0,0,0,0))
                        }
                    }
                }
            }
        }
    }
}
