import QtQuick 2.1
import QtQuick.Layouts 1.1
import QtQuick.Controls.Private 1.0
import "../"

CollapsibleItem {
    id: root
    property string title: qsTr("")
    property color headerOverlayColor: "transparent"
    property PanelStyle style: styleLoader.item
    default property alias data: content.data		
    property bool collapsible: false
    property Component header
    signal headerClicked(var mouse)
    signal headerPressAndHold(var mouse)
    collapsedHeight: 22
    collapsedWidth: 22
    
    Loader {
        id: styleLoader
        source: {
            var styleName = Settings.style.toString().split('/').pop()
            return styleName == "ECGStyle" ? "PanelStyleECG.qml" : "PanelStyle.qml"
        }
    }	
    
    Rectangle {
        anchors.fill: parent
        color: style.borderColor2
        border.width: 1
        border.color: style.backgroundColor
        Rectangle {
            anchors.fill: parent
            anchors.margins: 1
            color: "transparent"
            border.color: style.borderColor
            ColumnLayout {
                anchors.fill: parent
                anchors.margins: 2
                spacing: 1
                Rectangle {
                    id: header
                    Layout.fillWidth: true
                    height: 16
                    border.color: style.borderColor
                    color: headerOverlayColor == Qt.rgba(0, 0, 0, 0) ? style.headerColor : headerOverlayColor
                    clip: true
                    MouseArea {
                        anchors.fill: parent
                        acceptedButtons: Qt.LeftButton | Qt.RightButton
                        onClicked: root.headerClicked(mouse)
                        onPressAndHold: root.headerPressAndHold(mouse)
                    }
                    Text {
                        anchors.top: parent.top
                        anchors.bottom: parent.bottom
                        anchors.left: parent.left
                        anchors.right: collapsingButton.left
                        clip: true
                        verticalAlignment: Qt.AlignVCenter
                        anchors.leftMargin: 4
                        font.pixelSize: height - 4
                        text: root.title
                        color: root.style.textColor
                        renderType: Text.NativeRendering
                        opacity: (expanded || root.orientation != Qt.Horizontal) ? 1 : 0
                        Behavior on opacity {
                            NumberAnimation {duration: root.transitionDuration}
                            enabled: root.transitionEnabled
                        }
                    }
                    Rectangle {
                        id: collapsingButton
                        visible: root.collapsible
                        color: Qt.lighter(header.color, collapsingButtonArea.containsMouse ? 1.2 : 1)
                        border.width: 1
                        border.color: style.borderColor
                        radius: 1
                        width: 12
                        height: 12
                        anchors.top: header.top
                        anchors.right: header.right
                        anchors.rightMargin: 2
                        anchors.topMargin: 2					
                        Image {
                            id: collapsingButtonImage
                            source: style.expanderImageUrl
                            anchors.centerIn: parent
                            Binding on rotation {
                                value: {
                                    if(root.inTransition) return collapsingButtonImage.rotation
                                    if(root.orientation == Qt.Horizontal) return root.expanded ? 0 : 180
                                    else return root.expanded ? 90 : 270
                                }
                            }
                        }
                        MouseArea {
                            id: collapsingButtonArea
                            anchors.fill: parent
                            acceptedButtons: Qt.LeftButton
                            onClicked: root.flipState()
                            hoverEnabled: true
                        }
                    }
                    Loader {
                        id: headerLoader
                        anchors.fill: parent
                        sourceComponent: root.header
                    }
                }
                Rectangle {
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    color: style.backgroundColor
                    border.color: style.borderColor

                    Rectangle {
                        anchors.fill: parent
                        color: header.color
                        border.color: style.borderColor
                        opacity: (root.collapsed && root.orientation == Qt.Horizontal) ? 1 : 0
                        Behavior on opacity {
                            NumberAnimation {duration: 100}
                            enabled: root.transitionEnabled
                        }
                    }	
                    Item {
                        id: content
                        anchors.fill: parent
                        anchors.margins: 3
                    }
                }
            }
        }
    }
}
