import QtQuick 2.1
import QtQuick.Layouts 1.1
import QtQuick.Controls 1.1
import GuiControls 1.0
import MixerGuiControls 1.0
import Rpc 1.0
import "../"
import "../../"
import "../../ConvertColor.js" as ConvertColor

PropertyItem {
    id: root
    property bool editAlphaChannel: true
    property bool updateValueWhileChanging: true
    property bool yuv: false
    property string previewProperty
    property color  forcedPreviewColor: "transparent"
    property var pickingAreas: []
    signal pickingAreaClicked(var pickingArea, real mouseX, real mouseY)
    signal pickingAreaHovered(var pickingArea, real mouseX, real mouseY)
    property bool mouseInPickingArea: {
        for(var i = 0; i < pickingAreas.length; i++) {
            if(pickingAreas[i].containsMouse) return true
        }
        return false
    }    
    property alias pickingMode: pickingButton.checked
    readonly property alias dialogOpened: dialog.visible
    
    Item {
        id: colorControl
        implicitWidth: layout.implicitWidth
        implicitHeight: layout.implicitHeight
        property color color
        property color previewColor
        
        RpcBinding on color {
            rpcTarget: root.object
            rpcProperty: root.property
            send: root.enabled
        }
        
        RpcBinding on previewColor {
            rpcTarget: root.object
            rpcProperty: root.previewProperty
            send: root.enabled
        } 
 
        onColorChanged: previewColor = color
        onPreviewColorChanged: if(root.pickingMode && !root.mouseInPickingArea) previewColor = color
        
        RowLayout {
            id: layout
            Item {
                implicitWidth: 35
                implicitHeight: 17
                ChessBoard {
                    visible: colorIndicatorArea.containsMouse
                    anchors.fill: parent
                    anchors.margins: -1
                    colorWhite: Qt.rgba(191 / 255, 191 / 255, 191 / 255)
                    colorBlack: "black"        
                    checkSize: 1
                }            
                ChessBoard {
                    anchors.fill: parent
                    colorWhite: "white"
                    colorBlack: "gray"
                    checkSize: 6
                    layer.enabled: true

                    Rectangle {
                        id: colorIndicator
                        anchors.fill: parent
                        color: {
                            if (forcedPreviewColor!="#00000000") return forcedPreviewColor
                            return yuv ? ConvertColor.yuvToRgb(colorControl.previewColor) : colorControl.previewColor
                        }
                    }
                }
                MouseArea {
                    id: colorIndicatorArea
                    hoverEnabled: true
                    anchors.fill: parent
                    onClicked: {
                        root.pickingMode = false
                        dialog.color = yuv ? ConvertColor.yuvToRgb(colorControl.color) : colorControl.color
                        dialog.open()     
                    }
                }
            }
            
            Button {
                id: pickingButton
                style: root.propertyPanelStyle.toolButtonStyle
                visible: pickingAreas && pickingAreas.length > 0
                iconSource: "icons/Picker.png"
                checkable: true
                onClicked: forceActiveFocus()
                onActiveFocusChanged: if(!activeFocus) checked = false
            }
        }
        
        MixerColorDialog
        {
            id: dialog
            title: qsTr("Choose a color")
            onCurrentColorChanged: {
                colorControl.previewColor = yuv ? ConvertColor.rgbToYuv(currentColor) : currentColor
                if(root.updateValueWhileChanging) colorControl.color = colorControl.previewColor
            }
            onAccepted: {
                colorControl.color = yuv ? ConvertColor.rgbToYuv(color) : color
                root.propertyModified()
            }
            onRejected: {
                colorControl.color = yuv ? ConvertColor.rgbToYuv(color) : color
                colorControl.previewColor = colorControl.color
            }
            showAlphaChannel: root.editAlphaChannel
        }
             
        Repeater {
            model: root.pickingAreas.length
            Item {
                property var pickingArea: pickingAreas[model.index]
                Connections {
                    target: pickingArea
                    onClicked: {
                        if(root.pickingMode) {
                            root.pickingAreaClicked(pickingArea, pickingArea.mouseX, pickingArea.mouseY)
                            root.pickingMode = false
                        }
                    }
                    
                    onPositionChanged: {
                        if(root.pickingMode) root.pickingAreaHovered(pickingArea, pickingArea.mouseX, pickingArea.mouseY)
                    }
                }
            }
        }        
    }
    
    onPickingModeChanged: {
        for(var i = 0; i < pickingAreas.length; i++) pickingAreas[i].cursorShape = pickingMode ? Qt.CrossCursor : Qt.ArrowCursor
    }    
    
    onMouseInPickingAreaChanged: if(pickingMode && !mouseInPickingArea) colorControl.previewColor = colorControl.color
}
