import QtQuick 2.1
import GuiControls 1.0
import MixerGuiControls 1.0
import Rpc 1.0
import "../.."

PropertyItem {
    id: root
    property string type: "HV"     // "HV", "HS"
    
    property real minimumRadialValue: 0
    property real maximumRadialValue: 1    
    property real minimumAngularValue: 0
    property real maximumAngularValue: 1
    
    property real hue: 0
    property real value: 1
    property real saturation: 1

    property int wheelSize: width*0.75
    onWidthChanged: Qt.callLater(function() { wheelSize = width*0.75 })

    implicitWidth: parent.width
    implicitHeight: root.wheelSize

    controlAlignment: Qt.AlignCenter  
    hasLabel:false

    Item {
        implicitWidth: root.wheelSize
        implicitHeight: root.wheelSize

        MixerLabel {
            x:-8; y: 8
            text: root.label
        }

        ColorWheelHV {
            visible: root.type == "HV"
            anchors.fill: parent
            minValue: root.minimumRadialValue
            maxValue: root.maximumRadialValue
            minAngle: root.minimumAngularValue
            maxAngle: root.maximumAngularValue
            RpcBinding on value {
                rpcTarget: root
                rpcProperty: "value"
                send: root.type == "HV"              
            }
            RpcBinding on angle {
                rpcTarget: root
                rpcProperty: "hue"
                send: root.type == "HV"               
            }
            colorSaturation: root.saturation
        }
        ColorWheelHS {
            visible: root.type == "HS"
            anchors.fill: parent
            minValue: root.minimumRadialValue
            maxValue: root.maximumRadialValue
            minAngle: root.minimumAngularValue
            maxAngle: root.maximumAngularValue
            RpcBinding on value {
                rpcTarget: root
                rpcProperty: "saturation"
                send: root.type == "HS"             
            }
            RpcBinding on angle {
                rpcTarget: root
                rpcProperty: "hue"
                send: root.type == "HS"                
            }
            colorValue: root.value
        } 
    }

    RpcBinding on hue {
        rpcTarget: root.object
        rpcProperty: root.properties ? (root.properties[0] || "") : 0
        send: root.enabled
    }
    RpcBinding on value {
        rpcTarget: root.object
        rpcProperty: root.properties ? (root.properties[1] || "") : 0
        send: root.enabled && root.type == "HV"
        recv: root.type == "HV"
    }
    RpcBinding on saturation {
        rpcTarget: root.object
        rpcProperty: root.properties ? (root.properties[1] || "") : 0
        send: root.enabled && root.type == "HS"
        recv: root.type == "HS"
    }    
}