import QtQuick 2.1
import QtQuick.Controls 1.0
import Rpc 1.0
import GuiControls 1.0
import "../"

PropertyItem {
    id: root
    property var values: []
    property var strings
    property alias currentValue: comboBox.currentValue
    property alias currentIndex: comboBox.currentIndex
    property alias editableComboBox: comboBox.editable
    property alias comboBoxPressed: comboBox.pressed
    //property alias comboBoxImplicitWidth: comboBox.implicitWidth
    //property alias comboBoxImplicitHeight: comboBox.implicitHeight
    property var findItemFunction
    controlHorizontalOffset: comboBox.textHorizontalOffset

    signal activated(int index)

    onValuesChanged: {
        var newIndex = findCurrentIndex()
        applyNewIndex(newIndex)
    }

    function applyNewIndex(index) {
        if (index < 0)
            currentValue = undefined
        else if (root.values && root.values.length > 0)
            currentValue = root.values[index]
    }

    function findCurrentIndex() {
        var index = -1
        if (findItemFunction)
            index = findItemFunction(currentValue)
        else
            index = root.values.indexOf(currentValue)
        return index
    }

    Item {
        anchors.fill: parent
        CompactComboBox {
            id: comboBox
            //anchors.fill: parent
            editable: false
            width: parent.width
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: -textHorizontalOffset
            model: {
                var result = root.strings ? root.strings : root.values
                return result.map(function(value) {
                    return value ? value : qsTr("<empty>")
                })
            }

            property var currentValue

            onModelChanged: {
                currentIndex = findCurrentIndex()
            }

            onCurrentValueChanged: {
                currentIndex = findCurrentIndex()
            }

            RpcBinding on currentValue {
                rpcTarget: root.object
                rpcProperty: root.property
                send: root.enabled
            }
            onActivated: {
                applyNewIndex(index)
                root.activated(index)
                root.propertyModified()
            }
        }
    }
}
