import QtQuick 2.3
import QtQuick.Dialogs 1.2
import GuiControls 1.0
import QtQuick.Controls 1.1
import QtQuick.Layouts 1.1
import QtQuickUtils 1.0
import Rpc 1.0
import "../"

PropertyItem {
    id: root
    property alias clickable: hyperLinkLabel.clickable
    property alias nameFilters: fileDialog.nameFilters
    property alias selectedNameFilter: fileDialog.selectedNameFilter
    property alias selectFolder: fileDialog.selectFolder
    property alias selectExisting: fileDialog.selectExisting
    property alias category: fileDialogSaver.category
    property bool clearButtonVisible: false
    property bool reloadButtonVisible: false
    property bool showFullPath: false
    signal reload(url fileUrl)

    RowLayout {
        anchors.fill: parent
        Item {
            Layout.fillWidth: true
            Layout.fillHeight: true
            //implicitWidth: 150
            HyperLinkLabel {
                id: hyperLinkLabel
                property url fileUrl
                width: parent.width
                anchors.verticalCenter: parent.verticalCenter
                tooltip: fileUrl.toString().replace("file:///", "")
                text: showFullPath ? tooltip : fileUrl.toString().substr(fileUrl.toString().lastIndexOf("/") + 1)

                function getFileDirectory(fileUrl) {
                    var s = fileUrl.toString()
                    return s.substring(0, s.lastIndexOf("/"))
                }

                function getFileExtension(url) {
                    return url.toString().split(".").pop()
                }

                onClicked: {
                    var directory = getFileDirectory(fileUrl)
                    if(directory) fileDialog.folder = directory
                    var extension = getFileExtension(fileUrl)
                    if(extension) {
                        extension = "*." + extension
                        for(var i = 0; i < fileDialog.nameFilters.length; i++) {
                            var nameFilter = fileDialog.nameFilters[i]
                            if(nameFilter.indexOf(extension) != -1) {
                                fileDialog.selectedNameFilter = nameFilter
                                break
                            }
                        }
                }
                fileDialogSaver.loadingEnabled = !directory
                fileDialog.open()
            }
            
            RpcBinding on fileUrl {
                rpcTarget: root.object
                rpcProperty: root.property
                send: root.enabled
            }
            
            FileDialog {
                id: fileDialog
                title: qsTr("Please choose a file")
                modality: Qt.WindowModal
                folder: shortcuts.desktop
                onAccepted: {
                   hyperLinkLabel.fileUrl = fileUrl
                   root.propertyModified()
                }
            }
            FileDialogSaver {
                id: fileDialogSaver
                dialog: fileDialog
            }
        }
        }
        Button {
            style: root.propertyPanelStyle.toolButtonStyle
            visible: root.clearButtonVisible
            iconSource: "icons/Delete.png"
            enabled: hyperLinkLabel.text != ""
            tooltip: qsTr("Clear")
            onClicked: {
                hyperLinkLabel.fileUrl = ""
                root.propertyModified()
            }
        }
        Button {
            style: root.propertyPanelStyle.toolButtonStyle
            visible: root.reloadButtonVisible
            iconSource: "icons/Undo.png"
            enabled: hyperLinkLabel.text != ""
            tooltip: qsTr("Reload")
            onClicked: {
                reload(hyperLinkLabel.fileUrl)
                root.propertyModified()
            }
        }
    }
}
