import QtQuick 2.1
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.1
import QtQuick.Controls.Styles 1.1
import GuiControls 1.0
import MixerGuiControls 1.0
import Rpc 1.0
import "../"

PropertyItem {
    id: root
    property alias minimumValue: spinBox.minimumValue
    property alias maximumValue: spinBox.maximumValue
    property alias stepSize: spinBox.stepSize
    property alias decimals: spinBox.decimals
    property alias suffix: spinBox.suffix
    property alias spinValue: spinBox.value
    property alias compactText: spinBox.compactText
    controlHorizontalOffset: -spinBox.textHorizontalOffset
    property real valueScale: 1
    property bool showSlider: maximumValue<=1000 && minimumValue>=-1000
    property Component buttonDelegate: null

    RowLayout {
        id: layout
        spacing: MixerThemeManager.theme.itemSpacing
        anchors.fill: parent
        CompactSpinBox {
            id: spinBox
            Layout.fillWidth: false
            Layout.leftMargin: controlHorizontalOffset
            readonly property real objectValue: root.object ? (root.object[root.property] || 0) : 0
            property bool applyingObjectValue
            property bool applyingGuiValue

            onObjectValueChanged: applyObjectValue()
            onValueChanged: applyGuiValue()

            function applyObjectValue() {
                if(applyingObjectValue || applyingGuiValue) return
                applyingObjectValue = true
                value = objectValue * valueScale
                applyingObjectValue = false
            }

            function applyGuiValue() {
                if(!root.object || !root.object.hasOwnProperty(root.property)) return
                if(applyingObjectValue || applyingGuiValue) return
                applyingGuiValue = true
                root.object[root.property] = value / valueScale
                applyingGuiValue = false
            }

            property real valueBeforeEditing
            onFocusChanged: if(focus) valueBeforeEditing = objectValue
            onEditingFinished: {
                applyObjectValue()
                if(valueBeforeEditing != objectValue) {
                    root.propertyModified()
                    valueBeforeEditing = objectValue
                }
            }
            onWheel: if(!activeFocus) root.propertyModifiedWithDelay()
        }
        Loader {
            Layout.fillWidth: true
            active: showSlider && spinBox.activeFocus
            sourceComponent: Slider {
                 id: slider
                 minimumValue: root.minimumValue
                 maximumValue: root.maximumValue
                 value: spinBox.value
                 stepSize: root.stepSize
                 orientation: Qt.Horizontal
                 onValueChanged: spinBox.value = value
                 style: MixerSliderStyle {
                     lighterGroove: true
                     scaleVisible: false
                 }
             }
        }
        Loader {
            sourceComponent: root.buttonDelegate
        }

  }
    onLoseFocus: spinBox.focus = false
}
