import Scene3D 1.0
import QtQuick 2.1
import GuiControls 1.0
import QtQuick.Layouts 1.1
import Rpc 1.0

PropertyItem {
    id: root
    property real stepSize: 1
    property int decimals: 0
    property string suffix: "\u00B0"
    controlHorizontalOffset: spinBox0.textHorizontalOffset
    
    RowLayout {
        id: item
        property quaternion objectValue: root.object ? (root.object[root.property] || Qt.quaternion(0, 0, 0, 0)) : Qt.quaternion(0, 0, 0, 0)
        property bool applyingObjectValue
        property bool applyingGuiValue
        property quaternion valueBeforeEditing
        
        onObjectValueChanged: applyObjectValue()

        function applyObjectValue() {
            if(applyingObjectValue || applyingGuiValue) return
            applyingObjectValue = true
            var guiValue = Scene3DUtils.quaternionToEulerAngles(objectValue)
            spinBox0.value = guiValue.x
            spinBox1.value = guiValue.y
            spinBox2.value = guiValue.z
            applyingObjectValue = false
        }
        
        function applyGuiValue() {
            if(!root.object || !root.object.hasOwnProperty(root.property)) return
            if(applyingObjectValue || applyingGuiValue) return
            applyingGuiValue = true
            var guiValue = Qt.vector3d(spinBox0.value, spinBox1.value, spinBox2.value)
            root.object[root.property] = Scene3DUtils.eulerAnglesToQuaternion(guiValue)
            applyingGuiValue = false
        }
        
        function onEditingFinished() {
            applyObjectValue()
            if(objectValue != valueBeforeEditing) {
                root.propertyModified()
                valueBeforeEditing = objectValue
            }
        }

        CompactSpinBox {
            id: spinBox0
            minimumValue: -360
            maximumValue: 360
            stepSize: root.stepSize
            decimals: root.decimals
            suffix: root.suffix
            onValueChanged: item.applyGuiValue()
            onWheel: if(!activeFocus) root.propertyModifiedWithDelay()
            onEditingFinished: item.onEditingFinished()
        }
        CompactSpinBox {
            id: spinBox1
            minimumValue: -360
            maximumValue: 360
            stepSize: root.stepSize
            decimals: root.decimals
            suffix: root.suffix
            onValueChanged: item.applyGuiValue()
            onWheel: if(!activeFocus) root.propertyModifiedWithDelay()
            onEditingFinished: item.onEditingFinished()
        }
        CompactSpinBox {
            id: spinBox2
            minimumValue: -360
            maximumValue: 360
            stepSize: root.stepSize
            decimals: root.decimals
            suffix: root.suffix
            onValueChanged: item.applyGuiValue()
            onWheel: if(!activeFocus) root.propertyModifiedWithDelay()
            onEditingFinished: item.onEditingFinished()
        }
    }
    
    onFocusChanged: if(focus) item.valueBeforeEditing = item.objectValue
    onLoseFocus: {
        spinBox0.focus = false
        spinBox1.focus = false
        spinBox2.focus = false
    }
}