import QtQuick 2.1
import GuiControls 1.0
import QtQuick.Layouts 1.1
import Rpc 1.0

PropertyItem {
    id: root
    property size minimumValue: Qt.size(0, 0)
    property size maximumValue: Qt.size(1000000, 1000000)
    property real stepSize: 1
    property int decimals: 0
    property string suffix: ""
    controlHorizontalOffset: -spinBox0.textHorizontalOffset
    
    RowLayout {
        //anchors.fill: parent
        id: item
        property size objectValue: root.object ? (root.object[root.property] || Qt.size(0, 0)) : Qt.size(0, 0)
        property bool applyingObjectValue
        property bool applyingGuiValue
        property size valueBeforeEditing
        
        onObjectValueChanged: applyObjectValue()

        function applyObjectValue() {
            if(applyingObjectValue || applyingGuiValue) return
            applyingObjectValue = true
            spinBox0.value = objectValue.width
            spinBox1.value = objectValue.height
            applyingObjectValue = false
        }
        
        function applyGuiValue() {
            if(!root.object || !root.object.hasOwnProperty(root.property)) return
            if(applyingObjectValue || applyingGuiValue) return
            applyingGuiValue = true
            root.object[root.property] = Qt.size(spinBox0.value, spinBox1.value)
            applyingGuiValue = false
        }
        
        function onEditingFinished() {
            applyObjectValue()
            if(objectValue != valueBeforeEditing) {
                root.propertyModified()
                valueBeforeEditing = objectValue
            }
        }
        CompactSpinBox {
            id: spinBox0
            minimumValue: root.minimumValue.width
            maximumValue: root.maximumValue.width
            stepSize: root.stepSize
            decimals: root.decimals
            suffix: root.suffix
            onValueChanged: item.applyGuiValue()
            onWheel: if(!activeFocus) root.propertyModifiedWithDelay()
            onEditingFinished: item.onEditingFinished()
        }
        CompactSpinBox {
            id: spinBox1
            minimumValue: root.minimumValue.height
            maximumValue: root.maximumValue.height
            stepSize: root.stepSize
            decimals: root.decimals
            suffix: root.suffix
            onValueChanged: item.applyGuiValue()
            onWheel: if(!activeFocus) root.propertyModifiedWithDelay()
            onEditingFinished: item.onEditingFinished()
        }
        Item { Layout.fillWidth : true 	}
    }
    
    onFocusChanged: if(focus) item.valueBeforeEditing = item.objectValue
    onLoseFocus: {
        spinBox0.focus = false
        spinBox1.focus = false
    }  
}
