import QtQuick 2.1
import QtQuick.Controls 1.5
import GuiControls 1.0
import Serialization 1.0
import MixerGuiControls 1.0
import Qt.labs.settings 1.0

PropertyItem {
    id: root
    property int numValues: 20
    property var type: 'string'
    property var values: []
    property Settings settings: Settings {}

    function format(v) { 
        return type=='url' ? Serialization.urlFromUserInput(v) : v.toString() 
    }        

    function apply() { 
        recent.visible = false
        label.visible = true

        if(!object || !object.hasOwnProperty(property)) return
        var value = format(recent.editText).toString().trim()
        var index = values.indexOf(value)
        if (index> 0) values.splice(index,1)
        if (index!=0) values.splice(0,0,value)
        values.splice(numValues,values.length)
        values = values

        var modified = value != label.value
        object[property] = value
        if (modified) propertyModified()
    }

    function edit() {
        if (recent.visible) return
        recent.editText = format(label.text).toString()
        recent.visible = true
        label.visible = false
        recent.forceActiveFocus()
    }        

    onLoseFocus: apply()

    Item {
        anchors.fill: parent
        HyperLinkLabel {
            id: label
            width: parent.width
            anchors.verticalCenter: parent.verticalCenter
            readonly property var value: root.object && root.object[root.property] || values[0] || ""
            onClicked: edit()
            text: value.toString()
        }

        Item {
            width: parent.width

            MixerComboBox { 
                id: recent
                width: parent.width
                model: values
                editable: true
                visible: false
                menuBorder: 1
                onFocusChanged: if (!focus) root.apply()
            }
        }
    }
}
