import QtQuick 2.1
import GuiControls 1.0
import QtQuick.Layouts 1.1
import Rpc 1.0

PropertyItem {
    id: root
    property vector3d minimumValue: Qt.vector3d(-1000000, -1000000, -1000000)
    property vector3d maximumValue: Qt.vector3d(1000000, 1000000, 1000000)
    property real stepSize: 1
    property int decimals: 0
    property int digits: 0
    property string suffix: ""
    controlHorizontalOffset: -spinBox0.textHorizontalOffset
    
    RowLayout {
        id: item
				//anchors.fill: parent
        property vector3d objectValue: root.object ? (root.object[root.property] || Qt.vector3d(0, 0, 0)) : Qt.vector3d(0, 0, 0)
        property bool applyingObjectValue
        property bool applyingGuiValue
        property vector3d valueBeforeEditing
        
        onObjectValueChanged: applyObjectValue()

        function applyObjectValue() {
            if(applyingObjectValue || applyingGuiValue) return
            applyingObjectValue = true
            spinBox0.value = objectValue.x
            spinBox1.value = objectValue.y
            spinBox2.value = objectValue.z
            applyingObjectValue = false
        }
        
        function applyGuiValue() {
            if(!root.object || !root.object.hasOwnProperty(root.property)) return
            if(applyingObjectValue || applyingGuiValue) return
            applyingGuiValue = true
            root.object[root.property] = Qt.vector3d(spinBox0.value, spinBox1.value, spinBox2.value)
            applyingGuiValue = false
        }
        
        function onEditingFinished() {
            applyObjectValue()
            if(objectValue != valueBeforeEditing) {
                root.propertyModified()
                valueBeforeEditing = objectValue
            }
        }

        CompactSpinBox {
            id: spinBox0
            digits: root.digits
            minimumValue: root.minimumValue.x
            maximumValue: root.maximumValue.x
            stepSize: root.stepSize
            decimals: root.decimals
            suffix: root.suffix
            onValueChanged: item.applyGuiValue()
            onWheel: if(!activeFocus) root.propertyModifiedWithDelay()
            onEditingFinished: item.onEditingFinished()
        }
        CompactSpinBox {
            id: spinBox1
            digits: root.digits
            minimumValue: root.minimumValue.y
            maximumValue: root.maximumValue.y
            stepSize: root.stepSize
            decimals: root.decimals
            suffix: root.suffix
            onValueChanged: item.applyGuiValue()
            onWheel: if(!activeFocus) root.propertyModifiedWithDelay()
            onEditingFinished: item.onEditingFinished()
        }
        CompactSpinBox {
            id: spinBox2
            digits: root.digits
            minimumValue: root.minimumValue.z
            maximumValue: root.maximumValue.z
            stepSize: root.stepSize
            decimals: root.decimals
            suffix: root.suffix
            onValueChanged: item.applyGuiValue()
            onWheel: if(!activeFocus) root.propertyModifiedWithDelay()
            onEditingFinished: item.onEditingFinished()
        }
				Item { Layout.fillWidth : true 	}
    }
    
    onFocusChanged: if(focus) item.valueBeforeEditing = item.objectValue
    onLoseFocus: {
        spinBox0.focus = false
        spinBox1.focus = false
        spinBox2.focus = false
    }  
}
