import QtQuick 2.1
import GuiControls 1.0
import QtQuick.Layouts 1.2
import "Private"

Item {
    id: root
    property list<QtObject> content
    property string title
    property bool collapsible: true 
    property bool expanded: false
    property bool offseted: false
    signal loseFocus()
    signal propertyModified()
    
    property Item headerControl
    property list<Item> headerButtons
    
    property QtObject object: {
        if(parentPropertyGroup) return parentPropertyGroup.object
        if(propertyPanel) return propertyPanel.object
        return null
    }
    
    property PropertyPanelStyle propertyPanelStyle: {
        if(parentPropertyGroup) return parentPropertyGroup.propertyPanelStyle
        if(propertyPanel) return propertyPanel.style
        return _defaultPropertyPanelStyle
    }   
    readonly property PropertyPanelStyle _defaultPropertyPanelStyle: PropertyPanelStyle {}    
    
    property int offsetSize: {
        if(parentPropertyGroup) return parentPropertyGroup.offsetSize
        if(propertyPanel) return propertyPanel.groupOffsetSize
        return 20
    }
    
    property real columnRatio: {
        if(parentPropertyGroup) return parentPropertyGroup.columnRatio
        if(propertyPanel) return propertyPanel.columnRatio
        return 0.6
    }    
    property int spacing: {
        if(parentPropertyGroup) return parentPropertyGroup.spacing
        if(propertyPanel) return propertyPanel.spacing
        return 5
    }
    property int topSpacing: collapsible ? _topSpacingForCollapsible : 0
    readonly property int _topSpacingForCollapsible: {
        if(parentPropertyGroup) return parentPropertyGroup._topSpacingForCollapsible
        if(propertyPanel) return propertyPanel.groupTopSpacing
        return 5
    }    
    
    property int bottomSpacing: collapsible ? _bottomSpacingForCollapsible : 0
    readonly property int _bottomSpacingForCollapsible: {
        if(parentPropertyGroup) return parentPropertyGroup._bottomSpacingForCollapsible
        if(propertyPanel) return propertyPanel.groupTopSpacing
        return 5
    }    
    
    property int itemLeftSpacing: {
        if(parentPropertyGroup) return parentPropertyGroup.itemLeftSpacing
        if(propertyPanel) return propertyPanel.itemLeftSpacing
        return 5        
    }
    
    property int itemRightSpacing: {
        if(parentPropertyGroup) return parentPropertyGroup.itemRightSpacing
        if(propertyPanel) return propertyPanel.itemRightSpacing
        return 5
    }
    
    property int spacingBetweenLabelAndControl: {
        if(parentPropertyGroup) return parentPropertyGroup.spacingBetweenLabelAndControl
        if(propertyPanel) return propertyPanel.spacingBetweenLabelAndControl
        return 10
    }
    
    property int spacingBetweenControlAndExtendedControl: {
        if(parentPropertyGroup) return parentPropertyGroup.spacingBetweenControlAndExtendedControl
        if(propertyPanel) return propertyPanel.spacingBetweenControlAndExtendedControl
        return 5
    }
    
    property int minimumControlHeight: {
        if(parentPropertyGroup) return parentPropertyGroup.minimumControlHeight
        if(propertyPanel) return propertyPanel.minimumControlHeight
        return TextFieldSingleton.implicitHeight
    }       
    
    readonly property int offset: offseted ? offsetSize : 0
    readonly property int globalOffset: parentPropertyGroup ? parentPropertyGroup.globalOffset + offset : 0
    Layout.fillWidth: true    
    implicitHeight: columnLoader.implicitHeight
    
    readonly property string _tag: "propertyGroup"
    default property alias _content: root.content
    readonly property Item _column: columnLoader.item.column
    
    Binding on width {
        value: parent ? parent.width : 0
        when: root.parent && !root.visible
    }
    
    readonly property Item propertyPanel: {
        var item = parent
        while(item) {
            if(item._tag == "propertyPanel") return item
            item = item.parent
        }
        return null
    }
    
    readonly property Item topLevelPropertyGroup: {
        var item = parent
        var result = null
        while(item) {
            if(item._tag == "propertyGroup") result = item
            item = item.parent
        }
        return result
    }
    
    readonly property Item parentPropertyGroup: {
        var item = parent
        while(item) {
            if(item._tag == "propertyGroup") return item
            item = item.parent
        }
        return null
    }    

    implicitWidth: {
        var result = 0
        for(var i = 0; i < _column.children.length; i++) {
            var item = _column.children[i]
            if(!item.visible && item.hasOwnProperty("_tag") && item._tag == "propertyGroupLoader") continue
            result = Math.max(result, item.implicitWidth)
        }
        return result
    }    
    
    readonly property int localControlMinimumWidth: {
        var result = 0
        for(var i = 0; i < _column.children.length; i++) {
            var item = _column.children[i]
            if(item.hasOwnProperty("localControlMinimumWidth")) {
                result = Math.max(result, item.localControlMinimumWidth)
            }
        }
        return result
    }    
    readonly property int globalControlMinimumWidth: {
        if(propertyPanel) return propertyPanel.controlMinimumWidth
        if(topLevelPropertyGroup) return topLevelPropertyGroup.globalControlMinimumWidth
        return localControlMinimumWidth
    }
    
    readonly property int localControlPreferredWidth: {
        var result = 999999
        for(var i = 0; i < _column.children.length; i++) {
            var item = _column.children[i]
            if(item.hasOwnProperty("_tag") && item._tag == "propertyItem" && !item.hasLabel) continue
            if(item.hasOwnProperty("localControlPreferredWidth") && item.localControlPreferredWidth != 0) {
                result = Math.min(result, item.localControlPreferredWidth)
            }
        }
        return result == 999999 ? 0 : result
    }    
    readonly property int globalControlPreferredWidth: {
        if(propertyPanel) return propertyPanel.controlPreferredWidth
        if(topLevelPropertyGroup) return topLevelPropertyGroup.globalControlPreferredWidth
        return localControlMinimumWidth
    }
    
    readonly property int localLabelMinimumWidth: {
        var result = 0
        for(var i = 0; i < _column.children.length; i++) {
            var item = _column.children[i]
            if(item.hasOwnProperty("localLabelMinimumWidth")) {
                result = Math.max(result, item.localLabelMinimumWidth)
            }
        }
        return result
    }            
    readonly property int globalLabelMinimumWidth: {
        if(propertyPanel) return propertyPanel.labelMinimumWidth
        if(topLevelPropertyGroup) return topLevelPropertyGroup.globalLabelMinimumWidth
        return localLabelMinimumWidth
    }
    
    Component {
        id: columnComponent
        ColumnLayout {
            property var column: column
            spacing: 0
            ColumnLayout {
                id: column
                Layout.fillWidth: true
                Layout.topMargin: root.topSpacing
                Layout.bottomMargin: root.bottomSpacing
                data: root.content
                spacing: root.spacing
            }
        }
    }
    
    Component {
        id: collapsibleColumnComponent
        CollapsiblePanel {
            id: panel
            property alias column: column
            style: root.propertyPanelStyle.collapsiblePanelStyle
            title: root.title
            leftSpacing: 0
            rightSpacing: 0
            topSpacing: root.topSpacing
            bottomSpacing: root.bottomSpacing
            headerControl: root.headerControl
            headerButtons: root.headerButtons
            
            ColumnLayout {
                id: column
                data: root.content
                spacing: root.spacing
            }     
            Binding on state {
                value: root.expanded ? "expanded" : "collapsed"
            }
            onStateChanged: root.expanded = (state == "expanded")
        }
    }   

    Loader {
        id: columnLoader
        anchors.fill: parent
        anchors.leftMargin: root.offset
        sourceComponent: root.collapsible ? collapsibleColumnComponent : columnComponent
    }
    
    Connections {
        target: propertyPanel
        onLoseFocus: if(!parentPropertyGroup) root.loseFocus()
    }
    
    Connections {
        target: parentPropertyGroup
        onLoseFocus: root.loseFocus()
    }    
    
    onPropertyModified: {
        if(parentPropertyGroup) parentPropertyGroup.propertyModified()
        else if(propertyPanel) propertyPanel.propertyModified()
    }
}

