import QtQuick 2.1
import QtQuick.Layouts 1.1
import GuiControls 1.0
import "Private"

Item {
    id: root
    default property alias content: column.data
    implicitHeight: scrollPanel.implicitHeight
    
    property QtObject object
    property PropertyPanelStyle style: PropertyPanelStyle {}
    property int groupOffsetSize: 20
    property real columnRatio: 0.6
    property int spacing: 5
    property int groupTopSpacing: 5      // for collapsible group
    property int groupBottomSpacing: 5   // for collapsible group
    property int itemLeftSpacing: 5
    property int itemRightSpacing: 5
    property int spacingBetweenLabelAndControl: 10
    property int spacingBetweenControlAndExtendedControl: 5
    property int minimumControlHeight: TextFieldSingleton.implicitHeight
    signal loseFocus()
    signal propertyModified()

    readonly property string _tag: "propertyPanel"

    implicitWidth: {
        var result = 0
        for(var i = 0; i < column.children.length; i++) {
            var item = column.children[i]
            if(!item.visible && item.hasOwnProperty("_tag") && item._tag == "propertyGroupLoader") continue
            result = Math.max(result, item.implicitWidth)
        }
        return result + scrollPanel.scrollBarWidth + scrollPanel.scrollBarSpacing
    }
    
    readonly property int controlMinimumWidth: {
        var result = 0
        for(var i = 0; i < column.children.length; i++) {
            var item = column.children[i]
            if(item.hasOwnProperty("localControlMinimumWidth")) {
                result = Math.max(result, item.localControlMinimumWidth)
            }
        }
        return result
    }
    
    readonly property int controlPreferredWidth: {
        var result = 999999
        for(var i = 0; i < column.children.length; i++) {
            var item = column.children[i]
            if(item.hasOwnProperty("_tag") && item._tag == "propertyItem" && !item.hasLabel) continue
            if(item.hasOwnProperty("localControlPreferredWidth") && item.localControlPreferredWidth != 0) {
                result = Math.min(result, item.localControlPreferredWidth)
            }
        }
        return result == 999999 ? 0 : result
    }    
    
    readonly property int labelMinimumWidth: {
        var result = 0
        for(var i = 0; i < column.children.length; i++) {
            var item = column.children[i]
            if(item.hasOwnProperty("localLabelMinimumWidth")) {
                result = Math.max(result, item.localLabelMinimumWidth)
            }
        }
        return result
    }    
    
    MouseArea {
        anchors.fill: parent
        onClicked: loseFocus()
        onWheel: {
            if (!scrollPanel.showScrollBar) return
            scrollPanel.scrollBar.value += (wheel.angleDelta.y>0 ? -1:1)*30
        }    
    }

    ScrollPanel {
        id: scrollPanel
        style: root.style.scrollViewStyle
        background: root.style.background
        scrollBarSpacing: root.style.scrollBarSpacing
        anchors.fill: parent    
        ColumnLayout {
            id: column
            width: parent.width
            spacing: root.spacing
        }    
    }
}
