import QtQuick 2.3
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.1

SplitView {
    id: split
    property real ratio: 0.5
    property Item ratioItem: __items[0] || null
    property bool ratioEnabled: true
    property real minimumRatio: 0.0
    property real maximumRatio: 1.0
    
    property int minimumRatioItemWidth: 0
    property int maximumRatioItemWidth: 100000
    property int minimumRatioItemHeight: 0
    property int maximumRatioItemHeight: 100000    
    
    readonly property int handleWidth: __handles[0] ? __handles[0].width : 0
    readonly property int handleHeight: __handles[0] ? __handles[0].height : 0

    Connections {
        target: ratioItem
        onWidthChanged: if(split.resizing) split.updateRatio()
        onHeightChanged: if(split.resizing) split.updateRatio()        
    }
    
    onRatioChanged: if(!resizing) updateRatioItem()
    onWidthChanged: updateRatioItem()
    onHeightChanged: updateRatioItem()
    onMinimumRatioItemWidthChanged: updateRatioItem()
    onMaximumRatioItemWidthChanged: updateRatioItem()  
    onMinimumRatioItemHeightChanged: updateRatioItem()
    onMaximumRatioItemHeightChanged: updateRatioItem()      
    onOrientationChanged: updateRatioItem()
    onRatioItemChanged: updateRatioItem()
    
    function updateRatio() {
        if(!ratioItem) return
        if(orientation == Qt.Horizontal) {
            ratio = (ratioItem.width + handleWidth * 0.5) / width
        } else {
            ratio = (ratioItem.height + handleHeight * 0.5) / height
        }
    }
    
    function updateRatioItem() {
        if(!ratioItem) return
        if(orientation == Qt.Horizontal) {
            ratioItem.Layout.minimumWidth = ratioEnabled ? Math.max(minimumRatioItemWidth, minimumRatio * width - handleWidth * 0.5) : minimumRatioItemWidth
            ratioItem.Layout.maximumWidth = ratioEnabled ? Math.min(maximumRatioItemWidth, maximumRatio * width - handleWidth * 0.5) : maximumRatioItemWidth
            if(ratioEnabled) ratioItem.width = ratio * width - handleWidth * 0.5
        } else {
            ratioItem.Layout.minimumHeight = ratioEnabled ? Math.max(minimumRatioItemHeight, minimumRatio * height - handleHeight * 0.5) : minimumRatioItemHeight
            ratioItem.Layout.maximumHeight = ratioEnabled ? Math.min(maximumRatioItemHeight, maximumRatio * height - handleHeight * 0.5) : maximumRatioItemHeight
            if(ratioEnabled) ratioItem.height = ratio * height - handleHeight * 0.5
        }
    }
}   