import QtQuick 2.0

Item {
    id: root
    property real radius: 5
    property Item targetX
	property Item targetY
	property real posX: targetY ? targetY.x + targetY.width / 2 : 0
	property real posY: targetX ? (targetX.y + (targetX.height / 2)) : 0
	readonly property bool dragEnabled: ma.drag.active
	property bool swapDiagCursor: false
	property bool __shiftPressed: false
	property bool __controlPressed: false
	readonly property bool shiftPressed: __shiftPressed
	readonly property bool controlPressed: __controlPressed
	signal pressed(var mouse)
	Binding
	{
		target: targetX
		property: "dragActive"
		value: dragEnabled
	}
	Binding
	{
		target: targetY
		property: "dragActive"
		value: dragEnabled
	}
	
	Item {
		width: 0
		height: 0
        id: dummyRect
		Binding on x 
		{
			value: targetX ? targetX.x : 0
			when: targetX && !root.dragEnabled
		}
		Binding on x
		{
			value: posX
			when: !targetX && !root.dragEnabled
		}
		Binding on y
		{
			value: targetY ? targetY.y : 0
			when: targetY && !root.dragEnabled
		}
		Binding on y
		{
			value: posY
			when: !targetY && !root.dragEnabled
		}
    }	
	
	Binding
	{
		target: targetX
		property: "x"
		value: dummyRect.x
		when: root.dragEnabled
	}
	Binding
	{
		target: targetY
		property: "y"
		value: dummyRect.y
		when: root.dragEnabled
	}
	
	MouseArea {
		id: ma
        anchors.fill: dummyRect
		anchors.margins: -root.radius
        cursorShape: 
		{
			if (root.targetX && root.targetY)
			{
				return swapDiagCursor ? Qt.SizeBDiagCursor : Qt.SizeFDiagCursor
			}
			if (root.targetX)
			{
                return Qt.SizeHorCursor
			}
			if (root.targetY)
			{
				return Qt.SizeVerCursor
			}
		}
        drag.axis: {
			if (root.targetX && root.targetY)
			{
				return Drag.XandYAxis
			}
			if (root.targetX)
			{
				return Drag.XAxis
			}
			if (root.targetY)
			{
				return Drag.YAxis
			}
		}
        drag.target: dummyRect
        drag.threshold: 0		
		onPositionChanged:
		{
            if (objectCreator.__shiftPressed != (mouse.modifiers & Qt.ShiftModifier)) {
                objectCreator.__shiftPressed = (mouse.modifiers & Qt.ShiftModifier);
            }
            if (objectCreator.__controlPressed != (mouse.modifiers & Qt.ControlModifier)) {
                objectCreator.__controlPressed = (mouse.modifiers & Qt.ControlModifier);
            }
		}
        onPressed: {
            if (objectCreator.__shiftPressed != (mouse.modifiers & Qt.ShiftModifier)) {
                objectCreator.__shiftPressed = (mouse.modifiers & Qt.ShiftModifier);
            }
            if (objectCreator.__controlPressed != (mouse.modifiers & Qt.ControlModifier)) {
                objectCreator.__controlPressed = (mouse.modifiers & Qt.ControlModifier);
            }
            root.pressed(mouse)
        }
        onReleased:
        {
            if (objectCreator.__shiftPressed != (mouse.modifiers & Qt.ShiftModifier)) {
                objectCreator.__shiftPressed = (mouse.modifiers & Qt.ShiftModifier);
            }
            if (objectCreator.__controlPressed != (mouse.modifiers & Qt.ControlModifier)) {
                objectCreator.__controlPressed = (mouse.modifiers & Qt.ControlModifier);
            }
        }
	}

	Keys.onPressed:
	{
		if (event.modifiers & Qt.ShiftModifier)
		{
			root.__shiftPressed = true
		}
		if (event.modifiers & Qt.ControlModifier)
		{
			root.__controlPressed = true
		}
		event.accepted = false
	}
	Keys.onReleased:
	{
		if (!(event.modifiers & Qt.ShiftModifier))
		{
			root.__shiftPressed = false
		}
		if (!(event.modifiers & Qt.ControlModifier))
		{
			root.__controlPressed = false
		}
		event.accepted = false
    }
}
