import QtQuick 2.0

FocusScope {
	id: root
	property bool resizeEnabled: true
    property bool dragEnabled: true
	property real borderWidth: 2
	property bool selected: true
    property alias borderColor: frame.borderColor
    property alias anchorsColor: frame.anchorColor
	property real minimumX: -100000
	property real maximumX: 100000
	property real minimumY: -100000
	property real maximumY: 100000  
	property real minimumWidth: 1
	property real minimumHeight: 1
	readonly property bool hovered: ma.containsMouse
	property var target
	property bool keepAspect: false
	property real aspect: root.target ? (root.target.aspect ? root.target.aspect : 1.0) : 1.0
    property alias clickEnabled: ma.enabled
    readonly property rect targetRect: Qt.rect(x + targetItem.x, y + targetItem.y, targetItem.width, targetItem.height)
    signal pressed(var mouse, var item)
    signal released(var mouse, var item)
    signal clicked(var mouse, var item)
    signal pressAndHold(var mouse, var item)
	signal resizeStarted()
    property bool moveActive: false
    signal doubleClicked(var mouse, var item)
	antialiasing: true
	smooth: true
	focus: selected
    onTargetChanged: {
        if (!target) {
            selected = visible = false
        }
    }

	Binding on focus {
		value: root.selected
	}
	Item {
		id: targetItem		
		width: root.target ? root.target.width : 0
		height: root.target ? root.target.height : 0
		rotation: root.target ? root.target.rotation : 0
		transformOrigin: Item.Center
		Binding on x
		{
			value: root.target ? root.target.x : 0
			when: !ma.drag.active
		}
		Binding on y
		{
			value: root.target ? root.target.y : 0
			when: !ma.drag.active
		}
		Binding
		{
			target: root.target
			property: "x"
			value: targetItem.x
			when: ma.drag.active
		}
		Binding
		{
			target: root.target
			property: "y"
			value: targetItem.y
			when: ma.drag.active
		}		
		MouseArea {
			id: ma
            property bool parentIsSelected: (root.target && root.target.getSelectedParent() !== null)
			acceptedButtons: Qt.LeftButton | Qt.RightButton
			anchors.fill: targetItem			
            cursorShape: {
                if (root.target && root.target.freezed && root.target.selected) return Qt.ForbiddenCursor
                return enabled ? (root.selected || parentIsSelected ? Qt.SizeAllCursor : Qt.PointingHandCursor) : Qt.ArrowCursor
            }
            property real startX
			property real startY
			property bool dragActive: false
			onPressed: {
                root.pressed(mouse, this)
				if (root.target)
				{
					var newCoords = mapToItem(root, mouse.x, mouse.y)
					ma.startX = root.target.x - newCoords.x
					ma.startY = root.target.y - newCoords.y
				}
			}
            onReleased: root.released(mouse, this)
            onPressAndHold: {
                if (ma.dragActive) return
                root.pressAndHold(mouse,  this)
             }
			onClicked: {
                if (ma.dragActive) {
                    ma.dragActive = false;
                }
                else {
                    root.clicked(mouse, this)
                }
            }
			onDoubleClicked: {
                if (mouse.buttons & Qt.LeftButton) {
                    root.doubleClicked(mouse, this);
                }
            }
			onPositionChanged:
			{
                if (root.target && (mouse.buttons & Qt.LeftButton) && root.dragEnabled) {
                    var localTarget = root.target;
                    if (!root.target.selected) {
                        root.target = root.target.getSelectedParent();
                    }
                    if (root.target)
                    {
                        if (!ma.dragActive)
                        {
                            var newCoords = mapToItem(root, mouse.x, mouse.y)
                            if (Math.abs(ma.startX - (root.target.x - newCoords.x)) > 5 ||
                                Math.abs(ma.startY - (root.target.y - newCoords.y)) > 5)
                            {
                                ma.dragActive = true
                            }
                        }
                        if (ma.dragActive)
                        {
                            var newCoords = mapToItem(root, mouse.x, mouse.y)
                            root.target.x = ma.startX + newCoords.x
                            root.target.y = ma.startY + newCoords.y
                        }
                    }
                    root.target = localTarget;
                }
				mouse.accepted = false
			}
			onDragActiveChanged:
			{
                root.moveActive = ma.dragActive;
				if (ma.dragActive)
				{
                    root.resizeStarted()
				}
			}
		}		
	}
	
	onDragActiveChanged:
	{
        if (dragActive)
		{
			root.resizeStarted()
		}
	}
	
	Item
	{		
		id: dummy
		Binding on focus {
			value: root.focus
		}
        property var localTarget: root.target ? (root.target.selected ? root.target : root.target.getSelectedParent()) : null
		Keys.onLeftPressed:
		{
			event.accepted = false
            if (!localTarget || localTarget.freezed)
				return
			var delta = 1
			if (event.modifiers & Qt.ShiftModifier)
			{
				delta = 10
			}
			if (event.modifiers & Qt.ControlModifier)
			{
				rightAnchorLine.x -= delta
				localTarget.x = anchorRectangle.x
				localTarget.y = anchorRectangle.y
				localTarget.width = anchorRectangle.width
				localTarget.height = anchorRectangle.height
			}
			else
			{
				localTarget.x -= delta
			}
		}
		Keys.onRightPressed:
		{
			event.accepted = false
            if (!localTarget || localTarget.freezed)
                return
			var delta = 1
			if (event.modifiers & Qt.ShiftModifier)
			{
				delta = 10
			}
			if (event.modifiers & Qt.ControlModifier)
			{
				rightAnchorLine.x += delta
				localTarget.x = anchorRectangle.x
				localTarget.y = anchorRectangle.y
				localTarget.width = anchorRectangle.width
				localTarget.height = anchorRectangle.height
			}
			else
			{
				localTarget.x += delta
			}
		}
		Keys.onUpPressed:
		{
			event.accepted = false
            if (!localTarget || localTarget.freezed)
                return
			var delta = 1
			if (event.modifiers & Qt.ShiftModifier)
			{
				delta = 10
			}
			if (event.modifiers & Qt.ControlModifier)
			{
				bottomAnchorLine.y -= delta
				localTarget.x = anchorRectangle.x
				localTarget.y = anchorRectangle.y
				localTarget.width = anchorRectangle.width
				localTarget.height = anchorRectangle.height
			}
			else
			{
				localTarget.y -= delta
			}
		}
		Keys.onDownPressed:
		{
			event.accepted = false
            if (!localTarget || localTarget.freezed)
                return
			var delta = 1
			if (event.modifiers & Qt.ShiftModifier)
			{
				delta = 10
			}
			if (event.modifiers & Qt.ControlModifier)
			{
				bottomAnchorLine.y += delta
				localTarget.x = anchorRectangle.x
				localTarget.y = anchorRectangle.y
				localTarget.width = anchorRectangle.width
				localTarget.height = anchorRectangle.height
			}
			else
			{
				localTarget.y += delta
			}
		}
		Binding on x
		{
			value: root.target ? root.target.x + (root.target.width / 2) : 0
			when: !root.dragActive
		}
		Binding on y
		{
			value: root.target ? root.target.y + (root.target.height / 2) : 0
			when: !root.dragActive
		}
		rotation: root.target ? root.target.rotation : 0
        transformOrigin: Item.Center
		Item {
			id: leftAnchorLine
			y: -targetItem.height/2
			height: targetItem.height
			width: 0
			property bool dragActive: false
			Binding on x
			{
				value: -targetItem.width/2
				when: !root.dragActive
			}
			onDragActiveChanged:
			{
				if (dragActive)
				{
					Qt.binding(function () { return root.anchorX = rightAnchorLine.x });
				}
			}
		}
		Item {
			id: rightAnchorLine
			y: -targetItem.height/2
			height: targetItem.height
			width: 0
			property bool dragActive: false
			Binding on x
			{
				value: targetItem.width/2
				when: !root.dragActive
			}
			onDragActiveChanged:
			{
				if (dragActive)
				{
					root.anchorX = Qt.binding(function () { return leftAnchorLine.x });
				}
			}
		}
		Item {
			id: topAnchorLine
			x: -targetItem.width/2
			width: targetItem.width
			height: 0
			property bool dragActive: false
			Binding on y
			{
				value: -targetItem.height/2
				when: !root.dragActive
			}
			onDragActiveChanged:
			{
				if (dragActive)
				{
					root.anchorY = Qt.binding(function () { return bottomAnchorLine.y});
				}
			}
		}
		Item {
			id: bottomAnchorLine
			x: -targetItem.width/2
			width: targetItem.width
			height: 0
			property bool dragActive: false
			Binding on y
			{
				value: targetItem.height - (targetItem.height/2)
				when: !root.dragActive
			}
			onDragActiveChanged:
			{
				if (dragActive)
				{
					root.anchorY = Qt.binding(function () { return topAnchorLine.y});
				}
			}
		}
		
			
		Repeater
		{
			model: (resizeEnabled && selected) ? root.resizeAchorsModel : 0
			delegate:
				ObjectDragElement
				{
					id: resizeAchor
					targetX: modelData.targetX ? modelData.targetX : null
					targetY: modelData.targetY ? modelData.targetY : null
					radius: root.borderWidth * 15
					swapDiagCursor: modelData.swapDiagCursor ? modelData.swapDiagCursor : false
					/*
					Binding
					{
						target: root
						property: "keepAspect"
						value: resizeAchor.shiftPressed
						when: resizeAchor.targetX && resizeAchor.targetY
					}
					*/
					onDragEnabledChanged:
					{
						if (resizeAchor.targetX && resizeAchor.targetY)
							return
						root.keepAspect = false
					}
				}
		}

		Item
		{
			id: resizeRectangle
			property real leftX: Math.min(leftAnchorLine.x, rightAnchorLine.x)
			property real rightX: Math.max(leftAnchorLine.x, rightAnchorLine.x)
			property real topY: Math.min(topAnchorLine.y, bottomAnchorLine.y)
			property real bottomY: Math.max(topAnchorLine.y, bottomAnchorLine.y)			
			property bool flipX: root.anchorX > leftX
			property bool flipY: root.anchorY > topY
			property real _x: flipX ? rightX - _width: leftX
			property real _y: flipY ? bottomY - _height: topY		
			property real originalWidth: Math.max(root.minimumWidth, Math.abs(leftX - rightX))
			property real originalHeight: Math.max(root.minimumHeight, Math.abs(topY - bottomY))
			property real _aspectedWidth: originalWidth < originalHeight * root.aspect ? originalHeight * root.aspect: originalWidth
			property real _aspectedHeight: originalHeight < originalWidth / root.aspect ? originalWidth / root.aspect: originalHeight
			property real aspectedWidth: flipX ? Math.min((root.anchorX - root.minimumX),_aspectedWidth) : _aspectedWidth
			property real aspectedHeight: flipY ? Math.min((root.anchorY - root.minimumY),_aspectedHeight) : _aspectedHeight
			property real _width: keepAspectPrivate ? aspectedWidth : originalWidth
			property real _height: keepAspectPrivate ? aspectedHeight : originalHeight
			x: Math.min(root.maximumX, Math.max(root.minimumX, _x))
			y: Math.min(root.maximumY, Math.max(root.minimumY, _y))
			width: Math.min(root.maximumX - x, _width)
			height: Math.min(root.maximumY - y, _height)
		}
		property var mappedRect: dummyRotatedBack.mappedRect
		Item
		{
			rotation: root.target ? -root.target.rotation : 0
			transformOrigin: Item.Center
			x: resizeRectangle.x
			y: resizeRectangle.y
			width: resizeRectangle.width
			height: resizeRectangle.height
			
			Item{
				id: dummyRotatedBack
				anchors.fill:parent
				property var mappedRect: mapToItem(root, x, y, width, height)
				function updateMappedRect()
				{
					mappedRect = mapToItem(root, x, y, width, height)
					mappedRect.width = width
					mappedRect.height = height
				}
				onXChanged:	updateMappedRect()
				onYChanged:	updateMappedRect()
				onWidthChanged: updateMappedRect()
				onHeightChanged: updateMappedRect()
			}
		}
		onXChanged:	{dummyRotatedBack.updateMappedRect();}
		onYChanged:	{dummyRotatedBack.updateMappedRect();}
		onWidthChanged: {dummyRotatedBack.updateMappedRect();}
		onHeightChanged: {dummyRotatedBack.updateMappedRect();}
	}
	Item
	{
		id: anchorRectangle
		x: dummy.mappedRect.x
		y: dummy.mappedRect.y
		width: dummy.mappedRect.width
		height: dummy.mappedRect.height
	}	
	Binding
	{
		target: root.target
		property: "x"
		value: anchorRectangle.x
		when: root.dragActive
	}
	Binding
	{
		target: root.target
		property: "y"
		value: anchorRectangle.y
		when: root.dragActive
	}
	Binding
	{
		target: root.target
		property: "width"
		value: anchorRectangle.width
		when: root.dragActive
	}
	Binding
	{
		target: root.target
		property: "height"
		value: anchorRectangle.height
		when: root.dragActive
	}
	
	property real anchorX: 0
	property real anchorY: 0


			
	property alias leftAnchorLine: leftAnchorLine
	property alias rightAnchorLine: rightAnchorLine
	property alias topAnchorLine: topAnchorLine
	property alias bottomAnchorLine: bottomAnchorLine
	
	property var resizeAchorsModel:
			[
				{
					targetX: leftAnchorLine
				},
				{
					targetX: rightAnchorLine
				},
				{
					targetY: topAnchorLine
				},
				{
					targetY: bottomAnchorLine
				},
				{
					targetX: leftAnchorLine,
					targetY: topAnchorLine
				},
				{
					targetX: leftAnchorLine,
					targetY: bottomAnchorLine,
					swapDiagCursor: true
				},
				{
					targetX: rightAnchorLine,
					targetY: topAnchorLine,
					swapDiagCursor: true
				},
				{
					targetX: rightAnchorLine,
					targetY: bottomAnchorLine
				}
			]
	
	onKeepAspectChanged:
	{
		if (root.target && root.target.height != 0)
		{
			root.aspect = root.target.width / root.target.height;
		}
	}
	property bool keepAspectPrivate: keepAspect
    property bool dragActive:
		dragActiveHorizontal || 
		dragActiveVertical
		
	property bool dragActiveHorizontal: 
		leftAnchorLine.dragActive || 
		rightAnchorLine.dragActive
	property bool dragActiveVertical:
		bottomAnchorLine.dragActive || 
		topAnchorLine.dragActive
		
	readonly property bool active: dragActive || ma.pressedButtons & Qt.LeftButton

    Frame
    {
        id: frame
        borderWidth: root.borderWidth
        borderColor: root.color
        antialiasing: root.antialiasing
        transform: [
            Rotation {
                origin.x: root.target ? root.target.boundingBox.width / 2 : 0
                origin.y: root.target ? root.target.boundingBox.height / 2 : 0
                angle: root.target ? root.target.rotation : 0
            },
            Translate {
                x: root.target ? root.target.boundingBox.x : 0
                y: root.target ? root.target.boundingBox.y : 0
            }
        ]
        showAnchors: root.selected && resizeEnabled
        width:  root.target ? root.target.boundingBox.width  : 0
        height: root.target ? root.target.boundingBox.height : 0
        anchors.margins: -root.borderWidth/2
        visible: root.target && root.selected
    }
}
