import QtQuick 2.1
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.1

Item {
    id: root
    default property Item content
    property Component background: Item {
        implicitWidth: children.length == 1 ? children[0].implicitWidth : 0
        implicitHeight: children.length == 1 ? children[0].implicitHeight : 0    
    }
    property alias style: scrollView.style
    property int scrollBarSpacing: 0
    property var scrollBar: scrollView.__verticalScrollBar
    readonly property bool showScrollBar: scrollView.__verticalScrollBar.width > 0
    readonly property int scrollBarWidth: scrollView.__verticalScrollBar.implicitWidth
    
    implicitHeight: backgroundLoader.implicitHeight
    
    RowLayout {
        anchors.fill: parent
        spacing: 0
        
        Loader {
            id: backgroundLoader
            Layout.fillHeight: true
            Layout.fillWidth: true
            sourceComponent: root.background
            Binding {
                target: backgroundLoader.item
                property: "data"
                value: panel
            }
        }
        
        property Item _panel: Item {
            id: panel
            implicitWidth: content ? content.implicitWidth : 0
            implicitHeight: content ? content.implicitHeight : 0
            anchors.fill: parent
            clip: true
            Item {
                id: contentContainer
                y: -scrollView.flickableItem.contentY
                width: parent.width
                height: content ? content.height : 0
                data: content
            }
            Binding {
                target: content
                property: "width"
                value: panel.width
            }
        }
        
        Item {
            visible: showScrollBar
            implicitWidth: scrollBarSpacing
        }
        
        ScrollView {
            id: scrollView
            Layout.fillHeight: true
            Layout.preferredWidth: showScrollBar ? root.scrollBarWidth : 0
            horizontalScrollBarPolicy: Qt.ScrollBarAlwaysOff
            Item {
                height: root.height - panel.height + contentContainer.height
            }
        }
    }
}
