import QtQml 2.0

QtObject
{
	id: root
	property var items: []
	property bool exclusiveSelection: true
	property var selectedItems: getSelectedItems()
    property bool cover: true
    property bool invertSelection: true

	property int selectedCount: {
		var count = 0
		for (var index in items)
		{
			var item = items[index]						
			if (item.selected) {
				count++
			}
		}		
		return count
	}
	
	function resetSelection()
	{
		for (var index in items)
		{
			var item = items[index]							
			item.selected = false
		}
	}

	function selectAll()
	{		
		for (var index in items)
		{
			var item = items[index]							
			item.selected = true
		}
	}
	
	function selectItem(selectedItem, exclusiveSelection, invertSelection)
	{
        if (invertSelection == undefined)
			invertSelection = root.invertSelection
		if (exclusiveSelection == undefined)
			exclusiveSelection = root.exclusiveSelection
		if (exclusiveSelection)
		{
			for (var index in items)
			{
				var item = items[index]
				if (item != selectedItem)
					item.selected = false
			}			
		}
		if (selectedItem) {
            if (invertSelection) {			
                selectedItem.selected = !selectedItem.selected
            }
            else {
                selectedItem.selected = true;
            }
        }
	}
	
	function selectNextItem(shift)
	{
		if (exclusiveSelection)
		{			
			var selectedItems = getSelectedItems()
			if (selectedItems.length > 1)
				return
			var currentSelected = -1
			for (var index in items)
			{
				var item = items[index]				
				if (item.selected)
				{
					currentSelected = index
				}
				item.selected = false
			}
			currentSelected = items.length + parseInt(currentSelected) + shift
			currentSelected %= items.length
			items[currentSelected].selected = true
		}		
	}
	
	function selectItems(selectedItems, exclusiveSelection, invertSelection)
	{
        if (invertSelection == undefined)
			invertSelection = root.invertSelection
		if (exclusiveSelection == undefined)
			exclusiveSelection = root.exclusiveSelection
		if (exclusiveSelection)
		{
			for (var index in items)
			{
				var item = items[index]
				item.selected = false
			}
		}
		for (var index in selectedItems)
		{
			var item = selectedItems[index]
            if (invertSelection) {			
                item.selected = !item.selected
            }
            else {
                item.selected = true;
            }
		}		
	}
	
	function selectArea(rect, exclusiveSelection, invertSelection)
	{
        if (invertSelection == undefined)
			invertSelection = root.invertSelection
		var list = []
		for (var index in items)
		{
			var item = items[index]
			var boundingBox = intersectRects(rect, item.boundingBox)
            if (boundingBox) {
                if (cover) {
                    if (Math.abs(getBoundingBoxSquare(boundingBox) - getBoundingBoxSquare(item.boundingBox)) < 1.0) {
                        list.push(item);					
                    }
                }
                else {
                    list.push(item);
                }
            }
		}	
		selectItems(list, exclusiveSelection, invertSelection)
	}
	
	function getBoundingBoxSquare(bb)	
	{
		if (!bb)
			return 0
		return bb.width * bb.height
	}
	
	function hoverArea(rect)
	{
		var list = []
		for (var index in items)
		{
			var item = items[index]						
			var boundingBox = intersectRects(rect, item.boundingBox)	
            if (boundingBox) {
                if (cover) {
                    item.hovered = (boundingBox == item.boundingBox);
                }
                else {
                    item.hovered = true;
                }
            }
            else {
                item.hovered = false;
            }
		}
	}
	
	function intersectIntervals(int1, int2)
	{
		var left = Math.min(int1.y, Math.max(int1.x, int2.x))
		var right = Math.max(int1.x, Math.min(int1.y, int2.y))
		if (left < right)
			return Qt.vector2d(left, right)
		return null
	}
	
	function intersectRects(rect1, rect2)
	{
		var intX = intersectIntervals(Qt.vector2d(rect1.x, rect1.x+rect1.width), Qt.vector2d(rect2.x, rect2.x+rect2.width))
		var intY = intersectIntervals(Qt.vector2d(rect1.y, rect1.y+rect1.height), Qt.vector2d(rect2.y, rect2.y+rect2.height))
		if (intX && intY)
		{
			return Qt.rect(intX.x, intY.x, intX.y - intX.x, intY.y - intY.x)
		}
		return null
	}
	
	function getSelectedItems()
	{
		var result = []
		for (var index in items)
		{
			var item = items[index]
			if (item.selected)
				result.push(item)
		}
		return result
	}
    
    function getSelectedItemIndexes() {
		var result = []
		for (var index in items)
		{
			var item = items[index]
			if (item.selected)
				result.push(index)
		}
		return result
	}
}