import QtQml 2.0

QtObject
{
    id: root
    property var items: []
    property bool exclusiveSelection: true
    property var selectedItems: []
    property bool cover: true
    property bool invertSelection: true
    property bool recursSelection: false
    property int selectionLevel: (selectedItems.length !== 0) ? selectedItems[0].getParentCount() : 0

    property int selectedCount: {
        return selectedItems.length;
    }

    function updateSelectedItems() {
       selectedItems = getSelectedItems()
    }

    function resetSelection()
    {
        for (var index in items)
        {
            var item = items[index]
            item.deselect()
        }
        updateSelectedItems()
    }

    function selectAll()
    {
        resetSelection();
        for (var index in items)
        {
            var item = items[index]
            if (item.getParentCount() === 0) { // && !item.freezed) {
                item.select()
            }
        }
        updateSelectedItems()
    }

    function selectItem(selectedItem, exclusiveSelection, invertSelection, recursSelection)
    {
        if (invertSelection == undefined)
            invertSelection = root.invertSelection
        if (exclusiveSelection == undefined)
            exclusiveSelection = root.exclusiveSelection
        if (recursSelection == undefined)
            recursSelection = root.recursSelection
        //exclusive selection
        if (exclusiveSelection) {
            for (var index in items) {
                var item = items[index]
                if (!selectedItem.childOf(item))
                    item.deselect()
            }
        }
        if (selectedItem) {
            //invert selection
            if (invertSelection) {
                if (selectedItem.selected) {
                    selectedItem.deselect()
                    updateSelectedItems()
                    return;
                }
            }
            if (selectedItem.selected) {
                updateSelectedItems()
                return
            }
            //recurs selection
            if (recursSelection){
                selectItemRecurs(selectedItem);
            }
            else {
                selectedItem.select();
            }
            //clear selection on low levels
            var selectedObjects = privateGetSelectedItems();
            var maxLevel = getMaxSelectedLevel(selectedObjects);
            var selectedItemsOnLevel = [];
            for (var i = 0; i <= maxLevel; ++i) {
                selectedItemsOnLevel = getSelectedItemsOnLevel(selectedObjects, i);
                if (selectedItemsOnLevel.length > 1) {
                    clearSelectionOnLowLevels(i);
                    break
                }
            }
        }
        updateSelectedItems()
    }

    function selectItemRecurs(item) {
        if (!item || item.selected) return;

        if (item.objParent && (item.objParent.selected !== undefined)) {
            selectItemRecurs(item.objParent);
        }
        item.select();
    }
    function restoreSelection(items) {
        if (items.length === 0) return;
        for (var i in items) {
            var item = items[i];
            selectItemRecurs(item);
        }
        updateSelectedItems()
    }
    function selectItems(selectedItems, exclusiveSelection, invertSelection)
    {
        if (invertSelection == undefined)
            invertSelection = root.invertSelection
        if (exclusiveSelection == undefined)
            exclusiveSelection = root.exclusiveSelection
        if (exclusiveSelection)
        {
            for (var index in items)
            {
                var item = items[index]
                item.deselect()
            }
        }
        for (var index in selectedItems)
        {
            var item = selectedItems[index]
            if (invertSelection) {
                if (item.selected) {
                    item.deselect()
                }
                else {
                    item.select()
                }
            }
            else {
                item.select()
            }
        }
        updateSelectedItems()
    }

    function selectArea(rect, exclusiveSelection, invertSelection)
    {
        if (invertSelection == undefined)
            invertSelection = root.invertSelection
        var list = []
        for (var index in items)
        {
            var item = items[index]
            var boundingBox = intersectRects(rect, item.boundingBox)
            if (boundingBox) {
                if (cover) {
                    if (Math.abs(getBoundingBoxSquare(boundingBox) - getBoundingBoxSquare(item.boundingBox)) < 1.0) {
                        list.push(item);
                    }
                }
                else {
                    list.push(item);
                }
            }
        }

        selectItems(list, exclusiveSelection, invertSelection)
    }

    function getBoundingBoxSquare(bb)
    {
        if (!bb)
            return 0
        return bb.width * bb.height
    }

    function hoverArea(rect)
    {
        var list = []
        for (var index in items)
        {
            var item = items[index]
            var boundingBox = intersectRects(rect, item.boundingBox)
            if (boundingBox) {
                if (cover) {
                    item.hovered = (boundingBox == item.boundingBox);
                }
                else {
                    item.hovered = true;
                }
            }
            else {
                item.hovered = false;
            }
        }
    }

    function intersectIntervals(int1, int2)
    {
        var left = Math.min(int1.y, Math.max(int1.x, int2.x))
        var right = Math.max(int1.x, Math.min(int1.y, int2.y))
        if (left < right)
            return Qt.vector2d(left, right)
        return null
    }

    function intersectRects(rect1, rect2)
    {
        var intX = intersectIntervals(Qt.vector2d(rect1.x, rect1.x+rect1.width), Qt.vector2d(rect2.x, rect2.x+rect2.width))
        var intY = intersectIntervals(Qt.vector2d(rect1.y, rect1.y+rect1.height), Qt.vector2d(rect2.y, rect2.y+rect2.height))
        if (intX && intY)
        {
            return Qt.rect(intX.x, intY.x, intX.y - intX.x, intY.y - intY.x)
        }
        return null
    }
    function getSelectedItemsOnLevel(items, level) {
        var result = [];
        for (var i in items) {
            var object = items[i];
            var objectLevel = object.getParentCount();
            if (objectLevel === level) {
                result.push(object);
            }
        }
        if (result.length > 1) {
            if (!result[0].haveSameParentWithObjects(result)) {
                return [];
            }
        }
        return result;
    }
    function getMaxSelectedLevel(items) {
        var max = 0;
        for (var i in items) {
            var object = items[i];
            var objectLevel = object.getParentCount();
            if (objectLevel > max) {
                max = objectLevel;
            }
        }
        return max;
    }
    function clearSelectionOnLowLevels(level) {
        for (var i in items)  {
            var item = items[i];
            if (item.getParentCount() > level) {
                if (item.selected) {
                    item.deselect();
                }
            }
        }
    }
    function getSelectedItems()
    {
        var selectedObjects = privateGetSelectedItems();
        if (selectedObjects.length === 0) return [];
        var maxLevel = getMaxSelectedLevel(selectedObjects);
        var result = getSelectedItemsOnLevel(selectedObjects, maxLevel);
        //console.warn("getSelectedItems", result.length)
        return result;
    }
    function isInSelectedItems(object) {
        var list = selectedItems
        var index = list.indexOf(object);
        return (index !== -1);
    }
    function privateGetSelectedItems(){
        var result = []
        for (var index in items) {
            var item = items[index];
            if (item.selected) { // && item.visible) {
                result.push(item);
            }
        }
        return result;
    }
}
