import QtQuick 2.4
import QtQuick.Controls 1.1
import QtQuick.Layouts 1.1
import MixerGuiControls 1.0

ScrollView {
    id: root
    property var model: []
    property Component delegate: Component {
        MixerLabel {
            text: modelData.toString()
        }
    }
    property int columnIndent: MixerThemeManager.theme.propertyHeaderOffsetX
    property var currentItem: (model.length > 0) ? model[0] : null
    property string role: ""
    signal clicked(var mouse)
    signal doubleClicked(var mouse)
    property int spacing: 0

    property int rowHeight: MixerThemeManager.theme.propertyHeaderHeight
    property bool expandAll: true

    style: MixerScrollViewStyle {}
    horizontalScrollBarPolicy:  Qt.ScrollBarAlwaysOff

    function hasChildren(element) {
        if (!element) return false;
        var role = ((root.role !== "") ? root.role : "children");
        return (element[role] !== undefined);
    }

    contentItem: Loader {
        id: content
        width: flickableItem.width
        active: (root.model.length > 0)
        sourceComponent: (root.model.length > 0) ? treeBranchComponent : null
        onLoaded: {
            item.isRoot = true;
            item.children = Qt.binding( function() { return root.model } )
        }

        Component {
            id: treeBranchComponent

            Item {
                id: treeBranch
                property bool isRoot: false
                property int level: 0
                property var children: []
                implicitHeight: column.implicitHeight
                implicitWidth: column.implicitWidth

                Column {
                    id: column
                    spacing: root.spacing

                    Item {
                        height: isRoot ? 0 : rowHeight
                        width: 1
                    }

                    Repeater {
                        id: repeater
                        model: treeBranch.children
                        Item {
                            id: filler
                            property var child: treeBranch.children[model.index]
                            property int index: model.index
                            property bool hasChildren: root.hasChildren(child)
                            readonly property int shift: level * root.columnIndent
                            width: content.width - shift
                            height: Math.max(row.height, loader.height)
                            visible: load.item && load.item.enabled

                            MouseArea {
                                anchors.fill: parent
                                onClicked: {
                                    root.currentItem = child;
                                    root.clicked(mouse);
                                }
                                onDoubleClicked: root.doubleClicked(mouse)
                            }
                            RowLayout {
                                id: row
                                width: parent.width
                                height: root.rowHeight

                                Item {
                                    id: itemExpander
                                    Layout.preferredWidth: parent.height
                                    Layout.preferredHeight: parent.height
                                    opacity: filler.hasChildren ? 1 : 0
                                    Image {
                                        id: expander
                                        source: "expander_white.png"//expander_white.png / expander_black.png
                                        opacity: mouse.containsMouse ? 1 : 0.7
                                        anchors.centerIn: parent
                                        rotation: loader.expanded ? 90 : 0
                                        Behavior on rotation { NumberAnimation { duration: 120 } }
                                    }

                                    MouseArea {
                                        id: mouse
                                        anchors.fill: parent
                                        propagateComposedEvents: true
                                        onClicked: {
                                            if (itemExpander.opacity !== 0) {
                                                loader.expanded = !loader.expanded
                                            }
                                            else {
                                                mouse.accepted = false;
                                            }
                                        }
                                    }
                                }

                                Item {
                                    Layout.fillWidth: true
                                    Layout.fillHeight: true

                                    Loader {
                                        id: load
                                        anchors.verticalCenter: parent.verticalCenter
                                        width: parent.width
                                        active: root.delegate !== null
                                        property alias styleData: filler.child
                                        property alias index: filler.index
                                        property bool isSelected: root.currentItem && (root.currentItem === treeBranch.children[index])
                                        sourceComponent: root.delegate
                                    }
                                }
                            }

                            Loader {
                                id: loader
                                property bool expanded: root.expandAll
                                property string role: root.role !== "" ? root.role : "children"
                                x: columnIndent
                                active: filler.hasChildren && expanded
                                height: expanded ? implicitHeight : 0
                                sourceComponent: treeBranchComponent
                                onLoaded: {
                                    item.children = Qt.binding( function() { return filler.child[loader.role] } )
                                    item.level = treeBranch.level + 1;
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}
