import QtQuick 2.3
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.1
import QtQuick.Controls.Private 1.0

Item {
    id: root	
    default property Item contentItem
    property Item overlay
    property alias internalFocus: scrollView.focus
    property alias scrollViewStyle: scrollView.style
    
	function focusInternalArea()
	{
		scrollView.focus = true
	}
	property var forwardTo: []
    property real zoom: 1.0
    property real zoomSpeed: 1.1
    property real minimumZoom: 0.1
    property real maximumZoom: 9.99
    property real contentX: 0.0
    property real contentY: 0.0
	property bool autoFitMode
    property alias wheelEnabled: wheelArea.enabled
    readonly property rect viewport: Qt.rect(contentX, contentY, internal.flickable.width / zoom, internal.flickable.height / zoom)
    readonly property size contentSize: contentItem ? Qt.size(contentItem.width, contentItem.height) : Qt.size(0, 0)	
    Item {
        id: internal
        property real prevZoom: 1.0
        property alias flickable: scrollView.flickableItem
        property bool ownChange: false
        property bool contentFiting: false
    }
    
    
    function fitToContent()
    { 
        var zoomX = (root.width - 30) / Math.max(contentSize.width, 1)
        var zoomY = (root.height - 30) / Math.max(contentSize.height, 1)
        internal.contentFiting = true
        root.zoom = Math.min(zoomX, zoomY)     
        internal.contentFiting = false
    }
    function changeZoom(speed)
    {
		autoFitMode = false
        zoom = Math.max(minimumZoom, Math.min(zoom * speed, maximumZoom))
    }	
	Keys.onPressed: {
		//if (event.key == Qt.Key_Plus) changeZoom(zoomSpeed)
		//else if(event.key == Qt.Key_Minus) changeZoom(1/zoomSpeed)
		//else event.accepted = false
		event.accepted = false
	}
	onWidthChanged: if (root.autoFitMode) root.fitToContent()
	onHeightChanged: if (root.autoFitMode) root.fitToContent()
    onAutoFitModeChanged: if(root.autoFitMode) root.fitToContent()

	ScrollView {	
        id: scrollView
		focus: true
		anchors.fill: parent
		flickableItem.interactive: false
        frameVisible: true
		FocusScope {
			focus: scrollView.focus
			width: contentSize.width * zoomArea.zoom
			height: contentSize.height * zoomArea.zoom       
			Item {
				id: zoomArea
				property real zoom: 1.0
				transform: Scale {
					xScale: zoomArea.zoom
					yScale: zoomArea.zoom
				}
				data: root.contentItem
				width: contentSize.width
				height: contentSize.height
                onWidthChanged: if(root.autoFitMode) root.fitToContent()
                onHeightChanged: if(root.autoFitMode) root.fitToContent()
			}
		}		
	}
    WheelArea {
        id: wheelArea
        width: internal.flickable.width
        height: internal.flickable.height    
        onVerticalDeltaChanged: {
            if(verticalDelta > 0) changeZoom(zoomSpeed)
            else changeZoom(1/zoomSpeed)
        }
    }
    Item {
        id: overlayItem
        x: scrollView.viewport.x
        y: scrollView.viewport.y
        width: scrollView.viewport.width
        height: scrollView.viewport.height
        data: root.overlay
    }
    
    Connections {
        target: internal.flickable
        onContentXChanged: {
            internal.ownChange = true
            root.contentX = internal.flickable.contentX / zoom
            internal.ownChange = false
        }
        onContentYChanged: {
            internal.ownChange = true
            root.contentY = internal.flickable.contentY / zoom
            internal.ownChange = false
        }
    }
    Connections {
        target: root
        onContentXChanged: {
            if(internal.ownChange) return
           internal.flickable.contentX = root.contentX * zoom
        }
        onContentYChanged: {
            if(internal.ownChange) return
            internal.flickable.contentY = root.contentY * zoom
        }
        onWidthChanged: update()
        onHeightChanged: update()
        onContentSizeChanged: update()
        
        onZoomChanged: {
            update()
            if(!internal.contentFiting) root.autoFitMode = false
        }
        
        function update() {
            var prevCenterX = internal.flickable.contentX + internal.flickable.width * 0.5
            var prevCenterY = internal.flickable.contentY + internal.flickable.height * 0.5
            zoomArea.zoom = zoom
            var newContentX = (prevCenterX/internal.prevZoom) * zoom - internal.flickable.width * 0.5
            var newContentY = (prevCenterY/internal.prevZoom) * zoom - internal.flickable.height * 0.5
            if(internal.flickable.width > contentSize.width * zoom) newContentX = -(internal.flickable.width - contentSize.width * zoom) * 0.5
            else if(newContentX < 0) newContentX = 0
            else if(newContentX > contentSize.width * zoom - internal.flickable.width) newContentX = contentSize.width * zoom - internal.flickable.width
            if(internal.flickable.height > contentSize.height * zoom) newContentY = -(internal.flickable.height - contentSize.height * zoom) * 0.5
            else if(newContentY < 0) newContentY = 0
            else if(newContentY > contentSize.height * zoom - internal.flickable.height) newContentY = contentSize.height * zoom - internal.flickable.height
            internal.flickable.contentX = newContentX
            internal.flickable.contentY = newContentY
            internal.prevZoom = zoom        
        }
    }
}
