import QtQuick 2.1
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.1
import QtQuick.Controls.Styles 1.1
import GuiControls 1.0
import QtQuickUtils 1.0

Item {
    id: root
    width: 200
    height: width * 0.8
    layer.enabled: true
    
    property Item view
    property url iconFolder: "icons/"
    property alias buttonStyle: actualSizeButton.style
    property alias sliderStyle: zoomSlider.style
    
    onViewChanged: if(view) {
        zoomSlider.value = view.zoom
        autoFitButton.checked = view.autoFitMode
    }
    
    ColumnLayout {
        anchors.fill: parent
        spacing: 4
        Item {
            Layout.fillWidth: true
            Layout.fillHeight: true
            PreserveAspectFit {
                id: miniView
                clip: true
                aspect: (view.contentSize.height != 0) ? view.contentSize.width/view.contentSize.height : 1
                ShaderEffectSource {
                    anchors.fill: parent
                    sourceItem: root.view.contentItem
                    mipmap: true
                    textureSize: {
                        var zoom = Math.min(1, view.zoom)
                        return Qt.size(Math.max(sourceItem.width * zoom, width), Math.max(sourceItem.height * zoom, height))
                    }
                }
                Rectangle {
                    anchors.fill: parent
                    color: "transparent"
                    border.width: 1
                }
                Rectangle {
                    id: dragableFrame
                    color: "transparent"
                    border.width: 2
                    border.color: Qt.rgba(0.8, 0, 0)
                    width: (view.viewport.width / view.contentSize.width) * parent.width
                    height: (view.viewport.height / view.contentSize.height) * parent.height
                    Binding on x {
                        value: (view.viewport.x / view.contentSize.width) * miniView.width
                        when: !mouseArea.drag.active
                    }
                    Binding on y {
                        value: (view.viewport.y / view.contentSize.height) * miniView.height
                        when: !mouseArea.drag.active
                    }
                    Binding {
                        target: view
                        property: "contentX"
                        value: (dragableFrame.x / miniView.width) * view.contentSize.width
                        when: mouseArea.drag.active
                    }
                    Binding {
                        target: view
                        property: "contentY"
                        value: (dragableFrame.y / miniView.height) * view.contentSize.height
                        when: mouseArea.drag.active
                    }
                    MouseArea {
                        id: mouseArea
                        anchors.fill: parent
                        drag.target: parent
                        drag.axis: Drag.XandYAxis

                        drag.minimumX: (parent.width < miniView.width) ? 0 : -(parent.width - miniView.width) / 2
                        drag.minimumY: (parent.height < miniView.height) ? 0 : -(parent.height - miniView.height) / 2
                        drag.maximumX: (parent.width < miniView.width) ? miniView.width - parent.width : miniView.width + (parent.width - miniView.width) / 2 - parent.width
                        drag.maximumY: (parent.height < miniView.height) ? miniView.height - parent.height : miniView.height + (parent.height - miniView.height) / 2 - parent.width
                        cursorShape: Qt.SizeAllCursor
                    }
                }
            }
        }
        RowLayout {
            Button {
                id: actualSizeButton
                onClicked: root.view.zoom = 1.0
                tooltip: qsTr("Actual Size")
                iconSource: root.iconFolder + "Zoom100.png"
            }
            Button {
                id: autoFitButton
                tooltip: qsTr("Fit Content")
                style: actualSizeButton.style
                iconSource: root.iconFolder + "ZoomFit.png"
                
                checkable: true
                Connections {
                    target: root.view
                    onAutoFitModeChanged: autoFitButton.checked = root.view.autoFitMode
                }                
                onCheckedChanged: if(root.view) root.view.autoFitMode = checked
            }      
        
            Slider {
                id: zoomSlider
                Layout.fillWidth: true
                minimumValue: view.minimumZoom
                maximumValue: view.maximumZoom
                value: 1.0
                stepSize: 0.001
                Connections {
                    target: root.view
                    onZoomChanged: {
                        var value = root.view.zoom
                        zoomSlider.value = value
                        root.view.zoom = value
                    }
                }
                
                onValueChanged: if(root.view) root.view.zoom = value
            }
            Label {
                id: labelSizeMeasurer
                text: qsTr("999%")
                horizontalAlignment: Text.AlignRight
                visible: false
            }
            Label {
                text: (root.view.zoom * 100).toFixed() + qsTr("%")
                Layout.preferredWidth: labelSizeMeasurer.implicitWidth
                horizontalAlignment: Text.AlignRight
            }                
        }
    }
}
