import QtQuick 2.3

Item {
    id: root
    property Item source    
    property real loColorHue: 0.0
    property real loColorValue: 0.0
    property real hiColorHue: 0.0
    property real hiColorValue: 0.0
    property real midColorHue: 0.0
    property real midColorValue: 0.0
    property real liftValue: 0.0
    property real gainValue: 0.0
    property real gammaValue: 0.0
    property real satValue: 0.0

    implicitWidth: enabled && source ? source.width : 0
    implicitHeight: enabled && source ? source.height : 0

    ShaderEffect {
        anchors.fill: parent
        property Item source: root.enabled ? root.source : null
        property real  satK: satValue > 1.0 ? 1.0 : satValue        
        property color loColor:  Qt.hsla(root.loColorHue,  1,  satK*root.loColorValue, 1)
        property color hiColor:  Qt.hsla(root.hiColorHue,  1,  1.0-satK*(root.hiColorValue), 1)
        property color midColor: Qt.hsla(root.midColorHue, satK*root.midColorValue, 0.5, 1)

        property var  liftColor:  Qt.vector4d(root.liftValue, root.liftValue, root.liftValue, 1)
        property var  gainColor:  Qt.vector4d(root.gainValue, root.gainValue, root.gainValue, 1)
        property var  gammaColor: Qt.vector4d(root.gammaValue, root.gammaValue, root.gammaValue, 1)
        property var  saturMult: satValue;

        fragmentShader: "
            varying highp vec2 qt_TexCoord0;
            uniform highp float qt_Opacity;
            uniform lowp sampler2D source;

            uniform highp vec4 loColor;
            uniform highp vec4 midColor;
            uniform highp vec4 hiColor;
            uniform highp vec4 liftColor;
            uniform highp vec4 gainColor;
            uniform highp vec4 gammaColor;
            uniform highp float saturMult;

            #define  Pi 3.1415926
            #define  Pr  0.299
            #define  Pg  0.587
            #define  Pb  0.114

            highp float Overlay(highp float a, highp float b)
            {
                if (a>0.5)
                  return 2.0*a*b;
                else
                  return 1.0 - 2.0*(1.0-a)*(1.0-b);
            }
            highp vec4 MultSaturation(highp vec4 col, highp float kSat)
            {
              //  highp float Max = max(max(col.r, col.g), col.b);
                highp float Max = sqrt(col.r*col.r*Pr + col.g*col.g*Pg + col.b*col.b*Pb);

                highp vec4 res;
                res.r = Max - (Max-col.r)*kSat;
                res.g = Max - (Max-col.g)*kSat;
                res.b = Max - (Max-col.b)*kSat;
                if (res.r<0.0) res.r = 0.0;
                if (res.g<0.0) res.g = 0.0;
                if (res.b<0.0) res.b = 0.0;
                return res;
            }
            void main()
            {
                highp vec4 d0  = vec4(0.0,0.0,0.0,0.0);
                highp vec4 d1  = vec4(1.0,1.0,1.0,1.0);
                highp vec4 d2  = vec4(2.0,2.0,2.0,2.0);
                highp vec4 d05 = vec4(0.5,0.5,0.5,0.5);
                highp vec4 dPi = vec4(Pi,Pi,Pi,Pi);

                lowp  vec4 inpColor = texture2D(source, qt_TexCoord0);
                inpColor.rgb /= max(1.0/256.0, inpColor.a);
                highp vec4 outHi = inpColor*hiColor;
                highp vec4 outLo = d1 - (d1-inpColor)*(d1-loColor);
                highp vec4 outMid = vec4(Overlay(inpColor.r,midColor.r),
                                         Overlay(inpColor.g,midColor.g),
                                         Overlay(inpColor.b,midColor.b), 1.0);

                highp vec4 Khi  = inpColor;
                highp vec4 Klo  = d1 - Khi;
                highp vec4 Kmid = d1 - d2*abs(inpColor - d05);

                highp vec4 outColor;
                outColor = Khi*outHi   + (d1-Khi)*inpColor;
                outColor = Klo*outLo   + (d1-Klo)*outColor;
                outColor = Kmid*outMid + (d1-Kmid)*outColor;

                outColor = pow(max(d0, gainColor*(outColor + liftColor*(d1-outColor))), d1/gammaColor);

                outColor = MultSaturation(outColor, saturMult) * inpColor.a;
                outColor.a = inpColor.a;                
                gl_FragColor = outColor * qt_Opacity;
            }
        "
    }    
}
