import QtQuick 2.0
import QtQuickUtils 1.0

Item {
    id: root
    property CropSettings cropSettings
    property Item source
    readonly property bool keepOriginalSize: cropSettings ? cropSettings.keepOriginalSize : true
    implicitWidth: enabled && source ? (keepOriginalSize ? source.width : croppedRect.width) : 0
    implicitHeight: enabled && source ? (keepOriginalSize ? source.height : croppedRect.height) : 0
    property rect croppedRect: source ? crop(source.width, source.height) : Qt.rect(0, 0, 0, 0)

    function crop(w, h) {
        if(!cropSettings || !cropSettings.enabled) return Qt.rect(0, 0, w, h)
        var x1 = w * (cropSettings.left * 0.01)
        var x2 = w - w * (cropSettings.right * 0.01)
        var y1 = h * (cropSettings.top * 0.01)
        var y2 = h - h * (cropSettings.bottom * 0.01)
        return Qt.rect(x1, y1, x2 - x1, y2 - y1)    
    }
    
    TextureView {
        x: keepOriginalSize ? croppedRect.x : 0
        y: keepOriginalSize ? croppedRect.y : 0
        width: croppedRect.width
        height: croppedRect.height
        source: root.enabled ? root.source : null
        sourceRect: root.croppedRect
    }
}
