import QtQuick 2.3

Item {
    id: root
    property Item source
    property LevelsSettings rLevels: LevelsSettings{ }
    property LevelsSettings gLevels: LevelsSettings{ }
    property LevelsSettings bLevels: LevelsSettings{ }
    property LevelsSettings masterLevels: LevelsSettings{ }

    implicitWidth: enabled && source ? source.width : 0
    implicitHeight: enabled && source ? source.height : 0
    
    ShaderEffect {
        anchors.fill: parent
        property Item source: root.enabled ? root.source : null

        property vector4d inWhiteColor:
            Qt.vector4d(
                root.rLevels.enabled ? root.rLevels.inWhite : 1.0,
                root.gLevels.enabled ? root.gLevels.inWhite : 1.0,
                root.bLevels.enabled ? root.bLevels.inWhite : 1.0,
                root.masterLevels.enabled ? root.masterLevels.inWhite : 1.0)
        property vector4d outWhiteColor:
            Qt.vector4d(
                root.rLevels.enabled ? root.rLevels.outWhite : 1.0,
                root.gLevels.enabled ? root.gLevels.outWhite : 1.0,
                root.bLevels.enabled ? root.bLevels.outWhite : 1.0,
                root.masterLevels.enabled ? root.masterLevels.outWhite : 1.0)

        property vector4d inBlackColor:
            Qt.vector4d(
                root.rLevels.enabled ? root.rLevels.inBlack : 0.0,
                root.gLevels.enabled ? root.gLevels.inBlack : 0.0,
                root.bLevels.enabled ? root.bLevels.inBlack : 0.0,
                root.masterLevels.enabled ? root.masterLevels.inBlack : 0.0)
        property vector4d outBlackColor:
            Qt.vector4d(
                root.rLevels.enabled ? root.rLevels.outBlack : 0.0,
                root.gLevels.enabled ? root.gLevels.outBlack : 0.0,
                root.bLevels.enabled ? root.bLevels.outBlack : 0.0,
                root.masterLevels.enabled ? root.masterLevels.outBlack : 0.0)

        property vector4d gammaColor:
            Qt.vector4d(
                root.rLevels.enabled ? root.rLevels.gammaValue : 1.0,
                root.gLevels.enabled ? root.gLevels.gammaValue : 1.0,
                root.bLevels.enabled ? root.bLevels.gammaValue : 1.0,
                root.masterLevels.enabled ? root.masterLevels.gammaValue : 1.0)

        fragmentShader: 
        "
            varying highp vec2 qt_TexCoord0;
            uniform highp float qt_Opacity;
            uniform lowp sampler2D source;

            uniform highp vec4 inWhiteColor;
            uniform highp vec4 outWhiteColor;
            uniform highp vec4 inBlackColor;
            uniform highp vec4 outBlackColor;
            uniform highp vec4 gammaColor;

            highp vec4 LevelsCorrection(
                highp vec4 inputColor,
                highp vec4 inpBlack,
                highp vec4 inpWhite,
                highp vec4 outBlack,
                highp vec4 outWhite,
                highp vec4 gamma)
            {
                highp vec4 rgbLevels = (inputColor - inpBlack) / (inpWhite - inpBlack);
                highp vec4 rgbLevelsWithGamma = pow(clamp(rgbLevels, vec4(0.0, 0.0, 0.0, 0.0), vec4(1.0, 1.0, 1.0, 1.0)), vec4(1.0, 1.0, 1.0, 1.0)/gamma);
                highp vec4 rgbCorrected = outBlack + rgbLevelsWithGamma * (outWhite - outBlack);
                return rgbCorrected;
            }

            void main()
            {
                lowp  vec4 inpColor = texture2D(source, qt_TexCoord0);
                highp vec4 outColor = LevelsCorrection(inpColor, inBlackColor, inWhiteColor, outBlackColor, outWhiteColor, gammaColor);
                highp vec4 outColor2 = LevelsCorrection(outColor, inBlackColor.aaaa, inWhiteColor.aaaa, outBlackColor.aaaa, outWhiteColor.aaaa, gammaColor.aaaa);
                outColor2.a = inpColor.a;
                gl_FragColor = outColor2 * qt_Opacity;
            }
        "
    }    
}
