import QtQuick 2.0
import Serialization 1.0

QtObject {
    id: root
    property bool enabled: false

    property LevelsSettings rLevels: LevelsSettings {
        onInWhiteChanged: root.levelsColorWhite.r = inWhite
        onInBlackChanged: root.levelsColorBlack.r = inBlack
    }
    property LevelsSettings gLevels: LevelsSettings {
        onInWhiteChanged: root.levelsColorWhite.g = inWhite
        onInBlackChanged: root.levelsColorBlack.g = inBlack
    }
    property LevelsSettings bLevels: LevelsSettings {
        onInWhiteChanged: root.levelsColorWhite.b = inWhite
        onInBlackChanged: root.levelsColorBlack.b = inBlack
    }
    property LevelsSettings masterLevels: LevelsSettings{ }

    property color levelsColorWhite: "white"
    property color previewLevelsColorWhite: "white"
    property color levelsColorBlack: "black"
    property color previewLevelsColorBlack: "black"

    onLevelsColorWhiteChanged: {
        rLevels.inWhite = levelsColorWhite.r
        gLevels.inWhite = levelsColorWhite.g
        bLevels.inWhite = levelsColorWhite.b
    }
    
    onLevelsColorBlackChanged: {
        rLevels.inBlack = levelsColorBlack.r
        gLevels.inBlack = levelsColorBlack.g
        bLevels.inBlack = levelsColorBlack.b
    }
    
    function setDefault() {
        levelsColorWhite = Qt.rgba(1, 1, 1, 1)
        levelsColorBlack = Qt.rgba(0, 0, 0, 1)
        rLevels.setDefault()
        gLevels.setDefault()
        bLevels.setDefault()
        masterLevels.setDefault()
    }
    
    function setPerChannelCorrectionEnabled() {
        enabled = true
        rLevels.enabled = true
        gLevels.enabled = true
        bLevels.enabled = true
    }
    
    function save() {
        return {
            enabled: enabled,
            rLevels: rLevels.save(),
            gLevels: gLevels.save(),
            bLevels: bLevels.save(),
            masterLevels: masterLevels.save()
        }
    }
    function load(data) {
        enabled = data.enabled || false
        if(data.rLevels) rLevels.load(data.rLevels)
        if(data.gLevels) gLevels.load(data.gLevels)
        if(data.bLevels) bLevels.load(data.bLevels)
        if(data.masterLevels) masterLevels.load(data.masterLevels)
    }
}
