import QtQuick 2.3
import QtGraphicalEffects 1.0
import QtQuickUtils 1.0
import Serialization 1.0

Item {
    id: root
    enabled: false
    property Item source
    implicitWidth : enabled && source ? source.width : 0
    implicitHeight: enabled && source ? source.height: 0

    property var  info: []
    property var  pos0: []
    property var  pos: []
    property var  poses: []
    property var  active: false
    property var  clear: true
    property real size: (info.size||0)*height*2/1080
    property real removeSize: size*5
    property var  calibPoint: null
    property var  attached: null
    property var  loadCnt: 0
    property var  loadUrl: ''

    property var  delayFifo: []

    Timer {
        id: timer
        interval: 10
        running: true
        repeat: true
        onTriggered: {  
            if (!delayFifo.length) return
            var time = new Date().getTime()
            while (delayFifo.length && delayFifo[0].time<time) pointer_exec(delayFifo.shift())
        }    
    }

    function pointer(info) {
        //p('pointer '+root+' '+info.cmd+' '+info.attach)
        if (info.cmd=='attach') {
            while (delayFifo.length) delayFifo.shift()
            return setAttached(info.attach)
        }

        if (!info.delay) return pointer_exec(info)
        var m = {}; for (var i in info) m[i] = info[i];
        m.time = new Date().getTime()+info.delay
        delayFifo.push(m)
    }
    
    function pointer_exec(info) {
        root.info = info

        function getPos(v,f) {
            var a = 16/9;
            var w = source.width
            var h = source.height

            var aspectSettings = mixer.outputs[0].input.aspectSettings

            if (aspectSettings.fillMode == Image.Stretch) {
                return {x: v.x*w, y: v.y*h, w: w, h: h, f:f||0 }
            }

            if (aspectSettings.fillMode == Image.PreserveAspectCrop) 
            return (w*aspectSettings.aspect.height<h*aspectSettings.aspect.width) 
            ? {x: w*0.5+(v.x-0.5)*h*a, y: v.y*h, w: v.w*h*a, h: v.h*h, f:f||0 }
            : {y: h*0.5+(v.y-0.5)*w/a, x: v.x*w, h: v.h*w/a, w: v.w*w, f:f||0 }

            return (w<h*a) 
            ? {x: w*0.5+(v.x-0.5)*h*a, y: v.y*h, w: v.w*h*a, h: v.h*h, f:f||0 }
            : {y: h*0.5+(v.y-0.5)*w/a, x: v.x*w, h: v.h*w/a, w: v.w*w, f:f||0 }
        }

        switch (info.cmd) {
            case 'pen': 
                pos = getPos(info)
                active = true;
                enabled = true; 
                break
            case 'move': 
                pos = getPos(info,!active)
                if (!active) pos0 = pos; active = true;
                if (info.mode!='paint' && info.mode!='pen') clear = true
                enabled = true; 
                break
            case 'remove': 
                pos = getPos(info)
                active = true;
                if (info.mode!='paint' && info.mode!='remove') clear = true
                enabled = true; 
                break
            case 'rect': 
                var v = getPos(info);
                poses = [
                    {x:v.x    , y:v.y    , f:1},
                    {x:v.x+v.w, y:v.y    , f:0},
                    {x:v.x+v.w, y:v.y+v.h, f:0},
                    {x:v.x    , y:v.y+v.h, f:0},
                    {x:v.x    , y:v.y    , f:0}]
                active = false;
                clear  = true
                enabled = true; 
                break
            case 'points': 
                var v = []; for (var i in info.points) v.push(getPos(info.points[i],!v.length)) 
                v.push({x:v[0].x,y:v[0].y,f:0}) 
                poses  = v;
                active = false;
                clear  = true
                enabled = true; 
                break
            case 'stop' : if (info.mode=='point') enabled = false; active = false; return
            case 'clear': clear = true; active = false; enabled = false; break
        }
        
        canvas.requestPaint()
    }   

    function setAttached(attach) {
        if (attached) save(attached)
        load(attached=attach)
    }

    function remove() {
        if (!attached) return
        var file = attached.toString()+'.canvas.png';
        if (!Serialization.isFileExists(file)) return
        //p('remove '+file)
        Serialization.removeFile(file)
    }

    function save(url) {
        if (clear) return remove()
        var file = url.toString().replace('file:///','')+'.canvas.png';
        //p('save '+res+' '+file)
        var res = canvas.save(file)
    }

    function load(url) {
        enabled = true; clear = false; active = false
        loadUrl = url.toString()+'.canvas.png?r='+(++loadCnt);
        //p('load '+enabled+' '+loadUrl)
        canvas.loadImage(loadUrl)
    }

    TextureView {
        id: textureView
        anchors.fill: parent
        source: root.source
    }

    Canvas {
        id: canvas
        width:  root.width
        height: root.height

        property var m0;

        onImageLoaded: {
            var g = getContext("2d");
            g.reset();
            g.drawImage(loadUrl,0,0);
            canvas.requestPaint()
        }

        onPaint: {
            var g = getContext("2d");
            
            if (clear) {
                clear = false
                g.reset();
                if (active) return                
            }

            g.lineWidth = size/3
            g.lineCap = 'round'
            g.globalAlpha = 1
            g.shadowBlur = 1
            g.shadowColor = info.color

            if (info.cmd=='remove') {
                for( var y=-removeSize; y<removeSize; y++) {
                    var x = Math.sqrt(removeSize*removeSize-y*y);
                    g.clearRect(pos.x-x,pos.y+y, 2*x, 1);
                }
                return
            }        

            if (info.cmd=='move') {
                g.beginPath(); 
                g.strokeStyle = info.color
                g.moveTo(pos0.x,pos0.y); 
                g.lineTo(pos.x,pos.y);
                g.stroke();
                pos0 = pos
                return
            }        

            while (poses.length) {
                var m = poses.shift()
                if (m.f) m0 = m; else { 
                    g.beginPath(); 
                    g.strokeStyle = info.color
                    g.moveTo(m0.x,m0.y); 
                    g.lineTo(m.x,m.y); 
                    g.stroke();
                    m0 = m
                }
            }        
        }
    }

    property bool calibActive: info.mode=='calib' && textureView.width && textureView.height

    Rectangle {
        id: rect
        visible: calibActive
        x:0
        y:0
        width:40
        height:40
        radius:40
        opacity: 0.75
        border.width:12
        border.color:'red'
        color: 'transparent'
    }

    Timer {
        interval:20
        running: calibActive
        repeat: true
        onTriggered: {
            var w = textureView.width
            var h = textureView.height
            var point = ColorPicker.pickColor(textureView,Qt.rgba(0.4,0,0,0),Qt.rect(0,0,w,h))
            calibPoint = Qt.vector2d(point.x/w,point.y/h)
            rect.x = point.x-rect.width/2;
            rect.y = point.y-rect.height/2;
            //p('point '+w+'x'+h+' '+point)
        }    
    }

    Rectangle  {
        visible: active && info.cmd=='remove'
        color: "transparent"
        border.width: size/4
        border.color: "white"
        radius: removeSize
        width:  removeSize*2
        height: removeSize*2
        x: pos.x-width /2
        y: pos.y-height/2
    }   
    
    RectangularGlow  {
        visible: active && info.cmd!='remove'
        cached: true
        spread: 0.5
        color: info.color || ''
        x: pos.x-width /2
        y: pos.y-height/2
        glowRadius: size
    }   
}
