import QtQuick 2.0
import QtQuickUtils 1.0
import QtGraphicalEffects 1.0

Item {
    id: root
    property CropSettings cropSettings
    property AspectSettings aspectSettings
    property TransformSettings transformSettings
    property Item source
    
    readonly property size sourceSize: Qt.size(getw(source),geth(source))
    readonly property real angle: transformSettings.rotation * Math.PI / 180

    readonly property bool keepOriginalSize: true//cropSettings && cropSettings.enabled ? cropSettings.keepOriginalSize : true
    readonly property bool showWorkingArea : cropSettings && cropSettings.enabled ? cropSettings.showWorkingArea : false
    readonly property real feather         : cropSettings && cropSettings.enabled ? cropSettings.feather*Math.min(getw(sourceSize),geth(sourceSize))/400 : 0

    readonly property var sourceRect: Qt.rect(0,0,getw(rotate(aspectSize)),geth(rotate(aspectSize)))
    implicitWidth : getw(keepOriginalSize ? sourceRect : croppedRect)
    implicitHeight: geth(keepOriginalSize ? sourceRect : croppedRect)


//  onHeightChanged: p('onHeightChanged '+Math.round(width)+'x'+Math.round(height))

    function rotate(size) {
        if (!transformSettings.rotation || !size) return size

        var cos = Math.cos(angle)
        var sin = Math.sin(angle)
        function rot(x,y) { return Qt.point(x*cos-y*sin, x*sin+y*cos) }

        var w = getw(size), h = geth(size), x1 = 0, x2 = 0, y1 = 0, y2 = 0;
        var p = [Qt.point(0,0), rot(0,h), rot(w,h), rot(w,0)]
        for (var i in p) { 
            var x = p[i].x; if (x<x1) x1=x; if (x>x2) x2 = x; 
            var y = p[i].y; if (y<y1) y1=y; if (y>y2) y2 = y; 
        }

        return Qt.size(x2-x1, y2-y1)
    }

    function getw(size) { return !size ? 0 : size.width  }
    function geth(size) { return !size ? 0 : size.height }
   
    readonly property size aspectSize: {
        var w = enabled ? getw(source) : 0
        var h = enabled ? geth(source) : 0
        if(w <= 0 || h <= 0) return Qt.size(0, 0)
        if(!aspectSettings || aspectSettings.aspectType == 0) return Qt.size(w, h)
        var aw = getw(aspectSettings.aspect)
        var ah = geth(aspectSettings.aspect)
        var scale = Math.max(w/aw, h/ah)
        var result = Qt.size(aw * scale, ah * scale)
        var maxW = Math.min(w * 3, 8000)
        var maxH = Math.min(h * 3, 8000)
        if(result.width  > maxW) result = Qt.size(maxW, result.height / result.width * maxW)
        if(result.height > maxH) result = Qt.size(result.width / result.height * maxH, maxH)
        return result
    }
    
    readonly property rect croppedRect: crop(getw(sourceRect), geth(sourceRect))
    readonly property rect croppedSource: crop(getw(sourceSize), geth(sourceSize))
    
    function crop(w, h, r) {
        if(!cropSettings || !cropSettings.enabled) return Qt.rect(0, 0, w, h)
        var x1 = w * (cropSettings.left * 0.01)
        var x2 = w - w * (cropSettings.right * 0.01)
        var y1 = h * (cropSettings.top * 0.01)
        var y2 = h - h * (cropSettings.bottom * 0.01)
        return Qt.rect(x1, y1, x2 - x1, y2 - y1)
    }

    ShaderEffect {
        id: textureView
        property variant source: ShaderEffectSource {
            sourceItem: root.enabled ? root.source : null
        }
        property int dx: 1||keepOriginalSize ? croppedSource.x : 0
        property int dy: 1||keepOriginalSize ? croppedSource.y : 0

        property variant mask: ShaderEffectSource {
            sourceItem: Item {
                implicitWidth:  getw(sourceSize)
                implicitHeight: geth(sourceSize)

                RectangularGlow  {
                    cached: true
                    spread: 0.25
                    color: "#00ff00"
                    x: textureView.dx+feather
                    y: textureView.dy+feather
                    width:  croppedSource.width -2*feather
                    height: croppedSource.height-1*feather
                    glowRadius: feather
                }   

                Rectangle {
                    visible: root.showWorkingArea
                    x: textureView.dx
                    y: textureView.dy
                    width:  croppedSource.width
                    height: croppedSource.height
                    color: 'transparent'
                    border.color: '#ff0000'
                    border.width: 3
                    radius: 10
                }
            }   
        }

//        anchors.fill: parent
        width : getw(root.source)
        height: geth(root.source)
        smooth: true
        
        transform: [
            Translate { 
                x: -getw(sourceSize)/2 
                y: -geth(sourceSize)/2 
            },
            Scale { 
                origin.x: 0 
                origin.y: 0
                xScale: transformSettings.horizontalFlip ? -1:1
                yScale: transformSettings.verticalFlip   ? -1:1
            },
            Rotation {
                origin.y: 0
                origin.x: 0
                angle: transformSettings.rotation
            },
            Translate { 
                x: width /2
                y: height/2
            }
        ]            

        fragmentShader: "
            uniform lowp sampler2D source;
            uniform lowp sampler2D mask;
            varying highp vec2 qt_TexCoord0;
            void main() {
                lowp vec4 m = texture2D(mask, qt_TexCoord0);
                gl_FragColor = m.r>0.0 ? vec4(1,0,0,1) : texture2D(source, qt_TexCoord0) * m.g;
            }
        "
    }
}

