import QtQuick 2.1
import QtQuick.Controls 1.1
import QtQuick.Controls.Styles 1.0
import QtQuick.Layouts 1.1
import MixerGuiControls 1.0
import "AudioUtils.js" as AudioUtils

Item {
    id: root
    property var level: 0
    property real peakLevel: 0
    property real volume
    property alias pressed: slider.pressed
    property alias orientation: slider.orientation
    property bool handleVisible: true
    property bool indicatorEnabled: true    
    property bool peakVisible: false
    implicitWidth: orientation == Qt.Vertical ? slider.implicitWidth : 150
    implicitHeight: orientation == Qt.Vertical ? 150  : slider.implicitHeight
    
    readonly property real dbLinearEnd: -24

    Binding on volume {
        value: AudioUtils.scaleToLinear(slider.value,root.dbLinearEnd)
        when: slider.pressed
    }

    Slider {
        id: slider
        orientation: Qt.Vertical
        anchors.fill: parent
        enabled: root.handleVisible
        activeFocusOnTab: false
        minimumValue: 0
        maximumValue: AudioUtils.dbToScale(12, root.dbLinearEnd)
        Binding on value {
            value: AudioUtils.linearToScale(root.volume, root.dbLinearEnd)
            when: !slider.pressed
        }
        readonly property real pixelSize: orientation == Qt.Vertical ? slider.height : slider.width
        onValueChanged: if (slider.pressed && Math.abs(value - 1.0) < 5 * (maximumValue - minimumValue) / pixelSize) value = 1.0
        style: SliderStyle {
            groove: Item {
                id: grooveItem
                implicitHeight: background.implicitHeight
                StyledImage {
                    id: background
                    anchors.fill: parent
                    desc: MixerThemeManager.theme.imgAudioLevelSmallBgnd
                }
                Rectangle {
                    id: boundRect
                    anchors.fill: parent
                    anchors.topMargin: 4
                    anchors.bottomMargin: 3
                    anchors.leftMargin: 2
                    anchors.rightMargin: 2
                    radius: 3
                    antialiasing: false
                    clip: true
                    color: "transparent"
                    visible: root.peakVisible
                    function getColor(scale) {
                        if (AudioUtils.linearToDb(scale) > -1) return "red"
                        else if (AudioUtils.linearToDb(scale) > -12) return "#fafa6e"
                        else if (AudioUtils.linearToDb(scale) > -24) return "#adc049"
                        else return "#50641e"
                    }
                    function getPosition(peakLevel) {
                        return Math.min(width, width * Math.min(AudioUtils.linearToScale(peakLevel, root.dbLinearEnd), slider.maximumValue) / slider.maximumValue) - 2
                    }
                    Rectangle {
                        id: peakRect
                        width: 3
                        height: parent.height
                        color: parent.getColor(Math.max(peakLevel, level))
                        x: parent.getPosition(Math.max(peakLevel, level))
                    }
                }
                StyledImage {
                    id: indicator
                    visible: level > 0.0001
                    width: implicitWidth * Math.min(AudioUtils.linearToScale(level, root.dbLinearEnd), slider.maximumValue) / slider.maximumValue
                    height: parent.height
                    transform: Scale {
                        xScale: parent.width / indicator.implicitWidth
                    }
                    desc: root.indicatorEnabled ? MixerThemeManager.theme.imgAudioLevelSmallProgress : MixerThemeManager.theme.imgAudioLevelSmallProgressDisabled
                }
                StyledImage {
                    anchors.fill: parent
                    desc: MixerThemeManager.theme.imgAudioLevelSmallFront
                }
            }
            
            handle: StyledImage {
                visible: root.handleVisible
                desc: (control.hovered || control.pressed) ? MixerThemeManager.theme.imgSliderThumbSmallHovered : MixerThemeManager.theme.imgSliderThumbSmall
                mirror: slider.orientation == Qt.Vertical                
            }
        }
        BlockMouseWheel {
            anchors.fill: parent
        }
            
        Component.onCompleted: {        // обход бага, что у рукоятки перепутана ширина и высота в вертикальном положении
            var tmp = __panel.handleWidth
            __panel.handleWidth = __panel.handleHeight
            __panel.handleHeight = tmp
        }        
    }
    
}
