import QtQuick 2.0
import QtQuick.Controls 1.0
import MixerGuiControls 1.0
import "AudioUtils.js" as AudioUtils

Item {
    id: root
    implicitWidth: numbersRepeater.count > 0 ? numbersRepeater.itemAt(0).width : 0
    implicitHeight: 100

    property var scales: [-50, -25, -17.5, -12.5, -7.5, -2.5]
    property var numbers: [-70, -40, -30, -20, -15, -10, -5, 0]

    property int fontSize: MixerThemeManager.theme.promptFontSize
    property string fontFamily: MixerThemeManager.theme.fontFamily
    property bool  boldFont: false
    property color numbersColor: MixerThemeManager.theme.colNeutral

    property bool showSliderTicks: true
    property int  align: Qt.AlignRight

    property real dbLinearEnd: -20
    property real maximumValue: 1.0//AudioUtils.dbToScale(12, dbLinearEnd)

    function dbToPosition(value) {
        return (1 - Math.min(AudioUtils.dbToScale(value, dbLinearEnd), maximumValue) / maximumValue) * height
    }

    Repeater {
        model: scales
        StyledImage {
            desc: MixerThemeManager.theme.imgSliderScaleTick
            anchors.right: align==Qt.AlignRight? parent.right : undefined
            anchors.left:  align==Qt.AlignLeft?  parent.left  : undefined
            y: root.dbToPosition(modelData) - height * 0.5
        }
    }
    Repeater {
        id: numbersRepeater
        model: numbers
        Row {
            anchors.right: align==Qt.AlignRight? parent.right : undefined
            anchors.left:  align==Qt.AlignLeft?  parent.left  : undefined
            y: root.dbToPosition(modelData) - height * 0.5
            spacing: MixerThemeManager.theme.itemSpacing/2
            MixerLabel {
                text: modelData
                font.pixelSize: fontSize
                font.family: fontFamily
                font.bold: boldFont
                color: numbersColor
            }
            StyledImage {
                desc: MixerThemeManager.theme.imgSliderScaleTick
                anchors.verticalCenter: parent.verticalCenter
                visible: showSliderTicks
            }
        }
    }
}
