import QtQuick 2.0
import QtQuick.Controls 1.0
import QtQuick.Controls.Styles 1.0
import MixerGuiControls 1.0
import "AudioUtils.js" as AudioUtils

Item {
    id: root
    property real value
    property alias pressed: slider.pressed
    implicitWidth: slider.implicitWidth
    implicitHeight: slider.implicitHeight

    property real dbLinearEnd: -20
    property real maximumValue: AudioUtils.dbToScale(12, root.dbLinearEnd)

    Binding on value {
        value: AudioUtils.scaleToLinear(slider.value, root.dbLinearEnd)
        when: slider.pressed
    }

    Slider {
        id: slider
        anchors.fill: parent
        anchors.topMargin: 0
        anchors.bottomMargin: 0
        orientation: Qt.Vertical
        activeFocusOnTab: false
        minimumValue: 0
        maximumValue: root.maximumValue
        Binding on value {
            value: AudioUtils.linearToScale(root.value, root.dbLinearEnd)
            when: !slider.pressed
        }
        onValueChanged: {
            if(pressed && Math.abs(value - 1.0) < 5 * (maximumValue - minimumValue) / height) value = 1.0
        }
        style: SliderStyle {
            groove: Item {
                implicitWidth: grooveImage.implicitWidth
                implicitHeight: grooveImage.implicitHeight
                StyledImage {
                    id: grooveImage
                    anchors.fill: parent
                    anchors.leftMargin: 0
                    anchors.rightMargin: 0
                    desc: MixerThemeManager.theme.imgSliderGroove
                }
            }
            handle: StyledImage {
                desc: (control.hovered || control.pressed || control.activeFocus) ? MixerThemeManager.theme.imgSliderThumbSmallHovered : MixerThemeManager.theme.imgSliderThumbSmall
            }
        }
    }
    BlockMouseWheel {
        anchors.fill: slider
    }
}
