import QtQuick 2.0
import MixerGuiControls 1.0

Item {
    id: root
    implicitWidth:  hasBgnd ? background.implicitWidth : hoursAndMinutes.implicitWidth
    implicitHeight: hasBgnd ? background.implicitHeight: (twoRows ? (MixerThemeManager.theme.itemSpacing + hoursAndMinutes.implicitHeight*2) : hoursAndMinutes.implicitHeight)
    property date date: new Date()
    property var locale: Qt.locale("en_US")
    property color  textColor: MixerThemeManager.theme.colCommon
    property int    textSize: hasBgnd ? background.implicitHeight/3 : MixerThemeManager.theme.headerFontSize
    property string textFont: MixerThemeManager.theme.fontFamily
    property bool  hasBgnd: true
    property bool  showAP:  true
    property bool  twoRows:  true
    property bool  boldText: false
    property bool  showCurTime: true
    property alias timeText: hoursAndMinutes.text

    Timer {
        running: showCurTime
        interval: 100
        repeat: true
        onTriggered: root.date = new Date()
    }
    StyledImage {
        id: background
        anchors.fill: parent
        desc: MixerThemeManager.theme.imgClockDigitalBgnd
        visible: hasBgnd
    }
    Item {
        anchors.fill: parent
        anchors.margins: 6
        Text {
            id: hoursAndMinutes
            text: {
                var format = twoRows ? "hh:mm" : "hh:mm:ss"
                if (showAP)
                    format += " a"
                var res = date.toLocaleTimeString(locale, format);
                if (twoRows && showAP)
                    res = res.substring(0, twoRows ? 5 : 8)
                return res
            }
            font.pixelSize: textSize
            font.family: textFont
            font.bold: boldText
            color: textColor
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: twoRows ? undefined : parent.verticalCenter
            anchors.top: twoRows ? parent.top : undefined
        }
        Text {
            visible: twoRows
            text: root.date.toLocaleTimeString(locale, "ss")
            font.pixelSize: textSize
            font.family: textFont
            font.bold: boldText
            color: textColor
            anchors.left: showAP ? hoursAndMinutes.left : undefined
            anchors.horizontalCenter: showAP ? undefined : parent.horizontalCenter
            anchors.bottom: parent.bottom
        }    
        Text {
            visible: twoRows
            text: root.date.toLocaleTimeString(locale, "AP")
            font.pixelSize: textSize*0.62
            font.family: textFont
            font.bold: boldText
            color: textColor
            anchors.right: hoursAndMinutes.right
            anchors.bottom: parent.bottom
            anchors.bottomMargin: 1
        }        
    }
    StyledImage {
        anchors.fill: parent
        desc: MixerThemeManager.theme.imgClockDigitalFront
        visible: hasBgnd
    }
}
