import QtQuick 2.1
import QtQuick.Controls 1.1
import QtQuick.Controls.Private 1.0
import QtQuick.Controls.Styles 1.2
import QtGraphicalEffects 1.0
import MixerGuiControls 1.0

Button {
    id: root
    property string colorType: "common"    // "neutral", "common", "preset", "program"
    property string colorTypeUnpressed: ""
    property string fixedButColor: "" //"pressed", "unpressed"
    property string fixedTextColor: "" //"pressed", "unpressed"
    property var  contextMenu
    property var dragAndDropKeys: null
    property real iconRotation: 0
    property bool boldText: false
    property string icon: ""
    property bool dimmed: false
    property bool enableLongClick: false
    iconSource: icon!="" ? MixerThemeManager.theme.iconFolder + icon + ".png" : ""

    activeFocusOnTab: false

    signal contextMenuPopup(var item, real mouseX, real mouseY)
    signal dragAndDrop(var dragData)

    readonly property bool square: width == height
    readonly property var theme: MixerThemeManager.theme
    readonly property int defaultSize: theme.buttonSize
    readonly property int oneAndHalfSize: theme.oneAndHalfButtonSize
    readonly property int doubleSize: theme.doubleButtonSize

    function getColByType(type) {
        if(type=="preset")  return root.square ? theme.imgButtonSquarePreview : theme.imgButtonRectPreview
        if(type=="program") return root.square ? theme.imgButtonSquareProgram : theme.imgButtonRectProgram
        if(type=="common")  return root.square ? theme.imgButtonSquareCommon  : theme.imgButtonRectCommon
        return root.square ? theme.imgButtonSquareNeutral : theme.imgButtonRectNeutral
    }
    function getCurColorType(control) {
        if ((fixedButColor=="" && (control.pressed || control.checked)) || fixedButColor=="pressed")
            return root.colorType
        else if (colorTypeUnpressed != "")
           return root.colorTypeUnpressed
        return "neutral"
    }

    style: ButtonStyle {
        background: Item {
            implicitWidth: root.defaultSize
            implicitHeight: root.defaultSize
            readonly property bool hasIcon: control.iconSource != ""

            Item {
                id: backgroundItem
                anchors.fill: parent
                visible: true//!parent.hasIcon
                anchors.leftMargin: -buttonImage.anchors.leftMargin
                anchors.topMargin: -buttonImage.anchors.topMargin
                anchors.rightMargin: -buttonImage.anchors.rightMargin
                anchors.bottomMargin: -buttonImage.anchors.bottomMargin

                StyledImage {
                    id: buttonImage
                    anchors.fill: parent
                    anchors.leftMargin: desc.margins[0]
                    anchors.topMargin: desc.margins[1]
                    anchors.rightMargin: desc.margins[2]
                    anchors.bottomMargin: desc.margins[3]
                    desc: {
                        if (!control.enabled || control.dimmed)
                            return root.square ? theme.imgButtonSquareDisabled : theme.imgButtonRectDisabled
                        var curType = getCurColorType(control)
                        return getColByType(curType)
                    }
                }
            }
            Item {
                id: iconItem
                visible: false
                width: backgroundItem.width
                height: backgroundItem.height
                Item {
                    anchors.fill: parent
                    anchors.leftMargin: buttonImage.anchors.leftMargin
                    anchors.topMargin: buttonImage.anchors.topMargin
                    anchors.rightMargin: buttonImage.anchors.rightMargin
                    anchors.bottomMargin: buttonImage.anchors.bottomMargin
                    Image {
                        source: control.iconSource
                        anchors.centerIn: parent
                        width: 27
                        height: 27
                        rotation: root.iconRotation
                    }
                }
            }
            Blend {
                anchors.fill: parent
                anchors.leftMargin: -buttonImage.anchors.leftMargin
                anchors.topMargin: -buttonImage.anchors.topMargin
                anchors.rightMargin: -buttonImage.anchors.rightMargin
                anchors.bottomMargin: -buttonImage.anchors.bottomMargin
                visible: parent.hasIcon
                cached: true
                source: backgroundItem
                foregroundSource: iconItem
                mode: (control.pressed || control.checked) ? "multiply" : theme.buttonIconBlend
                opacity: (!enabled || dimmed)?  theme.colIconDisabled.a : 1.0
            }
            Rectangle {
                visible: control.hovered && !control.dimmed
                anchors.fill: parent
                color: "transparent"
                border.color: {
                    var cT = colorType
                    if (colorTypeUnpressed!="" && !(control.pressed || control.checked))
                        cT = colorTypeUnpressed;
                    theme.colText[cT] || theme.colCommon
                }
                border.width: 1
                opacity: 0.7
                radius: 2
            }

        }

        label: Item {
            width: control.width
            height: control.height
            anchors.centerIn: parent
            Text {
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                anchors.fill: parent
                elide: Text.ElideRight
                text: control.text
                wrapMode: Text.Wrap
                maximumLineCount: 2
                color: {
                    if(!control.enabled || control.dimmed)
                        return theme.colButtonTextDisabled
                    if ((fixedTextColor=="" && (control.pressed || control.checked)) || fixedTextColor=="pressed")
                       return theme.colPressedButtonText[root.colorType]
                    return theme.colButtonText[root.colorType]
                }
                font.family: theme.fontFamily
                font.pixelSize: {
                    var numbers = /^[0-9]+$/;
                    if ((text.length <= 2) && text.match(numbers)) //if(text.length == 1)
                       return theme.buttonDigitsFontSize
                    else if(textSizeCalculator.implicitWidth < width-4)
                        return theme.buttonTextFontSize
                    else if(textSizeCalculator2.implicitWidth < width-4)
                        return (theme.buttonTextFontSize + theme.systemFontSize)/2
                    else
                        return theme.systemFontSize
                }
                font.bold: boldText
                Text {
                    id: textSizeCalculator
                    visible: false
                    text: control.text
                    font.family: theme.fontFamily
                    font.pixelSize: theme.buttonTextFontSize
                }
                Text {
                    id: textSizeCalculator2
                    visible: false
                    text: control.text
                    font.family: theme.fontFamily
                    font.pixelSize: (theme.buttonTextFontSize + theme.systemFontSize)/2
                }
            }
        }


    }

    DropArea {
        id: dropArea
        anchors.fill: parent
        enabled: !!root.dragAndDropKeys
        keys: root.dragAndDropKeys ? root.dragAndDropKeys : []
        onDropped: root.dragAndDrop(drag.source.dragData)
    }
    Rectangle {
        anchors.fill: parent
        color: "white"
        opacity: 0.2
        visible: dropArea.containsDrag
    }

    Keys.onPressed: {
        if (event.key === Qt.Key_Space && !event.isAutoRepeat && !__behavior.pressed)
            __behavior.keyPressed = true;
    }

    onFocusChanged: if (!focus) __behavior.keyPressed = false

    Keys.onReleased: {
        if (event.key === Qt.Key_Space && !event.isAutoRepeat && __behavior.keyPressed) {
            __behavior.keyPressed = false;
            __action.trigger(button)
            __behavior.toggle()
        }
    }

    MouseArea {
        id: behavior
        property bool keyPressed: false
        property bool effectivePressed: pressed && containsMouse || keyPressed

        anchors.fill: parent
        enabled: !keyPressed

        function toggle() {
            if (root.checkable && !root.action && !(root.checked && root.exclusiveGroup))
                root.checked = !root.checked
        }
        onReleased: {
            if (containsMouse) {
                toggle()
                __action.trigger(root)
            }
        }
        onExited: Tooltip.hideText()
        onCanceled: Tooltip.hideText()
        onPressed: {
            if (activeFocusOnPress)
                root.forceActiveFocus()
        }
        onPressAndHold: if (enableLongClick && contextMenu) {
            contextMenu.popup(this, mouseX, mouseY)
            root.contextMenuPopup(this, mouseX, mouseY)
        }

        Timer {
            interval: 1000
            running: behavior.containsMouse && !pressed && tooltip.length && behavior.Window && behavior.Window.visibility !== Window.Hidden || false
            onTriggered: Tooltip.showText(behavior, Qt.point(behavior.mouseX, behavior.mouseY), tooltip)
        }
    }

    __behavior: behavior
    __effectivePressed: behavior.effectivePressed

    MouseArea {
        anchors.fill: parent
        acceptedButtons: Qt.RightButton
        onClicked: if(contextMenu) {
            contextMenu.popup(this, mouseX, mouseY)
            root.contextMenuPopup(this, mouseX, mouseY)
        }
    }

    z: (pressed || checked || !enabled || dimmed) ? 0 : 1
}
