import QtQuick 2.2
import QtQuick.Controls 1.2 as QC1
import QtQuick.Controls 2.4
import QtQuick.Controls.impl 2.4
import QtQuick.Window 2.11
import GuiControls 1.0
import QtQuick.Controls.Styles 1.0
import "."


ComboBox {
    id: control
    readonly property var theme: MixerThemeManager.theme
    property int menuPadding: theme.controlPadding
    property int menuBorder: 0
    property alias color: tf.textColor
    property bool autoArrow: true

    implicitHeight: Math.max(background ? background.implicitHeight : 0,
                             Math.max(contentItem.implicitHeight,
                                      indicator ? indicator.implicitHeight : 0) + topPadding + bottomPadding)
    baselineOffset: 0//contentItem.y + contentItem.baselineOffset


    font.family:    theme.fontFamily
    font.pixelSize: theme.menuFontSize

    onAccepted:  focus = false
    onActivated: focus = false

    background: Rectangle {
        color: theme.colNeutralDark
        border.color: theme.colNeutral
        border.width: 1
    }

    indicator: MixerIcon {
        anchors.verticalCenter: control.verticalCenter
        x: control.width - width - control.padding
        source: !autoArrow ? "icons/ArrowCombo.png" : (control.down ? "icons/ArrowUp.png" : "icons/ArrowDown.png")
        noMouseArea: false

        color: {
            if (!control.enabled) return theme.colTextDisabled
            if (control.pressed)  return theme.colCommon
            if (control.hovered)  return theme.colNeutralLight
            return theme.colNeutral
        }
        MouseArea {
            anchors.fill: parent
            onClicked: control.down ? popup.close() : popup.open()
        }
    }

    contentItem: QC1.TextField {
        id: tf
        text: control.editable ? control.editText : control.displayText
        enabled: control.enabled
        readOnly: control.down || !control.editable
        inputMethodHints: control.inputMethodHints
        validator: control.validator
        font: control.font
        verticalAlignment: Text.AlignVCenter
        onTextChanged: control.editText = text
        textColor: theme.colNeutral
        MouseArea {
            anchors.fill: parent
            visible: !editable || control.down
            onClicked: control.down ? popup.close() : popup.open() 
        }

        onAccepted: control.accepted()
    }

    delegate: ItemDelegate  {
        width: control.width
        padding: 0
        implicitHeight: theme.menuHeight
        //color: control.highlightedIndex === index ?  MixerThemeManager.theme.colCommon : MixerThemeManager.theme.colNeutral
        text: control.textRole ? (Array.isArray(control.model) ? modelData[control.textRole] : model[control.textRole]) : modelData
        contentItem: Text {
            text: parent.text
            font: control.font
            elide: Text.ElideRight
            verticalAlignment: Text.AlignVCenter
            color: highlighted ? theme.colCommon : theme.colNeutral
        }
        background: null
        highlighted: control.highlightedIndex === index
        hoverEnabled: control.hoverEnabled
    }


    popup: Popup {
        x: control.x //- menuPadding
        y: control.height //+ menuPadding
        width: control.width //+ menuPadding
        function min(v,h) { return v<h ? v:h*2 }
        implicitHeight: min(contentItem.implicitHeight, control.Window.height - topMargin - bottomMargin - control.height)
        leftPadding: menuPadding
        topPadding : 0
        bottomPadding: topPadding

        background: Rectangle {
            color: theme.colNeutralDark
            border.color: theme.colNeutral
            border.width: menuBorder
        }

        contentItem: ListView {
            clip: true
            implicitHeight: contentHeight
            spacing: 0
            model: control.delegateModel
            currentIndex: control.highlightedIndex
            ScrollIndicator.vertical: ScrollIndicator {  }
        }

        onClosed: if (control.editable) control.editText = control.currentText
     }
}
