import QtQuick 2.1
import QtQuick.Layouts 1.1
import MixerGuiControls 1.0

MixerWindow {
    id: root
    resizeEnabled: false
    modality: Qt.ApplicationModal
    default property Item content: null
    property var buttons: []
    property bool autoHide: true
    signal buttonClicked(string button, int buttonIndex)
    signal opened()
    signal contentReceivedFocus()

    ColumnLayout {
        Keys.onEscapePressed: root.close()
        anchors.fill: parent
        spacing: MixerThemeManager.theme.itemSpacing
        FocusScope {
            id: contentItem
            Layout.fillWidth: true
            data: content
            implicitWidth: content ? content.implicitWidth : 0
            implicitHeight: content ? content.implicitHeight : 0
            KeyNavigation.down: buttonRepeater.count > 0 ? buttonRepeater.itemAt(0) : null
            onActiveFocusChanged: if(activeFocus) root.contentReceivedFocus()
        }
        RowLayout {
            Layout.alignment: Qt.AlignHCenter
            spacing: MixerThemeManager.theme.dialogButtonHSpacing
            Repeater {
                id: buttonRepeater
                model: root.buttons
                DialogButton {
                    action: modelData.hasOwnProperty("shortcut") ? modelData : null
                    text: action ? action.text : modelData
                    KeyNavigation.right: model.index < buttonRepeater.count - 1 ? buttonRepeater.itemAt(model.index + 1) : null
                    KeyNavigation.left: model.index > 0 ? buttonRepeater.itemAt(model.index - 1) : null
                    KeyNavigation.up: contentItem
                    onClicked: {
                        root.buttonClicked(action ? action.text : modelData, model.index)
                        if(root.autoHide) root.hide()    
                    }
                }
            }
        }
    }
    
    function open() {
        show()
        width = Math.max(350, implicitWidth)
        height = implicitHeight
        if(buttonRepeater.count > 0) buttonRepeater.itemAt(0).forceActiveFocus()
        raise()
        requestActivate()
        opened()
    }
}