import QtQuick 2.1
import ".."

MixerTheme {
    themeID: "allmix_flat"
    name: qsTr("Dark Flat (Gray\Yellow)")

// Font and sizes
    fontFamily: "Tahoma"
    headerFontSize: 18
    footerFontSize: headerFontSize
    buttonTextFontSize: 18
    buttonDigitsFontSize: 24
    menuFontSize: 14
    systemFontSize: menuFontSize
    promptFontSize: 12

    panelSpacing: 8
    itemSpacing: 8
    inputsSpacingH: 26
    inputsSpacingV: 36

    buttonSize: 43
    oneAndHalfButtonSize: 58
    doubleButtonSize: 87
    iconSize: 25
    headerHeight: 28
    footerHeight: 28
    tabGroupHeight: 28
    menuHeight: 24
    statusHeight: 24
    mediaBarHeight: 34
    higlightFrameWidth: 3
    textOutlineSize: 1
    textGlowSize: 3

    propertyHeaderHeight: headerHeight
    propertyHeaderSpacing: 1
    propertyHeaderOffsetX: 14
    propertyHeaderItemSpacing: 7
    propertyListMargins: [14, 7, 14, 7]
    propertyListStep: 28
    propertyListFontSize: systemFontSize
    propertyListColumnRatio: 0.38

    dialogButtonWidth:	dialogButtonHeight * 3
    dialogButtonHeight: headerHeight
    dialogButtonHSpacing: propertyHeaderSpacing
    dialogButtonVSpacing: itemSpacing
    dialogHeaderHeight: 36

    toolbarHeaderHeight: 36
    toolbarButtonSize:	29
    toolbarButtonSpacing: 4
    toolbarGroupSpaсing: 14

    scrollBarSpacing: 1

// Colors
    colText: {
        "neutral": colNeutral,
        "preset": colPreset,
        "program": colProgram,
        "common" : colCommon
    }
    colTextHighlight: {
        "neutral": colNeutralLight,
        "preset": colPresetLight,
        "program": colProgramLight,
        "common" : colCommonLight
    }
    colTextOutline: {
        "neutral": colNeutralDark,
        "preset": colPresetDark,
        "program": colProgramDark,
        "common" : colCommonDark
    }
    colTextGlow: {
        "neutral": Qt.rgba(colNeutralLight.r, colNeutralLight.g, colNeutralLight.b, 0.4),
        "preset": Qt.rgba(colPresetLight.r, colPresetLight.g, colPresetLight.b, 0.4),
        "program": Qt.rgba(colProgramLight.r, colProgramLight.g, colProgramLight.b, 0.4),
        "common": Qt.rgba(colCommonLight.r, colCommonLight.g, colCommonLight.b, 0.4)
    }

    colTextInactive:	Qt.rgba(colNeutral.r, colNeutral.g, colNeutral.b, 0.6)
    colTextDisabled:	Qt.rgba(colNeutral.r, colNeutral.g, colNeutral.b, 0.4)

    colIcon: {
        "neutral": colText["neutral"],
        "preset": colText["preset"],
        "program": colText["program"],
        "common": colText["common"]
    }
    colIconHighlight: {
        "neutral": colNeutralLight,
        "preset": colPresetLight,
        "program": colProgramLight,
        "common": colCommonLight
    }
    colIconOutline: {
        "neutral": colTextOutline["neutral"],
        "preset": colTextOutline["preset"],
        "program": colTextOutline["program"],
        "common": colTextOutline["common"]
    }
    colIconGlow: {
        "neutral": colTextGlow["neutral"],
        "preset": colTextGlow["preset"],
        "program": colTextGlow["program"],
        "common": colTextGlow["common"]
    }

    colSelected: {
        "neutral": colText["neutral"],
        "preset": colText["preset"],
        "program": colText["program"],
        "common": colText["common"]
    }

    colIconInactive:	colTextInactive
    colIconDisabled:	colTextDisabled

    colButtonText: { 
        "neutral": colNeutralDark,
        "common":  colNeutralDark,
        "preset":  colNeutralDark,
        "program": colNeutralDark
		}

    colButtonTextDisabled: Qt.rgba(colNeutralBlack.r, colNeutralBlack.g, colNeutralBlack.b, 1)

    colButtonTextGlow: {
        "neutral": colNeutralLight,
        "common": colCommonLight,
        "preset": colPresetLight,
        "program": colProgramLight
    }
    colPressedButtonText: {
        "neutral": colNeutralBlack,
        "common":  colCommonBlack,
        "preset":  colPresetBlack,
        "program": colProgramBlack
    }

// Images
    //iconFolder: "icons/"

    imgAppBgnd: StyledImageDesc {
        //source: "Resources/AppBgnd.png"
        colFill: colNeutralBlack
    }
    imgPanelBgndFull: StyledImageDesc {
        source: "Resources/GroupBgndFull.png"
        borders: [4, 4, 4, 4]
    }
    imgPanelBgndHalfTop: imgPanelBgndFull
    imgPanelBgndHalfBottom: imgPanelBgndFull

    imgWindow:  StyledImageDesc {
        source: "Resources/Window.png"
        borders: [4, 4, 20, 20]
        margins: [0, 0, 16, 16]
    }
    imgMaximizedWindow: imgPanelBgndFull

    imgWindowGripper: StyledImageDesc {
        source: "Resources/GripperDots.png"
        horizontalTile: true
        borders: [1, 0, 1, 0]
    }
    imgWindowCorner: StyledImageDesc {
        source: "Resources/CornerDots.png"
    }

    imgClockAnalogBgnd: StyledImageDesc {
        source: "Resources/ClockAnalogBgnd.png"
    }
    imgClockAnalogFront: StyledImageDesc {
        source: "Resources/ClockAnalogFront.png"
    }
    imgClockDigitalBgnd: StyledImageDesc {
        source: "Resources/ClockDigitalBgnd.png"
    }
    imgClockDigitalFront: StyledImageDesc {
        source: "Resources/ClockDigitalFront.png"
    }
    imgAudioLevelBigBgnd: StyledImageDesc {
        source: "Resources/AudioLevelBigBgnd.png"
        borders: [10, 10, 10, 10]
        margins: [3, 0, 3, 0]
    }
    imgAudioLevelBigFront: StyledImageDesc {
        source: "Resources/AudioLevelBigFront.png"
        borders: [10, 10, 10, 10]
        margins: [3, 0, 3, 0]
    }
    imgAudioLevelBigProgress: StyledImageDesc {
        source: "Resources/AudioLevelBigProgress.png"
        margins: [3, 0, 3, 0]
    }
    imgAudioLevelBigProgressDisabled: StyledImageDesc {
        source: "Resources/AudioLevelBigProgressGray.png"
        margins: [3, 0, 3, 0]
    }
    imgSliderThumbBig: StyledImageDesc {
        source: "Resources/SliderThumbBig.png"
        borders: buttonBorders
    }
    imgSliderThumbBigHovered: StyledImageDesc {
        source: "Resources/SliderThumbBigHover.png"
        borders: buttonBorders
    }
    imgSliderThumbSmall: StyledImageDesc {
        source: "Resources/SliderThumbSmall.png"
        borders: buttonBorders
        margins: buttonMargins
    }
    imgSliderThumbSmallHovered: StyledImageDesc {
        source: "Resources/SliderThumbSmallHover.png"
        borders: buttonBorders
        margins: buttonMargins
    }


    imgAudioLevelSmallBgnd: StyledImageDesc {
        source: "Resources/AudioLevelSmallBgnd.png"
        borders: [7, 7, 7, 7]
    }
    imgAudioLevelSmallFront: StyledImageDesc {
        source: "Resources/AudioLevelSmallFront.png"
        borders: [7, 7, 7, 7]
    }
    imgAudioLevelSmallProgress: StyledImageDesc {
        source: "Resources/AudioLevelSmallProgress.png"
        borders: [0, 0, 0, 1]
        horizontalTile: true
    }
    imgAudioLevelSmallProgressDisabled: StyledImageDesc {
        source: "Resources/AudioLevelSmallProgressGray.png"
        borders: [0, 0, 0, 1]
        horizontalTile: true
    }
    imgLampOff: StyledImageDesc {
        source: "Resources/LampOff.png"
        margins: [6, 6, 5, 5]
    }
    imgLampOnPreview: StyledImageDesc {
        source: "Resources/LampOnGreen.png"
        margins: [6, 6, 5, 5]
    }
    imgLampOnProgram: StyledImageDesc {
        source: "Resources/LampOnOrange.png"
        margins: [6, 6, 5, 5]
    }

    buttonMargins: [1, 1, 1, 1]
    buttonBorders: [6, 6, 6, 6]
    imgButtonSquareNeutral: StyledImageDesc {
        source: "Resources/ButtonSquareWhite.png"
        margins: buttonMargins
        borders: buttonBorders
        colFill: colNeutral
    }
    imgButtonSquareCommon: StyledImageDesc {
        source: "Resources/ButtonSquareYellow.png"
        margins: buttonMargins
        borders: buttonBorders
        colFill: colCommon
    }
    imgButtonSquarePreview: StyledImageDesc {
        source: "Resources/ButtonSquareGreen.png"
        margins: buttonMargins
        borders: buttonBorders
        colFill: colPreset
    }
    imgButtonSquareProgram: StyledImageDesc {
        source: "Resources/ButtonSquareOrange.png"
        margins: buttonMargins
        borders: buttonBorders
        colFill: colProgram
    }
    imgButtonSquareDisabled: StyledImageDesc {
        source: "Resources/ButtonSquareDisabled.png"
        margins: buttonMargins
        borders: buttonBorders
    }
    imgButtonRectNeutral: StyledImageDesc {
        source: "Resources/ButtonRectWhite.png"
        margins: buttonMargins
        borders: buttonBorders
        colFill: colNeutralDark
    }
    imgButtonRectCommon: StyledImageDesc {
        source: "Resources/ButtonRectYellow.png"
        margins: buttonMargins
        borders: buttonBorders
        colFill: colCommon
    }
    imgButtonRectPreview: StyledImageDesc {
        source: "Resources/ButtonRectGreen.png"
        margins: buttonMargins
        borders: buttonBorders
        colFill: colPreset
    }
    imgButtonRectProgram: StyledImageDesc {
        source: "Resources/ButtonRectOrange.png"
        margins: buttonMargins
        borders: buttonBorders
        colFill: colProgram
    }
    imgButtonRectDisabled: StyledImageDesc {
        source: "Resources/ButtonRectDisabled.png"
        margins: buttonMargins
        borders: buttonBorders
    }

    imgOutputBgnd: StyledImageDesc {
        source: "Resources/VideoBgnd.png"
        borders: [4, 4, 4, 4]
    }
    imgMediaBarBgnd: StyledImageDesc {
        source: "Resources/MediaBarBgnd.png"
        borders: [4, 4, 4, 4]
    }


    imgSliderScaleTick: StyledImageDesc {
        source: "Resources/SliderScaleTick.png"
    }

    imgSeparatorHor: StyledImageDesc {
        source: "Resources/SeparatorHor.png"
    }
    imgSeparatorVert: StyledImageDesc {
        source: "Resources/SeparatorVert.png"
    }

    imgTBarGroove: StyledImageDesc {
        source: "Resources/TBarGroove.png"
        borders: [10, 4, 10, 4]
    }
    imgTBarThumb: StyledImageDesc {
        source: "Resources/TBarThumb.png"
        //margins: [8, 0, 0, 8]
    }

    imgSliderGroove: StyledImageDesc {
        source: "Resources/SliderGroove.png"
        borders: [10, 3, 10, 3]
    }

    imgSpinnerScale: StyledImageDesc {
        source: "Resources/SpinnerScale.png"
    }
    imgSpinnerHandle: StyledImageDesc {
        source: "Resources/SpinnerHandle.png"
    }
    imgSpinnerDot: StyledImageDesc {
        source: "Resources/SpinnerDotOrange.png"
    }

    imgButtonSensor: StyledImageDesc {
        source: "Resources/ButtonSensor.png"
        borders: [5, 5, 5, 5]
    }
    imgButtonSensorPressed: StyledImageDesc {
        source: "Resources/ButtonSensorPressed.png"
        borders: [5, 5, 5, 5]
    }

    imgProgressBarBgnd: StyledImageDesc {
        source: "Resources/ProgressBarBgnd.png"
        margins: [2, 2, 2, 2]
        borders: [10, 7, 10, 7]
    }
    imgProgressBarFront: StyledImageDesc {
        source: "Resources/ProgressBarFront.png"
        margins: [2, 2, 2, 2]
        borders: [10, 7, 10, 7]
    }
    imgProgressBarNeutral: StyledImageDesc {
        source: "Resources/ProgressBarCommon.png"
        margins: [2, 2, 2, 2]
        borders: [5, 7, 5, 7]
    }
    imgProgressBarPreview: StyledImageDesc {
        source: "Resources/ProgressBarGreen.png"
        margins: [2, 2, 2, 2]
        borders: [5, 7, 5, 7]
    }
    imgProgressBarProgram: StyledImageDesc {
        source: "Resources/ProgressBarOrange.png"
        margins: [2, 2, 2, 2]
        borders: [5, 7, 5, 7]
    }
    imgPropertyHeader: StyledImageDesc {
        source: "Resources/PropHeader.png"
        margins: buttonMargins
        borders: buttonBorders
    }
    imgPropertyHeaderPressed: StyledImageDesc {
        source: "Resources/PropHeaderPressed.png"
        margins: buttonMargins
        borders: buttonBorders
    }
    imgExpandOn: StyledImageDesc {
        source: "Resources/ExpandOn.png"
    }
    imgExpandOff: StyledImageDesc {
        source: "Resources/ExpandOff.png"
    }
    imgSwitchOn: StyledImageDesc {
        source: "Resources/SwitchOn.png"
    }
    imgSwitchOff: StyledImageDesc {
        source: "Resources/SwitchOff.png"
    }
    imgScrollerBgnd: StyledImageDesc {
        source: "Resources/ScrollerBgnd.png"
        borders: [4, 4, 4, 4]
    }
    imgScrollerBgndHorInner: StyledImageDesc {
        source: "Resources/ScrollerBgnd.png"
        margins: [3, -1, 3, 3]
        borders: [4, 4, 4, 4]
    }
    imgScrollerBgndVertInner: StyledImageDesc {
        source: "Resources/ScrollerBgnd.png"
        margins: [-1, 2, 3, 3]
        borders: [4, 4, 4, 4]
    }
    imgScrollerThumbHor: StyledImageDesc {
        source: "Resources/ScrollerThumbHor.png"
        borders: [5, 4, 6, 5]
        horizontalTile: true
    }
    imgScrollerThumbHorInner: StyledImageDesc {
        source: "Resources/ScrollerThumbHor.png"
        margins: [1, -1, 1, 0]
        borders: [5, 4, 6, 5]
        horizontalTile: true
    }
    imgScrollerThumbVert: StyledImageDesc {
        source: "Resources/ScrollerThumbVert.png"
        borders: [4, 5, 5, 6]
        verticalTile: true
    }
    imgScrollerThumbVertInner: StyledImageDesc {
        source: "Resources/ScrollerThumbVert.png"
        margins: [-1, 1, 0, 1]
        borders: [4, 5, 5, 6]
        verticalTile: true
    }
    imgThinScrollerBgnd: StyledImageDesc {
        source: "Resources/ThinScrollerBgnd.png"
        borders: [4, 4, 4, 4]
    }
    imgThinScrollerThumbHor: StyledImageDesc {
        source: "Resources/ThinScrollerThumbHor.png"
        borders: [5, 4, 6, 5]
        horizontalTile: true
    }
    imgThinScrollerThumbVert: StyledImageDesc {
        source: "Resources/ThinScrollerThumbVert.png"
        borders: [4, 5, 5, 6]
        verticalTile: true
    }
    imgDialogButton: imgPropertyHeader
    imgDialogButtonPressed: imgPropertyHeaderPressed

    imgKeyboardBackground: StyledImageDesc {
        source: "Resources/KeyboardBgnd.png"
    }
    imgTreeViewArrow: StyledImageDesc {
        source: "Resources/TreeViewArrow.png"
    }
}
