import QtQuick 2.11
import QtQuick.Controls 2.4
import MixerGuiControls 1.0

MenuItem {
    id: menuItem
    leftPadding: MixerThemeManager.theme.itemSpacing + indicator.width
    rightPadding: MixerThemeManager.theme.itemSpacing + arrow.width
    property color colBgnd:     MixerThemeManager.theme.colNeutralDark
    property color colText:     MixerThemeManager.theme.colNeutral
    property color colTextSel:  MixerThemeManager.theme.colCommon
    property color colTextDisab:MixerThemeManager.theme.colTextDisabled
    property bool noCloseMenu: false
    property var  action1 : null
    property string shortcut: action && action.shortcut ? action.shortcut :
                                                          action1 && action1.shortcut ? action1.shortcut : ""

    function formatMnemonic(text) {
        return text.replace("&","")
    }

    Binding on text      {  value: action1 ? formatMnemonic(action1.text) : "";  when: action1!=null  }
    Binding on enabled   {  value: action1 ? action1.enabled : false;     when: action1!=null  }
    Binding on checkable {  value: action1 ? action1.checkable : false;   when: action1!=null  }
    Binding on checked   {  value: action1 ? action1.checked : false;     when: action1!=null  }
    Binding on icon      {  value: action1 ? action1.icon : null;         when: action1!=null  && action1.icon!=null }

    onTriggered: {
        if (action) action.trigger()
        else if (action1) action1.trigger()
    }

    signal itemClicked()

    implicitWidth:  arrow.x + rightPadding//contentItem.implicitWidth + leftPadding + rightPadding
    implicitHeight: MixerThemeManager.theme.menuHeight//Math.max(contentItem.implicitHeight, indicator ? indicator.implicitHeight : 0) + topPadding + bottomPadding
    height: visible && text ? implicitHeight : 0
    baselineOffset: contentItem.y + contentItem.baselineOffset
    spacing: 0

    font.family: MixerThemeManager.theme.fontFamily
    font.pixelSize: MixerThemeManager.theme.menuFontSize

    //background: Rectangle { color: colBgnd    }
    background: null

    arrow: Text {
        text: menuItem.subMenu ? " >" : (menuItem.shortcut ?  "    " + menuItem.shortcut : "")
        x: menuItem.parent ? menuItem.parent.width - width - MixerThemeManager.theme.itemSpacing : 0
       // y: menuItem.topPadding + (menuItem.availableHeight - height) / 2
        anchors.verticalCenter: label.verticalCenter
        font: contentItem.font
        color: contentItem.color
    }

    indicator:  Item {
        x:  MixerThemeManager.theme.itemSpacing
        anchors.verticalCenter: label.verticalCenter
        width: checker.implicitWidth
        height: checker.implicitHeight
        MenuCheckBox  {
            id: checker
            checked: false
            visible: menuItem.checkable && !menuItem.autoExclusive
            onClicked: {
                if (!noCloseMenu) {
                    if (menuItem.action) menuItem.action.trigger()
                    else menuItem.triggered()
                }
                else menuItem.itemClicked()
            }
            Binding on checked {
                value: menuItem.checked
            }
        }
        Rectangle  {
            anchors.centerIn:  parent
            width:  label.height/2
            height: width
            radius: width/2
            color:  label.color
            visible: menuItem.autoExclusive && menuItem.checked
        }
    }

    contentItem: Text {
        id: label
        topPadding: 0
        bottomPadding: 0
        leftPadding: 0
        rightPadding: 0
        text: formatMnemonic(menuItem.text)
        font: menuItem.font
        color: {
            if (!menuItem.enabled) return  colTextDisab
            if (menuItem.selected || menuItem.highlighted) return  colTextSel
            return colText
        }
        horizontalAlignment: Text.AlignLeft
        verticalAlignment: Text.AlignVCenter
        MouseArea {
            anchors.fill: parent
            visible: noCloseMenu
            acceptedButtons: Qt.LeftButton
            onClicked: menuItem.itemClicked()
        }

        //elide: Text.ElideRight
    }
}
