import QtQuick 2.0
import QtQuick.Layouts 1.1
import MixerGuiControls 1.0

Item {
    id: root
    property string text
    property string colorType: "neutral"    // "neutral", "preset", "program"
    property string icon: ""
    property bool labelEnabled: MixerThemeManager.theme.headerTextEnabled
    property alias buttons: buttonLayout.data
    property int buttonVerticalOffset: 0
    property int buttonSpacing: 0
    implicitWidth: layout.implicitWidth
    implicitHeight: MixerThemeManager.theme.headerHeight
    property bool gripperVisible: false
    
    property Item iconItem: MixerIcon {
        visible: !!root.icon
        colorType: root.colorType
        icon: root.icon
    }
    
    RowLayout {
        id: layout
        spacing: 0
        anchors.fill: parent

        Item {
            data: iconItem
            implicitHeight: iconItem.height
            implicitWidth: iconItem.width
        }
        Item {
            implicitWidth: MixerThemeManager.theme.itemSpacing/2
            visible: iconItem.visible
        }
        MixerLabel {
            id: label
            Layout.fillWidth: true
            Layout.alignment: Qt.AlignVCenter
            Layout.maximumWidth: gripperVisible ? implicitWidth : -1
            enabled: root.labelEnabled
            text: root.text
            colorType: root.colorType
            font.pixelSize: MixerThemeManager.theme.headerFontSize
        }
        
        Item {
            implicitWidth: 10
            visible: gripperVisible
        } 
        
        StyledImage {
            id: gripper
            visible: gripperVisible
            Layout.fillWidth: true
            Layout.alignment: Qt.AlignVCenter
            desc: MixerThemeManager.theme.imgWindowGripper
        }
        
        Item {
            implicitWidth: 5
            visible: gripperVisible
        }         
        
        Item {
            implicitWidth: buttonLayout.implicitWidth
            implicitHeight: buttonLayout.implicitHeight
            RowLayout {
                id: buttonLayout
                anchors.centerIn: parent
                anchors.verticalCenterOffset: root.buttonVerticalOffset
                spacing: root.buttonSpacing
            }
        }
    }
}
