import QtQuick 2.1
import QtQuick.Layouts 1.1
import QtQuick.Controls 1.0
import MixerGuiControls 1.0

MixerDialog {
    id: root
    property string caption: ""
    signal entered(string value)
    signal rejected()
    
    property alias text: textField.text
    property bool allowEmptyString: false
    
    ColumnLayout {
        anchors.fill: parent
        MixerLabel {
            text: root.caption
        }
        TextField {
            id: textField
            Layout.fillWidth: true
            onAccepted: {
                entered(text)
                text = ""
                root.hide()
            }
        }        
    }
    
    readonly property Action okAction: Action {
        text: qsTr("OK")
        enabled: root.text || root.allowEmptyString
        onTriggered: {
            root.entered(root.text)
            root.text = ""
            root.hide()
        }
    }
    
    readonly property Action cancelAction: Action {
        text: qsTr("Cancel")
        onTriggered: {
            root.rejected()
            root.text = ""
            root.hide()        
        }
    }
    
    buttons: [okAction, cancelAction]
    
    onClosing: {
        rejected()
        text = ""
        hide()        
    }
    onOpened: textField.forceActiveFocus() 
}