import QtQuick 2.0
import QtQuick.Controls 1.0
import QtQuick.Controls.Styles 1.0
import MixerGuiControls 1.0

TableView {
    id: root
    frameVisible: false
    property Component itemOverlayDelegate: null
    property Component rowOverlayDelegate: null
    flickableItem.interactive: false

    style: TableViewStyle {
        backgroundColor: "transparent"

        property var scrollViewStyle: MixerScrollViewStyle {
            __externalScrollBars: false
        }
        scrollBarBackground: scrollViewStyle.scrollBarBackground
        corner: scrollViewStyle.corner
        handle: scrollViewStyle.handle
        incrementControl: scrollViewStyle.incrementControl
        decrementControl: scrollViewStyle.decrementControl        
    }

    headerDelegate: Item {
        height: root.headerVisible ? MixerThemeManager.theme.headerHeight  : 0
        MediaBarBackground {
            anchors.fill: parent
            MixerLabel {
                id: label
                anchors.fill: parent
                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: styleData.textAlignment
                anchors.leftMargin: styleData.textAlignment == Text.AlignLeft ? 8 : 0
                anchors.rightMargin: styleData.textAlignment == Text.AlignRight ? 12 : 0
                text: styleData.value
                elide: Text.ElideRight
                font.pixelSize: MixerThemeManager.theme.systemFontSize
            }
        }
    }
    
    
    contentFooter: Item {
        height: 5
    }
        
    Binding { // HACK: хотим сделать отсуп сверху, для этого стоит использовать contentHeader, но в Qt5.8 он сломан, поэтому добавляем отсуп в header
        target: root.__listView.headerItem
        property: "height"
        value: {
            var headerItem = root.__listView.headerItem
            return (headerItem && headerItem.visible ? headerItem.headerRow.height : 0) + 2
        }
    }
    
    rowDelegate: Item {
        height: MixerThemeManager.theme.listItemHeight
        property var _styleData: styleData
        Loader {
            property var styleData: parent._styleData
            anchors.fill: parent
            sourceComponent: root.rowOverlayDelegate
        }
    }
    
    itemDelegate: Item {
        MixerTableViewLabel {}
        property var _styleData: styleData
        Loader {
            anchors.fill: parent
            anchors.leftMargin: styleData.textAlignment == Text.AlignLeft ? 8 : 0
            anchors.rightMargin: styleData.textAlignment == Text.AlignRight ? 12 : 0
            property var styleData: parent._styleData
            sourceComponent: root.itemOverlayDelegate
        }        
    }    
}
