import QtQuick 2.1


QtObject {
    function colHSL(h, s, l)         { return Qt.hsva(h/360, s/100, l/100) }
    function colRGB(r, g, b)         { return Qt.rgba(r/255, g/255, b/255) }
    function colWAlpha(col, alpha)   { return Qt.rgba(col.r, col.g, col.b, alpha) }

    readonly property var tagToSysColor: {
        "program":      theme.colProgram,
        "preset":       theme.colPreset,
        "common":       theme.colCommon,
        "neutral":      theme.colNeutral,
        "program_dark": theme.colProgramDark,
        "preset_dark":  theme.colPresetDark,
        "common_dark":  theme.colCommonDark,
        "neutral_dark": theme.colNeutralDark,
        "program_light":theme.colProgramLight,
        "preset_light": theme.colPresetLight,
        "common_light": theme.colCommonLight,
        "neutral_light":theme.colNeutralLight,
    }

    property string themeID: "allmix"
    property string name: qsTr("Dark Realistic")

// Font and sizes
    property string fontFamily: "Tahoma"
    property int headerFontSize: 18
    property int footerFontSize: headerFontSize
    property int buttonTextFontSize: 18	
    property int buttonDigitsFontSize: 24
    property int menuFontSize: 14
    property int systemFontSize: menuFontSize
    property int promptFontSize: 12
    property bool headerTextEnabled: true
    property string buttonIconBlend: "multiply"

    property int panelSpacing: 14
    property int itemSpacing: 14
    property int inputsSpacingH: 28
    property int inputsSpacingV: 36
       
    property int buttonSize: 43
    property int oneAndHalfButtonSize: 58       
    property int doubleButtonSize: 87
    property int iconSize: 25    
    property int headerHeight: 28
    property int footerHeight: 28
    property int tabGroupHeight: 28
    property int menuHeight: 24
    property int statusHeight: 24
    property int mediaBarHeight: 34
    property int higlightFrameWidth: 3
    property int textOutlineSize: 1
    property int textGlowSize: 3
    property int headerButtonsOffset: 2
    property int listItemHeight: systemFontSize + controlPadding*2

    property int propertyHeaderHeight: headerHeight
    property int propertyHeaderSpacing: 1
    property int propertyHeaderOffsetX: 14
    property int propertyHeaderItemSpacing: 7
    property var propertyListMargins: [14, 7, 14, 7]
    property int propertyListStep: 28
    property int propertyListFontSize: systemFontSize
    property real propertyListColumnRatio: 0.38
    
    property int dialogButtonWidth:	dialogButtonHeight * 3
    property int dialogButtonHeight: headerHeight	
    property int dialogButtonHSpacing: propertyHeaderSpacing
    property int dialogButtonVSpacing: itemSpacing
    property int dialogHeaderHeight: 36
    
    property int toolbarHeaderHeight: 36	
    property int toolbarButtonSize:	29
    property int toolbarButtonSpacing: 4
    property int toolbarGroupSpaсing: 14
    
    property int scrollBarSpacing: 1
    property int scrollBarWidth: 23
    property int controlPadding: 4
    property int controlBorderSize: 1
    property int audioLedBigWidth:   scrollBarWidth-4
    property int audioLedSmallWidth: audioLedBigWidth/2-1

// Colors
    readonly property color colBlack: "black"
    readonly property color colWhite: "white"
    readonly property color colNone:  "transparent"

    property color colPreset:	    colRGB(229, 255, 102)
    property color colPresetDark:	colRGB(92 , 102, 41 )
    property color colPresetLight:	colRGB(244, 255, 191)
    property color colPresetBlack:	colRGB(42 , 51 , 0  )
    property color colProgram:	    colRGB(255, 178, 102)
    property color colProgramDark:	colRGB(102, 71 , 41 )
    property color colProgramLight:	colRGB(255, 223, 191)
    property color colProgramBlack:	colRGB(51 , 26 , 0  )
    property color colCommon:       colRGB(255, 234, 128)
    property color colCommonDark:   colRGB(102, 92 , 41 )
    property color colCommonLight:	colRGB(255, 244, 191)
    property color colCommonBlack:	colRGB(51 , 42 , 0  )
    property color colNeutral:	    colRGB(191, 191, 191)
    property color colNeutralDark:	colRGB(38 , 38 , 38 )
    property color colNeutralLight:	colRGB(255, 255, 255)
    property color colNeutralBlack:	colRGB(13 , 13 , 13 )

    property var colText: {
        "neutral": colNeutral,
        "preset": colPreset,
        "program": colProgram,
        "common" : colCommon
    }
    property var colTextHighlight: {
        "neutral": colNeutralLight,
        "preset": colPresetLight,
        "program": colProgramLight,
        "common" : colCommonLight
    }

    property var colTextOutline: {
        "neutral": colNeutralDark,
        "preset": colPresetDark,
        "program": colProgramDark,
        "common" : colCommonDark
    }

    property var colTextGlow: {
        "neutral": colWAlpha(colNeutralLight, 0.4),
        "preset":  colWAlpha(colPresetLight,  0.4),
        "program": colWAlpha(colProgramLight, 0.4),
        "common":  colWAlpha(colCommonLight,  0.4)
    }

    property color colTextInactive:	colWAlpha(colNeutral, 0.6)
    property color colTextDisabled:	colWAlpha(colNeutral, 0.4)

    property var colIcon: {
        "neutral": colText["neutral"],
        "preset": colText["preset"],
        "program": colText["program"],
        "common": colText["common"]
    }
    property var colIconHighlight: {
        "neutral": colNeutralLight,
        "preset": colPresetLight,
        "program": colProgramLight,
        "common": colCommonLight
    }
    property var colIconOutline: {
        "neutral": colTextOutline["neutral"],
        "preset": colTextOutline["preset"],
        "program": colTextOutline["program"],
        "common": colTextOutline["common"]
    }
    property var colIconGlow: {
        "neutral": colTextGlow["neutral"],
        "preset": colTextGlow["preset"],
        "program": colTextGlow["program"],
        "common": colTextGlow["common"]
    }

    property color colIconInactive:	colTextInactive
    property color colIconDisabled:	colTextDisabled

    property var colSelected: {
        "preset": colText["preset"],
        "program": colText["program"],
        "common": colText["common"]
    }

    property var colButtonText: {
        "neutral": colNeutralBlack,
        "common":  colCommonBlack,
        "preset":  colPresetBlack,
        "program": colProgramBlack
    }
    property color colButtonTextDisabled: colWAlpha(colNeutralDark, 0.5)
    
    property var colButtonTextGlow: {
        "neutral": colNeutralLight,    
        "common": colCommonLight,
        "preset": colPresetLight,
        "program": colProgramLight
    }      
    
    property var colPressedButtonText: {
        "neutral": colNeutralBlack,    
        "common": colCommonBlack,
        "preset": colPresetBlack,
        "program": colProgramBlack 
    }   
    
// Images    
    property url iconFolder: "icons/"

    property StyledImageDesc imgAppBgnd: StyledImageDesc {
        source: "MixerTheme/Resources/AppBgnd.png"
    }
    property StyledImageDesc imgPanelBgndFull: StyledImageDesc {
        source: "MixerTheme/Resources/GroupBgndFull.png"
        margins: [0, 0, 2, 2]
        borders: [7, 7, 7, 7]
    }
    property StyledImageDesc imgPanelBgndHalfTop: StyledImageDesc {
        source: "MixerTheme/Resources/GroupBgndHalfTop.png"
        margins: [0, 0, 2, 2]
        borders: [7, 7, 7, 7]
    }
    property StyledImageDesc imgPanelBgndHalfBottom: StyledImageDesc {
        source: "MixerTheme/Resources/GroupBgndHalfBottom.png"
        margins: [0, 0, 2, 2]
        borders: [7, 7, 7, 7]
    }    
    property StyledImageDesc imgWindow: StyledImageDesc {
        source: "MixerTheme/Resources/Window.png"
        margins: [2, 2, 20, 20]
        borders: [7, 7, 36, 36]
    }    
    property StyledImageDesc imgMaximizedWindow: StyledImageDesc {
        source: "MixerTheme/Resources/MaximizedWindow.png"
        margins: [2, 2, 2, 2]
        borders: [7, 7, 7, 7]
    }    
    property StyledImageDesc imgWindowGripper: StyledImageDesc {
        source: "MixerTheme/Resources/GripperDots.png"
        horizontalTile: true
        borders: [1, 0, 1, 0]
    }    
    property StyledImageDesc imgWindowCorner: StyledImageDesc {
        source: "MixerTheme/Resources/CornerDots.png"
    }    
    
    
    property StyledImageDesc imgClockAnalogBgnd: StyledImageDesc {
        source: "MixerTheme/Resources/ClockAnalogBgnd.png"
    } 
    property StyledImageDesc imgClockAnalogFront: StyledImageDesc {
        source: "MixerTheme/Resources/ClockAnalogFront.png"
    }
    property StyledImageDesc imgClockDigitalBgnd: StyledImageDesc {
        source: "MixerTheme/Resources/ClockDigitalBgnd.png"
    } 
    property StyledImageDesc imgClockDigitalFront: StyledImageDesc {
        source: "MixerTheme/Resources/ClockDigitalFront.png"
    }
    property StyledImageDesc imgAudioLevelBigBgnd: StyledImageDesc {
        source: "MixerTheme/Resources/AudioLevelBigBgnd.png"
        borders: [10, 10, 10, 10]
        margins: [3, 1, 3, -1]
    }
    property StyledImageDesc imgAudioLevelBigFront: StyledImageDesc {
        source: "MixerTheme/Resources/AudioLevelBigFront.png"
        borders: [10, 10, 10, 10]
        margins: [3, 1, 3, -1]
    }  
    property StyledImageDesc imgAudioLevelBigProgress: StyledImageDesc {
        source: "MixerTheme/Resources/AudioLevelBigProgress.png"
        margins: [3, 0, 3, 0]
    }
    property StyledImageDesc imgAudioLevelBigProgressDisabled: StyledImageDesc {
        source: "MixerTheme/Resources/AudioLevelBigProgressGray.png"
        margins: [3, 0, 3, 0]
    }    
    property StyledImageDesc imgSliderThumbBig: StyledImageDesc {
        source: "MixerTheme/Resources/SliderThumbBig.png"
        margins: [6, 2, 2, 6]
    }
    property StyledImageDesc imgSliderThumbBigHovered: StyledImageDesc {
        source: "MixerTheme/Resources/SliderThumbBigHover.png"
        margins: [6, 2, 2, 6]
    }    

    property StyledImageDesc imgAudioLevelSmallBgnd: StyledImageDesc {
        source: "MixerTheme/Resources/AudioLevelSmallBgnd.png"
        borders: [7, 7, 7, 7]
        margins: [3, 1, 3, -1]
    }
    property StyledImageDesc imgAudioLevelSmallFront: StyledImageDesc {
        source: "MixerTheme/Resources/AudioLevelSmallFront.png"
        borders: [7, 7, 7, 7]
        margins: [3, 1, 3, -1]
    }  
    property StyledImageDesc imgAudioLevelSmallProgress: StyledImageDesc {
        source: "MixerTheme/Resources/AudioLevelSmallProgress.png"
        borders: [0, 0, 0, 1]
        horizontalTile: true
    }
    property StyledImageDesc imgAudioLevelSmallProgressDisabled: StyledImageDesc {
        source: "MixerTheme/Resources/AudioLevelSmallProgressGray.png"
        borders: [0, 0, 0, 1]
        horizontalTile: true
    }      
    property StyledImageDesc imgSliderThumbSmall: StyledImageDesc {
        source: "MixerTheme/Resources/SliderThumbSmall.png"
        margins: [3, 1, 1, 3]
    }
    property StyledImageDesc imgSliderThumbSmallHovered: StyledImageDesc {
        source: "MixerTheme/Resources/SliderThumbSmallHover.png"
        margins: [3, 1, 1, 3]
    }
    
    property StyledImageDesc imgLampOff: StyledImageDesc {
        source: "MixerTheme/Resources/LampOff.png"
        margins: [6, 6, 5, 5]
    }
    property StyledImageDesc imgLampOnPreview: StyledImageDesc {
        source: "MixerTheme/Resources/LampOnGreen.png"
        margins: [6, 6, 5, 5]
    }
    property StyledImageDesc imgLampOnProgram: StyledImageDesc {
        source: "MixerTheme/Resources/LampOnOrange.png"
        margins: [6, 6, 5, 5]
    }    
    
    property var buttonMargins: [4, 4, 8, 8]
    property var buttonBorders: [24, 24, 28, 28]  
    property StyledImageDesc imgButtonSquareNeutral: StyledImageDesc {
        source: "MixerTheme/Resources/ButtonSquareWhite.png"
        margins: buttonMargins
        borders: buttonBorders
    }
    property StyledImageDesc imgButtonSquareCommon: StyledImageDesc {
        source: "MixerTheme/Resources/ButtonSquareYellow.png"
        margins: buttonMargins
        borders: buttonBorders
    }
    property StyledImageDesc imgButtonSquarePreview: StyledImageDesc {
        source: "MixerTheme/Resources/ButtonSquareGreen.png"
        margins: buttonMargins
        borders: buttonBorders
    }
    property StyledImageDesc imgButtonSquareProgram: StyledImageDesc {
        source: "MixerTheme/Resources/ButtonSquareOrange.png"
        margins: buttonMargins
        borders: buttonBorders
    }
    property StyledImageDesc imgButtonSquareDisabled: StyledImageDesc {
        source: "MixerTheme/Resources/ButtonSquareDisabled.png"
        margins: buttonMargins
        borders: buttonBorders
    }    
    property StyledImageDesc imgButtonRectNeutral: StyledImageDesc {
        source: "MixerTheme/Resources/ButtonRectWhite.png"
        margins: buttonMargins
        borders: buttonBorders
    }
    property StyledImageDesc imgButtonRectCommon: StyledImageDesc {
        source: "MixerTheme/Resources/ButtonSquareYellow.png"
        margins: buttonMargins
        borders: buttonBorders
    }
    property StyledImageDesc imgButtonRectPreview: StyledImageDesc {
        source: "MixerTheme/Resources/ButtonRectGreen.png"
        margins: buttonMargins
        borders: buttonBorders
    }
    property StyledImageDesc imgButtonRectProgram: StyledImageDesc {
        source: "MixerTheme/Resources/ButtonRectOrange.png"
        margins: buttonMargins
        borders: buttonBorders
    }
    property StyledImageDesc imgButtonRectDisabled: StyledImageDesc {
        source: "MixerTheme/Resources/ButtonRectDisabled.png"
        margins: buttonMargins
        borders: buttonBorders
    }     
    property StyledImageDesc imgOutputBgnd: StyledImageDesc {
        source: "MixerTheme/Resources/VideoBgnd.png"
        borders: [4, 4, 4, 4]
    } 
    property StyledImageDesc imgMediaBarBgnd: StyledImageDesc {
        source: "MixerTheme/Resources/MediaBarBgnd.png"
        borders: [5, 5, 5, 5]
    }     
    property StyledImageDesc imgSliderScaleTick: StyledImageDesc {
        source: "MixerTheme/Resources/SliderScaleTick.png"
    } 
    
    property StyledImageDesc imgSeparatorHor: StyledImageDesc {
        source: "MixerTheme/Resources/SeparatorHor.png"
        borders: [10, 2, 10, 2]
    }
    property StyledImageDesc imgSeparatorVert: StyledImageDesc {
        source: "MixerTheme/Resources/SeparatorVert.png"
        borders: [2, 10, 2, 10]
    }    
    
    property StyledImageDesc imgTBarGroove: StyledImageDesc {
        source: "MixerTheme/Resources/TBarGroove.png"
        borders: [10, 4, 10, 4]
    }
    property StyledImageDesc imgTBarThumb: StyledImageDesc {
        source: "MixerTheme/Resources/TBarThumb.png"
        margins: [0, 0, 8, 8]
    }  
    property StyledImageDesc imgTBarThumbVert: StyledImageDesc {
        source: "MixerTheme/Resources/TBarThumbVert.png"
        margins: [0, 0, 8, 8]
    }

    property StyledImageDesc imgSliderGroove: StyledImageDesc {
        source: "MixerTheme/Resources/SliderGroove.png"
        borders: [10, 3, 10, 3]
    }     
 
    property StyledImageDesc imgSpinnerScale: StyledImageDesc {
        source: "MixerTheme/Resources/SpinnerScale.png"
    } 
    property StyledImageDesc imgSpinnerHandle: StyledImageDesc {
        source: "MixerTheme/Resources/SpinnerHandle.png"
    }
    property StyledImageDesc imgSpinnerDot: StyledImageDesc {
        source: "MixerTheme/Resources/SpinnerDotOrange.png"
    }
    
    property StyledImageDesc imgButtonSensor: StyledImageDesc {
        source: "MixerTheme/Resources/ButtonSensor.png"
        borders: [5, 5, 5, 5]
    }
    property StyledImageDesc imgButtonSensorPressed: StyledImageDesc {
        source: "MixerTheme/Resources/ButtonSensorPressed.png"
        borders: [5, 5, 5, 5]
    }    

    property StyledImageDesc imgProgressBarBgnd: StyledImageDesc {
        source: "MixerTheme/Resources/ProgressBarBgnd.png"
        margins: [2, 2, 2, 2]
        borders: [10, 7, 10, 7]
    }
    property StyledImageDesc imgProgressBarFront: StyledImageDesc {
        source: "MixerTheme/Resources/ProgressBarFront.png"
        margins: [2, 2, 2, 2]
        borders: [10, 7, 10, 7]
    }    
    property StyledImageDesc imgProgressBarNeutral: StyledImageDesc {
        source: "MixerTheme/Resources/ProgressBarWhite.png"
        margins: [2, 2, 2, 2]
        borders: [5, 7, 5, 7]
    }        
    property StyledImageDesc imgProgressBarPreview: StyledImageDesc {
        source: "MixerTheme/Resources/ProgressBarGreen.png"
        margins: [2, 2, 2, 2]
        borders: [5, 7, 5, 7]
    }        
    property StyledImageDesc imgProgressBarProgram: StyledImageDesc {
        source: "MixerTheme/Resources/ProgressBarOrange.png"
        margins: [2, 2, 2, 2]
        borders: [5, 7, 5, 7]
    }
    
    property StyledImageDesc imgPropertyHeader: StyledImageDesc {
        source: "MixerTheme/Resources/PropHeader.png"
        margins: [0, 0, 3, 3]
        borders: [3, 3, 6, 6]
        horizontalTile: true
    }
    property StyledImageDesc imgPropertyHeaderPressed: StyledImageDesc {
        source: "MixerTheme/Resources/PropHeaderPressed.png"
        margins: [0, 0, 3, 3]
        borders: [3, 3, 6, 6]
        horizontalTile: true
    }  
    property StyledImageDesc imgExpandOn: StyledImageDesc {
        source: "MixerTheme/Resources/ExpandOn.png"
    }       
    property StyledImageDesc imgExpandOff: StyledImageDesc {
        source: "MixerTheme/Resources/ExpandOff.png"
    }       
    property StyledImageDesc imgSwitchOn: StyledImageDesc {
        source: "MixerTheme/Resources/SwitchOn.png"
    }       
    property StyledImageDesc imgSwitchOff: StyledImageDesc {
        source: "MixerTheme/Resources/SwitchOff.png"
    }      

    property StyledImageDesc imgScrollerBgnd: StyledImageDesc {
        source: "MixerTheme/Resources/ScrollerBgnd.png"
        borders: [4, 4, 4, 4]
    }
    property StyledImageDesc imgScrollerBgndHorInner: StyledImageDesc {
        source: "MixerTheme/Resources/ScrollerBgnd.png"
        margins: [3, -1, 3, 3]
        borders: [4, 4, 4, 4]
    }   
    property StyledImageDesc imgScrollerBgndVertInner: StyledImageDesc {
        source: "MixerTheme/Resources/ScrollerBgnd.png"
        margins: [-1, 2, 3, 3]
        borders: [4, 4, 4, 4]
    }
    
    
    property StyledImageDesc imgScrollerThumbHor: StyledImageDesc {
        source: "MixerTheme/Resources/ScrollerThumbHor.png"
        borders: [5, 4, 6, 5]
        horizontalTile: true
    }    
    property StyledImageDesc imgScrollerThumbHorInner: StyledImageDesc {
        source: "MixerTheme/Resources/ScrollerThumbHor.png"
        margins: [1, -1, 1, 0]
        borders: [5, 4, 6, 5]
        horizontalTile: true
    }        
    
    property StyledImageDesc imgScrollerThumbVert: StyledImageDesc {
        source: "MixerTheme/Resources/ScrollerThumbVert.png"
        borders: [4, 5, 5, 6]
        verticalTile: true
    }    
    property StyledImageDesc imgScrollerThumbVertInner: StyledImageDesc {
        source: "MixerTheme/Resources/ScrollerThumbVert.png"
        margins: [-1, 1, 0, 1]
        borders: [4, 5, 5, 6]
        verticalTile: true
    }
    
    property StyledImageDesc imgThinScrollerBgnd: StyledImageDesc {
        source: "MixerTheme/Resources/ThinScrollerBgnd.png"
        borders: [4, 4, 4, 4]
    }
    property StyledImageDesc imgThinScrollerThumbHor: StyledImageDesc {
        source: "MixerTheme/Resources/ThinScrollerThumbHor.png"
        borders: [5, 4, 6, 5]
        horizontalTile: true
    }        
    property StyledImageDesc imgThinScrollerThumbVert: StyledImageDesc {
        source: "MixerTheme/Resources/ThinScrollerThumbVert.png"
        borders: [4, 5, 5, 6]
        verticalTile: true
    }    
    
    property StyledImageDesc imgDialogButton: imgPropertyHeader
    property StyledImageDesc imgDialogButtonPressed: imgPropertyHeaderPressed
    

	property StyledImageDesc imgKeyboardBackground: StyledImageDesc {
        source: "MixerTheme/Resources/KeyboardBgnd.png"
    }
    
	property StyledImageDesc imgTreeViewArrow: StyledImageDesc {
        source: "MixerTheme/Resources/TreeViewArrow.png"
    }

}
