pragma Singleton
import QtQuick 2.1
import Qt.labs.settings 1.0

QtObject {
    readonly property MixerTheme theme: themeLoader.item


    readonly property string allThemesTag: "All"
    property var enabledThemes: [allThemesTag] //must be set outside Manager from features
    property string defaultAppThemeID: ''

    function setThemeID(id) {
       themeID = isEnabledTheme(id) ? id : defaultThemeID
    }

    property string themeID: ""

    readonly property string defaultThemeID: {
        if (!enabledThemes || !enabledThemes[0] || enabledThemes[0]==allThemesTag)
           return defaultAppThemeID!='' ? defaultAppThemeID : defaultAppThemeID//themesAll[0].id
        return enabledThemes[0]
    }


    readonly property int themeIndex: {
        for (var i in themesAll) if (themesAll[i].id==themeID) return i
        var defID = defaultThemeID
        for (var i in themesAll) if (themesAll[i].id==defID) return i
        return 0
    }

    readonly property Loader themeLoader: Loader {
        source: themesAll[themeIndex].file
    }
    readonly property var themesAll: [
        { id: "allcast",     desc: qsTr("Dark Flat"), file: "AllcastTheme\\AllcastTheme.qml"},
        { id: "allcast_cm",  desc: qsTr("Dark Gradiented Cy/Mg"),    file: "AllcastThemeCM\\AllcastThemeCM.qml" },
        { id: "allmix",      desc: qsTr("Dark Realistic"),     file: "MixerTheme.qml"                 },
        { id: "allmix_flat", desc: qsTr("Dark Flat Or/Yw"),    file: "MixerFlatTheme\\MixerFlatTheme.qml" },
        { id: "allmix20",    desc: qsTr("Dark Flat AllMix"),   file: "TricasterTheme\\TricasterTheme.qml"},
    ]


    readonly property var themeIds: {
        var result = []
        for (var i in themesAll)
            if (isEnabledTheme(themesAll[i].id))
                result.push(themesAll[i].id)
        if (!result.length)
           result.push(defaultAppThemeID != '' ? defaultAppThemeID : themesAll[0].id) //first found
        return result
    }

    function isEnabledTheme(id) {
        if (enabledThemes[0] && enabledThemes[0]==allThemesTag)
            return true
        return inArray(id, enabledThemes)
    }

    function inArray(v,a) {
        if (!v) return false;
        for (var i in a) if (v==a[i]) return true; return false;
    }

    function getDescByID(id) {
        for (var i in themesAll)
            if (themesAll[i].id==id) return themesAll[i].desc
        return null;
    }

    readonly property var themeDescs: {
        var result = []
        for (var i in themeIds) result.push(getDescByID(themeIds[i]))
        return result
    }
}
