import QtQuick 2.1
import GuiControls 1.0
import MixerGuiControls 1.0
import QtQuick.Window 2.2

FramelessWindow {
    id: root
    width: implicitWidth
    height: implicitHeight
    default property alias data: panel.data
    property alias header: panel.header
    property alias footer: panel.footer
    property alias leftSpacing: panel.leftSpacing
    property alias rightSpacing: panel.rightSpacing
    property alias closeButtonVisible: defaultHeader.closeButtonVisible
    property alias minimizeButtonVisible: defaultHeader.minimizeButtonVisible
    property alias maximizeButtonVisible: defaultHeader.maximizeButtonVisible
    property alias closeButtonEnabled: defaultHeader.closeButtonEnabled
    property alias minimizeButtonEnabled: defaultHeader.minimizeButtonEnabled
    property alias maximizeButtonEnabled: defaultHeader.maximizeButtonEnabled
    property alias icon: defaultHeader.icon
    property alias iconItem: defaultHeader.iconItem

    property real implicitWidth: panel.implicitWidth + margins[0] + margins[2]
    property real implicitHeight: panel.implicitHeight + margins[1] + margins[3]

    maximizeOnHeaderDoubleClick: maximizeButtonVisible && maximizeButtonEnabled

    color: "transparent"
    property string borderColor:  "" //"program", "preset", "common", "neutral"

    signal closing (var close)
    signal pressed (var event)
    signal released(var event)

    function close() {
        var event = {accepted: true}
        closing(event)
        if(event.accepted) {
            hide()
        }
    }

    MixerPanel {
        id: panel
        anchors.fill: parent
        backgroundType: root.visibility == Window.Maximized ? "maximized_window" : "window"
        focus: true
      //Keys.onEscapePressed: root.close()
        Keys.onPressed  : { root.pressed (event); }
        Keys.onReleased : { root.released(event); if (!event.accepted && event.key==Qt.Key_Escape) root.close() }

        header: MixerWindowHeader {
            id: defaultHeader
            text: root.title
            anchors.fill: parent
            windowVisibility: root.visibility
        }
    }
    Rectangle {
        anchors.fill: parent
        visible: !!borderColor
        color: "transparent"
        border.width: 2
        border.color:  MixerThemeManager.theme.colSelected[borderColor] || borderColor
        radius: MixerThemeManager.theme.panelSpacing / 2
    }

    StyledImage {
        visible: root.resizeEnabled && root.visibility != Window.Maximized
        desc: MixerThemeManager.theme.imgWindowCorner
        anchors.right: parent.right
        anchors.bottom: parent.bottom
    }


    Connections {
        target: root.header
        ignoreUnknownSignals: true
        onClose: root.close()
        onMinimize: root.visibility = Window.Minimized
        onMaximize: root.visibility = Window.Maximized
        onRestore: root.visibility = Window.Windowed

    }

    margins: root.visibility == Window.Maximized ? MixerThemeManager.theme.imgMaximizedWindow.margins : MixerThemeManager.theme.imgWindow.margins
    headerHeight: header ? header.height : 0
}
