import QtQuick 2.0
import QtQuick.Layouts 1.1
import "FormatTime.js" as FormatTime
import MixerGuiControls 1.0
import Multimedia 1.0

Item {
    id: root
    property string colorType: "neutral"

    property bool reconnectButtonVisible: false
    property bool playButtonVisible: false
    property bool stopButtonVisible: false

    property bool reconnectButtonEnabled: true
    property bool playButtonEnabled: true
    property bool stopButtonEnabled: true

    property int connectionState: MediaSession.Empty

    signal play()
    signal stop()
    signal reconnect()

    property int buttonSize: MixerThemeManager.theme.mediaBarHeight - 4
    implicitWidth: layout.implicitWidth
    implicitHeight: layout.implicitHeight + 4

    property bool _hasVisibleButtons: reconnectButtonVisible || playButtonVisible || stopButtonVisible

    MouseArea {
        anchors.fill: parent
    }

    MediaBarBackground {
        anchors.fill: parent
        RowLayout {
            id: layout
            anchors.fill: parent
            spacing: 0
            clip: true
            Layout.fillWidth: true
            Item {
                Layout.alignment: Qt.AlignLeft
                implicitWidth: 1
                implicitHeight: MixerThemeManager.theme.mediaBarHeight - 4
                visible: !_hasVisibleButtons
            }
            RowLayout {
                visible: _hasVisibleButtons
                Layout.alignment: Qt.AlignLeft
                spacing: 0
                SensorButton {
                    icon: "Play"
                    implicitWidth: root.buttonSize
                    implicitHeight: root.buttonSize
                    enabled: root.playButtonEnabled
                    visible: root.playButtonVisible
                    backgroundOnlyOnHovered: true
                    onClicked: root.play()
                    colorType: root.colorType
                }
                SensorButton {
                    icon: "Stop"
                    implicitWidth: root.buttonSize
                    implicitHeight: root.buttonSize
                    enabled: root.stopButtonEnabled
                    visible: root.stopButtonVisible
                    backgroundOnlyOnHovered: true
                    onClicked: root.stop()
                }
            }
            RowLayout {
                Layout.alignment: Qt.AlignRight
                SensorButton {
                    icon: "Redo"
                    implicitWidth: root.buttonSize
                    implicitHeight: root.buttonSize
                    enabled: root.reconnectButtonEnabled
                    visible: root.reconnectButtonVisible
                    backgroundOnlyOnHovered: true
                    onClicked: root.reconnect()
                    tooltip: qsTr("Reconnect")
                }
                MixerLabel {
                    id: positionLabel
                    text: qsTr("Status")
                    colorType: root.colorType
                    font.pixelSize: MixerThemeManager.theme.systemFontSize
                }
                Rectangle {
                    // TODO: заменить на что-нибудь красивое
                    implicitWidth: 10
                    implicitHeight: 10
                    radius: width/2
                    color: {
                        switch (root.connectionState) {
                            case MediaSession.Playing: return "green"
                            case MediaSession.Stopped: return "grey"
                            case MediaSession.Loading: return "yellow"
                            default: return "red"
                        }
                    }
                }
            }
            Item {
                Layout.alignment: Qt.AlignRight
                implicitWidth: 4
                implicitHeight: MixerThemeManager.theme.mediaBarHeight - 4
            }
        }
    }
}
