import QtQuick 2.0
import QtQuick.Layouts 1.1
import "FormatTime.js" as FormatTime
import MixerGuiControls 1.0

Item {
    id: root
    property string colorType: "neutral"
    property real currentSlideIndex: 0
    property real slideCount: 0
   
    property bool previousButtonEnabled: true
    property bool nextButtonEnabled: true    
    property bool stopButtonEnabled: true

    signal previous()        
    signal next()     
    signal stop()     
    
    property int buttonSize: MixerThemeManager.theme.mediaBarHeight - 4 
    implicitWidth: layout.implicitWidth
    implicitHeight: layout.implicitHeight + 4
    
    MouseArea {
        anchors.fill: parent
    }
    
    MediaBarBackground {
        anchors.fill: parent
        RowLayout {
            id: layout
            anchors.fill: parent
            spacing: 0
            Item {
                implicitWidth: 1
                implicitHeight: MixerThemeManager.theme.mediaBarHeight - 4
            }
                
            SensorButton {
                icon: "Stop"
                implicitWidth: root.buttonSize
                implicitHeight: root.buttonSize
                enabled: root.stopButtonEnabled
                backgroundOnlyOnHovered: true
                onClicked: root.stop()
            }

            SensorButton {
                icon: "Previous"
                implicitWidth: root.buttonSize
                implicitHeight: root.buttonSize           
                enabled: root.previousButtonEnabled
                backgroundOnlyOnHovered: true
                colorType: root.colorType
                onClicked: root.previous()
            }
            
            SensorButton {
                icon: "Next"
                implicitWidth: root.buttonSize
                implicitHeight: root.buttonSize           
                enabled: root.nextButtonEnabled
                backgroundOnlyOnHovered: true
                colorType: root.colorType
                onClicked: root.next()
            }
            
            Item {
                Layout.fillWidth: true
            }                
            
            MixerLabel {
                horizontalAlignment: Text.AlignRight
                text: root.currentSlideIndex + 1
                font.pixelSize: MixerThemeManager.theme.systemFontSize
                colorType: root.colorType                
                MouseArea {
                    anchors.fill: parent
                    onClicked: controller.notesAction.trigger()
                }
            }            
            MixerLabel {
                horizontalAlignment: Text.AlignRight
                text: " / " + root.slideCount
                font.pixelSize: MixerThemeManager.theme.systemFontSize                        
                MouseArea {
                    anchors.fill: parent
                    onClicked: controller.notesAction.trigger()
                }
            }
            
            Item {
                implicitWidth: 5
            }                
        }
    }
}
