import QtQuick 2.1
import MixerGuiControls 1.0

Item {
    id: root
    property real value
    property string colorType: "neutral"    // "neutral", "preset", "program"
    implicitWidth: 30
    implicitHeight: background.implicitHeight
    property bool seekEnabled: false
    property bool pressed: false
    property bool restMode: false
    
    StyledImage {
        id: background
        anchors.fill: parent
        desc: MixerThemeManager.theme.imgProgressBarBgnd
    }
    
    StyledImage {
        height: parent.height
        readonly property real curPos: parent.width * Math.max(0, Math.min(root.value, 1))
        width: !restMode ? curPos : parent.width - curPos
        x: !restMode ? 0 : curPos
        visible: width >= 1
        desc: {
            if      (root.colorType == "preset")    return MixerThemeManager.theme.imgProgressBarPreview
            else if (root.colorType == "program")   return MixerThemeManager.theme.imgProgressBarProgram
            else return MixerThemeManager.theme.imgProgressBarNeutral
        }
    }
    
    StyledImage {
        anchors.fill: parent
        desc: MixerThemeManager.theme.imgProgressBarFront
    }   

    MouseArea {
        id: seekArea
        enabled: root.seekEnabled
        anchors.fill: parent
        cursorShape: enabled ? Qt.PointingHandCursor : Qt.ArrowCursor
        onPressed: {
            root.pressed = true
            root.value = Math.max(0, Math.min(mouseX / width, 1))      
        }        
        onReleased: root.pressed = false
        onEnabledChanged: root.pressed = false
        onPositionChanged: root.value = Math.max(0, Math.min(mouseX / width, 1))                        
    }
    
}
