import QtQuick 2.1
import QtQuick.Layouts 1.1
import GuiControls 1.0
import MixerGuiControls 1.0

Item {
    id: root
    property var tabs: []
    property var icons: []
    property var tabsEnabled: []
    property int currentIndex: -1
    property int glowIndex: -1
    property bool flow: false
    property int spacing: 4
    property bool movableTabs: false
    property int separatorWidth:  MixerThemeManager.theme.itemSpacing * 1.5
    property bool sameTabsWidth: false
    implicitWidth: layoutLoader.implicitWidth
    implicitHeight: layoutLoader.implicitHeight + MixerThemeManager.theme.tabGroupHeight - dummyLabel.implicitHeight
    readonly property var tabOrigin: {
        var res = []
        for (var i=0; i<rep.count; i++) {
           res.push(rep.itemAt(i).x)
        }
        return res
    }
    readonly property var tabWidth: {
        var res = []
        for (var i=0; i<rep.count; i++) {
           var wid = rep.itemAt(i).width
           if (i==rep.count-1) wid += separatorWidth + root.spacing
           res.push(wid)
        }
        return res
    }

    MixerLabel {
        id: dummyLabel
        visible: false
    }

    Component {
        id: rowLayoutComponent
        RowLayout {
            spacing: root.spacing
        }
    }

    Component {
        id: flowComponent
        Flow {
            spacing: root.spacing
        }
    }

    Loader {
        id: layoutLoader
        width: parent.width
        anchors.verticalCenter: parent.verticalCenter
        sourceComponent: root.flow ? flowComponent : rowLayoutComponent
     }

    Binding {
        target: layoutLoader.item
        property: "data"
        value: content
    }
    signal moveTab(int fromIndex, int toIndex)

    readonly property Item content: Repeater {
        id: rep
        model: root.tabs
        RowLayout {
            id: element
            spacing: root.spacing
            MixerIcon {
                id: ico
                icon: icons[model.index] ? icons[model.index] : ""
                visible: icon!=""
                colorType: label.colorType
            }
            Item {
                implicitWidth:  MixerThemeManager.theme.itemSpacing/2
                visible: false//ico.visible
            }
            MixerLabel {
                id: label
                text: modelData
                horizontalAlignment: Text.AlignHCenter
                enabled: model.index < root.tabsEnabled.length ? root.tabsEnabled[model.index] : true
                colorType: {
                    if(model.index == root.glowIndex) return "preset"
                    if(model.index == root.currentIndex) return "common"
                    return "neutral"
                }
                highlighted: colorType == "neutral" && dragAndDropSource.containsMouse
                opacity: parent.isLastRow ? 0.2 : 1.0
                Layout.fillWidth: !flow
                Layout.preferredWidth: (!sameTabsWidth || flow)? label.implicitWidth : (root.parent.width - (rep.count-1)*(separatorWidth+spacing))/rep.count
                DragAndDropSource {
                    id: dragAndDropSource
                    anchors.fill: parent
                    dragAndDropLayer: root.movableTabs ? mainWindow.dragAndDropLayer : null
                    hoverEnabled: true
                    onClicked: currentIndex = model.index
                    dragItem: MixerLabel {
                        text: label.text
                        property int dragData: model.index
                        font.pixelSize: MixerThemeManager.theme.systemFontSize
                        Drag.keys: ["MixerTabs" + content]
                    }
                }
                DropArea {
                    id: dropArea
                    anchors.fill: parent
                    keys: ["MixerTabs" + content]
                    readonly property real dragX: drag.x
                    onDropped: {
                        var fromIndex = drag.source.dragData
                        var toIndex  = drop.x < label.width/2 ? model.index : model.index + 1
                        if(fromIndex == toIndex)  return
                        if(fromIndex < toIndex) toIndex--
                        moveTab(fromIndex, toIndex)
                   }
               }
                Rectangle {
                   readonly property bool leftPart: dropArea.dragX < label.width/2
                   x: leftPart ? (model.index > 0 ? -root.separatorWidth : 0)
                               : (label.width + (model.index < rep.count-1 ? root.spacing : -width/2))
                   implicitHeight: parent.height
                   width: MixerThemeManager.theme.itemSpacing/2
                   radius: width/2
                   color: MixerThemeManager.theme.colCommon
                   //opacity: 0.5
                   visible: dropArea.containsDrag
                }
            }
            Separator {
                id: sep
                orientation: Qt.Vertical
                implicitHeight: label.implicitHeight
                Layout.preferredWidth: root.separatorWidth
                visible: model.index != root.tabs.length - 1
            }
        }
    }
}
