import QtQuick 2.1
import ".."

MixerTheme {
    themeID: "tricaster"
    name: qsTr("Dark Flat AllMix 2.0")

// Font and sizes
    fontFamily: "Tahoma"
    headerFontSize: 14
    footerFontSize: headerFontSize
    buttonTextFontSize: 16
    buttonDigitsFontSize: 20
    menuFontSize: 14
    systemFontSize: 14
    promptFontSize: 12
    headerTextEnabled: false

    panelSpacing: 8
    itemSpacing: 8
    inputsSpacingH: 24
    inputsSpacingV: headerHeight + itemSpacing

    buttonSize: 43
    oneAndHalfButtonSize: 58
    doubleButtonSize: 87
    iconSize: 25
    headerHeight: 27
    footerHeight: headerHeight
    tabGroupHeight: headerHeight
    menuHeight: headerHeight
    statusHeight: headerHeight
    mediaBarHeight: 34
    higlightFrameWidth: 3
    textOutlineSize: 1
    textGlowSize: 3
    buttonIconBlend: "hardLight"
    headerButtonsOffset: 0

    propertyHeaderHeight: headerHeight
    propertyHeaderSpacing: 1
    propertyHeaderOffsetX: 14
    propertyHeaderItemSpacing: 7
    propertyListMargins: [itemSpacing, itemSpacing/2, itemSpacing, itemSpacing/2]
    propertyListStep: headerHeight
    propertyListFontSize: systemFontSize
    propertyListColumnRatio: 0.38

    dialogButtonWidth:	dialogButtonHeight * 3
    dialogButtonHeight: headerHeight
    dialogButtonHSpacing: propertyHeaderSpacing
    dialogButtonVSpacing: itemSpacing
    dialogHeaderHeight: 30

    toolbarHeaderHeight: 36
    toolbarButtonSize:	29
    toolbarButtonSpacing: 2
    toolbarGroupSpaсing: 14

    scrollBarSpacing: 1
    scrollBarWidth: 18
    audioLedBigWidth: scrollBarWidth-2
    audioLedSmallWidth: audioLedBigWidth/2-1

// Colors
    colPreset:      colHSL(110, 70, 100)
    colPresetDark:  colHSL(110, 70,  40)
    colPresetLight: colHSL(110, 25, 100)
    colPresetBlack: colHSL(110, 100, 20)

    colProgram:      colHSL(5, 75, 100)
    colProgramDark:	 colHSL(5, 70,  40)
    colProgramLight: colHSL(5, 25, 100)
    colProgramBlack: colHSL(5, 100, 20)

    colCommon:       colHSL(40, 70, 100)
    colCommonDark:	 colHSL(40, 70,  40)
    colCommonLight:	 colHSL(40, 25, 100)
    colCommonBlack:	 colHSL(40, 100, 20)

    colNeutral:       colHSL(0, 0,  70)
    colNeutralDark:   colHSL(0, 0,  18)
    colNeutralLight:  colHSL(0, 0, 100)
    colNeutralBlack:  colHSL(0, 0,   9)

    colText: {
        "neutral": colNeutral,
        "preset": colPreset,
        "program": colProgram,
        "common" : colCommon
    }
    colTextHighlight: {
        "neutral": colNeutralLight,
        "preset": colPresetLight,
        "program": colProgramLight,
        "common" : colCommonLight
    }
    colTextOutline: {
        "neutral": colNeutralDark,
        "preset": colPresetDark,
        "program": colProgramDark,
        "common" : colCommonDark
    }
    colTextGlow: {
        "neutral": colWAlpha(colNeutralLight, 0.2),
        "preset":  colWAlpha(colPresetLight,  0.2),
        "program": colWAlpha(colProgramLight, 0.2),
        "common":  colWAlpha(colCommonLight,  0.2)
    }

    colTextInactive:	colWAlpha(colNeutral, 0.65)
    colTextDisabled:	colWAlpha(colNeutral, 0.5)

    colIcon: {
        "neutral": colText["neutral"],
        "preset": colText["preset"],
        "program": colText["program"],
        "common":  colText["common"],
    }
    colIconHighlight: {
        "neutral": colNeutralLight,
        "preset": colPresetLight,
        "program": colProgramLight,
        "common": colCommonLight
    }
    colIconOutline: {
        "neutral": colTextOutline["neutral"],
        "preset": colTextOutline["preset"],
        "program": colTextOutline["program"],
        "common": colTextOutline["common"]
    }
    colIconGlow: {
        "neutral": colTextGlow["neutral"],
        "preset": colTextGlow["preset"],
        "program": colTextGlow["program"],
        "common": colTextGlow["common"]
    }

    colSelected: {
        "neutral": colText["neutral"],
        "preset": colText["preset"],
        "program": colText["program"],
        "common": colText["common"]
    }

    colIconInactive:	colTextInactive
    colIconDisabled:	colTextDisabled

    colButtonText: {
        "neutral": colNeutral,
        "common":  colCommon,
        "preset":  colPreset,
        "program": colProgram
    }
    colButtonTextGlow: {
        "neutral": colNeutralLight,
        "common": colCommonLight,
        "preset": colPresetLight,
        "program": colProgramLight
    }
    colPressedButtonText: {
        "neutral": colNeutralBlack,
        "common":  colCommonBlack,
        "preset":  colPresetBlack,
        "program": colProgramBlack
    }
    colButtonTextDisabled: colWAlpha(colNeutral, 0.3)

// Images
    //iconFolder: "icons/"
    //Windows-Panels -----------------------------------------------------
    imgAppBgnd: StyledImageDesc {
        colFill: colNeutralBlack
    }
    imgPanelBgndFull: StyledImageDesc {
        colFill:  colHSL(0, 0,  12)
    }
    imgPanelBgndHalfTop: imgPanelBgndFull
    imgPanelBgndHalfBottom: imgPanelBgndFull

    imgWindow:  StyledImageDesc {
        source: "Resources/Window.png"
        borders: [4, 4, 20, 20]
        margins: [0, 0, 16, 16]
    }
    imgMaximizedWindow: StyledImageDesc {
        colFill:  colHSL(0, 0, 12)
    }
    imgWindowGripper: StyledImageDesc {
        source: "Resources/GripperDots.png"
        horizontalTile: true
        borders: [0, 0, 1, 0]
    }
    imgWindowCorner: StyledImageDesc {
        source: "Resources/CornerDots.png"
    }

    imgOutputBgnd: StyledImageDesc {
        colFill: colNeutralBlack
    }
    imgMediaBarBgnd: StyledImageDesc {
        colFill: colNeutralDark
        opacity:  0.5
    }
    imgSeparatorHor: StyledImageDesc {
        colFill: colNeutralBlack
        height: 3
    }
    imgSeparatorVert: StyledImageDesc {
        colFill: colNeutralBlack
        width: 3
    }

    //Clocks ------------------------------------------------------------
    imgClockAnalogBgnd: StyledImageDesc {
        source: "Resources/ClockAnalogBgnd.png"
    }
    imgClockAnalogFront: StyledImageDesc {}

    imgClockDigitalBgnd: StyledImageDesc {
       // source: "Resources/ClockDigitalBgnd.png"
       width : 74
       height: 54
       colFill: colNeutralDark
       opacity: 0
    }
    imgClockDigitalFront: StyledImageDesc {  }

    //Sliders -----------------------------------------------------------
    imgSliderThumbBig: StyledImageDesc {
        source: "Resources/SliderThumbBig.png"
        borders: buttonBorders
    }
    imgSliderThumbBigHovered: StyledImageDesc {
        source: "Resources/SliderThumbBigHover.png"
        borders: buttonBorders
    }
    imgSliderThumbSmall: StyledImageDesc {
        source: "Resources/SliderThumbSmall.png"
        margins: [3, 1, 1, 3]
    }
    imgSliderThumbSmallHovered: StyledImageDesc {
        source: "Resources/SliderThumbSmallHover.png"
        margins: [3, 1, 1, 3]
    }

    //Audio Levels -----------------------------------------------------
    imgAudioLevelBigBgnd: StyledImageDesc {
        source: "Resources/AudioLevelBigBgnd.png"
        borders: [10, 10, 10, 10]
        margins: [3, 0, 3, 0]
    }
    imgAudioLevelBigFront: StyledImageDesc {}

    imgAudioLevelBigProgress: StyledImageDesc {
        source: "Resources/AudioLevelBigProgress.png"
        margins: [3, 0, 3, 0]
    }
    imgAudioLevelBigProgressDisabled: StyledImageDesc {
        source: "Resources/AudioLevelBigProgressGray.png"
        margins: [3, 0, 3, 0]
    }
    imgAudioLevelSmallBgnd: StyledImageDesc {
        source: "Resources/AudioLevelSmallBgnd.png"
        borders: [7, 7, 7, 7]
        margins: [3, 0, 3, 0]
    }
    imgAudioLevelSmallFront: StyledImageDesc {}

    imgAudioLevelSmallProgress: StyledImageDesc {
        source: "Resources/AudioLevelSmallProgress.png"
        borders: [0, 0, 0, 1]
        horizontalTile: true
    }
    imgAudioLevelSmallProgressDisabled: StyledImageDesc {
        source: "Resources/AudioLevelSmallProgressGray.png"
        borders: [0, 0, 0, 1]
        horizontalTile: true
    }

    //Buttons --------------------------------------------
    buttonMargins: [1, 1, 1, 1]
    buttonBorders: [6, 6, 6, 6]

    imgButtonSquareNeutral: imgButtonRectNeutral
    imgButtonSquareCommon:  imgButtonRectCommon
    imgButtonSquarePreview: imgButtonRectPreview
    imgButtonSquareProgram: imgButtonRectProgram
    imgButtonSquareDisabled: imgButtonRectDisabled

    imgButtonRectNeutral: StyledImageDesc {
        source: "Resources/ButtonRectNeutral.png"
        margins: buttonMargins
        borders: buttonBorders
        colFill: colNeutralDark
    }
    imgButtonRectCommon: StyledImageDesc {
        source: "Resources/ButtonRectCommon.png"
        margins: buttonMargins
        borders: buttonBorders
        colFill: colCommon
    }
    imgButtonRectPreview: StyledImageDesc {
        source: "Resources/ButtonRectPreset.png"
        margins: buttonMargins
        borders: buttonBorders
        colFill: colPreset
    }
    imgButtonRectProgram: StyledImageDesc {
        source: "Resources/ButtonRectProgram.png"
        margins: buttonMargins
        borders: buttonBorders
        colFill: colProgram
    }
    imgButtonRectDisabled: StyledImageDesc {
        colFill:  colNeutralDark
        opacity:  0.5
    }
    imgDialogButton: imgPropertyHeader
    imgDialogButtonPressed: imgPropertyHeaderPressed

    imgButtonSensor: StyledImageDesc {
        source: "Resources/ButtonSensor.png"
        borders: [5, 5, 5, 5]
    }
    imgButtonSensorPressed: StyledImageDesc {
        source: "Resources/ButtonSensorPressed.png"
        borders: [5, 5, 5, 5]
    }

    //Sliders ----------------------------------------------
    imgSliderScaleTick: StyledImageDesc {
        source: "Resources/SliderScaleTick.png"
    }
    imgTBarGroove: StyledImageDesc {
        colFill: colNeutralBlack
        height: 5
    }
    property StyledImageDesc imgTBarThumb: StyledImageDesc {
        source: "Resources/TBarThumb.png"
        margins: [0, 0, 8, 8]
    }
    property StyledImageDesc imgTBarThumbVert: StyledImageDesc {
        source: "Resources/TBarThumbVert.png"
        margins: [0, 0, 8, 8]
    }
    imgSliderGroove: StyledImageDesc {
        colFill: colNeutralBlack
        height: 4
    }
    imgSpinnerScale: StyledImageDesc {
        source: "Resources/SpinnerScale.png"
    }
    imgSpinnerHandle: StyledImageDesc {
        source: "Resources/SpinnerHandle.png"
    }
    imgSpinnerDot: StyledImageDesc {
        source: "Resources/SpinnerDotOrange.png"
    }
    imgLampOff: StyledImageDesc {
        source: "Resources/LampOff.png"
        margins: [6, 6, 5, 5]
    }
    imgLampOnPreview: StyledImageDesc {
        source: "Resources/LampOnPreset.png"
        margins: [6, 6, 5, 5]
    }
    imgLampOnProgram: StyledImageDesc {
        source: "Resources/LampOnProgram.png"
        margins: [6, 6, 5, 5]
    }

    //Progress Bar ---------------------------------------------
    imgProgressBarBgnd: StyledImageDesc {
        source: "Resources/ProgressBarBgnd.png"
        margins: [2, 2, 2, 2]
        borders: [10, 7, 10, 7]
    }
    imgProgressBarFront: StyledImageDesc { }

    readonly property var pgMargins: [2, 2, 2, 2]
    readonly property var pgBorders: [5, 7, 5, 7]

    imgProgressBarNeutral: StyledImageDesc {
        source: "Resources/ProgressBarNeutral.png"
        margins: pgMargins;   borders: pgBorders
        colFill: colNeutral
    }
    imgProgressBarPreview: StyledImageDesc {
        source: "Resources/ProgressBarPreset.png"
        margins: pgMargins;   borders: pgBorders
        colFill: colPreset
    }
    imgProgressBarProgram: StyledImageDesc {
        source: "Resources/ProgressBarProgram.png"
        margins: pgMargins;   borders: pgBorders
        colFill: colProgram
    }
    //Properties -----------------------------------
    imgPropertyHeader:        imgButtonRectNeutral
    imgPropertyHeaderPressed: imgButtonRectNeutral

    imgExpandOn: StyledImageDesc {
        source: "Resources/ExpandOn.png"
    }
    imgExpandOff: StyledImageDesc {
        source: "Resources/ExpandOff.png"
    }
    imgSwitchOn: StyledImageDesc {
        source: "Resources/SwitchOn.png"
    }
    imgSwitchOff: StyledImageDesc {
        source: "Resources/SwitchOff.png"
    }

    //Scroller bar -----------------------------------------
    imgScrollerBgnd: StyledImageDesc {
        source: "Resources/ScrollerBgnd.png"
        borders: [2, 2, 2, 2]
    }
    imgScrollerBgndHorInner:  imgScrollerBgnd
    imgScrollerBgndVertInner: imgScrollerBgnd

    imgScrollerThumbHor: StyledImageDesc {
        source: "Resources/ScrollerThumb.png"
        borders: [5, 5, 6, 5]
        margins: [0, 0, -1, 0] //trick to fix
        horizontalTile: true
    }
    imgScrollerThumbHorInner:  imgScrollerThumbHor
    imgScrollerThumbVert: StyledImageDesc {
        source: "Resources/ScrollerThumb.png"
        borders: [5, 5, 5, 6]
        margins: [0, 0, 0, -1]
        horizontalTile: true
    }
    imgScrollerThumbVertInner: imgScrollerThumbVert

    imgThinScrollerBgnd: StyledImageDesc {
        source: "Resources/ThinScrollerBgnd.png"
        borders: [4, 4, 4, 4]
    }
    imgThinScrollerThumbHor: StyledImageDesc {
        source: "Resources/ThinScrollerThumbHor.png"
        borders: [5, 4, 6, 5]
        horizontalTile: true
    }
    imgThinScrollerThumbVert: StyledImageDesc {
        source: "Resources/ThinScrollerThumbVert.png"
        borders: [4, 5, 5, 6]
        verticalTile: true
    }

    imgKeyboardBackground: StyledImageDesc {
        source: "Resources/KeyboardBgnd.png"
    }
    imgTreeViewArrow: StyledImageDesc {
        source: "Resources/TreeViewArrow.png"
    }
}
