﻿import QtQuick 2.0
import Multimedia 1.0
import QtQuickUtils 1.0


VideoView {
    id: root

    property QtObject input

    readonly property AudioLevelMeter audioLevelMeter: AudioLevelMeter{
        stream: enabled ? input.audioStream : null
    }    

    enabled: {
        for (var i in mixer.inputSlots) {
            var inp = mixer.inputSlots[i].input
            if (inp && inp.name=='Line-balance.mp4') return true
        }
        return false
    }  

    visible: enabled
    width:  640
    height: 480

    property int offset:0
    property int frame:0
    property var dropped:0
    property var time:0
    property var t0:0
    property var times: [0,0,0,0,0]
    property var averDelay  : new arr(20)
    property var averDropped: new arr(100)
    property var plevel: false

    function arr(n) { 
        this.aver = 0
        var cur=0, data = null
        this.add = function(v) {
            if (!data) data = new Array(n)
            cur = ++cur%n
            this.aver += (v-(data[cur]||0))/n
            data[cur] = v 
        }    
    }

    onFrameTimeChanged: {
        if (!enabled) return
        if (times.indexOf(frameTime)!=-1) return
        times.shift()
        times.push(frameTime)

        var t = new Date().getTime()
        var dt = t-t0; t0 = t;

        if (!time) time = frameTime
        var d = frameTime - time; time = frameTime

        var w = 600
        var cols = ColorPicker.pick(root,Qt.rect(0,10,width,0))
        var pos = 0, x = 0, len = cols.length; 

        //if (!root.frame) p('len='+len+' w='+width)

        for (var i=0; i<len; i++) {
            var c = cols[i]&255; if (c<32) continue
            x = i; 
            pos = Math.round((x+0.25)*640/len)-20
            break
        }

        var frame; 
        if ((frame = pos+offset) < root.frame-w/2) offset += w
        if ((frame = pos+offset) < root.frame-w/2) offset += w
        var delta = frame-root.frame-1; root.frame = frame;
        var tframe = Math.round(frameTime*60/1000)
        var d = frame-tframe
        var level0 = root.audioLevelMeter.levels[0]
        var level1 = root.audioLevelMeter.levels[1]
        var delay = level1>0.01 ? pos%60-7 : ''
        
        if (frame>100) {
            if (delay!='') averDelay.add(delay)
            averDropped.add(dropped += delta)
        }    

        var level = level0>0.02;
        if (level!=plevel) {
            plevel = level
            p('frame '+frame+' audio '+(plevel ? 'play':'paused'))
        }

        function s(v) { return (v<0?'':' ')+v }
        //if (delta || delay)
        var mess = 'pos='+pos+'('+x+') audio='+frame%60+' video='+frame+' '+frameTime+' ' +dt+' tframe='+tframe+s(d)+' delta='+s(delta)+' '+" dropped:"+s(dropped)+' levels='+level0.toFixed(3)+','+level1.toFixed(3)+' !'+delay;
        //p(mess)
        input.message = /*"dropped: "+averDropped.aver.toFixed(0)+*/ "\n\ndelay: "+averDelay.aver.toFixed(0)+'('+(averDelay.aver*1000/60).toFixed(0)+'ms)'
    }    
}            
