import QtQuick 2.0
import Multimedia 1.0
import GuiControls 1.0

PropertyGroup {
    property var mediaTypes
    property alias property: mediaTypesEnum.property
    onMediaTypesChanged: filtersGroup.object.reset()
    collapsible: false

    EnumProperty {
        id: mediaTypesEnum
        label: qsTr("Video Mode")
        readonly property var filteredTypes: {
            var result = []
            for (var i in mediaTypes) {
                var mediaType = mediaTypes[i]
                if (!filtersGroup.object.checkMatching(mediaType))
                    continue
                result.push(mediaType)
            }
            return result
        }
        strings: values.map( function(item){ return item.toString() })
        values: filteredTypes
        findItemFunction: function (value) {
            var index = -1
            if (values) index = values.indexOf(value)
            if (!!value && index < 0) index = 0
            return index
        }
        //comboBoxImplicitWidth: 200
    }

    PropertyGroup {
        id: filtersGroup
        collapsible: false
        offseted: true
        object: QtObject {
            property var frameSize
            property var frameRate
            property var subtype

            function reset() {
                frameSize = undefined
                frameRate = undefined
                subtype = undefined
            }

            function findUniqueTypes(checkMatchingFn, checkUnique) {
                var result = []
                result.push(undefined)
                for (var i in mediaTypes) {
                    var mediaType = mediaTypes[i]
                    if (!checkMatchingFn(mediaType))
                        continue

                    var isUnique = true
                    if (mediaType)
                        for (var j in result) {
                            if (result[j] && mediaType && checkUnique(result[j], mediaType)) {
                                isUnique = false
                                break
                            }
                        }
                    else
                        isUnique = false
                    if (isUnique)
                        result.push(mediaType)
                }
                return result
            }

            function checkMatchingFrameSize(mediaType) {
                if (!!frameSize && frameSize.width > 0 && frameSize.height > 0 &&
                        mediaType.frameSize != frameSize) {
                    return false
                }
                return true
            }

            function checkMatchingFrameRate(mediaType) {
                if (!!frameRate && (mediaType.getRate() != frameRate)) {
                    return false
                }
                return true
            }

            function checkMatchingSubtype(mediaType) {
                if (!!subtype && mediaType.subtypeName() != subtype) {
                    return false
                }
                return true
            }

            function checkMatching(mediaType) {
                return checkMatchingFrameSize(mediaType) && checkMatchingFrameRate(mediaType) && checkMatchingSubtype(mediaType)
            }

            function checkMatchingWoFrameSize(mediaType) {
                return checkMatchingFrameRate(mediaType) && checkMatchingSubtype(mediaType)
            }

            function checkMatchingWoFrameRate(mediaType) {
                return checkMatchingFrameSize(mediaType) && checkMatchingSubtype(mediaType)
            }

            function checkMatchingWoSubtype(mediaType) {
                return checkMatchingFrameSize(mediaType) && checkMatchingFrameRate(mediaType)
            }

        }
        EnumProperty {
            property: "frameSize"
            label: qsTr("Frame Size")
            strings: !values ? [] : values.map( function (item){ return !item ? qsTr("<Any>") : (item.width + " x " + item.height) })
            values: {
                var filteredMediaTypes = object.findUniqueTypes(object.checkMatchingWoFrameSize, function(mt1, mt2) { return mt1.frameSize == mt2.frameSize })
                return !filteredMediaTypes ? [] : filteredMediaTypes.map(function(mt) { return mt ? mt.frameSize : undefined })
            }
            findItemFunction: function (value) {
                var index = -1
                if (values) index = values.indexOf(value)
                if (!!value && index < 0) index = 0
                return index
            }
        }
        EnumProperty {
            property: "frameRate"
            label: qsTr("Frame Rate")
            strings: !values ? [] : values.map( function (item){ return !item ? qsTr("<Any>") : parseFloat(item.toFixed(2)).toString() + " fps"})
            values: {
                var filteredMediaTypes = object.findUniqueTypes(object.checkMatchingWoFrameRate, function(mt1, mt2) { return mt1.getRate() == mt2.getRate() })
                return !filteredMediaTypes ? [] : filteredMediaTypes.map(function(mt) { return mt ? mt.getRate() : undefined })
            }
            findItemFunction: function (value) {
                var index = -1
                if (values) index = values.indexOf(value)
                if (!!value && index < 0) index = 0
                return index
            }
        }
        EnumProperty {
            property: "subtype"
            label: qsTr("Format")
            strings: !values ? [] : values.map( function (item){ return !item ? qsTr("<Any>") : item })
            values: {
                var filteredTypes = object.findUniqueTypes(object.checkMatchingWoSubtype, function(mt1, mt2) { return mt1.subtype == mt2.subtype })
                return !filteredTypes ? [] : filteredTypes.map( function(mt) { return mt ? mt.subtypeName() : undefined})
            }
            findItemFunction: function (value) {
                var index = -1
                if (values) index = values.indexOf(value)
                if (!!value && index < 0) index = 0
                return index
            }
        }
    }
}

