import QtQuick 2.2
import QtQuick.Controls.Styles 1.0
import GuiStyling 1.0
import "." as ECGStyle
import MixerGuiControls 1.0

ComboBoxStyle {
    id: root
    readonly property var theme: MixerThemeManager.theme

    font.family:    theme.fontFamily
    font.pixelSize: theme.systemFontSize
    renderType: Text.QtRendering
    readonly property int _pad: theme.controlPadding

    padding {
        top: _pad; bottom: _pad
        left: _pad; right: _pad
    }

    dropDownButtonWidth: theme.propertyHeaderHeight

    background: TextEditBackground {
        implicitWidth:  200
        implicitHeight: theme.systemFontSize + padding.top + padding.bottom + borderWidth*2 //Math.max(20, Math.round(control.__contentHeight  * 1.2))
        focused: control.activeFocus
        enabled: control.enabled
        backgroundColor: theme.colNeutralDark
        cornerRadius: 0
        borderColor: theme.colNeutral
        disabledBorderColor: theme.colTextDisabled
        focusedBorderColor: theme.colCommon
        borderWidth: theme.controlBorderSize
        ButtonBackground {
            width: dropDownButtonWidth
            height: parent.height
            anchors.right: parent.right
            pressed: control.pressed
            hovered: control.hovered
            focused: control.activeFocus
            enabled: control.enabled
            gradientType: "none"
            normalColor:  theme.colNeutralDark
            pressedColor: normalColor
            hoveredColor: normalColor
            disabledColor: normalColor
            borderColor: theme.colNeutral
            disabledBorderColor: focused ? theme.colCommon : theme.colTextDisabled
            focusedBorderColor: theme.colCommon
            borderWidth: theme.controlBorderSize
            cornerRadius: 0
            MixerIcon {
                anchors.centerIn: parent
                source: "icons/black/arrow_down.png"
                color: {
                    if (!enabled) return theme.colTextDisabled
                    if (pressed)  return theme.colCommon
                    if (hovered)  return theme.colNeutralLight
                    return theme.colNeutral
                }
            }
        }
    }
    __editor: TextEditBackground {
        backgroundColor: "transparent"
        borderWidth: 0
    }

    __dropDownStyle: MixerMenuStyle {
        __maxPopupHeight: 400
        __menuItemType: "comboboxitem"
        __scrollerStyle: ECGStyle.ScrollViewStyle {}
    }
 }
