import QtQuick 2.2
import QtQuick.Controls.Styles 1.1
import MixerGuiControls 1.0

SpinBoxStyle {
    id: root
    readonly property var theme: MixerThemeManager.theme
    
    font.family:    theme.fontFamily
    font.pixelSize: theme.systemFontSize
    renderType: Text.QtRendering
    readonly property int _pad:  theme.controlPadding
    horizontalAlignment: Qt.AlignLeft

    Component {
        id: arrowControl
        ButtonBackground {
            id: bgnd
            property bool up: true
            implicitWidth: theme.propertyHeaderHeight
            pressed: up ? styleData.upPressed : styleData.downPressed
            hovered: up ? styleData.upHovered : styleData.downHovered
        focused: control.activeFocus
            enabled: control.enabled && (up ? styleData.upEnabled : styleData.downEnabled)
            gradientType: "none"
            normalColor:  theme.colNeutralDark
            pressedColor: normalColor
            hoveredColor: normalColor
            disabledColor: normalColor
            borderColor: theme.colNeutral
            disabledBorderColor: focused ? theme.colCommon : theme.colTextDisabled
            focusedBorderColor: theme.colCommon
            borderWidth: theme.controlBorderSize
            cornerRadius: 0
            MixerIcon {
            anchors.centerIn: parent      
                source: up ? "icons/black/arrow_up.png" : "icons/black/arrow_down.png"
                color: {
                    if (!enabled) return theme.colTextDisabled
                    if (pressed)  return theme.colCommon
                    if (hovered)  return theme.colNeutralLight
                    return theme.colNeutral
                }
        }
    }
        }        
    incrementControl: arrowControl
    decrementControl: Loader {
       id: decCtrl
       sourceComponent: arrowControl
       onLoaded:  { item.up = false }
    }
    padding {
        top: _pad; bottom:_pad
        left: _pad; right: _pad
    }
    background: TextEditBackground {
        id: textBgnd
        Text  {
            id: digits;
            font: root.font;
            visible: false
            text: control.digits ? -Math.pow(10,-control.digits):'';
        }
        implicitHeight: root.font.pixelSize + padding.top + padding.bottom + borderWidth*2 //Math.max(20, Math.round(styleData.contentHeight * 1.2))
        implicitWidth: (control.digits ? Math.max(control.textItem.implicitWidth, digits.implicitWidth) : styleData.contentWidth) + padding.left + padding.right + implicitHeight
        focused: control.activeFocus
        enabled: control.enabled
        backgroundColor: theme.colNeutralDark
        cornerRadius: 0
        borderColor: theme.colNeutral
        disabledBorderColor: theme.colTextDisabled
        focusedBorderColor: theme.colCommon
        borderWidth: theme.controlBorderSize
        baselineOffset: control.__baselineOffset
    }
}