import QtQuick 2.2
import QtQuick.Controls.Styles 1.0
import GuiStyling 1.0
import "." as ECGStyle

TableViewStyle {
    id: root
    property var headerTheme: StyleManager.theme.header
    property var textEditTheme: StyleManager.theme.textEdit
    backgroundColor: textEditTheme.backgroundColor
    
    headerDelegate: HeaderBackground {
        height: textItem.implicitHeight * 1.4
        pressed: false
        hovered: false
        enabled: control.enabled
        normalColor: headerTheme.normalColor
        pressedColor: headerTheme.pressedColor
        hoveredColor: headerTheme.hoveredColor
        disabledColor: headerTheme.disabledColor
        borderColor: StyleManager.theme.borderColor
        disabledBorderColor: StyleManager.theme.disabledBorderColor
        borderWidth: StyleManager.theme.borderWidth
  
        Text {
            id: textItem
            anchors.fill: parent
            verticalAlignment: Text.AlignVCenter
            horizontalAlignment: styleData.textAlignment
            anchors.leftMargin: implicitHeight * 0.5
            text: styleData.value
            elide: Text.ElideRight
            font: StyleManager.theme.font
            color: control.enabled ? headerTheme.textColor : headerTheme.disabledTextColor
            renderType: Text.NativeRendering
        }
        Rectangle {
            anchors.right: parent.right
            anchors.top: parent.top
            anchors.bottom: parent.bottom
            anchors.bottomMargin: 1
            anchors.topMargin: 1
            width: 1
            color: textEditTheme.backgroundColor
        }
    }
    
    highlightedTextColor: textEditTheme.highlightedTextColor
    rowDelegate: Rectangle {
        height: Math.round(StyleManager.textHeight * 1.2)
        color: styleData.alternate ? textEditTheme.backgroundColor2 : textEditTheme.backgroundColor
        Rectangle {
            visible: styleData.selected
            anchors.fill: parent
            color: textEditTheme.highlightColor
        }
    }
    
    itemDelegate: Item {
        height: Math.max(16, label.implicitHeight)
        property int implicitWidth: label.implicitWidth + label.implicitHeight * 0.2
        Text {
            id: label
            objectName: "label"
            width: parent.width
            anchors.leftMargin: implicitHeight * 0.5
            anchors.left: parent.left
            anchors.right: parent.right
            horizontalAlignment: styleData.textAlignment
            anchors.verticalCenter: parent.verticalCenter
            anchors.verticalCenterOffset: 1
            elide: styleData.elideMode
            text: styleData.value !== undefined ? styleData.value : ""
            color: styleData.textColor
            font: StyleManager.theme.font
            renderType: Text.NativeRendering
        }
    }    
        
    // ScrollView
    property var scrollViewStyle: ECGStyle.ScrollViewStyle {}
    scrollBarBackground: scrollViewStyle.scrollBarBackground
    corner: scrollViewStyle.corner
    handle: scrollViewStyle.handle
    incrementControl: scrollViewStyle.incrementControl
    decrementControl: scrollViewStyle.decrementControl
}