import QtQuick 2.2
import QtQuick.Controls.Styles 1.0
import QtQuick.Controls.Private 1.0
import GuiStyling 1.0

ToolButtonStyle {
    id: root
    property var toolButtonTheme: StyleManager.theme.toolButton
    property bool pressed: control.pressed || (control.checkable && control.checked)
    panel: ButtonBackground {
        implicitWidth:  hasIcon ? 18 : Math.max(label.implicitWidth, 18)
        implicitHeight: hasIcon ? 18 : Math.max(label.implicitHeight, 18)      
        readonly property bool hasIcon: icon.status === Image.Ready || icon.status === Image.Loading

        pressed: root.pressed
        hovered: control.hovered
        focused: control.activeFocus
        enabled: control.enabled
        gradientType: pressed ? toolButtonTheme.pressedGradientType : toolButtonTheme.normalGradientType
        normalColor: toolButtonTheme.normalColor
        normalColor2: toolButtonTheme.normalColor2
        pressedColor: toolButtonTheme.pressedColor
        pressedColor2: toolButtonTheme.pressedColor2
        hoveredColor: toolButtonTheme.hoveredColor
        disabledColor: toolButtonTheme.disabledColor
        borderColor: toolButtonTheme.borderColor
        disabledBorderColor: toolButtonTheme.disabledBorderColor
        focusedBorderColor: StyleManager.theme.focusedBorderColor
        borderWidth: toolButtonTheme.borderWidth
        cornerRadius: toolButtonTheme.cornerRadius
        
        Item {
            anchors.fill: parent
            clip: true
            Text {
                id: label
                visible: !hasIcon
                anchors.centerIn: parent
                font: StyleManager.theme.font
                text: StyleHelpers.stylizeMnemonics(control.text)
                color: !control.enabled ? toolButtonTheme.disabledTextColor : (root.pressed ? toolButtonTheme.pressedTextColor : toolButtonTheme.textColor)
            }
            Image {
                id: icon
                anchors.centerIn: parent
                source: control.iconSource
            }
        }       
        
    }
}