import QtQuick 2.0

Item {
    id: root
    property real aspect: 1.0
    property int fillMode: Image.PreserveAspectFit
    property int alignment: Qt.AlignVCenter|Qt.AlignHCenter
    default property alias content: contentItem.data
    
    clip: fillMode == Image.PreserveAspectCrop
    
    function calcAlignedX(w) {
        if(alignment & Qt.AlignLeft) return 0
        else if(alignment & Qt.AlignRight) return root.width - w
        else return root.width/2 - w/2
    }    
    function calcAlignedY(h) {
        if(alignment & Qt.AlignTop) return 0
        else if(alignment & Qt.AlignBottom) return root.height - h
        else return root.height/2 - h/2
    }
    function calcPreserveAspectFitRect() {
        var x, y, w, h
        if(aspect > root.width/root.height) {
            w = root.width
            x = 0
            h = root.width / aspect
            y = calcAlignedY(h)
        }
        else {
            w = aspect * root.height
            x = calcAlignedX(w)
            h = root.height
            y = 0
        }
        return Qt.rect(x, y, w, h)
    }
    function calcPreserveAspectCropRect() {
        var x, y, w, h
        if(aspect > root.width/root.height) {
            w = aspect * root.height
            x = calcAlignedX(w)
            h = root.height
            y = 0	
        }
        else {
            w = root.width
            x = 0        
            h = root.width / aspect
            y = calcAlignedY(h)
        }
        return Qt.rect(x, y, w, h)        
    }
    readonly property rect contentRect: {
        if(aspect == 0) return Qt.rect(0, 0, root.width, root.height)
        if(fillMode == Image.PreserveAspectFit) return calcPreserveAspectFitRect()
        if(fillMode == Image.PreserveAspectCrop) return calcPreserveAspectCropRect()
        else return Qt.rect(0, 0, root.width, root.height)
    }
    readonly property rect croppedContentRect: {
        var x0 = Math.max(0, contentRect.x)
        var y0 = Math.max(0, contentRect.y)
        var x1 = Math.min(root.width, contentRect.x + contentRect.width)
        var y1 = Math.min(root.height, contentRect.y + contentRect.height)
        return Qt.rect(x0, y0, x1 - x0, y1 - y0)
    }
    Item {
        id: contentItem
        x: contentRect.x
        y: contentRect.y
        width: contentRect.width
        height: contentRect.height
    }
}
