import QtQuick 2.0
import QtQml 2.2
import "packer.growing.js" as P

Item {
    id: root
    readonly property alias needRepack: d.needRepack
    signal packed()
    
    function pack() {
        d.pack()
    }

    onChildrenChanged: d.needRepack = true
    
    QtObject {
        id: d
        property bool needRepack: true
        readonly property Instantiator connections: Instantiator {
            model: root.children
            Connections {
                target: modelData
                onImplicitWidthChanged: d.needRepack = true
                onImplicitHeightChanged: d.needRepack = true
            }
        }

        function pack() {
            if(!needRepack) return
            if(root.children.length == 0) {
                root.width = 0
                root.height = 0
                return
            }
            
            var packer = P.GrowingPacker()
            var rects = []
            for(var i = 0; i < root.children.length; i++) {
                var item = root.children[i]
                rects.push({w: item.implicitWidth, h: item.implicitHeight, item: item})
            }
            rects.sort(function(obj1, obj2) {
                var s1 = obj1.w * obj1.h
                var s2 = obj2.w * obj2.h
                return s2 - s1
            })
            packer.fit(rects)

            var W = 0
            var H = 0
            for(var i = 0; i < rects.length; i++) {
                var r = rects[i]
                var item = r.item
                item.x = r.fit.x
                item.y = r.fit.y
                item.width = r.w
                item.height = r.h
                W = Math.max(W, r.fit.x + r.w)
                H = Math.max(H, r.fit.y + r.h)
            }
            root.width = W
            root.height = H
            
            needRepack = false
            packed()            
        }
    }
    
   
}
