import QtQuick 2.0
import "boxpack.js" as P

Item {
    id: root
    readonly property alias scale: d.scale
    readonly property alias needRepack: d.needRepack
    signal packed()
    
    function pack() {
        d.pack()
    }
    
    onWidthChanged: d.needRepack = true
    onHeightChanged: d.needRepack = true
        
    QtObject {
        id: d
        property bool needRepack: true
        property real scale: 1.0
        readonly property var childrenInfo: {
            var result = []
            for(var i = 0; i < root.children.length; i++) {
                var item = root.children[i]
                result.push({width: item.implicitWidth, height: item.implicitHeight, item: item})
            }
            result.sort(function(obj1, obj2) {
                var s1 = Math.max(obj1.width, obj1.height)
                var s2 = Math.max(obj2.width, obj2.height)
                return s2 - s1
            })        
            return result
        }
    
        onChildrenInfoChanged: needRepack = true

        function getSortFunction(aspect) {
            return function(a, b) {
                var d1 = Math.max(a.x, a.y * aspect)
                var d2 = Math.max(b.x, b.y * aspect)
                return d1 - d2
            }
        }
        
        function pack() {
            if(!needRepack || childrenInfo.length < 0) return
            var packer = P.boxpack({algo: getSortFunction(root.width/root.height)})
            var result = packer.pack(childrenInfo)
            var W = 0
            var H = 0
            for(var i = 0; i < result.length; i++) {
                var obj = result[i]
                W = Math.max(W, obj.x + obj.width)
                H = Math.max(H, obj.y + obj.height)
            }
            
            var scale = Math.min(root.width/W, root.height/H)
            if(scale > 1.0) scale = 1.0
            for(var i = 0; i < result.length; i++) {
                var obj = result[i]
                var item = obj.item
                item.x = obj.x * scale
                item.y = obj.y * scale
                item.width = obj.width * scale
                item.height = obj.height * scale
            }
            d.scale = scale
            needRepack = false
            packed()
        }
    }  
}
