import QtQuick 2.0
import Serialization 1.0

QtObject {
    id: root
    property QtObject object
    property url fileUrl
    readonly property alias savedData: d.delayedSavedData
    
    readonly property QtObject internal: QtObject {
        id: d
        property bool initialized: false
        property bool loading: false
        property bool saveRequestedCounter: false // у сохраняемого объекта может быть сигнал saveRequested, по которому меняем saveRequestedCounter, чтобы вызвать сохранение
        readonly property var savedData: {
            if(!initialized || loading) return undefined
            saveRequestedCounter
            return object.save()
        }
        property var delayedSavedData
        readonly property Timer saveTimer: Timer {
            interval: 100
            onTriggered: d.saveToFile()
        }
        readonly property Connections connections: Connections {
            target: object
            onSaveRequested: d.saveRequestedCounter = !d.saveRequestedCounter
            ignoreUnknownSignals: true
        }
        onSavedDataChanged: if(initialized && !loading) saveTimer.restart()
        
        function saveToFile() {
            delayedSavedData = savedData
            if(!savedData) return
            Serialization.writeToJsonFileAsync(fileUrl, savedData, null)
        }
        
        function loadFromFile() {
            loading = true
            var data = Serialization.readFromJsonFile(fileUrl) || {}
            if(data) {
                object.load(data)
            } else {
                print("Can't load file " + fileUrl)
            }
            loading = false
        }
    }

    function init() {
        if(Serialization.isFileExists(fileUrl)) {
            d.loadFromFile()
        } else {
            if(object.createDefault !== undefined) object.createDefault()
            d.saveToFile()
        }
        d.initialized = true
    }
    
    Component.onDestruction: {
        if(d.saveTimer.running) {
            d.saveTimer.stop()
            d.saveToFile()
        }
    }
}
