import QtQuick 2.4
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.0
import "utils/FormatTime.js" as FormatTime

MixerPanelSimple {
    id: root
    title: qsTr("Events")

    readonly property var events: archivePlayer && archivePlayer.events || []

    function getTypeColor(type) {
        for (var i in availableTypes) if (availableTypes[i].name==type) return availableTypes[i].color
        return 'gray'
    }

    TableView {
        id: tableView
        anchors.fill: parent
        model: root.events
        headerVisible: false
        backgroundVisible:false
        alternatingRowColors:false 
        frameVisible: root.events.length
        horizontalScrollBarPolicy: Qt.ScrollBarAlwaysOff
        selectionMode: 0

        TableViewColumn {}

        rowDelegate: Rectangle {
            height: 16
            color: 'transparent' 
        }

        itemDelegate: Item {
            readonly property var event: modelData
            RowLayout {
                anchors.fill: parent
                anchors.leftMargin : 5
                anchors.rightMargin: 5
                spacing: 4

                Text {
                    font.pixelSize: 11
                    verticalAlignment: Text.AlignVCenter
                    text: {
                        if(!event) return ""
                        var time = (timeState.timeMode == 1 && event.boardTime > -0.5) ? event.boardTime : timeState.trainingTimeToCurrentModeTime(event.time)
                        return "[" + FormatTime.millisecondsToString(time) + "]"
                    }
                    color: "white"
                }
                Text {
                    font.pixelSize: 12
                    verticalAlignment: Text.AlignVCenter
                    text: event ? event.comment : ""
                    color: getTypeColor(event && event.type)
                    styleColor: '#606060'
                    elide: Text.ElideRight
                    Layout.fillWidth: true
                }
            }
            MouseArea {
                id: mouseArea
                anchors.fill: parent
                hoverEnabled: true
                cursorShape: Qt.PointingHandCursor
                onClicked: archivePlayer.setPosition(event.time)
            }
        }
    }
}
