import QtQuick 2.0
import QtQuick.Layouts 1.1
import QtQuick.Controls 1.0
import QtQuick.Controls.Styles 1.2
import "utils/Conversion.js" as Conversion
import "utils/FormatTime.js" as FormatTime

MixerPanelSimple {
    id: root
    title: qsTr("Archive")

    readonly property var recordedArchive: archiveRecorder && archiveRecorder.archive || false
    readonly property var currentArchive: archiveCollection && archiveCollection.currentArchive
    readonly property var archive: archiveCollection && archiveCollection.archives && archiveCollection.archives[0] || {}
    
    readonly property var isClosing: postPlayStatus==2 
    readonly property var diskSize: archive.diskSize || 0 
    readonly property var duration: archive.duration || 0 
    readonly property var date    : archive.creationDate || new Date() 

    Button {
        x:0; y:3
        activeFocusOnTab: false
        //anchors.verticalCenter: parent.verticalCenter
        style: ButtonStyle {
            padding {left: 0; right: 0; top: 0; bottom: 0}
            background: Image {
                source: recordedArchive ? "RecordWriting.png" : currentArchive ? "RecordReading.png" : "RecordDefault.png"
            }
            label: Item {}
        }
        action: archiveCollection && archiveCollection.currentArchive ? closeArchiveAction : openArchiveAction
        Component.onCompleted: __behavior.cursorShape = Qt.PointingHandCursor
    }

    Text {
        x:5; y:60
        text: date.toLocaleString(Qt.locale(), "d MMMM yyyy г. hh:mm:ss")
        font.pixelSize: fontSize-1
        color: colNeutral
    }

    Text {
        x:5; y:80
        text: qsTr("Size - ") + Conversion.bytesToMB(diskSize).toFixed() + " MB"
        font.pixelSize: fontSize-1
        color: colNeutral
        visible: !isClosing
    }            

    Text {
        x:5; y:80
        text: qsTr('Registrator is down')
        color: 'red'
        font.pixelSize: fontSize-1
        visible: isClosing
    }            

    Action {
        id: openArchiveAction
        text: qsTr("Open")
        onTriggered: {
            archiveCollection.setCurrent(0)
            //archiveRecorder.start('ttt')
        }
    }

    Action {
        id: closeArchiveAction
        text: qsTr("Close")
        onTriggered: {
            archiveCollection.setCurrent(-1)
            //app.doWithQuestion(qsTr("Do you really want to stop recording?"), archiveCollection.setCurrent(-1))
        }
    }

    Connections {
        target: archiveRecorder.recordingSettings || null
        onValidated: {
            if(error) {
                app.openErrorDialog(error)
                return
            }
            archiveCollection.removeEmptyArchives()
            startRecordingDialog.text = qsTr("Training")
            startRecordingDialog.open()
        }
    }
        
}
