import QtQuick 2.0
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.0
import "utils"

MixerPanelSimple {
    id: root
    property int selectedIndex: -1
    property bool archived: false

    title: archived ? qsTr("Archived Streams") : qsTr("Video Streams")

    readonly property var streams: (videoStreamCollection && videoStreamCollection.streams || []).filter(function(stream) { return stream.archived==archived; })

    ListView {
        id: scrollView
        anchors.fill: parent
        //width: 270
        flickableDirection: Flickable.VerticalFlick 
        interactive: !mainWindow.dragAndDropLayer.dragActive
        currentIndex: streams.indexOf(videoStreamCollection.currentStream)

        spacing: 0
        clip: true
        model: streams
        delegate: Item {
            implicitWidth: videoStreamPreview.implicitWidth + 6
            implicitHeight: videoStreamPreview.implicitHeight + 6
            VideoStreamPreview {
                id: videoStreamPreview
                anchors.centerIn: parent
                stream: modelData
                isSelected: stream && stream == videoStreamCollection.currentStream
                onRightButtonClicked: {
                    selectedIndex = videoStreamCollection.streams.indexOf(modelData)
                    contextMenu.popup()
                }
                onClicked: videoStreamCollection.toggleCurrentStream(stream,false,currentMonitorIndex)
                onDropped: {
                    var from = videoStreamCollection.streams.indexOf(drag.source.dragData)
                    var to   = videoStreamCollection.streams.indexOf(target)
                    if (to>=0 && drag.x > width/2) to += 1;
                    if (from<0 || to<0 || from==to) return;
                    videoStreamCollection.move(from,to)
                }
            }
        }
    }

/*
    ListView {
        id: scrollView
        anchors.fill: parent
        //width: 270
        spacing: 0
        clip: true
        model: (videoStreamCollection && videoStreamCollection.streams || []).filter(function(stream) { return stream.archived==archived; } )
        delegate: Item {
            implicitWidth: videoStreamPreview.implicitWidth + 6
            implicitHeight: videoStreamPreview.implicitHeight + 6
            VideoStreamPreview {
                id: videoStreamPreview
                anchors.centerIn: parent
                stream: modelData
                isSelected: stream && stream == videoStreamCollection.currentStream
                onRightButtonClicked: {
                    selectedIndex = videoStreamCollection.streams.indexOf(modelData)
                    contextMenu.popup()
                }
                onClicked: videoStreamCollection.toggleCurrentStream(stream,false,currentMonitorIndex)
                onDropped: {
                    var from = videoStreamCollection.streams.indexOf(drag.source.dragData)
                    var to   = videoStreamCollection.streams.indexOf(target)
                    if (to>=0 && drag.x > width/2) to += 1;
                    if (from<0 || to<0 || from==to) return;
                    videoStreamCollection.move(from,to)
                }
            }
        }
    }
*/    
/*
    ScrollView {
        id: scrollView
        anchors.fill: parent
        //style: MixerScrollViewStyle {}
        verticalScrollBarPolicy: Qt.ScrollBarAlwaysOff
        horizontalScrollBarPolicy: Qt.ScrollBarAlwaysOff
        frameVisible: true
        Flow {
            width: 270//Math.max(300, scrollView.viewport.width)
            spacing: 0
            Repeater {
                id: repeater
                model: (videoStreamCollection && videoStreamCollection.streams || []).filter(function(stream) { return stream.archived==archived; } )
                Item {
                    implicitWidth: videoStreamPreview.implicitWidth + 6
                    implicitHeight: videoStreamPreview.implicitHeight + 6
                    VideoStreamPreview {
                        id: videoStreamPreview
                        anchors.centerIn: parent
                        stream: modelData
                        isSelected: stream && stream == videoStreamCollection.currentStream
                        onRightButtonClicked: {
                            selectedIndex = videoStreamCollection.streams.indexOf(modelData)
                            contextMenu.popup()
                        }
                        onClicked: videoStreamCollection.toggleCurrentStream(stream,false,currentMonitorIndex)
                        onDropped: {
                            var from = videoStreamCollection.streams.indexOf(drag.source.dragData)
                            var to   = videoStreamCollection.streams.indexOf(target)
                            if (to>=0 && drag.x > width/2) to += 1;
                            if (from<0 || to<0 || from==to) return;
                            videoStreamCollection.move(from,to)
                        }
                    }
                }
            }
        }
    }
*/


/*
    RowLayout {
        anchors.fill: parent
        anchors.margins: 7
        spacing: 6
        Item {
            id: streamList
            Layout.fillWidth: true
            Layout.fillHeight: true
            clip: true
            readonly property bool atBeginning: currentIndex <= 0
            readonly property bool atEnd: currentIndex >= maximumIndex
            readonly property bool scrollNeeded: content.width > root.width
            property int count: 0
            property int currentIndex: -1
            readonly property int maximumIndex: {
                if(repeater._reloading) return count - 1
                for(var i = 0; i < count; i++) {
                    var item = repeater.itemAt(i)
                    if(item && item.x >= content.width - streamList.width) return i
                }
                return count - 1
            }
            ColumnLayout {
                id: content
                height: parent.height
                spacing: 6
                
                Behavior on x {
                    NumberAnimation { duration: 200 }
                }
                
                Repeater {
                    id: repeater
                	model: (videoStreamCollection && videoStreamCollection.streams || []).filter(function(stream) { return stream.archived==archived; } )
                    VideoStreamPreview {
                        stream: modelData
                        fontSize: 13
                        iconHeight: 93
                    }
                }
                onWidthChanged: streamList.updateContentPosition()
            }
            
            function moveToNext() {
                currentIndex = Math.min(currentIndex + 1, maximumIndex)
            }
            
            function moveToPrevious() {
                currentIndex = Math.max(currentIndex - 1, 0)
            }
            
            function moveToEnd() {
                currentIndex = maximumIndex
            }
            
            function updateContentPosition() {
                var item = repeater.itemAt(currentIndex)
                if(!item || !scrollNeeded) {
                    content.x = 0
                    return
                }
                content.x = -Math.min(item.x, content.width - streamList.width)
            }
            
            onWidthChanged: updateContentPosition()
            onCurrentIndexChanged: updateContentPosition()
            onScrollNeededChanged: updateContentPosition()
            onMaximumIndexChanged: currentIndex = Math.min(currentIndex, maximumIndex)
        }
    }
*/    
    DropArea {
        id: streamDropArea
        anchors.fill: parent
        keys: ["FromMosaicPanel"]
        Rectangle {
            anchors.fill: parent
            color: Qt.rgba(147 / 255, 222 / 255, 255 / 255, 0.2)
            border.color: "black"
            visible: streamDropArea.containsDrag
        }
        onDropped: if(drag.source.dropCallback) drag.source.dropCallback(null)
    } 
}