import QtQuick 2.0
import QtQuick.Controls 1.1
import Rpc 1.0
import "../../common/utils"

import Serialization 1.0

Item {
    id: root
    readonly property bool isAndroid: true
    readonly property alias server: rpcServer.object
    readonly property var videoStreamCollection: server && server.videoStreamCollection || []
    readonly property var audioStreamCollection: server && server.audioStreamCollection || []
    readonly property var availableMedias      : server && server.availableMedias       || []
    readonly property var templateCollection   : server && server.templateCollection    || []
    readonly property var mosaicPresets        : server && server.mosaicPresets         || []
    readonly property var archiveCollection    : server && server.archiveCollection     || []
    readonly property var archiveRecorder      : server && server.archiveRecorder       || []
    readonly property var archivePlayer        : server && server.archivePlayer         || []
    readonly property var eventTypeCollection  : server && server.eventTypeCollection   || []
    readonly property var hotkeyCollection     : server && server.hotkeyCollection      || []
    readonly property var mosaicCollection     : server && server.mosaicCollection      || []
    readonly property var operationLog         : server && server.operationLog          || []
    readonly property var captionsSettings     : server && server.captionsSettings      || []
    readonly property var networkEventReceiver : server && server.networkEventReceiver  || []
    readonly property var previewAtlasSender   : server && server.previewAtlasSender    || []
    readonly property var output               : server && server.output                || []

    readonly property var postPlayStatus       : server && server.postPlayStatus        || 0
    readonly property var postPlayService      : server && server.postPlayService       || []
    readonly property var availableTypes       : server && server.availableTypes        || []
    readonly property var availableVideos      : server && server.availableVideos       || []

    readonly property var currentArchive       : archiveCollection.currentArchive       || null
    readonly property var currentMosaic        : mosaicCollection && mosaicCollection.mosaics && mosaicCollection.mosaics[currentMosaicIndex] || null

    readonly property alias videoStreamAtlas: videoStreamAtlas
    readonly property alias mainWindow: mainWindow
    readonly property alias timeState: timeState

    readonly property var colProgram: "#ffb266"
    readonly property var colNeutral: "#a0a0a0"
    readonly property int fontSize: 12
    readonly property int mosaicHeight: 79
    readonly property int mosaicWidth: mosaicHeight * 16/9

    readonly property var ipAddress: rpcServer.localAddress
    readonly property int isLocal: 0
    readonly property int isNetShare: 1

    property alias config: configLoader.config
    
    readonly property var monitors: server && server.monitors || []
    property var monitorIndexes: mosaicCollection && mosaicCollection.monitorIndexes || []
    readonly property bool loaded: !!server && !!monitors && !!mosaicCollection && monitorIndexes.length
    property int currentMonitorIndex: 0
    property int currentMosaicIndex: 0
    
    property var monitorMosaics: {
        if (!loaded) return []
        var res = []; for (var i=0; i<monitors.length; i++) {
            if (i>=monitorIndexes.length) monitorIndexes.push(0)
            res.push(mosaicCollection.mosaics[monitorIndexes[i]])
        }    
        return res
    } 

    onLoadedChanged: { 
        currentMosaicIndex = monitorIndexes[currentMonitorIndex]
        updateMonitor() 
    }

    onCurrentMonitorIndexChanged: {
        if (!loaded) return
        currentMosaicIndex = monitorIndexes[currentMonitorIndex]
    }

    onMonitorIndexesChanged: {
        if (!loaded) return
        currentMosaicIndex = monitorIndexes[currentMonitorIndex]
    }
    
    function updateMonitor() { 
        if (!loaded) return
        mosaicCollection.currentIndex = currentMosaicIndex
        monitorIndexes[currentMonitorIndex] = currentMosaicIndex
        mosaicCollection.monitorIndexes = monitorIndexes 
        updateMonitorInfo()
    }

    function updateMonitorInfo() {
        if (server) server.updateMonitorInfo({monitor:currentMonitorIndex,quality:10})
    }
    onAvailableVideosChanged: {
        p('onAvailableVideosChanged',availableVideos)
    }        

    onServerChanged: {
        updateMonitorInfo()
        p("availableTypes",availableTypes)
        p("availableVideos",availableVideos)
    }

    function p(s,v,l,f,r) { Debug.p(s,v,l,f,r) }
    function pf(s,v,l,f,r) { Debug.pf(s,v,l,f,r) }

    ConfigLoader {
        id: configLoader
        fileName: "config.json"
        defaultConfig: ({
            "rpcAddress": "26.68.143.146",
            "rpcPort": 5657,
            "streamingPort": 8888
        })
    }
    GlobalClientObject {
        id: rpcServer
        port: config.rpcPort || 5657
        address: isLocal ? "127.0.0.1" : config.rpcAddress 
        name: "Server"
    }

    TimeState {
        id: timeState
    }    

    VideoStreamAtlas {
        id: videoStreamAtlas
        url: "udp://127.0.0.1:%1".arg(config.streamingPort)
    } 

    MainWindow { 
        id:mainWindow 
    }

}
