import QtQuick 2.1
import QtQuick.Controls 1.1
import QtQuick.Window 2.0

Item {
    id: root
    property alias exitAction: exitAction
    property alias fullscreenMosaicAction: fullscreenMosaicAction
    property alias fullscreenWindowAction: fullscreenWindowAction
    property alias openMosaicPresetsWindowAction: openMosaicPresetsWindowAction
    property alias openTemplateCollectionWindowAction: openTemplateCollectionWindowAction
    property alias openArchiveCollectionWindowAction: openArchiveCollectionWindowAction
    property alias openEventTypeCollectionWindowAction: openEventTypeCollectionWindowAction
    property alias openHotkeyCollectionWindowAction: openHotkeyCollectionWindowAction
    property alias openRecordingSettingsWindowAction: openRecordingSettingsWindowAction
    property alias openOperationLogWindowAction: openOperationLogWindowAction
    property alias openAudioSettingsWindowAction: openAudioSettingsWindowAction
    property alias openOutputSettingsWindowAction: openOutputSettingsWindowAction
    property alias openStorageSettingsWindowAction: openStorageSettingsWindowAction
    property alias openArchiveAction: openArchiveAction
    property alias closeArchiveAction: closeArchiveAction
    property alias openExportDialogAction: openExportDialogAction
    property alias openNetworkSettingsDialogAction: openNetworkSettingsDialogAction
    property alias cancelExportAction: cancelExportAction
    signal actionWithHotkey(string hotkey)
    
    Action {
        id: exitAction
        text: qsTr("Exit")
        shortcut: "Alt+F4"
        onTriggered: {
            actionWithHotkey(shortcut)
            //if(archiveRecorder && archiveRecorder.running) app.doWithQuestion(qsTr("Recording is running. Do you really want to exit?"), app.exit); else 
            if(exporter && exporter.running) app.doWithQuestion(qsTr("Exporting is running. Do you really want to exit?"), app.exit); else 
            app.exit()
        }
    }

    Action {
        id: fullscreenMosaicAction
        text: qsTr("Fullscreen Mosaic")
        shortcut: "Ctrl+D"
        checkable: true
        checked: mainWindow.fullscreenMosaic
        onTriggered: {
            actionWithHotkey(shortcut)
            mainWindow.fullscreenMosaic = checked
        }
    }
    
    Action {
        id: fullscreenWindowAction
        text: qsTr("Fullscreen")
        shortcut: "Ctrl+F"
        checkable: true
        checked: mainWindow.visibility == Window.FullScreen
        onTriggered: {
            actionWithHotkey(shortcut)
            mainWindow.setFullScreen(checked)
        }
    }

    Action {
        id: openMosaicPresetsWindowAction
        text: qsTr("Mosaic Presets...")
        onTriggered: mosaicPresetsWindow.show()
    }
    
    Action {
        id: openTemplateCollectionWindowAction
        text: qsTr("Templates...")
        onTriggered: templateCollectionWindow.show()
    }
    
    Action {
        id: openArchiveCollectionWindowAction
        text: qsTr("Open...")
        onTriggered: archiveCollectionWindow.show()
    }
    
    Action {
        id: openEventTypeCollectionWindowAction
        text: qsTr("Events...")
        onTriggered: eventTypeCollectionWindow.show()
    }
    
    Action {
        id: openHotkeyCollectionWindowAction
        text: qsTr("Hotkeys...")
        onTriggered: hotkeyCollectionWindow.show()
    }
    
    Action {
        id: openRecordingSettingsWindowAction
        text: qsTr("Recording...")
        onTriggered: recordingSettingsWindow.show()
        enabled: archiveRecorder
    }    

    Action {
        id: openStorageSettingsWindowAction
        text: qsTr("Storages...")
        onTriggered: storageSettingsWindow.show()
    }      

    Action {
        id: openOperationLogWindowAction
        text: qsTr("Operation Log...")
        onTriggered: operationLogWindow.show()
    }
    
    Action {
        id: openAudioSettingsWindowAction
        text: qsTr("Audio...")
        tooltip: qsTr("Audio Settings")
        onTriggered: audioSettingsWindow.open()
    }

    Action {
        id: openOutputSettingsWindowAction
        text: qsTr("Output...")
        tooltip: qsTr("Output Settings")
        onTriggered: outputSettingsWindow.open()
    }
    
    Action {
        id: openArchiveAction
        text: qsTr("Open")
        enabled: !archiveCollection || !archiveCollection.currentArchive
        onTriggered: archiveCollection.setCurrent(0)
    }

    Action {
        id: closeArchiveAction
        text: qsTr("Close")
        enabled: archiveCollection && archiveCollection.currentArchive
        onTriggered: archiveCollection.setCurrent(-1)
    }
    
    Action {
        id: openExportDialogAction
        text: qsTr("Export...")
        onTriggered: {
            exportDialog.open()
            exporterSettings.selectIntervalMode = 0
        }
        enabled: archiveCollection && archiveCollection.currentArchive && exporter && exporter.available && !exporter.running
    }
    Action {
        id: cancelExportAction
        text: qsTr("Cancel Export")
        onTriggered: exporter.cancelExport()
        enabled: exporter && exporter.running
    }  

    Action {
        id: openNetworkSettingsDialogAction
        text: qsTr("Network...")
        onTriggered: {
            networkSettingsDialog.open()
        }
    }
  
}
