import QtQuick 2.0
import QtQuick.Layouts 1.0
import QtQuick.Controls 1.0
import MixerGuiControls 1.0

MixerDialog {
    id: root
    title: qsTr("Add Mosaic")
    readonly property var templates: (templateCollection && templateCollection.templates) ? templateCollection.templates : []
    property int selectedIndex: -1
    property alias name: nameTextField.text
    readonly property QtObject template: templates[selectedIndex] || null

    readonly property int templateCount: templates.length
    onTemplateCountChanged: {
        if(selectedIndex == -1 && templateCount > 0) selectedIndex = 0
        else if(selectedIndex >= templateCount) selectedIndex = templateCount - 1
    }
    
    signal accepted()
    signal rejected()

    ColumnLayout {
        id: layout
        MixerLabel {
            text: qsTr("Mosaic name: ")
        }
        TextField {
            id: nameTextField
            Layout.fillWidth: true
        }
        Item {
            implicitWidth: 2
        }
        MixerLabel {
            text: qsTr("Template: ")
        }
        Flow {
            Layout.preferredWidth: 422
            spacing: 6
            Repeater {
                model: templates
                TemplatePreview {
                    template: modelData
                    isSelected: selectedIndex == model.index
                    onLeftButtonClicked: selectedIndex = model.index
                    onDoubleClicked: {
                        selectedIndex = model.index
                        accepted()
                        hide()
                    }           
                }
            }
        }
    }

    buttons: [qsTr("OK"), qsTr("Cancel")]
    
    onButtonClicked: {
        if(buttonIndex == 0) accepted()
        else rejected()
        hide()
    }
    
    onClosing: rejected()
        
    onOpened: {
        nameTextField.text = ""
        nameTextField.forceActiveFocus()
    }
    
    onImplicitHeightChanged: {
        height = implicitHeight
    }
}