import QtQuick 2.0
import QtQuick.Layouts 1.0
import QtQuick.Controls 1.0
import MixerGuiControls 1.0
import "utils/Conversion.js" as Conversion

MixerDialog {
    id: root
    title: qsTr("Add Storage")
    property alias name: nameTextField.text
    property var drive: postPlayService.drives && postPlayService.drives[driveComboBox.currentIndex] || null
    property real size: Conversion.mbToBytes(sizeSpinBox.value)
    
    signal accepted()
    signal rejected()

    ColumnLayout {
        id: layout
        MixerLabel {
            text: qsTr("Name: ")
        }
        TextField {
            id: nameTextField
            Layout.preferredWidth: 300
        }
        Item {
            implicitWidth: 2
        }
        MixerLabel {
            text: qsTr("Drive: ")
        }
        MixerComboBox {
            id: driveComboBox
            Layout.preferredWidth: 90
            model: postPlayService.drives
            textRole: "name"
        }
        Item {
            implicitWidth: 2
        }
        MixerLabel {
            text: qsTr("Size (MB): ")
        }
        RowLayout {
            SpinBox {
                id: sizeSpinBox
                Layout.preferredWidth: 90
                minimumValue: 1
                maximumValue: drive ? Conversion.bytesToMB(drive.freeSize) : 0
                stepSize: 100
            }
            MixerLabel {
                Layout.fillWidth: true
                horizontalAlignment: Text.AlignRight
                text: drive ? qsTr("Available size: ") + Conversion.bytesToMB(drive.freeSize).toFixed() + qsTr(" MB") : ""
                font.pixelSize: MixerThemeManager.theme.systemFontSize
            }
        }
    }

    buttons: [qsTr("OK"), qsTr("Cancel")]
    
    onButtonClicked: {
        if(button == qsTr("OK")) accepted()
        else rejected()
        hide()
    }
    
    onClosing: rejected()
        
    onOpened: {
        nameTextField.text = ""
        driveComboBox.currentIndex = 0
        sizeSpinBox.value = 1000
        nameTextField.forceActiveFocus()
    }
}
