import QtQuick 2.0
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.0
import QtQuick.Window 2.0
import MixerGuiControls 1.0
import "utils/Conversion.js" as Conversion
import "utils/FormatTime.js" as FormatTime

MixerWindow {
    id: root
    width: 480
    height: 200
    minimumWidth: 280
    minimumHeight: 200
    modality: Qt.ApplicationModal
    title: qsTr("Archive")
    readonly property int selectedIndex: tableView.rowCount > 0 ? tableView.currentRow : -1
    onVisibleChanged: if(visible) archiveCollection.removeEmptyArchives()
    
    ColumnLayout {
        anchors.fill: parent
        MixerTableView {
            id: tableView
            Layout.fillWidth: true
            Layout.fillHeight: true
            headerVisible: true
            horizontalScrollBarPolicy: Qt.ScrollBarAlwaysOff
            
            property var modelTmp: archiveCollection ? archiveCollection.archives : []
            onModelTmpChanged: {
                var row = currentRow
                model = modelTmp
                currentRow = Math.min(row, rowCount - 1)
                if(currentRow == -1 && rowCount > 0) currentRow = 0
                selection.clear()
                if(currentRow >= 0 && rowCount > 0) selection.select(currentRow)            
            }
            /*
            TableViewColumn {
                title: qsTr("Name")
                role: "name"
                width: tableView.width - 370
                movable: false
                resizable: false
                delegate: Item {
                    anchors.fill: parent
                    anchors.leftMargin: 18
                    MixerLabel {
                        id: archiveLabel
                        verticalAlignment: Text.AlignVCenter
                        text: styleData.value
                        elide: styleData.elideMode
                        colorType: styleData.selected ? "program" : "neutral"
                        Item {
                            anchors.left: parent.right
                            anchors.leftMargin: 10
                            height: parent.height
                            RowLayout {
                                anchors.fill: parent
                                MixerLabel {
                                    verticalAlignment: Text.AlignVCenter
                                    font.pixelSize: archiveLabel.font.pixelSize - 4
                                    text: qsTr("<Recording>")
                                    color: "red"
                                    visible: archiveCollection && archiveCollection.archives && (archiveCollection.archives[styleData.row] == currentRecordingArchive)
                                }
                                MixerLabel {
                                    verticalAlignment: Text.AlignVCenter
                                    font.pixelSize: archiveLabel.font.pixelSize - 4
                                    text: qsTr("<Viewing>")
                                    colorType: "program"
                                    visible: archiveCollection && archiveCollection.archives && (archiveCollection.archives[styleData.row] == currentArchive)
                                }
                            }
                        }
                    }
                }
            }
            */    
            TableViewColumn {
                title: qsTr("Creation date")
                role: "creationDate"
                width: root.width*0.45
                delegate: MixerLabel {
                    anchors.fill: parent
                    anchors.leftMargin: 8
                    verticalAlignment: Text.AlignVCenter
                    anchors.topMargin: styleData.row == 0 ? 6 : 0
                    text: styleData.value.toLocaleString(Qt.locale(), "d MMMM yyyy г. hh:mm:ss")
                    elide: styleData.elideMode
                    colorType: styleData.selected ? "program" : "neutral"
                }
            }
            
            TableViewColumn {
                title: qsTr("Duration")
                role: "duration"
                width: root.width*0.25

                delegate: MixerLabel {
                    anchors.fill: parent
                    anchors.leftMargin: 8
                    verticalAlignment: Text.AlignVCenter
                    anchors.topMargin: styleData.row == 0 ? 6 : 0
                    text: FormatTime.millisecondsToString(styleData.value)
                    elide: styleData.elideMode
                    colorType: styleData.selected ? "program" : "neutral"
                }                
            }
            
            TableViewColumn {
                title: qsTr("Size (MB)")
                role: "size"
                width: root.width*0.30
                delegate: MixerLabel {
                    anchors.fill: parent
                    anchors.leftMargin: 8
                    verticalAlignment: Text.AlignVCenter
                    anchors.topMargin: styleData.row == 0 ? 6 : 0
                    text: Conversion.bytesToMB(styleData.value).toFixed()
                    
                    elide: styleData.elideMode
                    colorType: styleData.selected ? "program" : "neutral"
                }                
            }            
                        
            rowOverlayDelegate: MouseArea {
                height: MixerThemeManager.theme.systemFontSize + 4 + (styleData.row == 0 ? 6 : 0)
                acceptedButtons: Qt.LeftButton | Qt.RightButton
                onPressed: {
                    if(styleData.row == undefined) return
                    tableView.currentRow = styleData.row
                    if(mouse.button == Qt.RightButton) contextMenu.popup()
                }
                onDoubleClicked: {
                    if(styleData.row == undefined) return
                    if(mouse.button == Qt.LeftButton) {
                        tableView.currentRow = styleData.row
                        openAction.trigger()
                    }
                }
            }
            
            onCurrentRowChanged: {
                selection.clear()
                if(currentRow >= 0 && currentRow < rowCount) selection.select(currentRow)            
            }
        }
        
        RowLayout {
            Layout.alignment: Qt.AlignHCenter
            DialogButton { action: openAction }
//          DialogButton { action: renameAction }
        }
    }
    
    MixerMenu {
        id: contextMenu
        MixerMenuItem { action1: openAction }
        MixerMenuItem { action1: renameAction }
    }
    
    Action {
        id: openAction
        enabled: selectedIndex != -1
        text: qsTr("Open")
        onTriggered: {
            archiveCollection.setCurrent(selectedIndex)
            root.hide()
        }
    }
    
    Action {
        id: renameAction
        enabled: selectedIndex != -1
        text: qsTr("Rename")
        onTriggered: {
            var archive = tableView.model[selectedIndex]
            if(!archive) return
            renameDialog.text = archive.name || ""
            renameDialog.open()
        }
    } 

    MixerStringInputDialog {
        id: renameDialog
        title: qsTr("Rename archive")
        caption: qsTr("Name: ")
        onEntered: archiveCollection.rename(selectedIndex, value)
    }
}
