import QtQuick 2.0
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.1
import MixerGuiControls 1.0

MixerDialog {
    id: root
    title: qsTr("Audio Settings")
    signal accepted()
    signal rejected()
    readonly property var streams: audioStreamCollection && audioStreamCollection.streams || []
    readonly property var streamSources: streams.map(function(stream) {
        return stream.source
    })

    ColumnLayout {
        id: layout
        spacing: 10
        Repeater {
            id: repeater
            model: streams.length
            ColumnLayout {
                readonly property int modelIndex: model.index
                property alias source: sourceSelector.source
                MixerLabel {
                    text: qsTr("Source") + " " + (model.index + 1) + ": "
                }
                FdInputSelector {
                    id: sourceSelector
                    hasEmpty: true
                    Layout.preferredWidth: 300
                    readonly property QtObject stream: streams[parent.modelIndex] || null
                    onSourceChanged: {
                        if(root.visible && stream) {
                            stream.source = source
                        }
                    }
                }
            }
        }
    }
    
    buttons: [qsTr("OK"), qsTr("Cancel")]
    
    onButtonClicked: {
        if(buttonIndex == 0) accepted()
        else rejected()
        hide()
    }
    
    onClosing: rejected()
    
    readonly property QtObject prevSettings: QtObject {
        property var sources: []
    }

    onOpened: {
        prevSettings.sources = []
        for(var i = 0; i < streams.length; i++) {
            var source = streamSources[i]
            prevSettings.sources.push(source)
            repeater.itemAt(i).source = source
        }
    }
    
    onRejected: {
        for(var i = 0; i < streams.length; i++) {
            streams[i].source = prevSettings.sources[i]
        }
        prevSettings.sources = []
    }

}
