import QtQuick 2.0
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.1
import MixerGuiControls 1.0
import "utils/FormatTime.js" as FormatTime

Item {
    id: root
    readonly property Item header: MixerPanelHeader {
        anchors.fill: parent
        text: qsTr("Time")
    }    
    
    readonly property var titles: [qsTr("SYSTEM"), qsTr("TRAINING")]
    readonly property var times: timeState ? [timeState.systemTime, timeState.trainingTime] : [0, 0]
    anchors.bottomMargin: 20

    function toMode(index) { 
        switch (index) {
            case 0: return 0
            case 1: return 2
        }            
        return 2
    }

    ColumnLayout {
        anchors.centerIn: parent
        spacing : 5
        
        Repeater {
            model: times.length
            Item {
                implicitWidth: rowLayout.implicitWidth
                implicitHeight: rowLayout.implicitHeight
                RowLayout {
                    id: rowLayout
                    spacing: 0
                    readonly property color textColor: (timeState && toMode(model.index) == timeState.timeMode) ? Qt.lighter("white", 0.75) : Qt.lighter("white", 0.5)
                    Text {
                        Layout.preferredWidth: 100
                        text: titles[model.index]
                        font.pixelSize: 14
                        color: rowLayout.textColor
                        Layout.alignment: Qt.AlignVCenter
                    }
                    Text {
                        text: FormatTime.millisecondsToString(times[model.index])
                        font.pixelSize: 34
                        color: rowLayout.textColor
                        Layout.alignment: Qt.AlignBaseline
                    }
                }
                MouseArea {
                    anchors.fill: parent
                    onClicked: if(timeState) timeState.timeMode = toMode(model.index)
                }                
            }
        }
    }
    
}