import QtQuick 2.0
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.0
import QtQuick.Window 2.0
import MixerGuiControls 1.0

MixerWindow {
    id: root
    width: 380
    height: 500
    minimumWidth: 380
    minimumHeight: 300
    modality: Qt.ApplicationModal
    title: qsTr("Events Filtering")
    readonly property int selectedIndex: tableView.rowCount > 0 ? tableView.currentRow : -1
    
    ColumnLayout {
        anchors.fill: parent
        MixerTableView {
            id: tableView
            Layout.fillWidth: true
            Layout.fillHeight: true
            headerVisible: false
            horizontalScrollBarPolicy: Qt.ScrollBarAlwaysOff
                   
            property var modelTmp: eventTypeCollection ? eventTypeCollection.eventTypes : []
            onModelTmpChanged: {
                var row = currentRow
                model = modelTmp
                currentRow = Math.min(row, rowCount - 1)
                if(currentRow == -1 && rowCount > 0) currentRow = 0
                selection.clear()
                if(currentRow >= 0 && rowCount > 0) selection.select(currentRow)            
            }
            
            TableViewColumn { 
                role: "visible"
                width: 50
                delegate: Item {
                    anchors.fill: parent
                    CheckBox {
                        x: 18
                        anchors.verticalCenter: parent.verticalCenter
                        Binding on checked {
                            value: styleData.value
                        }
                        onClicked: {
                            var eventType = eventTypeCollection.eventTypes[styleData.row]
                            if(!eventType) return
                            eventType.visible = checked
                        }
                    }
                }
            }
            
            TableViewColumn {
                role: "type"
                width: 250
                delegate: Item {
                    anchors.fill: parent
                    MixerLabel {
                        anchors.fill: parent
                        anchors.leftMargin: 18
                        anchors.rightMargin: 6
                        verticalAlignment: Text.AlignVCenter
                        text: styleData.value
                        elide: styleData.elideMode
                        colorType: styleData.selected ? "program" : "neutral"
                    }
                    MouseArea {
                        anchors.fill: parent
                        acceptedButtons: Qt.LeftButton | Qt.RightButton
                        onPressed: {
                            tableView.currentRow = styleData.row
                            if(mouse.button == Qt.RightButton) contextMenu.popup()
                        }
                    }
                }
            }
                
            TableViewColumn {
                role: "color"
                width: 50
                delegate: Item {
                    anchors.fill: parent
                    Rectangle {
                        anchors.verticalCenter: parent.verticalCenter
                        implicitWidth: 35
                        implicitHeight: 17
                        radius: 1
                        border.width: 1
                        color: styleData.value
                        MouseArea {
                            anchors.fill: parent
                            onClicked: {
                                var eventType = eventTypeCollection.eventTypes[styleData.row]
                                if(!eventType) return
                                colorDialog.target = eventType
                                colorDialog.open()
                            }
                        }
                   }
               }
            }
            
            onCurrentRowChanged: {
                selection.clear()
                if(currentRow >= 0 && currentRow < rowCount) selection.select(currentRow)            
            }
        }
        
        RowLayout {
            Layout.alignment: Qt.AlignHCenter
            DialogButton { action: addAction }
            DialogButton { action: editAction }
            DialogButton { action: deleteAction }
        }
    }
    
    Action {
        id: addAction
        text: qsTr("Add")
        onTriggered: {
            eventTypeDialog.eventTypeIndex = -1
            eventTypeDialog.open()
        }
    }
    
    Action {
        id: editAction
        enabled: selectedIndex > 0
        text: qsTr("Edit")
        onTriggered: {
            eventTypeDialog.eventTypeIndex = selectedIndex
            eventTypeDialog.open()
        }
    }
    
    Action {
        id: deleteAction
        enabled: selectedIndex > 0
        text: qsTr("Delete")
        shortcut: "Del"
        onTriggered: {
            doWithQuestion(qsTr("Do you really want to delete this event type?"), function() {
                eventTypeCollection.remove(selectedIndex)
            })
        }         
    }
    
    MixerMenu {
        id: contextMenu
        MixerMenuItem { action1: editAction }
        MixerMenuItem { action1: deleteAction }
    }    

    readonly property QtObject selectedEventType: (eventTypeCollection && eventTypeCollection.eventTypes) ? (eventTypeCollection.eventTypes[selectedIndex] || null) : null
    readonly property QtObject selectedEventTypeProperties: QtObject { // HACK: биндимся к свойствам, чтобы они успели прийти, до того как откроется eventTypeDialog
        readonly property string type: selectedEventType ? selectedEventType.type : ""
    }

}
