import QtQuick 2.0
import QtQuick.Layouts 1.0
import QtQuick.Controls 1.0
import MixerGuiControls 1.0

MixerDialog {
    id: root
    property int eventTypeIndex
    title: eventTypeIndex == -1 ? qsTr("Add Event") : qsTr("Edit Event")
    GridLayout {
        anchors.fill: parent
        columns: 2

        MixerLabel {
            text: qsTr("Type: ")
        }
        
        MixerComboBox {
            id: typeControl
            Layout.fillWidth: true
            editable: true
            model: availableTypes.map(function(type) { return type.name; })
            onCurrentIndexChanged: {
                var color = availableTypes && availableTypes[currentIndex] &&  availableTypes[currentIndex].color
                if (color) colorControl.color = color
            }
        }        
        
        RowLayout {
            Layout.columnSpan: 2
            MixerLabel {
                text: qsTr("Visible: ")
            }
            CheckBox {
                id: visibleCheckBox
                checked: true
            }
            Item {
                implicitWidth: 20
            }
            MixerLabel {
                text: qsTr("Color: ")
            }
            Rectangle {
                id: colorControl
                implicitWidth: 35
                implicitHeight: 17
                radius: 1
                border.width: 1
                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        colorDialog.target = colorControl
                        colorDialog.open()
                    }
                }
            }
        }
        
    }
        
    buttons: [qsTr("OK"), qsTr("Cancel")]

    onButtonClicked: {
        if(buttonIndex == 0) {
            var eventType = eventTypeCollection.eventTypes[eventTypeIndex] || null
            if(!eventType) {
                eventTypeCollection.add(undefined, {
                    type: typeControl.editText,
                    visible: visibleCheckBox.checked,
                    color: colorControl.color
                })
            }
            else {
                eventType.type = typeControl.editText
                eventType.visible = visibleCheckBox.checked
                eventType.color = colorControl.color
            }
        }
        clear()
        hide()
    }
    
    onClosing: clear()
        
    onOpened: {
        var eventType = eventTypeCollection.eventTypes[eventTypeIndex] || null 
        if(eventType) {
            typeControl.editText = eventType.type
            visibleCheckBox.checked = eventType.visible
            colorControl.color = eventType.color
        }
        typeControl.forceActiveFocus()
    }
    
    function clear() {
        eventTypeIndex = -1
        visibleCheckBox.checked = true
        typeControl.editText = availableTypes && availableTypes[0] && availableTypes[0].name || ""
        colorControl.color = availableTypes && availableTypes[0] && availableTypes[0].color || "white"
    }
}
