import QtQuick 2.4
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.0
import MixerGuiControls 1.0
import DonSouzMediaRecorder 1.0
import "utils/FormatTime.js" as FormatTime

Item {
    id: root
    readonly property alias eventModel: eventModel
    readonly property Item header: MixerPanelHeader {
        anchors.fill: parent
        text: qsTr("Events")
        buttons: [
            MixerIconButton {
                icon: "Settings"
                action: actions.openEventTypeCollectionWindowAction
                tooltip: qsTr("Events Settings")
            }
        ]
    }

    MixerLabel {
        id: dummyNameLabel
        font.weight: Font.Normal
        visible: false
    }

    MixerTableView {
        id: tableView
        anchors.fill: parent
        model: EventModel {
            id: eventModel
            eventCollection: archivePlayer ? archivePlayer.eventCollection : null
            nameFont: dummyNameLabel.font
        }

        headerVisible: false
        horizontalScrollBarPolicy: Qt.ScrollBarAlwaysOff

        TableViewColumn {}

        itemDelegate: Item {
            readonly property var event: model
            RowLayout {
                anchors.fill: parent
                anchors.leftMargin: 10
                anchors.rightMargin: 6
                spacing: 10

                MixerLabel {
                    verticalAlignment: Text.AlignVCenter
                    text: {
                        if(!event) return ""
                        var time = (timeState.timeMode == 1 && event.boardTime > -0.5) ? event.boardTime : timeState.trainingTimeToCurrentModeTime(event.time)
                        return "[" + FormatTime.millisecondsToString(time) + "]"
                    }
                    color: mouseArea.containsMouse ? "white" : Qt.rgba(0.8, 0.8, 0.8)
                    style: Text.Normal
                    font.pixelSize: nameLabel.font.pixelSize * 0.8
                    font.weight: Font.Light
                }
                MixerLabel {
                    id: nameLabel
                    verticalAlignment: Text.AlignVCenter
                    text: event ? event.type : ""
                    color: event ? event.color : Qt.rgba(0, 0, 0, 1)
                    style: Text.Normal
                    elide: Text.ElideRight
                    font.weight: mouseArea.containsMouse ? Font.DemiBold : Font.Normal
                    Layout.preferredWidth: eventModel.maxNameWidth
                }
                MixerLabel {
                    verticalAlignment: Text.AlignVCenter
                    text: event ? event.comment : ""
                    color: mouseArea.containsMouse ? "white" : Qt.rgba(0.8, 0.8, 0.8)
                    style: Text.Normal
                    font.pixelSize: nameLabel.font.pixelSize * 0.8
                    font.weight: Font.Light
                    elide: Text.ElideRight
                    Layout.fillWidth: true
                }
            }
            MouseArea {
                id: mouseArea
                anchors.fill: parent
                acceptedButtons: Qt.LeftButton | Qt.RightButton
                hoverEnabled: true
                cursorShape: Qt.PointingHandCursor
                onClicked: {
                    if(mouse.button == Qt.LeftButton) {
                        archivePlayer.setPosition(event.time)
                    }
                    else if(mouse.button == Qt.RightButton) {
                        editEventDialog.eventIndex = styleData.row
                        contextMenu.popup()
                    }
                }
            }
        }
    }

    MixerLabel {
        anchors.centerIn: parent
        text: {
            var status = archivePlayer ? archivePlayer.eventCollectionStatus : EventCollection.Ready
            if(status == EventCollection.Loading) return qsTr("Loading...")
            if(status == EventCollection.Filtering) return qsTr("Filtering...")
            return ""
        }
        opacity: text ? 1 : 0
        Behavior on opacity {
            NumberAnimation { duration: 200 }
        }
    }

    Action {
        id: editEventAction
        text: qsTr("Edit")
        onTriggered: editEventDialog.open()
    }

    MixerMenu {
        id: contextMenu
        MixerMenuItem { action1: editEventAction }
    }

    MixerStringInputDialog {
        id: editEventDialog
        property int eventIndex
        title: qsTr("Edit event")
        caption: qsTr("Comment: ")
        allowEmptyString: true

        onVisibleChanged: if(!visible) eventIndex = -1
        onOpened: text = eventModel.getValue(eventIndex, "comment") || ""
        onEntered: if(eventModel.eventCollection) {
            eventModel.eventCollection.setComment(eventIndex, text)
            if (!archivePlayer || !archivePlayer.archive) return
            var event = {
                time:eventModel.getValue(eventIndex, "time"),
                type:eventModel.getValue(eventIndex, "type"),
                boardTime:eventModel.getValue(eventIndex, "boardTime"),
                comment:text
            } 
            archivePlayer.archive.setEvent(event)
        }                
    }
}
