import QtQuick 2.0
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.0
import QtQuick.Window 2.0
import MixerGuiControls 1.0

MixerWindow {
    id: root
    width: 570
    height: 600
    minimumWidth: 400
    minimumHeight: 300
    modality: Qt.ApplicationModal
    title: qsTr("Hotkeys")
    readonly property int selectedIndex: tableView.rowCount > 0 ? tableView.currentRow : -1
    
    ColumnLayout {
        anchors.fill: parent
        MixerTableView {
            id: tableView
            Layout.fillWidth: true
            Layout.fillHeight: true
            headerVisible: true
            horizontalScrollBarPolicy: Qt.ScrollBarAlwaysOff
                   
            property var modelTmp: hotkeyCollection ? hotkeyCollection.hotkeys : []
            onModelTmpChanged: {
                var row = currentRow
                model = modelTmp
                currentRow = Math.min(row, rowCount - 1)
                if(currentRow == -1 && rowCount > 0) currentRow = 0
                selection.clear()
                if(currentRow >= 0 && rowCount > 0) selection.select(currentRow)            
            }
                    
            TableViewColumn {
                role: "sequence"
                title: qsTr("Keyboard")
                width: 100
            }
            TableViewColumn {
                role: "type"
                title: qsTr("Type")
                width: 150
/*
                delegate: MixerLabel {
                    anchors.fill: parent
                    anchors.leftMargin: 10
                    anchors.rightMargin: 6
                    verticalAlignment: Text.AlignVCenter
                    text: availableTypes[styleData.value ||0].name
                    elide: styleData.elideMode
                    color: availableTypes[styleData.value ||0].color
                }   
*/             
            }
            TableViewColumn {
                role: "comment"
                title: qsTr("Comment")
                width: {
                    var result = tableView.width
                    for(var i = 0; i < tableView.columnCount - 1; i++) {
                        result -= tableView.getColumn(i).width
                    }
                    result -= tableView.__verticalScrollBar.width
                    return Math.max(result, 20)
                }
            }

            itemDelegate:  MixerLabel {
                anchors.fill: parent
                anchors.leftMargin: 10
                anchors.rightMargin: 6
                verticalAlignment: Text.AlignVCenter
                text: styleData.value
                elide: styleData.elideMode
                colorType: styleData.selected ? "program" : "neutral"
            }

            onCurrentRowChanged: {
                selection.clear()
                if(currentRow >= 0 && currentRow < rowCount) selection.select(currentRow)            
            }
            rowOverlayDelegate: MouseArea {
                anchors.fill: parent
                acceptedButtons: Qt.LeftButton | Qt.RightButton
                onPressed: {
                    if(styleData.row == undefined) return
                    tableView.currentRow = styleData.row
                    if(mouse.button == Qt.RightButton) contextMenu.popup()
                }
            }
        }
        
        RowLayout {
            Layout.alignment: Qt.AlignHCenter
            DialogButton { action: addAction }
            DialogButton { action: editAction }
            DialogButton { action: deleteAction }
        }
    }
    
    Action {
        id: addAction
        text: qsTr("Add")
        onTriggered: {
            hotkeyDialog.hotkey = null
            hotkeyDialog.open()
        }
    }
    
    Action {
        id: editAction
        enabled: selectedIndex >= 0
        text: qsTr("Edit")
        onTriggered: {
            hotkeyDialog.hotkey = selectedHotkey
            hotkeyDialog.open()
        }
    }
    
    Action {
        id: deleteAction
        enabled: selectedIndex >= 0
        text: qsTr("Delete")
        shortcut: "Del"
        onTriggered: {
            doWithQuestion(qsTr("Do you really want to delete this hotkey?"), function() {
                hotkeyCollection.remove(selectedIndex)
            })
        }         
    }
    
    MixerMenu {
        id: contextMenu
        MixerMenuItem { action1: editAction }
        MixerMenuItem { action1: deleteAction }
    }    

    readonly property QtObject selectedHotkey: (hotkeyCollection && hotkeyCollection.hotkeys) ? (hotkeyCollection.hotkeys[selectedIndex] || null) : null
}
