import QtQuick 2.0
import QtQuick.Layouts 1.0
import QtQuick.Controls 1.0
import MixerGuiControls 1.0

MixerDialog {
    id: root
    property QtObject hotkey
    title: hotkey ? qsTr("Edit Hotkey") : qsTr("Add Hotkey")
    GridLayout {
        anchors.fill: parent
        columns: 2

        MixerLabel {
            text: qsTr("Keyboard: ")
        }
        RowLayout {
            id: sequenceControl
            readonly property string value: sequenceControl1.currentText + "+" + sequenceControl2.currentText
            function setValue(value) {
                var modifier = value.substr(0, value.length - 2)
                var letter = value.substr(-1)
                sequenceControl1.currentIndex = sequenceControl1.model.indexOf(modifier)
                sequenceControl2.currentIndex = sequenceControl2.model.indexOf(letter)
            }
            MixerComboBox {
                id: sequenceControl1
                model: ["Ctrl", "Shift", "Alt", "Ctrl+Shift"]
            }
            MixerLabel {
                text: " + "
            }
            MixerComboBox {
                id: sequenceControl2
                model: {
                    var result = []
                    for(var i = 0; i < 26; i++) {
                        result.push(String.fromCharCode(65 + i))
                    }
                    return result
                }
            }
        }

        MixerLabel {
            text: qsTr("Type: ")
        }

        MixerComboBox {
            id: typeControl
            function getValue() { return model[currentIndex] }
            function setValue(value) { currentIndex = find(value) }
            model: availableTypes.map(function(type) { return type.name; })
            Layout.fillWidth: true
        }        
        
        MixerLabel {
            text: qsTr("Comment: ")
        }
        TextField {
            id: commentTextField
            Layout.fillWidth: true
        }
        
    }
        
    buttons: [qsTr("OK"), qsTr("Cancel")]

    onButtonClicked: {
        if(buttonIndex == 0) {
            if(!hotkey) {
                hotkeyCollection.add(undefined, {
                    sequence: sequenceControl.value,
                    type: typeControl.getValue(),
                    comment: commentTextField.text
                })
            }
            else {
                hotkey.sequence = sequenceControl.value
                hotkey.type = typeControl.getValue()
                hotkey.comment = commentTextField.text
            }
        }
        clear()
        hide()
    }
    
    onClosing: clear()
        
    onOpened: {
        if(hotkey) {
            sequenceControl.setValue(hotkey.sequence)
            typeControl.setValue(hotkey.type)
            commentTextField.text = hotkey.comment
        }
        typeControl.forceActiveFocus()
    }
    
    function clear() {
        hotkey = null
        sequenceControl1.currentIndex = 0
        sequenceControl2.currentIndex = 0
        typeControl.currentIndex = 0
        commentTextField.text = ""
    }
}
