import QtQuick 2.1
import QtQuick.Layouts 1.1
import QtQuick.Window 2.0
import QtQuick.Controls 2.4
import MixerGuiControls 1.0
import GuiControls 1.0
import QtQuickUtils 1.0

ApplicationWindow {
    id: mainWindow
    visible: true
    minimumWidth: 1200
    minimumHeight: 850
    width: 1820
    height: 945
    title: mainWindowTitle.text
    property alias fullscreenMosaic: mainWindowLayout.fullscreenMosaic
    
    readonly property alias mainWindow: mainWindow
    readonly property alias actions: actions    
    readonly property alias dragAndDropLayer: dragAndDropLayer
    readonly property alias eventsPanel: eventsPanel

    Component.onCompleted: Qt.callLater(function() { if (x<0 || x>=1920) x = (1920-width)/2; })
    
    onClosing: {
        close.accepted = false
        actions.exitAction.trigger()
    }
    
    property bool restoreToFullScreen: false
    property bool maximizeFromFullScreen: false
    
    MainWindowTitle {
        id: mainWindowTitle
    }
    
    menuBar: MainMenu {
        id: menuBar
        visible: !mainWindowLayout.fullscreenMosaic || mainWindowLayout.upPanelVisible
        onMinimizeButtonClicked: {
            mainWindow.visibility = Window.Maximized
            mainWindow.visibility = Window.Minimized
            mainWindow.restoreToFullScreen = true
        }
        onRestoreButtonClicked: mainWindow.visibility = Window.Windowed
        onMaximizeButtonClicked: mainWindow.visibility = Window.Maximized
        onCloseButtonClicked: actions.exitAction.trigger()
        
        minimizeButtonVisible: mainWindow.visibility == Window.FullScreen
        closeButtonVisible: mainWindow.visibility == Window.FullScreen
        restoreButtonVisible: mainWindow.visibility == Window.FullScreen && !maximizeFromFullScreen
        maximizeButtonVisible: mainWindow.visibility == Window.FullScreen && maximizeFromFullScreen
    }
    
    function setFullScreen(value) {
        if(visibility == Window.Maximized) maximizeFromFullScreen = true
        else if(visibility == Window.Windowed) maximizeFromFullScreen = false
        mainWindow.visibility = value ? Window.FullScreen : (maximizeFromFullScreen ? Window.Maximized : Window.Windowed)
    }
    
    onVisibilityChanged: {
        if((visibility == Window.Maximized || visibility == Window.Windowed) && restoreToFullScreen) {
            restoreToFullScreen = false
            setFullScreen(true)
        }
    }
    
    Actions {
        id: actions
        onActionWithHotkey: hotkeyEventGenerator.processHotkey(hotkey)
    }

    Action {
        shortcut: 'F9'
        onTriggered: archivePlayer.switchStatus()
    }    

    Action {
        shortcut: 'F2'
        onTriggered: aboutDialog.open()
    }    

    MixerMessageDialog {
        id: aboutDialog
        title: mainWindow.title
        captionHeight: 80
        caption: qsTr("Version: ") + Qt.application.version + "\n" 
    }
   
    MainWindowLayout {
        id: mainWindowLayout
        mainContent: MainContent {
            id: mainContent
            anchors.fill: parent
        }
        mosaicsPanel: MosaicsPanel {
            anchors.fill: parent
        }
        monitorsPanel: MonitorsPanel {
            anchors.fill: parent
        }
        videoStreamsPanel: VideoStreamsPanel {
            anchors.fill: parent
        }
        videoStreamsArchivedPanel: VideoStreamsPanel {
            archived: true
            anchors.fill: parent
        }
        recordingPanel: MediaRecordingPanel { 
            anchors.fill: parent
        }
        playbackPanel: PlaybackPanel {
            anchors.fill: parent
        }
        eventsPanel: EventsPanel {
            id: eventsPanel
            anchors.fill: parent
        }
        clockPanel: ClockPanel {
            anchors.fill: parent
        }
        audioPanel: AudioPanel {
            anchors.fill: parent
        }
        statusBar: MainStatusBar {}
    }
    
    HotkeyEventGenerator {
        id: hotkeyEventGenerator
    }
    
    DragAndDropLayer {
        id: dragAndDropLayer
        anchors.fill: parent
    }    
    
    WindowPositionSaver {
        window: mainWindow
        name: "MosaicRecorder_mainWindow"
    }
    WindowUpdater{}
}
