import QtQuick 2.1
import QtQuick.Layouts 1.0
import QtQuick.Controls 1.0
import MixerGuiControls 1.0
import Serialization 1.0
import "utils"

Item {
    id: root
    anchors.fill: parent
    property bool fullscreenMosaic: false
    property Item mainContent
    property Item mosaicsPanel
    property Item monitorsPanel
    property Item videoStreamsPanel
    property Item videoStreamsArchivedPanel
    property Item clockPanel
    property Item audioPanel
    property Item recordingPanel
    property Item eventsPanel
    property Item playbackPanel
    property Item statusBar
    property real spacing: MixerThemeManager.theme.panelSpacing
    
    readonly property bool upPanelVisible: upMoveInItem.moveProgress == 1
    
    property alias leftPanelSize: leftMoveInItem.width
    property alias upPanelSize: upMoveInItem.height
    property alias rightPanelSize: rightMoveInItem.width
    
    MixerBackground {
        anchors.fill: parent
        anchors.topMargin: -MixerThemeManager.theme.menuHeight
    }
    
    PanelVisibilityControl {
        id: panelVisibilityControl
        anchors.fill: parent
        anchors.topMargin: -MixerThemeManager.theme.menuHeight
        enabled: root.fullscreenMosaic
    }
    
    Item {
        id: mainContentPanel
        readonly property int playbackPanelHeight: 110
        parent: root.fullscreenMosaic ? fullscreenContentItem : nonfullscreenContentItem
        anchors.fill: parent
        Item {
            anchors.fill: parent
            anchors.bottomMargin: root.fullscreenMosaic ? 0 : mainContentPanel.playbackPanelHeight
            data: root.mainContent
        }
        MoveIn {
            moveEnabled: root.fullscreenMosaic
            moveDirection: Qt.vector2d(0, -1)
            show: panelVisibilityControl.showDownPanel
            
            width: parent.width
            height: mainContentPanel.playbackPanelHeight
            anchors.bottom: parent.bottom
            MediaBarBackground {
                anchors.fill: parent
                anchors.leftMargin: -2
                anchors.rightMargin: -2
                anchors.bottomMargin: -2
                data: root.playbackPanel
            }
        }

        NetworkRemoteAddress {}
    }
    
    Item {
        id: fullscreenContentItem
        anchors.fill: parent
        anchors.topMargin: -MixerThemeManager.theme.menuHeight
    }
    
    Rectangle {
        width: parent.width
        height: MixerThemeManager.theme.menuHeight + 2
        y: -MixerThemeManager.theme.menuHeight
        visible: root.fullscreenMosaic && root.upPanelVisible
        color: "black"
    }
    
    ColumnLayout {
        anchors.fill: parent
        anchors.topMargin: root.fullscreenMosaic ? -MixerThemeManager.theme.menuHeight : 0        
        anchors.leftMargin: root.fullscreenMosaic ? 0 : root.spacing
        anchors.rightMargin: root.fullscreenMosaic ? 0 : root.spacing
        anchors.bottomMargin: root.fullscreenMosaic ? 0 : 4
        spacing: 0

        Item {
            Layout.fillWidth: true
            Layout.fillHeight: true
            SplitView {
                id: horizontalSplitView
                anchors.fill: parent
                orientation: Qt.Horizontal
                handleDelegate : Item {
                    implicitWidth: root.spacing
                }
                
                Binding {
                    target: horizontalSplitView.__handles[0]
                    property: "enabled"
                    value: !root.fullscreenMosaic || panelVisibilityControl.showLeftPanel
                }
                Binding {
                    target: horizontalSplitView.__handles[1]
                    property: "enabled"
                    value: !root.fullscreenMosaic || panelVisibilityControl.showRightPanel
                }
                
                MoveIn {
                    id: leftMoveInItem
                    moveEnabled: root.fullscreenMosaic
                    moveDirection: Qt.vector2d(1, 0)
                    show: panelVisibilityControl.showLeftPanel
                    
                    Layout.minimumWidth: 340
                    Layout.maximumWidth: 500
                    Layout.fillHeight: true
                    ColumnLayout {
                        anchors.fill: parent
                        spacing: root.fullscreenMosaic ? 0 : root.spacing
                        MixerPanel {
                            header: root.clockPanel.header
                            Layout.fillWidth: true
                            implicitHeight: 146
                            Item {
                                anchors.fill: parent
                                data: clockPanel
                            }
                        }
                        MixerPanel {
                            header: root.eventsPanel.header
                            Layout.fillWidth: true
                            Layout.fillHeight: true
                            Item {
                                anchors.fill: parent
                                data: root.eventsPanel
                            }
                        }
                    }
                }
                
                SplitView {
                    id: verticalSplitView
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    orientation: Qt.Vertical
                    handleDelegate : Item {
                        implicitHeight: root.spacing
                    }
                    Binding {
                        target: verticalSplitView.__handles[0]
                        property: "enabled"
                        value: !root.fullscreenMosaic || panelVisibilityControl.showUpPanel
                    }
                    MoveIn {
                        id: upMoveInItem
                        moveEnabled: root.fullscreenMosaic
                        moveDirection: Qt.vector2d(0, 1)
                        show: panelVisibilityControl.showUpPanel
                        
                        Layout.fillWidth: true
                        Layout.minimumHeight: 146
                        Layout.maximumHeight: 242
                        
                        RowLayout {
                            anchors.fill: parent

                            MixerPanel {
                                Layout.fillHeight: true
                                Layout.fillWidth: true                                
                                header: root.mosaicsPanel.header

                                Item {
                                    anchors.fill: parent
                                    data: root.mosaicsPanel
                                }
                            }
                            
                            MixerPanel {
                                Layout.fillHeight: true
                                Layout.preferredWidth: root.monitorsPanel.implicitWidth
                                header: root.monitorsPanel.header

                                Item {
                                    anchors.fill: parent
                                    data: root.monitorsPanel
                                }
                            }
                        }    
                    }
                    
                    Item {
                        Layout.fillWidth: true
                        Layout.fillHeight: true
                        OutputBackground {
                            id: nonfullscreenContentItem
                            anchors.fill: parent
                            visible: !root.fullscreenMosaic
                        }
                    }
                }
                
                MoveIn {
                    id: rightMoveInItem
                    moveEnabled: root.fullscreenMosaic
                    moveDirection: Qt.vector2d(-1, 0)
                    show: panelVisibilityControl.showRightPanel
                    
                    Layout.minimumWidth: 305
                    Layout.maximumWidth: 500
                    Layout.fillHeight: true
                    ColumnLayout {
                        anchors.fill: parent
                        spacing: root.fullscreenMosaic ? 0 : root.spacing

                        MixerPanel {
                            header: root.recordingPanel.header
                            Layout.fillWidth: true
                            implicitHeight: 82
                            Item {
                                anchors.fill: parent
                                data: root.recordingPanel
                            }
                        }
                        MixerPanel {
                            header: root.audioPanel.header
                            Layout.fillWidth: true
                            implicitHeight: 56
                            Item {
                                anchors.fill: parent
                                data: root.audioPanel
                            }
                        }

                        MixerPanel {
                            header: root.videoStreamsPanel.header
                            Layout.fillWidth: true
                            Layout.fillHeight: true
                            Item {
                                anchors.fill: parent
                                data: root.videoStreamsPanel
                            }
                        }
                        
                        MixerPanel {
                            header: root.videoStreamsArchivedPanel.header
                            Layout.fillWidth: true
                            Layout.fillHeight: true
                            Item {
                                anchors.fill: parent
                                data: root.videoStreamsArchivedPanel
                            }
                        }
                    }
                }
            }
        }
        Item {
            Layout.fillWidth: true
            visible: !root.fullscreenMosaic
            implicitHeight: statusBar ? statusBar.implicitHeight : 0 
            data: statusBar
        }
    }
 
    function save() {
        return {
            fullscreenMosaic: fullscreenMosaic,
            leftPanelSize: leftPanelSize,
            upPanelSize: upPanelSize,
            rightPanelSize: rightPanelSize
        }
    }
    
    function load(data) {
        fullscreenMosaic = data.fullscreenMosaic || false
        leftPanelSize = data.leftPanelSize || 0
        upPanelSize = data.upPanelSize || 0
        rightPanelSize = data.rightPanelSize || 0
    }    
    
    ObjectSaver {
        id: objectSaver
        object: root
        fileUrl: Serialization.applicationDataDirPath + "/gui_settings.json"
    }

    Component.onCompleted: objectSaver.init() 
}
