import QtQuick 2.0
import QtQuick.Layouts 1.1
import QtQuick.Controls 1.0
import QtQuick.Controls.Styles 1.2
import MixerGuiControls 1.0
import "utils/Conversion.js" as Conversion
import "utils/FormatTime.js" as FormatTime

Item {
    readonly property Item header: MixerPanelHeader {
        anchors.fill: parent
        text: qsTr("Archive")
        /*
        buttons: [
            MixerIconButton {
                icon: "Settings"
                action: actions.openRecordingSettingsWindowAction
                tooltip: qsTr("Recording Settings")
            }
        ]
        */
    }

    readonly property var recordedArchive: archiveRecorder && archiveRecorder.archive || false
    readonly property var currentArchive: archiveCollection && archiveCollection.currentArchive
    readonly property var archive: archiveCollection && archiveCollection.archives && archiveCollection.archives[0] || {}
    
    readonly property var isClosing: postPlayStatus==2 
    readonly property var diskSize: archive.diskSize || 0 
    readonly property var duration: archive.duration || 0 
    readonly property var date    : archive.creationDate || new Date() 

    RowLayout {
        spacing: 10
        Button {
			activeFocusOnTab: false
            //anchors.verticalCenter: parent.verticalCenter
            style: ButtonStyle {
                padding {left: 0; right: 0; top: 0; bottom: 0}
                background: Image {
                    source: recordedArchive ? "RecordWriting.png" : currentArchive ? "RecordReading.png" : "RecordDefault.png"
                }
                label: Item {}
            }
            action: archiveCollection && archiveCollection.currentArchive ? closeArchiveAction : openArchiveAction
            Component.onCompleted: __behavior.cursorShape = Qt.PointingHandCursor
        }
        ColumnLayout {
            MixerLabel {
                text: date.toLocaleString(Qt.locale(), "d MMMM yyyy г. hh:mm:ss")
                font.pixelSize: MixerThemeManager.theme.systemFontSize
                active: recordedArchive
            }
            MixerLabel {
                text: qsTr("Size - ") + Conversion.bytesToMB(diskSize).toFixed() + " MB"
                font.pixelSize: MixerThemeManager.theme.systemFontSize
                active: recordedArchive
                visible: !isClosing
            }            
            MixerLabel {
                text: qsTr('Registrator is down')
                color: '#ff3030'
                font.pixelSize: MixerThemeManager.theme.systemFontSize
                visible: isClosing
            }
        }
    }
    
    Action {
        id: openArchiveAction
        text: qsTr("Open")
        onTriggered: {
            archiveCollection.setCurrent(0)
        }
    }

    Action {
        id: closeArchiveAction
        text: qsTr("Close")
        onTriggered: {
            archiveCollection.setCurrent(-1)
            //app.doWithQuestion(qsTr("Do you really want to stop recording?"), archiveCollection.setCurrent(-1))
        }
    }

    Connections {
        target: archiveRecorder && archiveRecorder.recordingSettings || null
        onValidated: {
            if(error) {
                app.openErrorDialog(error)
                return
            }
            archiveCollection.removeEmptyArchives()
            startRecordingDialog.text = qsTr("Training")
            startRecordingDialog.open()
        }
    }
        
}
