import QtQml 2.2
import QtQuick 2.0
import QtQuick.Controls 1.1
import QtQuick.Layouts 1.1
import MixerGuiControls 1.0

MixerWindow {
    id: root
    signal accepted(url source, string name)
    property bool archived: false
    readonly property var mediasData: archived ? availableVideos : availableMedias && availableMedias.data ? availableMedias.data : []
    onMediasDataChanged: listModel.update()

    modality: Qt.WindowModal
    
    minimumWidth: 300
    minimumHeight: 400
    width: 500
    height: 400
    leftSpacing: MixerThemeManager.theme.itemSpacing + 2
    
    readonly property var typeToLabel: ({
        "fd": qsTr("FD Boards"),
        "dl": qsTr("DeckLink Boards"),
        "dshow": qsTr("DirectShow Video Capture"),
        "ip": qsTr("IP Cameras"),
        "video": qsTr("Video"),
        "test": qsTr("Test"),
        "archive": qsTr("Archive")
    })

    function updateInfo(media) {
        var source = media && media.source + '';
        if (!source) return
        var scheme = source.split('://')[0]
        var device = scheme;
        if (['rtsp','rtmp','udp','http'].indexOf(scheme)!=-1) device = 'ip'; 
        if (['file'       ].indexOf(scheme)!=-1) device = 'video'; 
        media.deviceType = device
    }
    
    Action {
        id: selectAction
        text: qsTr("OK")
        enabled: tableView.currentRow != -1
        onTriggered: {
            var media = mediasData[tableView.currentRow]
            if(media) accepted(media.source, media.name)
            root.hide()
        }
    }
    
    Action {
        id: cancelAction
        text: qsTr("Cancel")
        onTriggered: root.hide()
    }

    ListModel {
        id: listModel
        function update() {
            clear()
            for(var i = 0; i < mediasData.length; i++) {
                var media = mediasData[i]
                updateInfo(mediasData[i])
                append(mediasData[i])
            }
        }
    }

    ColumnLayout {
        anchors.fill: parent
        Keys.onEscapePressed: root.close()
        spacing: MixerThemeManager.theme.itemSpacing
        OutputBackground {                    
            Layout.fillHeight: true
            Layout.fillWidth: true
            MixerTableView {
                id: tableView
                anchors.fill: parent
                headerVisible: false

                TableViewColumn { role: "name"; title: "" }
                model: listModel
                itemDelegate: MixerTableViewLabel {
                    anchors.leftMargin: 12
                }
                onDoubleClicked: {
                    if(selectAction.enabled) selectAction.trigger()
                }
                section.property: "deviceType"
                section.delegate: MixerLabel {
                    x: 4
                    text: root.typeToLabel[section] || qsTr("Unknown")
                    font.pixelSize: MixerThemeManager.theme.systemFontSize
                    enabled: false
                }
            }
        }
        
        RowLayout {
            Layout.fillWidth: true
            Layout.alignment: Qt.AlignHCenter
            spacing: MixerThemeManager.theme.dialogButtonHSpacing
            DialogButton {
                action: selectAction
            }
            DialogButton { 
                action: cancelAction 
            }
        }        
    }
    
    function open() {
        root.show()
    }

    // хак: чтобы все нужные проперти MediaSettings пришли через Rpc
   readonly  property Instantiator __hack: Instantiator {
        model: mediasData
        QtObject {
            readonly property string mediaName: modelData.name
            readonly property string deviceType: modelData.deviceType
            onMediaNameChanged: listModel.update()
            onDeviceTypeChanged: listModel.update()
        }
    }
}
