import QtQuick 2.1
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.1
import MixerGuiControls 1.0
import "../../common/utils"

Item {
    property int selectedIndex: -1
    visible: monitorMosaics && monitorMosaics.length>1

    implicitWidth: visible ? flow.width+23 : 0

    readonly property Item header: MixerPanelHeader {
        anchors.fill: parent
        anchors.rightMargin: 3
        text: qsTr("Monitors")
    }
    
    OutputBackground {
        anchors.fill: parent
        ScrollViewWithButtons {
            anchors.fill: parent
            scrollStep: repeater.count > 0 ? repeater.itemAt(0).height : 1
            Flow {
                id:flow
                spacing: 0
                Repeater {
                    id: repeater
                    model: monitorMosaics
                    Item {
                        implicitWidth: mosaicPreview.implicitWidth + 6
                        implicitHeight: mosaicPreview.implicitHeight + 6
                        MosaicPreview {
                            id: mosaicPreview
                            title: qsTr("Monitor")+' '+(model.index+1)
                            anchors.centerIn: parent
                            mosaic: modelData
                            monitor: model.index
                            isSelected: currentMonitorIndex == model.index
                            onLeftButtonClicked: {
                                currentMonitorIndex = model.index
                                currentMosaicIndex = monitorIndexes[currentMonitorIndex]
                                videoStreamCollection.currentStream = null
                                updateMonitor();
                            }
                        }
                    }
                }
            }
        }
    }
    
    MixerMenu {
        id: contextMenu
        MixerMenuItem { action1: renameAction }
        MixerMenuItem { action1: deleteAction }
    }
    
    Action {
        id: addAction
        enabled: !!mosaicCollection
        text: qsTr("Add Mosaic")
        onTriggered: addDialog.open()
    }
    
    Action {
        id: renameAction
        enabled: !!mosaicCollection && selectedIndex != -1
        text: qsTr("Rename")
        onTriggered: {
            renameDialog.text = mosaicCollection.mosaics[selectedIndex].name
            renameDialog.open()
        }
    }
    
    Action {
        id: deleteAction
        enabled: !!mosaicCollection && selectedIndex != -1
        text: qsTr("Delete")
        onTriggered: mosaicCollection.remove(selectedIndex)
    }
    
    AddMosaicDialog {
        id: addDialog
        onAccepted: if(template) mosaicCollection.add(name || qsTr("Unnamed"), {template: template})
    }
    
    MixerStringInputDialog {
        id: renameDialog
        title: qsTr("Rename mosaic")
        caption: qsTr("Name: ")
        onEntered: mosaicCollection.rename(selectedIndex, value)
    }
}
