import QtQuick 2.0
import GuiControls 1.0

Item {
    id: root
    property QtObject mosaic
    readonly property var streams: (mosaic && mosaic.streams) ? mosaic.streams : []
    Item {
        id: streamDragAndDropSources
        anchors.fill: parent
        Repeater { 
            model: mosaic ? mosaic.rects : []
            DragAndDropSource {
                id: streamDragAndDropSource
                dragAndDropLayer: mainWindow.dragAndDropLayer
                property rect rect: modelData
                property QtObject stream: root.streams[model.index] || null
                visible: !!stream
                x: rect.x * streamDragAndDropSources.width
                y: rect.y * streamDragAndDropSources.height
                width: rect.width * streamDragAndDropSources.width
                height: rect.height * streamDragAndDropSources.height
                onClicked: {
                    if(videoStreamCollection) videoStreamCollection.toggleCurrentStream(stream,true,currentMonitorIndex)
                    if(dragAndDropLayer) dragAndDropLayer.drop()
                }
                dragItem: Rectangle {
                    implicitWidth: implicitHeight * 16 / 9
                    implicitHeight: mosaicHeight * 1.5
                    color: "black"
                    property QtObject dragData: streamDragAndDropSource.stream
                    property var dropCallback: function(dropCallbackData) {
                        if(mosaic) mosaic.setStream(model.index, dropCallbackData)
                    }
                    Drag.keys: ["VideoStream", "FromMosaic"]
                    VideoStreamPreviewIcon {
                        anchors.fill: parent
                        stream: streamDragAndDropSource.stream
                    }
                }
            }         
        }
    }
    
    DropArea {
        id: streamDropArea
        anchors.fill: parent
        keys: ["VideoStream"]
        onEntered: if(videoStreamCollection) videoStreamCollection.toggleCurrentStream(null,true,currentMonitorIndex)
        
        property var rects: (mosaic && mosaic.rects) ? mosaic.rects : []
        property int rectIndex: {
            var x = streamDropArea.drag.x / streamDropArea.width
            var y = streamDropArea.drag.y / streamDropArea.height
            for(var i = rects.length - 1; i >= 0; i--) {
                var r = rects[i]
                if(x >= r.x && x <= r.x + r.width && y >= r.y && y <= r.y + r.height) return i
            }
            return -1
        }

        Rectangle {
            color: Qt.rgba(147 / 255, 222 / 255, 255 / 255, 0.2)
            property rect rect: streamDropArea.rects[streamDropArea.rectIndex] || Qt.rect(0, 0, 0, 0)
            x: rect.x * streamDropArea.width
            y: rect.y * streamDropArea.height
            width: rect.width * streamDropArea.width
            height: rect.height * streamDropArea.height
            visible: streamDropArea.containsDrag && streamDropArea.rectIndex != -1
        }
        onDropped: {
            if(rectIndex == -1) return
            var streamUnderDrop = mosaic.streams[rectIndex]
            if(mosaic) mosaic.setStream(rectIndex, drag.source.dragData)
            if(drag.source.dropCallback) drag.source.dropCallback(streamUnderDrop)
        }
    }

    DropArea {
        id: mosaicDropArea
        anchors.fill: parent
        keys: ["Mosaic","Monitor"]
        
        Rectangle {
            anchors.fill: parent
            color: Qt.rgba(147 / 255, 222 / 255, 255 / 255, 0.2)
            visible: parent.containsDrag
        }

        onDropped: {
            var monitor = drag.source.dragData.monitor

            if (monitor!=null) {
                currentMonitorIndex = monitor
                currentMosaicIndex = monitorIndexes[currentMonitorIndex]
            } else {
                var index = mosaicCollection.mosaics.indexOf(drag.source.dragData.mosaic)
                if (index<0) return;
                currentMosaicIndex = index
            }

            videoStreamCollection.currentStream = null
            updateMonitor();
        }
    }
}
