import QtQuick 2.0
import "utils"
import "../../common/utils"

Item {
    id: root

    FillModeItem {
        anchors.fill: parent
        aspect: 16/9
        Rectangle {
            anchors.fill: parent
            color: "black"
            visible: !video.visible
        }
        ShaderEffect {
            id: video
            anchors.fill: parent
            property ShaderEffectSource source: videoStreamAtlas.mosaicElement
            implicitWidth: videoStreamAtlas.mosaicElement.width
            implicitHeight: videoStreamAtlas.mosaicElement.height
            visible: !!server && implicitWidth > 0 && video.implicitHeight > 0
        }
        Item {
            id: streamDragAndDropSources
            anchors.fill: parent
            Repeater {
                model: currentMosaic && currentMosaic.rects ? currentMosaic.rects : []
                DragAndDropSource {
                    id: streamDragAndDropSource
                    dragAndDropLayer: mainWindow.dragAndDropLayer
                    property rect rect: modelData
                    property QtObject stream: currentMosaic && currentMosaic.streams ? currentMosaic.streams[model.index] || null : null
                    visible: !!stream
                    x: rect.x * streamDragAndDropSources.width
                    y: rect.y * streamDragAndDropSources.height
                    width: rect.width * streamDragAndDropSources.width
                    height: rect.height * streamDragAndDropSources.height

                    onClicked: if(videoStreamCollection) videoStreamCollection.toggleCurrentStream(stream,true,currentMonitorIndex)

                    dragItem: Rectangle {
                        property real aspect: streamDragAndDropSource.stream ? streamDragAndDropSource.stream.aspect : 0
                        implicitWidth: implicitHeight * Math.max(4 / 3, aspect)
                        implicitHeight: mosaicHeight * 1.5
                        border.color: "black"
                        property QtObject dragData: streamDragAndDropSource.stream
                        property var dropCallback: function(dropCallbackData) {
                            if(currentMosaic) currentMosaic.setStream(model.index, dropCallbackData)
                        }
                        Drag.keys: ["VideoStream", "FromMosaicPanel"]
                        VideoStreamAtlasElement {
                            anchors.fill: parent
                            anchors.margins: 1
                            stream: streamDragAndDropSource.stream
                            fillMode: Image.Stretch
                        }
                    }
                }         
            }
        }
        
        DropArea {
            id: streamDropArea
            anchors.fill: parent
            keys: ["VideoStream"]
            
            property var rects: currentMosaic && currentMosaic.rects ? currentMosaic.rects : []
            property int rectIndex: {
                var x = streamDropArea.drag.x / streamDropArea.width
                var y = streamDropArea.drag.y / streamDropArea.height
                for(var i = rects.length - 1; i >= 0; i--) {
                    var r = rects[i]
                    if(x >= r.x && x <= r.x + r.width && y >= r.y && y <= r.y + r.height) return i
                }
                return -1
            }
            property QtObject streamUnderDrop: currentMosaic && currentMosaic.streams ? currentMosaic.streams[rectIndex] || null : null
            
            Rectangle {
                color: Qt.rgba(147 / 255, 222 / 255, 255 / 255, 0.2)
                border.color: "black"
                property rect rect: streamDropArea.rects[streamDropArea.rectIndex] || Qt.rect(0, 0, 0, 0)
                x: rect.x * streamDropArea.width
                y: rect.y * streamDropArea.height
                width: rect.width * streamDropArea.width
                height: rect.height * streamDropArea.height
                visible: streamDropArea.containsDrag && streamDropArea.rectIndex != -1
            }
            onDropped: {
                if(rectIndex == -1) return
                if(drag.source.dropCallback) drag.source.dropCallback(streamDropArea.streamUnderDrop)
                if(currentMosaic) currentMosaic.setStream(rectIndex, drag.source.dragData)
            }
        }
        DropArea {
            id: mosaicDropArea
            anchors.fill: parent
            keys: ["Mosaic"]
            Rectangle {
                anchors.fill: parent
                color: Qt.rgba(147 / 255, 222 / 255, 255 / 255, 0.2)
                border.color: "black"
                visible: mosaicDropArea.containsDrag
            }
            onDropped: setMonitorMosaic(drag.source.dragData)
        }
    }
    //ServerRunningControl {
    //    anchors.fill: parent
    //}
}
