import QtQuick 2.0
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.0
import QtQuick.Window 2.0
import MixerGuiControls 1.0
import "TemplateEditor"

MixerWindow {
    id: root
    width: 500
    height: 600
    minimumWidth: 500
    minimumHeight: 300
    modality: Qt.ApplicationModal
    title: qsTr("Mosaic Presets")
    readonly property int currentIndex: mosaicPresets ? mosaicPresets.currentIndex : -1
    readonly property int selectedIndex: tableView.rowCount > 0 ? tableView.currentRow : -1
    
    ColumnLayout {
        anchors.fill: parent
        MixerTableView {
            id: tableView
            Layout.fillWidth: true
            Layout.fillHeight: true
            TableViewColumn{ role: "data" }	
            headerVisible: false
            horizontalScrollBarPolicy: Qt.ScrollBarAlwaysOff
            
            property var modelTmp: mosaicPresets ? mosaicPresets.presetNames : []
            onModelTmpChanged: {
                var row = currentRow
                model = modelTmp
                currentRow = Math.min(row, rowCount - 1)
                if(currentRow == -1 && rowCount > 0) currentRow = 0
                selection.clear()
                if(currentRow >= 0 && rowCount > 0) selection.select(currentRow)            
            }
            
            itemDelegate: Item {
                MixerLabel {
                    anchors.fill: parent
                    anchors.leftMargin: 18
                    verticalAlignment: Text.AlignVCenter
                    text: styleData.value
                    elide: styleData.elideMode
                    colorType: styleData.selected ? "program" : "neutral"
                    MixerLabel {
                        text: "•"
                        verticalAlignment: Text.AlignVCenter
                        height: parent.height
                        colorType: parent.colorType
                        visible: styleData.row == currentIndex
                        x: -width * 1.5
                    }
                }
                MouseArea {
                    anchors.fill: parent
                    acceptedButtons: Qt.LeftButton | Qt.RightButton
                    onPressed: {
                        tableView.currentRow = styleData.row
                        if(mouse.button == Qt.RightButton) contextMenu.popup()
                    }
                    onDoubleClicked: {
                        if(mouse.button == Qt.LeftButton) {
                            tableView.currentRow = styleData.row
                            selectAction.trigger()
                        }
                    }
                }
            }
            
            onCurrentRowChanged: {
                selection.clear()
                if(currentRow >= 0 && currentRow < rowCount) selection.select(currentRow)            
            }
        }
        
        RowLayout {
            Layout.alignment: Qt.AlignHCenter
            DialogButton { action: selectAction }
            DialogButton { action: addAction }
            DialogButton { action: renameAction }
            DialogButton { action: deleteAction }
        }
    }
    
    MixerMenu {
        id: contextMenu
        MixerMenuItem { action1: selectAction }
        MixerMenuItem { action1: renameAction }
        MixerMenuItem { action1: deleteAction }
    }
    
    Action {
        id: addAction
        text: qsTr("Add")
        onTriggered: addDialog.open()
    }
    
    Action {
        id: selectAction
        enabled: selectedIndex != -1
        text: qsTr("Select")
        onTriggered: {
            mosaicPresets.currentIndex = selectedIndex
            root.hide()
        }
    }
    
    Action {
        id: renameAction
        enabled: selectedIndex != -1
        text: qsTr("Rename")
        onTriggered: {
            renameDialog.text = tableView.model[selectedIndex] || ""
            renameDialog.open()
        }
    }
    
    Action {
        id: deleteAction
        enabled: selectedIndex != -1
        text: qsTr("Delete")
        shortcut: "Del"
        onTriggered: {
            doWithQuestion(qsTr("Do you really want to delete this mosaic preset?"), function() {
                mosaicPresets.remove(selectedIndex)
            })
        }        
    } 
    
    MixerStringInputDialog {
        id: addDialog
        title: qsTr("Add preset")
        caption: qsTr("Name: ")
        allowEmptyString: true
        onEntered: mosaicPresets.add(value || qsTr("Unnamed"), {})
    }
    
    MixerStringInputDialog {
        id: renameDialog
        title: qsTr("Rename preset")
        caption: qsTr("Name: ")
        onEntered: mosaicPresets.rename(selectedIndex, value)
    }    
    
}
