import QtQuick 2.0
import QtQuick.Layouts 1.0
import QtQuick.Controls 1.0
import GuiControls 1.0
import MixerGuiControls 1.0

Item {
    id: root
    property QtObject mosaic
    property bool isSelected
    property var monitor
    property var title
    signal leftButtonClicked()
    signal rightButtonClicked()
    signal dropped(var target, var drag)

    implicitWidth: icon.implicitWidth
    implicitHeight: layout.implicitHeight
       
    ColumnLayout {
        id: layout
        anchors.fill: parent
        spacing: 3       
        
        MosaicPreviewIcon {
            id: icon
            mosaic: root.mosaic
            OutputBorder {
                anchors.fill: parent
                anchors.margins: -4
                colorType: "program"
                visible: isSelected
                borderWidth: 2
            }

            DropArea {
                anchors.fill: parent
                keys: ["Mosaic"]
                onDropped: root.dropped(root.mosaic,drag)

                Rectangle {
                    anchors.fill: parent
                    color: Qt.rgba(147 / 255, 222 / 255, 255 / 255, 0.2)
                    border.color: "black"
                    visible: parent.containsDrag
                }
            }

            DragAndDropSource {
                anchors.fill: parent
                dragAndDropLayer: mainWindow.dragAndDropLayer
                onClicked: root.leftButtonClicked()

                dragItem: Rectangle {
                    implicitWidth: implicitHeight * 16/9
                    implicitHeight: mosaicHeight
                    color: "black"
                    border.width: 1
                    property QtObject dragData: root
                    Drag.keys:  [monitor!=null ? "Monitor":"Mosaic"]
                    
                    MosaicPreviewIcon {
                        mosaic: root.mosaic
                        anchors.fill: parent
                        anchors.margins: 1
                    }
                }    
            }

            MouseArea {
                anchors.fill: parent
                acceptedButtons: Qt.RightButton
                onClicked: root.rightButtonClicked()
            }
        }
        MixerLabel {
            Layout.fillWidth: true
            text: root.title || mosaic.name
            colorType: isSelected ? "program" : "neutral"
            font.pixelSize: MixerThemeManager.theme.systemFontSize
            horizontalAlignment: Text.AlignHCenter
        }        
    }        
}