import QtQuick 2.1
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.1
import MixerGuiControls 1.0

Item {
    property int selectedIndex: -1

    readonly property Item header: MixerPanelHeader {
        anchors.fill: parent
        anchors.rightMargin: 3
        text: qsTr("Mosaics")
        buttons: [
            MixerIconButton {
                icon: "Add"
                action: addAction
            }
        ]
    }
    
    OutputBackground {
        anchors.fill: parent
        ScrollViewWithButtons {
            anchors.fill: parent
            scrollStep: repeater.count > 0 ? repeater.itemAt(0).height : 1
            Flow {
                width: parent.width
                spacing: 0
                Repeater {
                    id: repeater
                    model: mosaicCollection ? mosaicCollection.mosaics : []
                    Item {
                        implicitWidth: mosaicPreview.implicitWidth + 6
                        implicitHeight: mosaicPreview.implicitHeight + 6
                        MosaicPreview {
                            id: mosaicPreview
                            anchors.centerIn: parent
                            mosaic: modelData
                            isSelected: currentMosaicIndex == model.index
                            onLeftButtonClicked: {
                                currentMosaicIndex = model.index
                                videoStreamCollection.currentStream = null
                                updateMonitor();
                            }
                            onRightButtonClicked: {
                                selectedIndex = model.index
                                contextMenu.popup()
                            }
                            onDropped: {
                                var from = mosaicCollection.mosaics.indexOf(drag.source.dragData.mosaic)
                                var to   = mosaicCollection.mosaics.indexOf(target)
                                if (to>=0 && drag.x > width/2) to += 1;
                                if (from<0 || to<0 || from==to) return;
                                mosaicCollection.move(from,to)
                            }
                        }
                    }
                }
            }
        }
        MixerLabel {
            text: qsTr("Mosaic preset is not selected")
            anchors.centerIn: parent
            visible: !mosaicCollection
        }
    }
    
    
    MixerMenu {
        id: contextMenu
        MixerMenuItem { action1: renameAction }
        MixerMenuItem { action1: deleteAction }
    }
    
    Action {
        id: addAction
        enabled: !!mosaicCollection
        text: qsTr("Add Mosaic")
        onTriggered: addDialog.open()
    }
    
    Action {
        id: renameAction
        enabled: !!mosaicCollection && selectedIndex != -1
        text: qsTr("Rename")
        onTriggered: {
            renameDialog.text = mosaicCollection.mosaics[selectedIndex].name
            renameDialog.open()
        }
    }
    
    Action {
        id: deleteAction
        enabled: !!mosaicCollection && selectedIndex != -1
        text: qsTr("Delete")
        onTriggered: mosaicCollection.remove(selectedIndex)
    }
    
    AddMosaicDialog {
        id: addDialog
        onAccepted: if(template) mosaicCollection.add(name || qsTr("Unnamed"), {template: template})
    }
    
    MixerStringInputDialog {
        id: renameDialog
        title: qsTr("Rename mosaic")
        caption: qsTr("Name: ")
        onEntered: mosaicCollection.rename(selectedIndex, value)
    }
}
