import QtQuick 2.0
import QtQuick.Layouts 1.0
import QtQuick.Controls 1.0
import MixerGuiControls 1.0
import GuiControls 1.0
import UdpProtocol 1.0

MixerDialog {
    id: root
    title: qsTr("Network settings")
    
    GridLayout {
        anchors.fill: parent
        columns: 2
        
        MixerLabel {
            text: qsTr("Local Address: ")
        }
       MixerComboBox {
            id: localAddressComboBox
            function setValue(value) {
                for(var i = 0; i < NetworkAddressEnumerator.addresses.length; i++) {
                    if(NetworkAddressEnumerator.addresses[i] == value) {
                        currentIndex = i
                        return
                    }
                    currentIndex = -1
                }
            }
            model: {
                var result = []
                for(var i = 0; i < NetworkAddressEnumerator.addresses.length; i++) {
                    result.push(NetworkAddressEnumerator.addresses[i])
                }
                return result
            }
        }
                
        MixerLabel {
            text: qsTr("Local port: ")
        }
        TextField {
            id: localPortTextField
            validator: IntValidator {
                bottom: 0
                top: 99999
            }
        }
                
        MixerLabel {
            text: qsTr("Remote port: ")
        }
        TextField {
            id: remotePortTextField
            validator: IntValidator {
                bottom: 0
                top: 99999
            }
        }
    }
    
    buttons: [qsTr("OK"), qsTr("Cancel")]
    
    onButtonClicked: {
        if(buttonIndex == 0) {
            networkEventReceiver.localAddress = localAddressComboBox.currentText
            networkEventReceiver.localPort = localPortTextField.text
            networkEventReceiver.remotePort = remotePortTextField.text       
        }
        hide()
    }

    readonly property QtObject serverValues: QtObject {
        property string localAddress: networkEventReceiver ? networkEventReceiver.localAddress : ""
        property string localPort: networkEventReceiver ? networkEventReceiver.localPort : 0
        property string remotePort: networkEventReceiver ? networkEventReceiver.remotePort : 0
    }
        
    onOpened: {
        localAddressComboBox.setValue(serverValues.localAddress)
        localPortTextField.text = serverValues.localPort
        remotePortTextField.text = serverValues.remotePort
        localPortTextField.forceActiveFocus()
    }
}
