import QtQuick 2.0
import QtQuick.Controls 1.3
import QtQuick.Layouts 1.0
import QtQuick.Window 2.0
import MixerGuiControls 1.0
import "utils/FormatTime.js" as FormatTime

MixerWindow {
    id: root
    width: 600
    height: 750
    minimumWidth: 400
    minimumHeight: 300
    modality: Qt.ApplicationModal
    title: {
        var name = operationLog ? operationLog.currentName : ""
        return qsTr("Operation Log") + (name ? " (" + name + ")" : "")
    }
    
    ColumnLayout {
        anchors.fill: parent
        spacing: 10
        MixerTableView {
            id: tableView
            Layout.fillWidth: true
            Layout.fillHeight: true
            headerVisible: true
            selectionMode: SelectionMode.NoSelection
            model: {
                if(!operationLog || !operationLog.elements) return
                if(typeComboBox.currentValue) return operationLog.elements.filter(function(element) {
                    return element.type == typeComboBox.currentValue
                })
                return operationLog.elements
            }
            horizontalScrollBarPolicy: Qt.ScrollBarAlwaysOff
            TableViewColumn {
                id: timeColumn
                title: qsTr("Time")
                role: "date"
                width: 80
                movable: false
                resizable: false
                delegate: MixerLabel {
                    anchors.fill: parent
                    text: styleData.value.toLocaleString(Qt.locale(), "hh:mm:ss")
                    verticalAlignment: Text.AlignVCenter
                    horizontalAlignment: Text.AlignHCenter
                    elide: styleData.elideMode
                    colorType: styleData.selected ? "program" : "neutral"
                    font.pixelSize: MixerThemeManager.theme.systemFontSize
                }
            }
            
            TableViewColumn {
                title: qsTr("Message")
                role: "message"
                width: tableView.width - timeColumn.width - typeColumn.width
                movable: false
                resizable: false
            }
            
            TableViewColumn {
                id: typeColumn
                title: qsTr("Type")
                role: "type"
                width: 100
                movable: false
                resizable: false
                delegate: MixerLabel {
                    anchors.fill: parent
                    anchors.leftMargin: 8
                    verticalAlignment: Text.AlignVCenter
                    colorType: styleData.selected ? "program" : "neutral"
                    font.pixelSize: MixerThemeManager.theme.systemFontSize
                    text: {
                        if(styleData.value == "info") return qsTr("Info")
                        if(styleData.value == "error") return qsTr("Error")
                        return ""
                    }
                }
            }       
        }
        
        RowLayout {
            MixerLabel {
                text: qsTr("Day")
                font.pixelSize: MixerThemeManager.theme.systemFontSize
                Layout.alignment: Qt.AlignBaseline
            }
            TextField {
                id: dayTextField
                text: operationLog ? operationLog.currentName : ""
                readOnly: true
                implicitWidth: 90
                Layout.alignment: Qt.AlignBaseline
            }
            Button {
                text: "..."
                implicitWidth: dayTextField.implicitHeight
                implicitHeight: dayTextField.implicitHeight
                onClicked: calendarWindow.show()
            }

            Item {
                implicitWidth: 10
            }
            
            MixerLabel {
                text: qsTr("Type")
                font.pixelSize: MixerThemeManager.theme.systemFontSize
                Layout.alignment: Qt.AlignBaseline
            }
            MixerComboBox {
                id: typeComboBox
                Layout.alignment: Qt.AlignBaseline
                model: [qsTr("All"), qsTr("Info"), qsTr("Error")]
                readonly property string currentValue: {
                    if(currentIndex == 1) return "info"
                    if(currentIndex == 2) return "error"
                    return ""
                }
            }            
        }
    }
    MixerWindow {
        id: calendarWindow
        width: 400
        height: 360
        resizeEnabled: false
        title: qsTr("Select a date")
        
        Calendar {
            anchors.fill: parent   
            style: DarkCalendarStyle {}
            maximumDate: new Date()
            onClicked: {
                operationLog.currentDate = date
                calendarWindow.close()
            }
        }
    }
}
