import QtQuick 2.1
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.1
import MixerGuiControls 1.0
import "Timeline"
import "utils/FormatTime.js" as FormatTime

Item {
    id: root    
    ColumnLayout {
        anchors.fill: parent
        anchors.leftMargin: 12
        anchors.rightMargin: 12
        anchors.topMargin: 10
        anchors.bottomMargin: 0
        spacing: 2
        enabled: archivePlayer && archivePlayer.archive
        Timeline {
            id: timeline
            Layout.fillWidth: true
            Layout.fillHeight: true
            position: archivePlayer ? archivePlayer.position : 0
            duration: archivePlayer ? archivePlayer.duration : 0
            playing: archivePlayer ? archivePlayer.isPlaying : false
            recording: archiveRecorder ? archiveRecorder.running : false
            
            intervals: {
                if(!exporterSettings) return []
                if(!exportDialog.visible && exporterSettings.selectIntervalMode == 0) return []
                return exporterSettings.intervals || []
            }
            
            onBeforeSeek: if(archivePlayer) archivePlayer.setSeeking(true)
            onAfterSeek: if(archivePlayer) archivePlayer.setSeeking(false)
            onSeek: if(archivePlayer) archivePlayer.seek(time)
        }
        MixerToolbar {
            Layout.fillWidth: true
            ToolbarButton {
                icon: "Play"
                action: playAction
            }
            ToolbarButton {
                icon: "Pause"
                action: pauseAction
            }                
            ToolbarButton {
                icon: "Stop"
                action: stopAction
            }
            
            ToolbarSeparator {}
            
            ToolbarButton {
                icon: "ArrowLeft"
                action: stepBackAction
            }
            ToolbarButton {
                icon: "ArrowRight"
                action: stepForwardAction
            }   

            ToolbarButton {
                icon: "ToBegin"
                action: moveToBeginAction
            }
            ToolbarButton {
                icon: "ToEnd"
                action: moveToEndAction
            }    

            ToolbarSeparator {}
            
            Item {
                implicitWidth: 5
            }
            
            RateSlider {
                id: rateSlider
                onRateChanged: if(pressed && archivePlayer) archivePlayer.setRate(rate)
                Binding on value {
                    value: rateSlider.rateToValue(archivePlayer ? archivePlayer.rate : 0)
                }
            }
            MixerLabel {
                text: "x" + rateSlider.rate
                font.pixelSize: MixerThemeManager.theme.systemFontSize
            }
            
            Item {
                Layout.fillWidth: true
            }
            
            ToolbarButton {
                icon: "ZoomFit"
                action: fitZoomAction
            }
            ToolbarButton {
                icon: "ZoomIn"
                action: zoomInAction
            }
            ToolbarButton {
                icon: "ZoomOut"
                action: zoomOutAction
            }   
            
            ToolbarSeparator {}
            
            MixerLabel {
                text: qsTr("Duration - ") + FormatTime.millisecondsToString(archivePlayer ? archivePlayer.duration : 0)
                font.pixelSize: MixerThemeManager.theme.systemFontSize
            }
        }
    }
    
    DialogButton {
        id: selectExportPositionButton
        text: qsTr("Select")
        visible: exporterSettings && exporterSettings.selectIntervalMode > 0
        y: -height + 5
        x: {
            timer.tick
            exporterSettings && exporterSettings.selectIntervalMode // обновление биндинга
            var d = 2
            var w = selectExportPositionButton.width
            var value = timeline.mapPositionIndicatorToItem(root) - w/2
            return Math.max(d, Math.min(value, root.width - w - d))
        }
        onClicked: {
            exportDialog.open()
        }
        property Timer timer: Timer {
            running: selectExportPositionButton.visible
            repeat: true
            interval: 10
            property bool tick
            onTriggered: tick = !tick
        }
    }
    
    Action {
        id: playAction
        enabled: archivePlayer && !archivePlayer.isPlaying
        onTriggered: archivePlayer.play()
        text: qsTr("Play")
    }
    
    Action {
        id: pauseAction
        enabled: archivePlayer && archivePlayer.isPlaying
        onTriggered: archivePlayer.pause()
        text: qsTr("Pause")
    }
    
    Action {
        id: stepBackAction
        enabled: archivePlayer && !archivePlayer.isPlaying
        text: qsTr("Step Back")
        onTriggered: archivePlayer.stepBack()
    }
    
    Action {
        id: stepForwardAction
        enabled: archivePlayer && !archivePlayer.isPlaying
        text: qsTr("Step Forward")
        onTriggered: archivePlayer.stepForward()
    }   

    Action {
        id: moveToBeginAction
        text: qsTr("Move To Begin")
        enabled: archivePlayer
        onTriggered: archivePlayer.setPosition(0)
    }
    
    Action {
        id: moveToEndAction
        text: qsTr("Move To End")
        enabled: archivePlayer
        onTriggered: archivePlayer.setPosition(archivePlayer.duration)
    }       
    
    Action {
        id: stopAction
        enabled: archivePlayer && !archivePlayer.isStopped
        onTriggered: archivePlayer.stop()
        text: qsTr("Stop")
    }
    
    Action {
        id: fitZoomAction
        onTriggered: timeline.fitZoom()
        text: qsTr("Fit Zoom")
    }
    
    Action {
        id: zoomInAction
        enabled: timeline.zoomFactor < timeline.maximumZoomFactor
        onTriggered: timeline.zoomIn()
        text: qsTr("Zoom In")
    } 

    Action {
        id: zoomOutAction
        enabled: timeline.zoomFactor > timeline.minimumZoomFactor
        onTriggered: timeline.zoomOut()
        text: qsTr("Zoom Out")
    } 
}
