import QtQuick 2.0
import QtQuick.Layouts 1.0
import QtQuick.Controls 1.0
import MixerGuiControls 1.0

MixerDialog {
    id: root
    property int elementIndex
    property bool readonly
    title: elementIndex == -1 ? qsTr("Add Source") : qsTr("Edit Source")
    GridLayout {
        anchors.fill: parent
        columns: 2
        
        MixerLabel {
            text: qsTr("Type: ")
        }
        MixerComboBox {
            id: typeComboBox
            Layout.fillWidth: true
            model: [qsTr("Video"), qsTr("Audio")]
            enabled: !root.readonly
            onCurrentIndexChanged: {
                audioSourceSelector.currentIndex = 0
                videoSourceSelector.currentIndex = 0
                bitrateSpinBox.value = bitrateSpinBox.defaultBitrates[currentIndex]
                hardwareCheckBox.checked = false
                Qt.callLater(function() {
                    root.height = root.implicitHeight
                })
            }
        }
        
        MixerLabel {
            text: qsTr("Source: ")
            Layout.alignment: Qt.AlignTop
        }
        MediaUrlComboBox {
            id: videoSourceSelector
            visible: typeComboBox.currentIndex == 0
            enabled: !root.readonly
            Layout.fillWidth: true
        }
        FdInputSelector {
            id: audioSourceSelector
            visible: typeComboBox.currentIndex == 1
            enabled: !root.readonly
            Layout.fillWidth: true
        }

        MixerLabel {
            text: qsTr("Storage: ")
        }
        PostPlayStorageSelector {
            id: storageSelector
            enabled: !root.readonly
            Layout.fillWidth: true
        }
        TrackProperty {
            object: root.stream
            property: "mediaType"
            label: qsTr("Media Type")
            tracks: root.stream ? root.stream.supportedMediaTypes : []
        }
        MixerLabel {
            text: qsTr("Bitrate: ")
            visible: typeComboBox.currentIndex == 0
        }
        SpinBox {
            id: bitrateSpinBox
            visible: typeComboBox.currentIndex == 0
            enabled: !root.readonly
            readonly property var defaultBitrates: [5000, 100]
            value: defaultBitrates[0]
            minimumValue: 1
            maximumValue: 99999
            stepSize: 10
            suffix: qsTr(" Kb/s")
        }
        
        MixerLabel {
            text: qsTr("Hardware: ")
            visible: typeComboBox.currentIndex == 0
        }
        CheckBox {
            id: hardwareCheckBox
            visible: typeComboBox.currentIndex == 0
            enabled: !root.readonly
        }
    }
    
    buttons: [qsTr("OK"), qsTr("Cancel")]
    
    onButtonClicked: {
        if(!root.readonly && buttonIndex == 0) {
            var element = recordingSettings.elements[elementIndex] || null
            if(!element) {
                recordingSettings.add(undefined, {
                    type: typeComboBox.currentIndex,
                    source: (typeComboBox.currentIndex == 0) ? videoSourceSelector.source : audioSourceSelector.source,
                    storageGuid: storageSelector.storage ? storageSelector.storage.guid : 0,
                    bitrate: bitrateSpinBox.value,
                    hardware: hardwareCheckBox.checked
                })
            }
            else {
                element.type = typeComboBox.currentIndex
                element.source = (typeComboBox.currentIndex == 0) ? videoSourceSelector.source : audioSourceSelector.source
                element.storageGuid = storageSelector.storage ? storageSelector.storage.guid : 0
                element.bitrate = bitrateSpinBox.value
                element.hardware = hardwareCheckBox.checked
            }
        }
        clear()
        hide()
    }
    
    onClosing: clear()
        
    onOpened: {
        var element = recordingSettings.elements[elementIndex] || null
        if(element) {
            typeComboBox.currentIndex = element.type
            if(element.type == 0) videoSourceSelector.source = element.source
            else audioSourceSelector.source = element.source
            storageSelector.setCurrentStorageByGuid(element.storageGuid)
            bitrateSpinBox.value = element.bitrate
            hardwareCheckBox.checked = element.hardware
        }
    }
    
    function clear() {
        elementIndex = -1
        typeComboBox.currentIndex = 0
        videoSourceSelector.currentIndex = 0
        audioSourceSelector.currentIndex = 0
        storageSelector.currentIndex = 0
        bitrateSpinBox.value = bitrateSpinBox.defaultBitrates[0]
        hardwareCheckBox.checked = false
    }
}
