import QtQuick 2.0
import QtQuick.Layouts 1.0
import QtQuick.Controls 1.0

import Multimedia 1.0
import GuiControls 1.0
import MixerGuiControls 1.0

import "../../common/utils"

PropertyGroup {
    id: root
    expanded: true
    collapsible: false

    property QtObject recorder: null
    readonly property int elementType: object ? object.type : 0
    readonly property var elementStorage: object ? object.storage : undefined
    readonly property url elementSource: object ? object.source : ""

    property var mediaSettings: recorder ? recorder.mediaSettings : null
    readonly property variant currentVideoType: mediaSettings ? mediaSettings.videoType : undefined

    EnumProperty {
        id: typeSelector
        enabled: false
        label: qsTr("Type")
        property: "type"
        values: [ 0, 1 ]
        strings: [ qsTr("Video"), qsTr("Audio") ]
    }

    MediaUrlProperty {
        enabled: false
        visible: root.elementType == 0
        //comboBoxImplicitWidth: 300
    }

    FdInputProperty {
        enabled: false
        visible: root.elementType == 1
        //comboBoxImplicitWidth: 300
    }

    PostPlayStorageProperty {
        label: qsTr("Storage")
    }

    VideoTypeProperty {
        object: root.mediaSettings
        property: "videoType"
        mediaTypes: root.recorder ? root.recorder.supportedMediaTypes : []
        visible: !!mediaTypes && mediaTypes.length && (typeSelector.currentIndex == 0)
    }

    NumberProperty {
        label: qsTr("Bitrate")
        property: "bitrate"
        minimumValue: 1
        maximumValue: 99999
        stepSize: 10
        suffix: qsTr(" Kb/s")
    }

    BoolProperty {
        label: qsTr("Hardware")
        property: "hardware"
    }
}
