import QtQuick 2.3
import QtQuick.Window 2.2
import QtQuick.Layouts 1.1
import QtQuick.Controls 1.4

import Multimedia.FDExt 1.0
import GuiControls 1.0
import MixerGuiControls 1.0

MixerWindow {
    id: root
    width: 600
    height: 400
    minimumWidth: 500
    minimumHeight: 300
    modality: Qt.ApplicationModal
    title: qsTr("Recorded Sources")

    readonly property bool readonly: archiveRecorder && archiveRecorder.running
    readonly property var recorders: archiveRecorder ? archiveRecorder.recorders : []
    readonly property int selectedIndex: tableView.rowCount > 0 ? tableView.currentRow : -1
    readonly property QtObject selectedElement: (recordingSettings && recordingSettings.elements) ? (recordingSettings.elements[selectedIndex] || null) : null
    readonly property var defaultStorage: (postPlayService.storages && postPlayService.storages.length) ? postPlayService.storages[0] : null

    ColumnLayout {
        anchors.fill: parent

        SplitView {
            Layout.fillWidth: true
            Layout.fillHeight: true

            MixerTableView {
                id: tableView
                Layout.fillHeight: true
                Layout.minimumWidth: implicitWidth
                Layout.alignment: Qt.AlignLeft
                headerVisible: false
                horizontalScrollBarPolicy: Qt.ScrollBarAlwaysOff

                property var modelTmp: recordingSettings ? recordingSettings.elements : []
                onModelTmpChanged: {
                    var row = currentRow
                    model = modelTmp
                    currentRow = Math.min(row, rowCount - 1)
                    if(currentRow == -1 && rowCount > 0) currentRow = 0
                    selection.clear()
                    if(currentRow >= 0 && rowCount > 0) selection.select(currentRow)
                }

                TableViewColumn {
                    role: "sourceString"
                    title: qsTr("Source")
                    movable: false
                    resizable: false
                    width: tableView.width
                    delegate: RowLayout {
                        Item {
                            width: 10
                        }
                        MixerLabel {
                            Layout.fillWidth: true
                            text: (styleData.value && styleData.value.length) ? styleData.value : qsTr("<Unknown>")
                            font.pixelSize: 14
                            colorType: (model && (model.index == tableView.currentRow)) ? "program" : "neutral"
                        }
                    }
                }

                rowOverlayDelegate: MouseArea {
                    height: MixerThemeManager.theme.systemFontSize + 4
                    acceptedButtons: Qt.LeftButton
                    onPressed: {
                        if(styleData.row == undefined) return
                        tableView.currentRow = styleData.row
                    }
                }

                onCurrentRowChanged: {
                    selection.clear()
                    if(currentRow >= 0 && currentRow < rowCount) selection.select(currentRow)
                }
            }
            MixerPropertyPanel {
                Layout.fillHeight: true
                Layout.minimumWidth: implicitWidth
                RecordingSettingsElementPanel {
                    id: recordingSettingsPanel
                    enabled: !root.readonly
                    visible: object
                    object: root.selectedElement
                    recorder: !!root.recorders ? (root.recorders[root.selectedIndex] || null) : null
                }
            }
        }
        RowLayout {
            Layout.alignment: Qt.AlignBottom | Qt.AlignHCenter
            DialogButton {
                enabled: !root.readonly
                text: qsTr("Add")
                onClicked: addMenu.popup()
             }
            DialogButton { action: deleteAction }
        }
    }
  
   /* Action {
        id: addAction
        text: qsTr("Add")
        tooltip: qsTr("Add Source")
        enabled: !root.readonly
        onTriggered: {
            recordingSettings.add(undefined, {
                type: 0,
                source: "",
                storageGuid: 0,
                bitrate: 5000,
                hardware: false
            })
        }
    }*/
    MixerMenu {
        id: addMenu
        MixerMenu {
            id: addVideoMenu
            title: qsTr("Video")
            Instantiator {
                model:  {
                    var result = []
                    if (availableMedias && availableMedias.data && availableMedias.data.length) {
                        for (var i = 0; i < availableMedias.data.length; ++i) {
                            result.push(availableMedias.data[i])
                        }
                    }
                    if (!result.length) result.push({ "name" : qsTr("<None>"), "source" : "" })
                    return result
                }
                MixerMenuItem {
                    text: modelData.name
                    onTriggered: {
                        if (modelData.source) {
                            recordingSettings.add(undefined, {
                                type: 0,
                                source: modelData.source,
                                storageGuid: defaultStorage ? defaultStorage.guid : 0,
                                bitrate: 5000,
                                hardware: false
                            })
                        }
                    }
                }
                onObjectAdded: addVideoMenu.insertItem(index, object)
                onObjectRemoved: addVideoMenu.removeItem(object)
            }
        }
        MixerMenu {
            id: addAudioMenu
            title: qsTr("Audio")
            Instantiator {
                model:  {
                    var result = []
                    if (FD.inputs && FD.inputs.length) {
                        for (var i = 0; i < FD.inputs.length; ++i) {
                            result.push(FD.inputs[i])
                        }
                    }
                    if (!result.length) result.push({ "name" : qsTr("<None>"), "source" : "" })
                    return result
                }
                MixerMenuItem {
                    text: modelData.name
                    onTriggered: {
                        if (modelData.name) {
                            recordingSettings.add(undefined, {
                                type: 1,
                                source: "fd:///" + encodeURIComponent(modelData.name),
                                storageGuid: defaultStorage ? defaultStorage.guid : 0,
                                bitrate: 128,
                                hardware: false
                            })
                        }
                    }
                }
                onObjectAdded: addAudioMenu.insertItem(index, object)
                onObjectRemoved: addAudioMenu.removeItem(object)
            }
        }
    }

    Action {
        id: deleteAction
        enabled: !root.readonly && selectedIndex != -1
        text: qsTr("Delete")
        shortcut: "Del"
        onTriggered: {
            doWithQuestion(qsTr("Do you really want to delete this source?"), function() {
                recordingSettings.remove(selectedIndex)
            })
        }
    }
}
