import QtQuick 2.1
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.1
import MixerGuiControls 1.0

Item {
    id: root
    property real scrollStep: 100
    default property alias data: content.data
    RowLayout {
        anchors.fill: parent
        Item {
            id: viewport
            clip: true
            Layout.fillWidth: true
            Layout.fillHeight: true
            property real contentY: 0
            Item {
                id: content
                y: viewport.contentY
                Behavior on y {
                    NumberAnimation { duration: 200 }
                }
                width: parent.width
                implicitHeight: children.length == 1 ? children[0].implicitHeight : 0
                onHeightChanged: viewport.correctContentY()
            }
            
            onHeightChanged: correctContentY()
            
            function correctContentY() {
                if(contentY < height - content.height) {
                    contentY = Math.min(0, Math.floor((height - content.height) / scrollStep) * scrollStep)
                }
            }
        }
        ColumnLayout {
            visible: scrollDownAction.enabled || scrollUpAction.enabled
            Layout.fillHeight: true
            spacing: 2
            
            DialogButton {
                implicitWidth: 25
                Layout.fillHeight: true
                icon: "WindowUp"
                action: scrollUpAction
            }
            DialogButton {
                implicitWidth: 25
                Layout.fillHeight: true
                icon: "WindowDown"
                action: scrollDownAction
            }
        }
    }
    
    Action {
        id: scrollDownAction
        enabled: viewport.contentY > viewport.height - content.height
        onTriggered: {
            viewport.contentY -= scrollStep
        }
    }
    Action {
        id: scrollUpAction
        enabled: viewport.contentY < 0
        onTriggered: {
            viewport.contentY += scrollStep
        }
    }   
    
}