import QtQuick 2.0
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.0
import QtQuick.Window 2.0
import MixerGuiControls 1.0
import "utils/Conversion.js" as Conversion

MixerWindow {
    id: root
    width: 510
    height: 600
    minimumWidth: 510
    minimumHeight: 300
    modality: Qt.ApplicationModal
    title: qsTr("Storages")
    readonly property int selectedIndex: tableView.rowCount > 0 ? tableView.currentRow : -1

    ColumnLayout {
        anchors.fill: parent
        MixerTableView {
            id: tableView
            Layout.fillWidth: true
            Layout.fillHeight: true
            horizontalScrollBarPolicy: Qt.ScrollBarAlwaysOff
            
            TableViewColumn { 
                role: "name"
                title: qsTr("Name")
                width: tableView.width / 3
                resizable: false
                movable: false
            }
            TableViewColumn {
                role: "driveName"
                title: qsTr("Drive")
                width: tableView.width / 3
                resizable: false
                movable: false
            }
            TableViewColumn {
                title: qsTr("Size (MB)")
                width: tableView.width / 3
                resizable: false
                movable: false
                delegate: Item {
                    MixerLabel {
                        anchors.fill: parent
                        anchors.leftMargin: 8
                        text: styleData.value ? Conversion.bytesToMB(styleData.value.size).toFixed() : 0
                        verticalAlignment: Text.AlignVCenter
                        colorType: styleData.selected ? "program" : "neutral"
                        font.pixelSize: MixerThemeManager.theme.systemFontSize 
                    }
                }
            }
                   
            property var modelTmp: {
                var result = []
                for(var i in postPlayService.storages) {
                    var storage = postPlayService.storages[i]
                    result.push({name: storage.name, driveName: storage.driveName, size: storage.size})
                }
                return result
            }
            
            onModelTmpChanged: {
                var row = currentRow
                model = modelTmp
                currentRow = Math.min(row, rowCount - 1)
                if(currentRow == -1 && rowCount > 0) currentRow = 0
                selection.clear()
                if(currentRow >= 0 && rowCount > 0) selection.select(currentRow)            
            }
            
            onCurrentRowChanged: {
                selection.clear()
                if(currentRow >= 0 && currentRow < rowCount) selection.select(currentRow)            
            }
        }
        
        RowLayout {
            Layout.alignment: Qt.AlignHCenter
            DialogButton { action: addAction }
            DialogButton { action: deleteAction }
        }
    }
    
    Action {
        id: addAction
        text: qsTr("Add")
        onTriggered: addDialog.open()
    }
    
    Action {
        id: deleteAction
        enabled: selectedIndex >= 0
        text: qsTr("Delete")
        shortcut: "Del"
        onTriggered: {
            doWithQuestion(qsTr("Do you really want to delete this storage?"), function() {
                var result = postPlayService.deleteStorage(postPlayService.storages[selectedIndex])
                if(!result) app.openErrorDialog(qsTr("Failed to delete storage"))
            })
        }        
    }
    
    MixerMenu {
        id: contextMenu
        MixerMenuItem { action1: deleteAction }
    }
    
    function hasStorageWithName(name) {
        for(var i in postPlayService.storages) {
            if(postPlayService.storages[i].name == name) return true
        }
        return false
    }
    
    function makeUniqueStorageName(baseName) {
        var result = baseName
        var i = 1
        while(hasStorageWithName(result)) {
            result = "%1 (%2)".arg(baseName).arg(i)
            i++
        }
        return result
    }
    
    AddStorageDialog {
        id: addDialog
        onAccepted: {
            var result = postPlayService.createStorage(makeUniqueStorageName(name || qsTr("Unnamed")), drive, size)
            if(!result) app.openErrorDialog(qsTr("Failed to create storage"))
        }
    }
}
