import QtQuick 2.0
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.0
import QtQuick.Window 2.0
import MixerGuiControls 1.0

MixerWindow {
    id: root
    width: 610
    height: 500
    minimumWidth: 610
    modality: Qt.ApplicationModal
    title: qsTr("Templates")
    readonly property var templates: (templateCollection && templateCollection.templates) ? templateCollection.templates : []
    readonly property int templateCount: templates.length
    property int selectedIndex: -1
    readonly property QtObject selectedTemplate: templates[selectedIndex] || null
    onTemplateCountChanged: {
        if(selectedIndex == -1 && templateCount > 0) selectedIndex = 0
        else if(selectedIndex >= templateCount) selectedIndex = templateCount - 1
    }
        
    ColumnLayout {
        anchors.fill: parent
        OutputBackground {
            Layout.fillWidth: true
            Layout.fillHeight: true
            Item {
                anchors.fill: parent
                clip: true
                Flow {
                    anchors.fill: parent
                    anchors.margins: 6
                    spacing: 6
                    Repeater {
                        model: templates
                        TemplatePreview {
                            template: modelData
                            isSelected: selectedIndex == model.index
                            onLeftButtonClicked: selectedIndex = model.index
                            onRightButtonClicked: {
                                selectedIndex = model.index
                                contextMenu.popup()
                            }                        
                        }
                    }
                }
            }
        }
        RowLayout {
            Layout.alignment: Qt.AlignHCenter
            DialogButton { action: editAction }
            DialogButton { action: addAction }
            DialogButton { action: renameAction }   
            DialogButton { action: deleteAction }
        }
    }
  
    MixerMenu {
        id: contextMenu
        MixerMenuItem { action1: editAction }
        MixerMenuItem { action1: renameAction }
        MixerMenuItem { action1: deleteAction }
    }
    
    Action {
        id: editAction
        text: qsTr("Edit")
        enabled: selectedIndex != -1
        onTriggered: templateEditor.show()
    }    
    
    Action {
        id: addAction
        text: qsTr("Add")
        onTriggered: addDialog.open()
    }
    
    Action {
        id: deleteAction
        enabled: selectedIndex != -1
        text: qsTr("Delete")
        shortcut: "Del"
        onTriggered: {
            doWithQuestion(qsTr("Do you really want to delete this template?"), function() {
                templateCollection.remove(selectedIndex)
            })
        }        
    }
    
    Action {
        id: renameAction
        enabled: selectedIndex != -1
        text: qsTr("Rename")
        onTriggered: {
            renameDialog.text = selectedTemplate.name
            renameDialog.open()
        }
    }  

    MixerStringInputDialog {
        id: addDialog
        title: qsTr("Add template")
        caption: qsTr("Name: ")
        allowEmptyString: true
        onEntered: {
            templateCollection.add(value || qsTr("Unnamed"), {})
            selectedIndex = templateCount - 1
        }
    }
    
    MixerStringInputDialog {
        id: renameDialog
        title: qsTr("Rename template")
        caption: qsTr("Name: ")
        onEntered: templateCollection.rename(selectedIndex, value)
    }
        
    onVisibleChanged: if(visible) {
        selectedIndex = templateCount > 0 ? 0 : -1
    }
    
}
