import QtQuick 2.0

Item {
    id: root
    property point startPoint
    property point currentPoint
    property int gridX
    property int gridY
    readonly property real gridCellWidth: width / gridX
    readonly property real gridCellHeight: height / gridY
    property bool isDrawing
    signal drawingFinished(rect rect)
    
    readonly property point p0: Qt.point(Math.min(startPoint.x, currentPoint.x), Math.min(startPoint.y, currentPoint.y))
    readonly property point p1: Qt.point(Math.max(startPoint.x, currentPoint.x), Math.max(startPoint.y, currentPoint.y))
    
    readonly property rect snappedRect: {
        var x0 = Math.floor(p0.x / gridCellWidth) * gridCellWidth
        var y0 = Math.floor(p0.y / gridCellHeight) * gridCellHeight
        var x1 = Math.floor(p1.x / gridCellWidth + 1) * gridCellWidth
        var y1 = Math.floor(p1.y / gridCellHeight + 1) * gridCellHeight
        return Qt.rect(x0, y0, x1 - x0, y1 - y0)
    }

    MouseArea {
        id: mouseArea
        anchors.fill: parent
        acceptedButtons: Qt.RightButton|Qt.LeftButton
        onPressed: {
            if(isDrawing && mouse.button == Qt.RightButton) {
                isDrawing = false
            } else if(mouse.button == Qt.LeftButton) {
                isDrawing = true
                startPoint = Qt.point(mouse.x, mouse.y)
                currentPoint = startPoint
            }
        }
        onPositionChanged: currentPoint = Qt.point(Math.max(0, Math.min(mouse.x, width - 1)), Math.max(0, Math.min(mouse.y, height - 1)))

        onReleased: if(isDrawing && mouse.button == Qt.LeftButton) {
            isDrawing = false
            if(snappedRect.width < 0.001 || snappedRect.height < 0.001) return
            drawingFinished(Qt.rect(snappedRect.x / width, snappedRect.y / height, snappedRect.width / width, snappedRect.height / height))
        }
    }
  
    Rectangle {
        id: snappedFrame
        visible: isDrawing
        x: snappedRect.x
        y: snappedRect.y
        width: snappedRect.width
        height: snappedRect.height   
        color: "gray"
        border.color: "black"  
    } 
    
    Rectangle {
        id: frame
        visible: isDrawing
        x: p0.x
        y: p0.y
        width: p1.x - p0.x
        height: p1.y - p0.y
        color: Qt.rgba(255 / 255, 220 / 255, 134 / 255)
        opacity: 0.3
    }      
}
