import QtQuick 2.0

Item {
    id: root
    property TemplateElement element
    property int gridX
    property int gridY    
    ResizebleFrame {
        id: frame
        visible: !!root.element
        color: Qt.rgba(255 / 255, 220 / 255, 134 / 255)
        minimumX: 0
        minimumY: 0
        maximumX: root.width
        maximumY: root.height
        minimumWidth: root.width / gridX
        minimumHeight: root.height / gridY
        onMouseReleased: root.allignFrameToElement()
        onDragged: root.moveElement()
    }
    
    function getSnappedFrameRect() {
        var gridCellWidth = root.width / gridX
        var gridCellHeight = root.height / gridY
        var x0 = Math.round(frame.x / gridCellWidth) * gridCellWidth / root.width
        var y0 = Math.round(frame.y / gridCellHeight) * gridCellHeight / root.height
        var x1 = Math.round((frame.x + frame.width) / gridCellWidth) * gridCellWidth / root.width
        var y1 = Math.round((frame.y + frame.height) / gridCellHeight) * gridCellHeight / root.height
        var w = Math.max(1/gridX, x1 - x0)
        var h = Math.max(1/gridY, y1 - y0)
        return Qt.rect(x0, y0, w, h)    
    }
    
    function moveElement() {
        if(!element) return
        element.rect = getSnappedFrameRect()
    }
        
    function allignFrameToElement() {
        if(!element) return
        frame.x = element.rect.x * root.width
        frame.y = element.rect.y * root.height
        frame.width = element.rect.width * root.width
        frame.height = element.rect.height * root.height
    }
    
    onElementChanged: allignFrameToElement()
    onWidthChanged: allignFrameToElement()
    onHeightChanged: allignFrameToElement()
}