import QtQuick 2.0

Item {
    id: root
    property real borderWidth: 1
    property bool selected: true
    property color color: "white"
    property real minimumX: -100000
    property real maximumX: 100000
    property real minimumY: -100000
    property real maximumY: 100000  
    property real minimumWidth: 1
    property real minimumHeight: 1
    readonly property bool pressed: {
        if(mouseArea.pressed) return true
        for(var i = 0; i < anchorRepeater.count; i++) {
            var anchor = anchorRepeater.itemAt(i)
            if(anchor.pressed) return true
        }
        return false
    }
    signal dragged()
    signal mousePressed()
    signal mouseReleased()
    
    onXChanged: if(pressed) dragged()
    onYChanged: if(pressed) dragged()    
    onWidthChanged: if(pressed) dragged()
    onHeightChanged: if(pressed) dragged()
    
    Rectangle {
        color: root.color
        anchors.left: parent.left
        anchors.top: parent.top
        anchors.bottom: parent.bottom
        width: borderWidth
        visible: root.selected
    }
    Rectangle {
        color: root.color
        anchors.right: parent.right
        anchors.top: parent.top
        anchors.bottom: parent.bottom
        width: borderWidth
        visible: root.selected
    }
    Rectangle {
        color: root.color
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: parent.top
        height: borderWidth
        visible: root.selected
    }
    Rectangle {
        color: root.color
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.bottom: parent.bottom
        height: borderWidth
        visible: root.selected
    }
    MouseArea {
        id: mouseArea
        anchors.fill: parent
        acceptedButtons: Qt.LeftButton
        drag.target: root
        drag.axis: Drag.XandYAxis
        drag.minimumX: root.minimumX
        drag.minimumY: root.minimumY
        drag.maximumX: root.maximumX - root.width
        drag.maximumY: root.maximumY - root.height
        cursorShape: Qt.SizeAllCursor
        propagateComposedEvents: true
        onPressed: root.mousePressed()
        onReleased: root.mouseReleased()
    }
    Repeater {
        model: 8
        id: anchorRepeater
        ResizebleFrameAnchor {
            dir: model.index
            target: root
            radius: root.borderWidth * 2
            visible: root.selected
            color: root.color
            onMousePressed: root.mousePressed()
            onMouseReleased: root.mouseReleased()            
        }
    }   
}
