import QtQuick 2.0

Item {
    id: root
    property int dir
    property real radius: 5
    property color color
    property Item target
    readonly property bool pressed: mouseArea.pressed
    signal mousePressed()
    signal mouseReleased()    
    property var __dirToCursor: [
        Qt.SizeFDiagCursor,
        Qt.SizeVerCursor, 
        Qt.SizeBDiagCursor,
        Qt.SizeHorCursor, 
        Qt.SizeFDiagCursor,
        Qt.SizeVerCursor, 
        Qt.SizeBDiagCursor,
        Qt.SizeHorCursor
    ]
    property var __dirToDragTypes: [
        Drag.XandYAxis,
        Drag.YAxis,
        Drag.XandYAxis,
        Drag.XAxis,
        Drag.XandYAxis,
        Drag.YAxis,
        Drag.XandYAxis,
        Drag.XAxis
    ]
    property var __dirToXDir: [
        -1, 0, 1, 1, 1, 0, -1, -1
    ]
    property var __dirToYDir: [
        -1, -1, -1, 0, 1, 1, 1, 0
    ]
    
    x: (__dirToXDir[dir] + 1) * target.width / 2
    y: (__dirToYDir[dir] + 1) * target.height / 2
    
    onXChanged: {
        var xDir = __dirToXDir[dir]
        if(xDir < 0 && x != 0) {
            target.x += x
            target.width -= x
        }
        if(xDir > 0 && x != target.width) {
            target.width = x  
        }
    }
    
    onYChanged: {
        var yDir = __dirToYDir[dir]
        if(yDir < 0 && y != 0) {
            target.y += y
            target.height -= y
        }
        if(yDir > 0 && y != target.height) {
            target.height = y  
        }
    }
    
    Rectangle {
        id: rect
        color: root.color
        x: -root.radius
        y: -root.radius
        width: root.radius * 2
        height: root.radius * 2
    }
    
    MouseArea {
        id: mouseArea
        anchors.fill: rect
        acceptedButtons: Qt.LeftButton
        cursorShape: __dirToCursor[dir]
        drag.axis: __dirToDragTypes[dir]
        drag.target: root
        drag.minimumX: __dirToXDir[dir] > 0 ? target.minimumWidth : target.minimumX - target.x
        drag.maximumX: __dirToXDir[dir] < 0 ? target.width - target.minimumWidth : target.maximumX - target.x
        drag.minimumY: __dirToYDir[dir] > 0 ? target.minimumHeight : target.minimumY - target.y
        drag.maximumY: __dirToYDir[dir] < 0 ? target.height - target.minimumHeight : target.maximumY - target.y
        drag.threshold: 0
        onPressed: root.mousePressed()
        onReleased: root.mouseReleased()        
    }
}
