import QtQuick 2.0
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.0
import QtQuick.Window 2.0
import GuiControls 1.0
import QtQuickUtils 1.0
import MixerGuiControls 1.0

MixerWindow {
    id: root
    width: 860
    height: 550
    minimumWidth: 460
    minimumHeight: 325
    modality: Qt.ApplicationModal
    
    title: qsTr("Template Editor") + (name ? ": " + name : "")
    property QtObject template
    readonly property string name: template && template.name ? template.name : ""
    readonly property int gridX: template ? template.gridX : 0
    readonly property int gridY: template ? template.gridY : 0
    readonly property var rects: template && template.rects ? template.rects : []
    onRectsChanged: loadTemplate()
    onClosing: saveTemplate()   
    onGridXChanged: gridXSpinBox.value = gridX
    onGridYChanged: gridYSpinBox.value = gridY
    
    function loadTemplate() {
        templateView.load(rects)    
    }

    function saveTemplate() {
        if(!template) return
        var data = {
            name: name,
            gridX: gridXSpinBox.value,
            gridY: gridYSpinBox.value,
            elements: templateView.save()
        }
        template.load(data)
    }
    
    Action {
        id: clearTemplateAction
        shortcut: "Ctrl+N"
        tooltip: qsTr("New template")
        onTriggered: templateView.clear()
    }
    
    Action {
        id: deleteElementAction
        enabled: templateView.selectedElement
        shortcut: "Del"
        tooltip: qsTr("Delete element")
        onTriggered: templateView.selectedElement.destroy()
    }
    
    Action {
        id: bringElementToFrontAction
        enabled: templateView.selectedElement
        tooltip: qsTr("Bring to front")
        onTriggered: templateView.bringElementToFront(templateView.selectedElement)
    }
    
    Action {
        id: bringElementToBackAction
        enabled: templateView.selectedElement
        tooltip: qsTr("Send to back")
        onTriggered: templateView.bringElementToBack(templateView.selectedElement)
    }
    
    ExclusiveGroup {
        id: toolsExclusiveGroup
    }
    
    Action {
        id: drawingAction
        exclusiveGroup: toolsExclusiveGroup
        checkable: true
        checked: true
        tooltip: qsTr("Drawing")
    }
    
    Action {
        id: selectionAction
        exclusiveGroup: toolsExclusiveGroup
        checkable: true
        tooltip: qsTr("Selection")
    }
 
    ColumnLayout {
        anchors.fill: parent
        anchors.bottomMargin: 10
        anchors.rightMargin: 10
        spacing: 0
        MixerToolbar {
            Item {
                implicitWidth: 30
            }       
            ToolbarSeparator {}
            ToolbarButton {
                action: clearTemplateAction
                icon: "New"
            }
            ToolbarButton {
                action: deleteElementAction
                icon: "Delete"
            }
            ToolbarSeparator {}
            
            ToolbarButton {
                action: bringElementToFrontAction
                icon: "BringTop"
            }            
            ToolbarButton {
                action: bringElementToBackAction
                icon: "BringBottom"
            }
            ToolbarSeparator {}
            
            Label {
                text: qsTr("Grid: ")
            }
            SpinBox {
                id: gridXSpinBox
                maximumValue: 100
                minimumValue: 1
                value: 8
                activeFocusOnPress: false
            }
            Label {
                text: "x"
            }
            SpinBox {
                id: gridYSpinBox
                maximumValue: 100
                minimumValue: 1
                value: 8
                activeFocusOnPress: false
            }
        }
        RowLayout {
            spacing: 0
            MixerToolbar {
                Layout.alignment: Qt.AlignTop
                orientation: Qt.Vertical
                ToolbarSeparator {}
                ToolbarButton {
                    action: drawingAction
                    icon: "ToolRect"
                }               
                ToolbarButton {
                    action: selectionAction
                    icon: "SelectArrow"
                }
            }
            OutputBackground {
                Layout.fillWidth: true
                Layout.fillHeight: true
                FillModeItem {
                    anchors.fill: parent
                    aspect: 16/9
                    TemplateView {
                        id: templateView
                        anchors.fill: parent
                        canSelect: selectionAction.checked
                    }
                    DrawingLayer {
                        id: drawingLayer
                        anchors.fill: parent
                        visible: drawingAction.checked
                        onDrawingFinished: templateView.addElement(rect)
                        gridX: gridXSpinBox.value
                        gridY: gridYSpinBox.value
                    }
                    ElementMovingLayer {
                        anchors.fill: parent
                        element: templateView.selectedElement
                        gridX: gridXSpinBox.value
                        gridY: gridYSpinBox.value
                    }
                    SnappingGrid {
                        anchors.fill: parent
                        gridX: gridXSpinBox.value
                        gridY: gridYSpinBox.value  
                    }                    
                }
            }
        }        
    }
}
