import QtQuick 2.0
import Serialization 1.0
import "../../common/utils"

Object {
    id: root
    property int timeMode: 0 // 0 - системное; 1 - бортовое; 2 - тренировка
    readonly property var currentAchive: archivePlayer && archivePlayer.archive || null
    
    readonly property real systemTime: {
        timer.tick
        var date = isRealTime() ? new Date() : new Date(currentAchive.creationDate.getTime() + archivePlayer.position)
        return dateToMilliseconds(date)
    }
    
    readonly property real boardTime: {
        timer.tick
        if(isRealTime()) return dateToMilliseconds(new Date()) + (networkEventReceiver ? networkEventReceiver.deltaWithSystemTime : 0)
        else return trainingTimeToBoardTime(archivePlayer.position)
    }

    readonly property real trainingTime: isRealTime() ? 0 : archivePlayer.position

    function trainingTimeToSystemTime(time) { 
        return isRealTime() ? time : dateToMilliseconds(new Date(currentAchive.creationDate.getTime() + time))
    }
    
    function trainingTimeToBoardTime(time) {
        if(isRealTime() || !mainWindow.eventsPanel) return time
        var eventModel = mainWindow.eventsPanel.eventModel
        if(eventModel.timestampCount == 0) return 0 // binding
        var result = eventModel.getBoardTime(time)
        return result
    }

    function trainingTimeToCurrentModeTime(time) {
        if(timeMode == 0) return trainingTimeToSystemTime(time)
        else if(timeMode == 1) return trainingTimeToBoardTime(time)
        else return time
    }
        
    function dateToMilliseconds(date) {
        return ((date.getHours() * 60 + date.getMinutes()) * 60 + date.getSeconds()) * 1000 + date.getMilliseconds()
    }

    function isRealTime() {
        return !currentAchive || !currentAchive.creationDate.getTime()
    }
    
    Timer {
        id: timer
        property bool tick
        running: isRealTime()
        repeat: true
        interval: 1000
        onTriggered: tick = !tick
    }
    
    function save() {
        return {
            timeMode: timeMode
        }
    }
    
    function load(data) {
        timeMode = data.timeMode || 0
    }    
    
    ObjectSaver {
        id: objectSaver
        object: root
        fileUrl: Serialization.applicationDataDirPath + "/time_settings.json"
    }

    Component.onCompleted: objectSaver.init()    
}



