import QtQuick 2.1

MouseArea {
    id: root
    property real scrollK: 0.6
    property real maxScrollDelta: 20
    
    signal seek(real seekX)
    signal beforeSeek()
    signal afterSeek()
    signal scroll(real delta)
    
    property bool seeking: false
    
    onPressed: {
        seeking = true
        root.beforeSeek()
        seek(mouse.x)
    }
    onReleased: {
        root.afterSeek()
        seeking = false
    }
    onPositionChanged: {
        var x = Math.max(0, Math.min(mouse.x, width - 1))
        seek(x)
    }
    
    Timer {
        running: root.seeking
        interval: 10
        repeat: true
        
        onTriggered: {
            if(root.mouseX < 0) {
                var delta = Math.max(-maxScrollDelta, scrollK * root.mouseX)
                root.scroll(delta)
                root.seek(0)
            } else if(root.mouseX > root.width - 1) {
                var delta = Math.min(scrollK * (root.mouseX - root.width + 1), maxScrollDelta)
                root.scroll(delta)
                root.seek(root.width - 1)
            }
        }
    }
}