import QtQuick 2.1
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.1
import MixerGuiControls 1.0

Item {
    id: root
    property alias duration: timelineScale.duration
    property alias position: timelineScale.currentTime
    property alias intervals: timelineScale.intervals
    
    property bool playing: false
    property bool recording: false
    signal seek(real time)
    signal beforeSeek()
    signal afterSeek()
    
    property real zoomFactor: 0
    readonly property int minimumZoomFactor: -13
    readonly property int maximumZoomFactor: 2
    
    TimelineScrollView {
        id: scrollView
        anchors.fill: parent
        targetX: timelineScale.timeToPixel(root.position)
        follow: !seekingArea.seeking && !scrollView.handlePressed && root.playing
        smoothFollowing: seekingArea.seeking || scrollView.handlePressed
        
        onHandlePressedChanged: {
            if(handlePressed) beforeSeek()
            else afterSeek()
        }
        
        onContentXChanged: if(scrollView.handlePressed && root.playing) {
            var time0 = timelineScale.pixelToTime(contentX)
            var time1 = timelineScale.pixelToTime(contentX + scrollView.viewport.width - 1)
            if(root.position < time0) root.seek(time0)
            else if(root.position > time1) root.seek(time1)
        }
        
        TimelineScale {
            id: timelineScale
            width: Math.max(implicitWidth + (root.recording ? scrollView.viewport.width * 0.5 : 0), scrollView.viewport.width)
            height: scrollView.viewport.height
            visibleAreaX: scrollView.flickableItem.contentX
            visibleAreaWidth: scrollView.viewport.width
            millisecondsPerSmallInterval: Math.max(1, 100 / Math.pow(2, zoomFactor))
            decimals: Math.max(zoomFactor, 0)
            antialiasing: scrollView.follow
        }
    }
    
    function clampTime(t) {
        return
    }
    
    function mapPositionIndicatorToItem(item) {
        return timelineScale.mapPositionIndicatorToItem(item)
    }
    
    SeekingArea {
        id: seekingArea
        anchors.fill: parent
        anchors.topMargin: 2
        anchors.leftMargin: 2
        anchors.rightMargin: 2
        anchors.bottomMargin: 18
        
        onSeek: {
            var time = timelineScale.pixelToTime(scrollView.contentX + seekX)
            time = Math.max(0, Math.min(time, root.duration))
            root.seek(time)
        }
        
        onBeforeSeek: root.beforeSeek()
        onAfterSeek: root.afterSeek()
        onScroll: scrollView.scroll(delta)
    }
    
    Connections {
        target: archivePlayer
        onRequestTimelineFollowing: scrollView.scrollToPosition(timelineScale.timeToPixel(time))
        onRequestTimelineMoving: scrollView.scroll(timelineScale.timeToPixel(deltaTime))
    }
    
    function scrollToCurrentPosition() {
        scrollView.scrollToPosition(timelineScale.timeToPixel(root.position))
    }
    
    function zoomIn() {
        zoomFactor = Math.round(zoomFactor)
        zoomFactor++
        scrollView.followTarget(1)
    }
    
    function zoomOut() {
        zoomFactor = Math.round(zoomFactor)
        zoomFactor--
        scrollToCurrentPosition()
    }
    
    function fitZoom() {
        var zoomFactor0 = minimumZoomFactor
        var zoomFactor1 = maximumZoomFactor
        for(var i = 0; i < 300; i++) {
            zoomFactor = (zoomFactor0 + zoomFactor1) * 0.5
            if(timelineScale.implicitWidth >= scrollView.viewport.width - 1) {
                zoomFactor1 = zoomFactor
            } else {
                zoomFactor0 = zoomFactor
            }
        }
        
        for(var i = 0; i < 300; i++) {
            if(timelineScale.implicitWidth < scrollView.viewport.width - 1) break
            zoomFactor -= 0.0001
        }
        scrollToCurrentPosition()
    }
}