import QtQuick 2.1
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.1
import MixerGuiControls 1.0
import "../utils/FormatTime.js" as FormatTime

Rectangle {
    id: root
    color: Qt.rgba(5 / 255, 5 / 255, 5 / 255, 0.2)
    clip: true

    property real duration: 0
    property real currentTime: 0
    property int decimals: 0
    property int millisecondsPerSmallInterval: 100
    property int pixelsPerSmallInterval: 10
    property bool antialiasing: false
    
    property int visibleAreaX: 0
    property int visibleAreaWidth: 1000
    
    property var intervals: []
    
    property bool seeking: false
    signal seek(real time)
    signal beforeSeek()
    signal afterSeek()
    
    property int implicitWidth: duration / millisecondsPerBigInterval * pixelsPerBigInterval
    readonly property int millisecondsPerBigInterval: millisecondsPerSmallInterval * 10
    readonly property int pixelsPerBigInterval: pixelsPerSmallInterval * 10    
    readonly property int drawAreaX: Math.floor(visibleAreaX / pixelsPerBigInterval) * pixelsPerBigInterval - 1 * pixelsPerBigInterval
    readonly property int drawAreaWidth: Math.floor(visibleAreaWidth / pixelsPerBigInterval) * pixelsPerBigInterval + 3 * pixelsPerBigInterval
    readonly property int drawBeginTime: drawAreaX / pixelsPerSmallInterval * millisecondsPerSmallInterval

    function pixelToTime(x) {
        return x / pixelsPerSmallInterval * millisecondsPerSmallInterval
    }
    
    function timeToPixel(t) {
        return t / millisecondsPerSmallInterval * pixelsPerSmallInterval
    }
    
    function mapPositionIndicatorToItem(item) {
        return positionIndicator.mapToItem(item, positionIndicator.width/2, 0).x
    }
    
    Repeater {
        model: Math.round(drawAreaWidth / pixelsPerBigInterval)
        Item {
            id: element
            width: pixelsPerBigInterval
            height: root.height
            x: drawAreaX + model.index * pixelsPerBigInterval
            
            Rectangle {
                width: 1
                height: element.height - 25
                y: 21
                color: Qt.rgba(200 / 255, 200 / 255, 200 / 255)
                opacity: element.x < root.implicitWidth ? 1 : 0.3
                antialiasing: root.antialiasing
            }
            Repeater {
                model: 10
                Rectangle {
                    width: 1
                    height: element.height - 35
                    y: 31
                    x: pixelsPerSmallInterval * model.index
                    color: Qt.rgba(200 / 255, 200 / 255, 200 / 255)
                    opacity: element.x + x < root.implicitWidth ? 1 : 0.3
                    antialiasing: root.antialiasing
                }
            }
            Label {
                property int time: drawBeginTime + millisecondsPerBigInterval * model.index
                visible: time != 0
                text: FormatTime.millisecondsToString(timeState.trainingTimeToCurrentModeTime(time), root.decimals)
                x: -width * 0.5
                y: 3
                color: Qt.rgba(200 / 255, 200 / 255, 200 / 255)
                font.pixelSize: 12
                opacity: element.x < root.implicitWidth ? 1 : 0.3
            }
        }
    }
    
    Rectangle {
        x: root.implicitWidth
        width: root.width - x
        height: root.height
        opacity: 0.2
        color: "white"
    }   
    
    Rectangle {
        id: positionIndicator
        width: 3
        height: root.height
        color: "orange"
        opacity: 0.5
        x: Math.min(root.implicitWidth - 1, timeToPixel(currentTime)) - 1
        visible: root.implicitWidth > 0
    }
    
    Repeater {
        model: root.intervals
        Rectangle {
            height: root.height
            color: "orange"
            opacity: modelData.isCurrent ? 0.15 : 0.08
            x: timeToPixel(modelData.from)
            width: Math.max(2, timeToPixel(modelData.to - modelData.from))
        }        
    }
    
}