import QtQuick 2.1
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.1
import MixerGuiControls 1.0

ScrollView {
    id: root
    anchors.fill: parent
    style: MixerScrollViewStyleThin {}
    frameVisible: true
    verticalScrollBarPolicy: Qt.ScrollBarAlwaysOff
    horizontalScrollBarPolicy: Qt.ScrollBarAlwaysOn
    readonly property bool handlePressed: __horizontalScrollBar.handlePressed
    readonly property real contentX: flickableItem.contentX
    readonly property real contentWidth: contentItem.width
    
    property real targetX: 0
    property bool follow: false
    property bool smoothFollowing: true
    property real followK: 0.3
    property real _currentFollowK: 1
    
    onTargetXChanged: if(follow) followTarget(_currentFollowK)
    
    onSmoothFollowingChanged: {
        if(smoothFollowing) {
            smoothFollowingTimer.stop()
            _currentFollowK = followK
        } else {
            smoothFollowingTimer.restart()
        }
    }
    
    property Timer smoothFollowingTimer: Timer {
        id: smoothFollowingTimer
        running: false
        interval: 10
        repeat: true
        onTriggered: {
            root._currentFollowK += 0.05
            if(root._currentFollowK >= 1) {
                root._currentFollowK = 1
                smoothFollowingTimer.stop()
            }
        }
    }
    
    property var _opacityBinding: Binding {
        target: root.__horizontalScrollBar
        property: "opacity"
        value: root.viewport.width >= root.contentItem.width ? 0.5 : 1
    }

    function clampToBounds(x) {
        if(viewport.width >= contentItem.width) return 0
        return Math.max(0, Math.min(x, contentItem.width - viewport.width))
    }
    
    function followTarget(k) {
        var newX = clampToBounds(targetX - viewport.width * 0.5)
        flickableItem.contentX = flickableItem.contentX * (1 - k) + newX * k
    }
    
    function scrollToPosition(value) {
        flickableItem.contentX = clampToBounds(value - viewport.width * 0.5)
    }    
    
    function scroll(delta) {
        flickableItem.contentX = clampToBounds(flickableItem.contentX + delta)
    }
}