import QtQuick 2.0
import QtQml 2.2
import "../../common/utils"

Item {
    id: root
    property url url
    property alias enabled: video.enabled
    property alias elements: elementsCreator.objects
    property alias emptyElement: emptyElement
    property alias mosaicElement: mosaicElement
    readonly property var params: []
    readonly property var rects: atlas && atlas.rects || []
    readonly property var names: atlas && atlas.names || []
    readonly property var atlas: server && server.videoStreamAtlases && server.videoStreamAtlases[0] || null

    readonly property var audioStream: video.audioStream

    function getByName(name) {
        return elements[names.indexOf(name)] || emptyElement
    }

    function getByStream(stream) {
        return !stream ? emptyElement : getByName(stream.name)
    }

    Component.onCompleted: p('isAndroid='+isAndroid)

    NetworkStreamReceiver {
        id: video
        source: root.url
        visible: false
    }

    Instantiator {
        id: elementsCreator
        model: rects.length
        delegate: ShaderEffectSource {
            sourceItem: (sourceRect.width > 0 && sourceRect.height > 0) ? video : null
            sourceRect: rects[model.index] || Qt.rect(0, 0, 0, 0)
            width: sourceRect.width
            height: sourceRect.height
            visible: false
            mipmap: true
        }
        property var objects: {
            var result = []
            for(var i = 0; i < model; i++) {
                var object = objectAt(i)
                if(object) result.push(object)
            }
            return result
        }        
    }    
    ShaderEffectSource {
        id: emptyElement
        sourceItem: Rectangle {
            color: "transparent"
            width: 1
            height: 1
        }
        width: 1
        height: 1
        visible: false
    }
    ShaderEffectSource {
        id: mosaicElement
        sourceItem: (sourceRect.width > 0 && sourceRect.height > 0) ? video : null
        sourceRect: Qt.rect(0, 0, video.height * 16 / 9, video.height) 
        width: sourceRect.width
        height: sourceRect.height
        visible: false
    }
}
