import QtQuick 2.0
import QtQuick.Layouts 1.0
import QtQuick.Controls 1.0
import GuiControls 1.0
import MixerGuiControls 1.0

Item {
    id: root
    property QtObject stream
    property bool isSelected
    property bool isDropped

    readonly property bool isArchived: stream && stream.archived
    readonly property var borderColor: isSelected ? 'program': isArchived && archivePlayer && archivePlayer.color || "dimgray"

    implicitWidth: icon.implicitWidth
    implicitHeight: layout.implicitHeight
    signal rightButtonClicked()
    signal clicked()
    signal doubleClicked()
    signal dropped(var target, var drag)

    ColumnLayout {
        id: layout
        anchors.fill: parent
        spacing: 3
        Rectangle {
            id: icon
            implicitWidth: implicitHeight * 16/9
            implicitHeight: mosaicHeight
            Layout.alignment: Qt.AlignHCenter
            color: "black"
            VideoStreamPreviewIcon {
                anchors.fill: parent
                stream: root.stream
            }

            OutputBorder {
                anchors.fill: parent
                colorType: borderColor  
                borderWidth: isSelected ? 2:1
            }

            DropArea {
                anchors.fill: parent
                keys: ["VideoStream-"+stream.archived]
                onDropped: {
                    isDropped = true; 
                    root.dropped(root.stream,drag)
                }    
                Rectangle {
                    anchors.fill: parent
                    color: Qt.rgba(147 / 255, 222 / 255, 255 / 255, 0.2)
                    border.color: "black"
                    visible: parent.containsDrag
                }
            }

            DragAndDropSource {
                anchors.fill: parent
                dragAndDropLayer: mainWindow.dragAndDropLayer
                onDoubleClicked: root.doubleClicked()
                onEntered: isDropped = false;
                onClicked: if (!isDropped) root.clicked(); 
                dragItem: Rectangle {
                    implicitWidth: implicitHeight * 16/9
                    implicitHeight: mosaicHeight*1.5
                    color: "black"
                    border.width: 1
                    border.color: borderColor
                    property QtObject dragData: stream
                    Drag.keys: ["VideoStream","VideoStream-"+stream.archived]
                    VideoStreamPreviewIcon {
                        id: dragVideo
                        anchors.fill: parent
                        anchors.margins: 1
                        stream: root.stream
                    }
                }    
            }
            MouseArea {
                anchors.fill: parent
                acceptedButtons: Qt.RightButton
                onClicked: root.rightButtonClicked()
            }
        }
        MixerLabel {
            Layout.fillWidth: true
            text: stream.caption
            colorType: isSelected ? "program" : "neutral"
            font.pixelSize: MixerThemeManager.theme.systemFontSize
            horizontalAlignment: Text.AlignHCenter
        }
    }    
}
