import QtQuick 2.0
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.1
import MixerGuiControls 1.0
import GuiControls 1.0
import Multimedia 1.0

MixerDialog {
    id: root
    title: qsTr("Video Stream Settings")
    property QtObject stream
    property QtObject mediaSettings: stream ? stream.mediaSettings : null
    readonly property bool recordingRunning: archiveRecorder && archiveRecorder.running
    property bool archived: false
   
    signal accepted()
    signal rejected()

    ColumnLayout {
        id: layout

        MixerLabel {
            text: qsTr("Name: ")
        }
        TextField {
            id: captionTextField
            Layout.preferredWidth: 350
        }
        Item {
            implicitHeight: 2
        }

        MixerLabel {
            text: qsTr("Source: ")
        }
        MediaUrlComboBox {
            id: sourceSelector
            Layout.preferredWidth: 350
            archived: root.archived
            onSourceChanged: if(root.visible) stream.source = source
        }
        TrackProperty {
            enabled: !root.recordingRunning
            object: root.mediaSettings
            property: "videoType"
            label: qsTr("Media Type")
            tracks: root.stream ? root.stream.supportedMediaTypes : []
        }
        Item {
            implicitHeight: 8
        }
        
        MixerLabel {
            text: ' '+qsTr("Cropping: ")
        }      
        GridLayout {
            Layout.alignment: Qt.AlignHCenter
            Layout.fillWidth: true
            columns: 5
            Label {
                text: qsTr("Left: ")
            }
            SpinBox {
                id: croppingLeftSpinBox
                maximumValue: 9999
                onValueChanged: stream.cropping.x = value
            }
            Item {
                implicitWidth: 25
            }
            Label {
                text: qsTr("Top: ")
            }
            SpinBox {
                id: croppingTopSpinBox
                maximumValue: 9999
                onValueChanged: stream.cropping.z = value
            }
            
            Label {
                text: qsTr("Right: ")
            }
            SpinBox {
                id: croppingRightSpinBox
                maximumValue: 9999
                onValueChanged: stream.cropping.y = value
            }
            Item {
                implicitWidth: 20
            }            
            Label {
                text: qsTr("Bottom: ")
            }
            SpinBox {
                id: croppingBottomSpinBox
                maximumValue: 9999
                onValueChanged: stream.cropping.w = value
            }
        }
        Item {
            implicitHeight: 4
        }
    }
    
    buttons: [qsTr("OK"), qsTr("Cancel")]
    
    onButtonClicked: {
        if(buttonIndex == 0) accepted()
        else rejected()
        hide()
    }
    
    onClosing: rejected()
    
    readonly property QtObject prevSettings: QtObject {
        property url source
        property string caption
        property vector4d cropping
        property variant mediaType // variant а не var, иначе хранит по ссылке
    }

    onOpened: {
        prevSettings.source = stream.source
        prevSettings.caption = stream.caption
        prevSettings.cropping = stream.cropping
        prevSettings.mediaType = stream.mediaType

        captionTextField.text = stream.caption
        sourceSelector.source = stream.source
        croppingLeftSpinBox.value = stream.cropping.x
        croppingTopSpinBox.value = stream.cropping.z
        croppingRightSpinBox.value = stream.cropping.y
        croppingBottomSpinBox.value = stream.cropping.w
    }
    
    onRejected: {
        stream.source = prevSettings.source
        stream.caption = prevSettings.caption
        stream.cropping = prevSettings.cropping
        stream.mediaType = prevSettings.mediaType
        stream = null
    }
    
    onAccepted: {
        stream.caption = captionTextField.text
        //if(captionTextField.text != stream.caption) videoStreamCollection.renameObject(stream, captionTextField.text || qsTr("Unnamed"))
        stream = null
    }
}
