import QtQuick 2.0
import Serialization 1.0

Object {
    id: root
    property var objects: []
    property Component objectComponent
    
    signal added(QtObject object)
    signal removed(int index, QtObject object)
    signal renamed(int index, string oldName, string newName)
    signal moved(int fromIndex, int toIndex)
    signal swapped(int index1, int index2)

    function getByName(name) {
        if(!name) return null
        for (var i = 0; i < objects.length; i++) {
            if(objects[i].name == name) return objects[i]
        }
        return null
    }

    function sid(i) { return (i<10 ? '0':'')+i }

    function makeUniqueName(baseName) {
        var result = baseName
        var i = 1
        while(getByName(result)) {
            result = "%1 (%2)".arg(baseName).arg(sid(i))
            i++
        }
        return result
    }

    function create(name, params) {
        var paramsWithName = {}
        if(params){
            for (var key in params) {
                paramsWithName[key] = params[key]
            }
        }
        paramsWithName["name"] = name
        var object = objectComponent.createObject(root, paramsWithName)
        return object
    }
    
    function add(name, params) {
        var object = create(name != undefined ? makeUniqueName(name) : undefined, params)
        objects.push(object)
        objects = objects
        added(object)
        return object
    }

    function remove(index) {
        var object = objects[index]
        if(!object) return
        objects.splice(index, 1)
        objects = objects
        removed(index, object)
        object.destroy()
    }

    function renameObject(object, name) {
        rename(objects.indexOf(object), name)
    }

    function rename(index, name) {
        var object = objects[index]
        if(!object || object.name == name) return
        var oldName = object.name
        object.name = makeUniqueName(name)
        objects = objects
        renamed(index, oldName, object.name)
    }

    function move(fromIndex, toIndex) {
		if(fromIndex == toIndex) return
		if(fromIndex < 0 || fromIndex > objects.length - 1) return
		if(toIndex < 0 || toIndex > objects.length) return
        var object = objects[fromIndex]
        if(fromIndex < toIndex) {
            objects.splice(toIndex, 0, object)
            objects.splice(fromIndex, 1)
        } else {
            objects.splice(fromIndex, 1)
            objects.splice(toIndex, 0, object)
        }
        objects = objects
        moved(fromIndex, toIndex)
    }
    
	function swap(index1, index2) {
		if(index1 == index2) return
		if(index1 < 0 || index1 > objects.length - 1) return
		if(index2 < 0 || index2 > objects.length - 1) return
		var tmp = objects[index1]
		objects[index1] = objects[index2]
		objects[index2] = tmp
		objects = objects
        swapped(index1, index2)
	}

    function clear() {
        var prevObjects = objects
        objects = []
        for (var i = 0; i < prevObjects.length; i++) {
            removed(i, prevObjects[i])
            prevObjects[i].destroy()
        }
    }
    
    function saveObjects() {
        var objectsData = []
        for (var i = 0; i < objects.length; i++) {
            objectsData.push(objects[i].save())
        }
        return objectsData
    }

    function loadObjects(data) {
        clear()
        if(!data) return
        for(var i = 0; i < data.length; i++) {
            var object = create("")
            object.load(data[i])
            objects.push(object)
            added(object)
        }
        objects = objects
    }
}
