import QtQuick 2.0
import Multimedia 1.0
import Multimedia.FFmpeg 1.0

MediaSession {
    id: root
    property var settings:({})
    readonly property real progress: source.duration > 0 ? Math.min(source.frameTime / source.duration, 1) : 0
    
    autoPlay: true
    QtQuickSource {
        id: source
        Scene {
            id: scene
            anchors.fill: parent
            recordingTime: source.frameTime
            settings: root.settings
        }
        frameSize: root.settings.frameSize
        frameRate: root.settings.frameRate
        offlineMode: true
        duration: {
            var result = 0
            for(var i = 0; i < root.settings.intervals.length; i++) {
                var interval = root.settings.intervals[i]
                result += interval.to - interval.from
            }
            return result
        }
        audioStream: scene.audioStream
    }
    VideoEncoder {
        id: videoEncoder
        codec: VideoEncoder.H264
        bitrate: root.settings.bitrate * 1024
    }
    AudioEncoder {
        id: audioEncoder
    }
    FFmpegSink {
        id: sink
        url: root.settings.fileUrl
    }    
    NodeConnection {
        from: source.videoOutput
        to: videoEncoder.input
    }
    NodeConnection {
        from: videoEncoder.output
        to: sink.videoInput
    }
    NodeConnection {
        from: source.audioOutput
        to: audioEncoder.input
    }
    NodeConnection {
        from: audioEncoder.output
        to: sink.audioInput
    }
}
