import QtQuick 2.0
import Multimedia 1.0

Item {
    id: root
    property var settings: ({})
    property real recordingTime
    property AudioStream audioStream: AudioMixer {
        streams: {
            var result = []
            for(var i = 0; i < audioPlayersRepeater.count; i++) {
                var player = audioPlayersRepeater.itemAt(i)
                result.push(player.audioStream)
            }
            return result
        }
    }
    onRecordingTimeChanged: {
        var t = recordingTime
        var index = 0
        for(var i = 0; i < root.settings.intervals.length; i++) {
            var interval = root.settings.intervals[i]
            var duration = interval.to - interval.from
            if(t < duration) break
            t -= duration
            index++
        }
        root.currentIntervalIndex = index
        root.time = (currentInterval ? currentInterval.from : 0) + t
    }
    
    property real time
    property int currentIntervalIndex
    readonly property var currentInterval: settings.intervals[currentIntervalIndex] || null
    property var currentDateInterval: null
    property bool enablePlayers: false
    property bool enableViews: false

    onCurrentIntervalChanged: {
        enableViews = false
        enablePlayers = false
        if(currentInterval) {
            currentDateInterval = {
                from: new Date(settings.archiveCreationDate.getTime() + currentInterval.from),
                to: new Date(settings.archiveCreationDate.getTime() + currentInterval.to + 10000)
            }
        }
        else {
            currentDateInterval = null
        }
        if(currentDateInterval) {
            enablePlayers = true
            enableViews = true
        }
    }
    
    Repeater {
        id: playerRepeater
        model: {
            if(!root.enablePlayers) return []
            var videoPlayersSettings = []            
            function isDuplicate(videoStreamsSettings) {
                for(var i = 0; i < videoPlayersSettings.length; i++) {
                    if(videoPlayersSettings[i].streamName == videoStreamsSettings.streamName && videoPlayersSettings[i].storage == videoStreamsSettings.storage) return true
                }
                return false
            }
            var videoStreamsSettings = settings.videoStreams || []
            for(var i = 0; i < videoStreamsSettings.length; i++) {
                var videoStreamSettings = videoStreamsSettings[i]
                if(!videoStreamSettings.streamName || !videoStreamSettings.storage) continue
                if(isDuplicate(videoStreamSettings)) continue
                videoPlayersSettings.push({streamName: videoStreamSettings.streamName, storage: videoStreamSettings.storage})
            }
            return videoPlayersSettings
        }
        VideoPlayer {
            settings: modelData
            interval: root.currentDateInterval
        }
    }
    
    readonly property int captionFontSize: settings.fontSize
    
    Repeater {
        model: {
            if(!root.enableViews) return []
            return settings.videoStreams || []
        }
        VideoStreamView {
            settings: modelData
            captionFontSize: root.captionFontSize
            player: {
                if(!settings.streamName || !settings.storage) return null
                for(var i = 0; i < playerRepeater.count; i++) {
                    var player = playerRepeater.itemAt(i)
                    if(player.settings.streamName == settings.streamName && player.settings.storage == settings.storage) return player
                }
                return null
            }
        }
    }
    
    Titles {
        anchors.fill: parent
        time: root.time
        settings: root.settings
    }
    Repeater {
        id: audioPlayersRepeater
        model: {
            if(!root.enablePlayers) return []
            return settings.audioStreams || []
        }
        AudioPlayer {
            settings: modelData
            interval: root.currentDateInterval
        }
    }
}
