import QtQuick 2.0
import QtQuick.Layouts 1.0
import DonSouzMediaRecorder 1.0
import Multimedia 1.0

Item {
    id: root
    property real time
    property var settings: ({})

    EventCollection {
        id: eventCollection
        fileUrl: settings.exportEventCaptions ? settings.eventsUrl : ""
        filters: settings.eventFilters
        previewTime: settings.exportEventCaptions ? root.time : -1
    }

    TimeSchedulerLocker {
        locked: eventCollection.status != EventCollection.Ready
    }

    readonly property int captionCount: settings.exportEventCaptions ? eventCollection.maxPreviewedEventCount : 0

    ColumnLayout {
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 10
        Repeater {
            model: root.captionCount
            Text {
                id: eventCaption
                Layout.preferredWidth: root.width - 20

                text: {
                    var event = eventCollection.previewedEvents[model.index]
                    if(!event) return ""
                    var text = ""
                    if (event.name && event.name.length) text += event.name + ": "
                    if (event.caption && event.caption.length) text += event.caption
                    return text
                }
                visible: !!text

                color: "white"
                style: Text.Outline
                font.pixelSize: settings.titlesFontSize
                wrapMode: Text.WordWrap
                horizontalAlignment: Text.AlignHCenter
            }
        }
    }

    
    Text {
        anchors.right: parent.right
        anchors.top: parent.top
        anchors.rightMargin: 5
        color: "white"
        style: Text.Outline
        font.pixelSize: settings.fontSize
        visible: settings.exportDateTimeCaption
        text: {
            if(!settings.archiveCreationDate || !visible) return ""
            var date = new Date(settings.archiveCreationDate.getTime() + root.time)
            return Qt.formatDateTime(date, "dd.MM.yyyy hh:mm:ss")
        }
    }
}
