import QtQuick 2.0
import QtQuickUtils 1.0
import Multimedia 1.0
import GuiControls 1.0

Item {
    id: root
    property VideoPlayer player
    property var settings
    property int captionFontSize: 20
    x: settings.rect.x * (parent ? parent.width : 0)
    y: settings.rect.y * (parent ? parent.height : 0)
    width: settings.rect.width * (parent ? parent.width : 0)
    height: settings.rect.height * (parent ? parent.height : 0)
    
    property size resolution: settings.resolution || Qt.size(0, 0)
    readonly property size currentResolution: hasData ? Qt.size(videoItem.implicitWidth, videoItem.implicitHeight) : Qt.size(0, 0)
    onCurrentResolutionChanged: if(currentResolution.width > 0 && currentResolution.height > 0) resolution = currentResolution
    
    readonly property bool hasSource: settings.hasSource
    readonly property bool hasData: videoItem.implicitWidth > 0 && videoItem.implicitHeight > 0
    
    FillModeItem {
        anchors.fill: parent
        anchors.margins: 1
        aspect: resolution.height > 0 ? resolution.width/resolution.height : 0
        CroppingItem {
            id: videoItem
            anchors.fill: parent
            cropping: settings.cropping
            visible: hasData
            VideoView {
                id: videoView
                source: player ? player.sink : null
            }
        }
        ChessBoard {
            anchors.fill: parent
            visible: !hasSource
            Text {
                anchors.centerIn: parent
                text: "Нет источника"
                color: Qt.rgba(251 / 255, 145 / 255, 104 / 255)
                style: Text.Outline
                font.pixelSize: captionFontSize
            }
        }
        Text {
            id: caption
            visible: hasSource
            x: captionFontSize * 0.5
            text: settings.caption || ""
            color: "white"
            style: Text.Outline
            font.pixelSize: captionFontSize
        }
        Text {
            anchors.centerIn: parent
            text: "Нет сигнала"
            color: Qt.rgba(0.9, 0.1, 0.1, 1.0)
            font.pixelSize: captionFontSize
            visible: hasSource && !hasData
        }
    }
    Rectangle {
        anchors.fill: parent
        anchors.margins: -1
        color: "black"
        z: -1
    }
}
