import QtQuick 2.0
import QtQml 2.2
import Serialization 1.0
import Multimedia.PostPlay 1.0
import "../../common/utils"

Object {
    id: root

    property string name
    property date creationDate: PostPlayService.startTime
    property real duration: PostPlayService.duration
    property real durationForSaving: PostPlayService.duration

    readonly property real size: {
        var result = 0
        for(var i = 0; i < rangeInstantiator.count; i++) {
            var size = rangeInstantiator.objectAt(i).size
            if(size >= 0) result += size
        }
        return result
    }
    readonly property bool sizeIsDefined: {
        for(var i = 0; i < rangeInstantiator.count; i++) {
            var size = rangeInstantiator.objectAt(i).size
            if(size < 0) return false
        }
        return true
    }
    
    property real diskSize: PostPlayService.diskSize
    readonly property date finishDate: new Date(creationDate.getTime() + duration)
    readonly property RecordingSettings recordingSettings: RecordingSettings {}
    property url eventsUrl
    property var events

    function setEvent(event) {
        p('Archive::setEvent',event)
        PostPlayService.setEvent(event)
    }
    function setRecordingSettings(settings) {
        recordingSettings.load(settings.save())
    }
    
    Instantiator {
        id: rangeInstantiator
        model: recordingSettings.elements
        Range {
            from: root.creationDate
            to: (root == archiveRecorder.archive) ? new Date(root.creationDate.getTime() + 48*60*60*1000) : root.finishDate
            stream: {
                var storage = modelData.storage
                if(!storage || !storage.streams) return null
                //return storage.getStreamByName(modelData.streamName)
                return PostPlayService.getStreamByName(modelData.streamName)
            }
        }
    }

    function save() {
        return {
            name: name,
            creationDate: creationDate,
            duration: durationForSaving,
            recordingSettings: recordingSettings.save(),
            events: events,
            eventsUrl: eventsUrl
        }
    }

    function load(data) {
        name = data.name || ""
        creationDate = data.creationDate || ""
        duration = data.duration || 0
        durationForSaving = duration
        recordingSettings.load(data.recordingSettings || {})
        eventsUrl = data.eventsUrl || ""
        events = data.events || []
//p('Archive::load', root)
    }
}
