import QtQuick 2.0
import Multimedia 1.0
import "../../common/utils"

Object {
    id: root
    property string name
    property url source
    property real volume: 0.5
    property bool mute: false
    readonly property real level: Math.max(audioLevelMeter.peakLevels[0], audioLevelMeter.peakLevels[1])
    readonly property alias audioStream: audioStream

    FdAudioPlayer {
        id: realtimePlayer
        source: root.source
        active: !archivePlayer.active
    }
    
    readonly property int playerIndex: archivePlayer.archive ? archivePlayer.archive.recordingSettings.getAudioIndex(root.source) : -1
    readonly property var activePlayer: {
        if(realtimePlayer.active) return realtimePlayer
        return archivePlayer.players[playerIndex] || null
    }

    MediaSessionAudioStream {
        id: audioStream
        source: (activePlayer && activePlayer.audioSink) ? activePlayer.audioSink : null
        mute: root.mute
        volume: root.volume
    }

    AudioLevelMeter {
        id: audioLevelMeter
        stream: root.audioStream
    }
    
    function save() {
        return {
            name: name,
            source: source,
            volume: volume,
            mute: mute
        }
    }

    function load(data) {
        name = data.name || ""
        source = data.source || ""
        volume = data.volume || 0
        mute = data.mute || false
    }   
}
