import QtQuick 2.0
import Serialization 1.0
import Multimedia 1.0 as M
import "../../common/utils"

ObjectCollection {
    id: root
    property var source: availableAudios && availableAudios[0] && availableAudios[0].source
    readonly property int maxCount: 2
    property alias streams: root.objects
    property alias mixedAudioStream: mixedAudioStream
    objectComponent: AudioStream{}

    property var volumeAudioStream: streams && streams[0] && streams[0].audioStream 
    property var originAudioStream: streams && streams[1] && streams[1].audioStream 

    M.AudioMixer {
        id: mixedAudioStream
        streams: root.streams.map(function(stream) {
            return stream.audioStream
        })
    }

    ObjectSaver {
        id: objectSaver
        object: root
        fileUrl: Serialization.applicationDataDirPath + "/audio_streams.json"
    }
    
    function getBySource(source) {
        for(var i = 0; i < streams.length; i++) {
            if(streams[i].source == source) return streams[i]
        }
        return null
    }

    function init() {
        objectSaver.init()
    }
    
    function save() {
        return {
            streams: saveObjects()
        }
    }

    function correct(data) {
        data[0].source = source
        data[1].source = source
        data[1].volume = 1.0
    }

    function load(data) {
        correct(data.streams)
        loadObjects(data.streams)
    }
    
    function createDefault() {
        add("Stream 1", {})
        add("Stream 2", {})
        correct(objects)
    }
    
}
