import QtQuick 2.0
import Multimedia 1.0
import Multimedia.FDExt 1.0
import Multimedia.DeckLink 1.0
import Multimedia.DirectShow 1.0
import Qt.labs.folderlistmodel 2.1
import Serialization 1.0
import "../../common/utils"

Object {
    id: root
    property var data: []

    readonly property var videos: Serialization.isFileExists(config.videos) && config.videos || (Serialization.applicationDataDirPath + "/videos") 
    
    readonly property Component mediaSettingsComponent: Component {
        id: settingsComponent
        MediaSettings {}
    }

    ObjectSaver {
        id: objectSaver
        object: root
        //fileUrl: Serialization.applicationDataDirPath + "/medias.json"
    }

    function init() {
        objectSaver.init()
    }

    function save() {
        var medias = []
        for (var i in data) {
            medias.push(data[i].save())
        }
        return { medias: medias }
    }

    property var savedSettings: ({})
    property var currentMedias: enumerateMedias()

    onCurrentMediasChanged: load()

    function load(savedData) {
        if (savedData && savedData.medias) {
            for (var i in savedData.medias) {
                var settings = mediaSettingsComponent.createObject(null, {})
                if (settings) {
                    settings.load(savedData.medias[i])
                    savedSettings[settings.source] = settings
                }
            }
        }

        for (var i in currentMedias) {
            var currentMedia = currentMedias[i]
            var settings = savedSettings[currentMedia.source]
            if (!settings || settings.videoType === undefined) continue
            currentMedia.videoType = settings.videoType; 
        }

        data = currentMedias
    }

    function enumerateMedias() {
        var result = data

        function has(source) {
            for(var i in result) if (result[i].source == source) return true
            return false
        }
        
        if (1) {
            var ip_cameras = Serialization.readFromJsonFile(Serialization.applicationDataDirPath + "/ip_cameras.json") || {}
            var from_json  = Serialization.readFromJsonFile(Serialization.applicationDataDirPath + "/available_videos.json") || {}
            var medias  = availableVideos.concat(from_json.video||[]).concat(ip_cameras.streams||[])

            for (var i in medias) {
                var media = medias[i]
                if (has(media.source)) continue
                var scheme = media.source.toString().split('://')[0]
                if (['rtsp','rtmp','udp','http'].indexOf(scheme)==-1) continue
                media.deviceType = "ip";
                result.push(settingsComponent.createObject(null, media))
            }
        }    
        for(var i = 0; i < FD.inputs.length; i++) {
            var name = FD.inputs[i].name
            var url = Serialization.stringToUrl("fd:///" + encodeURIComponent(name))
            if (!has(url)) result.push(settingsComponent.createObject(null, {name: name, source: url, deviceType: "fd"}))
        }
        for(var i = 0; i < DeckLink.inputs.length; i++) {
            var device = DeckLink.inputs[i]
            var url = Serialization.stringToUrl("dl:///" + encodeURIComponent(device.id))
            if (!has(url)) result.push(settingsComponent.createObject(null, {name: device.name, source: url, deviceType: "dl"}))
        }
        for(var i = 0; i < DirectShowDevices.videoInputs.length; i++) {
            var device = DirectShowDevices.videoInputs[i]
            var url = Serialization.stringToUrl("dshow:///" + encodeURIComponent(device.id))
            if (!has(url)) result.push(settingsComponent.createObject(null, {name: device.name, source: url, deviceType: "dshow"}))
        }

        for(var i in config.testMedias) {
            var name = "Test " + config.testMedias[i]
            var url = Serialization.stringToUrl("test:///" + name)
            if (!has(url)) result.push(settingsComponent.createObject(null, {name: name, source: url, deviceType: "test"}))
        }

        if (Serialization.isFileExists(videos)) { // HACK: когда папки не существует, FolderListModel использует папку приложения
            for(var i = 0; i < videoFilesModel.count; i++) {
                var fileName = videoFilesModel.get(i, "fileName")
                var url = Serialization.stringToUrl(videoFilesModel.folder + "/" + fileName)
                if (!has(url)) result.push(settingsComponent.createObject(null, {name: fileName, source: url, deviceType: "video"}))
            }
        }    
        
        if (0&&archivePlayer.archive) {
            for(var i = 0; i < archivePlayer.archive.recordingSettings.elements.length; i++) {
                var element = archivePlayer.archive.recordingSettings.elements[i]
                if(element.type != 0) continue
                if (!has(element.source)) result.push(settingsComponent.createObject(null, {name: element.source.toString(), source: element.source, deviceType: "archive"}))
            }
        }

        return result
    }

    function getNameByUrl(url) {
        for(var i = 0; i < data.length; i++) {
            var element = data[i]
            if(element.source == url) return element.name
        }
        return url.toString()
    }

    function get(source) {
        if (!source) return null
        for (var i in data) {
            var settings = data[i]
            if (settings.source === source) return settings
        }
        return null
    }

    FolderListModel {
        id: videoFilesModel
        folder: videos
        showDirs: false
    }
}
