import QtQuick 2.0
import Serialization 1.0
import "../../common/utils"

Object {
    id: root
    property url fileUrl
    property size frameSize: Qt.size(1280, 720)
    property size frameRate: Qt.size(25, 1)
    property real bitrate: 5000
    property bool exportEventCaptions: true
    property bool exportDateTimeCaption: true
    property int selectIntervalMode: 0
    readonly property alias intervals: intervalCollection.objects
    property alias currentIntervalIndex: intervalCollection.currentIndex
    readonly property QtObject currentInterval: intervals[intervalCollection.currentIndex] || null

    ObjectCollection {
        id: intervalCollection
        property int currentIndex: -1
        objectComponent: QtObject {
            property real from
            property real to
            readonly property bool isCurrent: this == root.currentInterval
        }
        
        onRemoved: {
            if(objects.length == 0) currentIndex = -1
            else if(currentIndex > index || (currentIndex == index && index == objects.length)) currentIndex = Math.max(0, currentIndex - 1)
        }
        
        onAdded: if(currentIndex == -1) currentIndex = 0
        
        onSwapped: {	
            if(currentIndex == index1) currentIndex = index2
            else if(currentIndex == index2) currentIndex = index1	
        }        
    }
    
    ObjectSaver {
        id: objectSaver
        object: root
        fileUrl: Serialization.applicationDataDirPath + "/exporter_settings.json"
    }

    Component.onCompleted: objectSaver.init()
    
    function addInterval(params) {
        intervalCollection.add(undefined, params)
    }
    
    function removeInterval() {
        if(currentIntervalIndex < 0) return
        intervalCollection.remove(currentIntervalIndex)
    }
    
    function moveIntervalUp() {
        if(currentIntervalIndex <= 0) return
        intervalCollection.swap(currentIntervalIndex - 1, currentIntervalIndex)
    }
    
    function moveIntervalDown() {
        if(currentIntervalIndex < 0 || currentIntervalIndex >= intervals.length - 1) return
        intervalCollection.swap(currentIntervalIndex, currentIntervalIndex + 1)
    }
    
    function save() {
        return {
            frameSize: frameSize,
            frameRate: frameRate,
            bitrate: bitrate,
            exportEventCaptions: exportEventCaptions,
            exportDateTimeCaption: exportDateTimeCaption
        }
    }
    
    function load(data) {
        frameSize = data.frameSize || Qt.size(1280, 720)
        frameRate = data.frameRate || Qt.size(25, 1)
        bitrate = data.bitrate || 5000
        exportEventCaptions = data.exportEventCaptions || false
        exportDateTimeCaption = data.exportDateTimeCaption || false
    }
    
    Connections {
        target: archivePlayer
        onArchiveChanged: {
            currentIntervalIndex = -1
            selectIntervalMode = 0
            intervalCollection.clear()
        }
        onPositionChanged: {
            if(!currentInterval) return
            if(selectIntervalMode == 1) {
                currentInterval.from = archivePlayer.position 
                if(currentInterval.to < archivePlayer.position) currentInterval.to = archivePlayer.position
            }
            else if(selectIntervalMode == 2) {
                currentInterval.to = archivePlayer.position
                if(currentInterval.from > archivePlayer.position) currentInterval.from = archivePlayer.position
            }
        }
    }
    
    
}
