import QtQuick 2.0
import Multimedia 1.0
import "utils"
import "../../common/utils"

SharedObject {
    id: root
    readonly property alias mediaSession: mediaSessionLoader.item
    readonly property QtQuickSink videoSink: mediaSession ? mediaSession.videoSink : null
    readonly property real aspect: (mediaSession && mediaSession.aspect) ? mediaSession.aspect : 0
    readonly property variant mediaType: mediaSession ? mediaSession.mediaType : defaultMediaType
    readonly property var supportedMediaTypes: mediaSession ? mediaSession.supportedMediaTypes : []
    readonly property int status: mediaSession ? mediaSession.status : MediaSession.Empty
    readonly property bool hasData: videoSink ? videoSink.hasTexture : false
    property var stream

    property MediaSettings mediaSettings: availableMedias.get(root.source)
    onMediaSettingsChanged: {
        mediaSessionLoader.updateSource()
    }

    property var defaultMediaType: Multimedia.videoType({subtype: "RGB32", frameSize: Qt.size(1280, 720), frameRate: Qt.size(60, 1)})

    readonly property Loader mediaSessionLoader: Loader {
        id: mediaSessionLoader
        function updateSource() {
            source = ""
            var sourceString = root.source.toString()

            if (!mediaSettings) return;

            var params = {
                mediaType: Qt.binding(function() { return mediaSettings.videoType || root.mediaType }) 
            }

            if(sourceString.indexOf("fd:///") == 0) {
                params["deviceId"] = decodeURIComponent(sourceString.substring("fd:///".length))
                params.stream = root.stream
                setSource("medias/FdMediaSession.qml", params)
            }
            else if(sourceString.indexOf("dl:///") == 0) {
                params["deviceId"] = decodeURIComponent(sourceString.substring("dl:///".length))
                setSource("medias/DlMediaSession.qml", params)
            }
            else if(sourceString.indexOf("dshow:///") == 0) {
                params["deviceId"] = decodeURIComponent(sourceString.substring("dshow:///".length))
                setSource("medias/DShowMediaSession.qml", params)
            }
            else if(sourceString.indexOf("test:///") == 0) {
                setSource("medias/TestMediaSession.qml", params)
            }
            else {
                params["source"] = root.source
                setSource("medias/FFmpegMediaSession.qml", params)
            }
        }
    }
    
    onSourceChanged: mediaSessionLoader.updateSource()
    onStatusChanged: if(status == MediaSession.Error) console.warn(qsTr("Error in input device: ") + root.source)
}
