import QtQuick 2.0
import "../../common/utils"

ObjectCollection {
    id: root
    property alias mosaics: root.objects
    objectComponent: Mosaic{}
    
    property string name
    property int currentIndex: -1

    property var monitorIndexes: []

    function monitorMosaicIndex(monitorIndex) {
        for (var i=0; i<10; i++) if (monitorIndex>=monitorIndexes.length) monitorIndexes.push(0)
        return monitorIndexes[monitorIndex] || 0
    }
    
    function monitorMosaic(monitorIndex) {
        return mosaics[monitorMosaicIndex(monitorIndex)] || null
    }
    
    onAdded: if(currentIndex == -1) currentIndex = 0
    onRemoved: if(currentIndex >= mosaics.length) currentIndex = mosaics.length - 1
    onMoved: {
		if(currentIndex == fromIndex) currentIndex = fromIndex < toIndex ? toIndex - 1 : toIndex
        else if(currentIndex > fromIndex && currentIndex < toIndex) currentIndex--
        else if(currentIndex < fromIndex && currentIndex >= toIndex) currentIndex++    
    }

    function removeStream(stream) {
        for (var i = 0; i < mosaics.length; i++) {
            mosaics[i].removeStream(stream)
        }
    }
    
    function save() {
        return {
            name: name,
            monitorIndexes: monitorIndexes,
            mosaics: saveObjects()
        }
    }

    function load(data) {
        name = data.name || ""
        monitorIndexes = data.monitorIndexes || [0]
        loadObjects(data.mosaics)
    }
    
    function createDefault() {
        add("Мозаика 1", {template: templateCollection.getByName("Шаблон 1")})
        add("Мозаика 2", {template: templateCollection.getByName("Шаблон 2")})
        add("Мозаика 3", {template: templateCollection.getByName("Шаблон 3")})    
        currentIndex = 0    
    }
}
