import QtQuick 2.0
import Serialization 1.0
import "../../common/utils"

ObjectCollection {
    id: root
    property alias presets: root.objects
    objectComponent: MosaicCollection{}
    
    property int currentIndex: -1
    readonly property MosaicCollection currentPreset: presets[currentIndex] || null
    
    readonly property var presetNames: {
        var result = []
        for(var i = 0; i < presets.length; i++) {
            result.push(presets[i].name)
        }
        return result
    }
    
    onRemoved: {
        if(presets.length == 0) currentIndex = -1
        else if(currentIndex > index || (currentIndex == index && index == presets.length)) currentIndex = Math.max(0, currentIndex - 1)
    }
    
    onAdded: if(currentIndex == -1) currentIndex = 0
    
    onSwapped: {	
        if(currentIndex == index1) currentIndex = index2
		else if(currentIndex == index2) currentIndex = index1	
    }

    ObjectSaver {
        id: objectSaver
        object: root
        fileUrl: Serialization.applicationDataDirPath + "/mosaics.json"
    }

    function init() {
        objectSaver.init()
    }
    
    function save() {
        return {
            currentIndex: currentIndex,
            presets: saveObjects()
        }
    }

    function load(data) {
        loadObjects(data.presets)
        currentIndex = data.currentIndex !== undefined ? data.currentIndex : -1
    }
    
    function createDefault() {
        var preset
        preset = add("Набор 1")
        preset.createDefault()
        currentIndex = 0  
    }
}
