import QtQuick 2.0

Item {
    id: root
    property Mosaic mosaic
    property int monitor: -1
    property VideoStream previewStream: currentMonitorIndex==monitor && videoStreamCollection && videoStreamCollection.currentStream || null
    property real captionFontSize: 20
    property int border: 1
    property real previewSize: 0.9
    
    Repeater {
        model: mosaic && mosaic.template ? mosaic.template.elements : []

        VideoStreamView {
            x: modelData.rect.x * root.width  + border
            y: modelData.rect.y * root.height + border
            width : modelData.rect.width  * root.width  -2*border
            height: modelData.rect.height * root.height -2*border
            stream: mosaic && mosaic.streams[model.index] || null
            captionFontSize: root.captionFontSize

            Rectangle {
                anchors.fill: parent
                anchors.margins: -root.border
                border.width: root.border
                border.color: stream && stream.archived && archivePlayer && archivePlayer.color || "gray"  
                color: "black"
                z: -1
            }
        }
    }

    VideoStreamView {
        id: preview
        x: (1-previewSize)/2 * root.width  + border
        y: (1-previewSize)/2 * root.height + border
        width : previewSize * root.width  -2*border
        height: previewSize * root.height -2*border
        captionFontSize: root.captionFontSize
        stream: previewStream
        enabled: !!previewStream
        visible: !!previewStream

        Rectangle {
            anchors.fill: parent
            anchors.margins: -root.border
            border.width: root.border
            border.color: previewStream && previewStream.archived && archivePlayer && archivePlayer.color || "gray"  
            color: "black"
            z: -1
        }
    }
}
