import QtQuick 2.0
import UdpProtocol 1.0
import Serialization 1.0
import "../../common/utils"

Object {
    id: root
    property real deltaWithSystemTime: 0
    
    property alias localAddress: client.addrLocal
    property alias localPort: client.portControl
    property alias remotePort: client.portFeedback
        
    UdpProtocolClient {
        id: client
		addrLocal: NetworkAddressEnumerator.addresses[0]
                
        function dateToMilliseconds(date) {
            return ((date.getHours() * 60 + date.getMinutes()) * 60 + date.getSeconds()) * 1000 + date.getMilliseconds()
        }
        
        onEventReceived: {
            var boardTime = dateToMilliseconds(event.timeOnBoard)
            root.deltaWithSystemTime = boardTime - dateToMilliseconds(new Date())
            if(!archiveRecorder.running) return
            archiveRecorder.addEvent({
                source: event.source.trim(),
                value: event.value.trim(),
                caption: event.description.trim(),
                boardTime: boardTime
            })
        }
    }
      
    ObjectSaver {
        id: objectSaver
        object: root
        fileUrl: Serialization.applicationDataDirPath + "/network_settings.json"
    }
    
    function save() {
        return {
            localAddress: localAddress,
            localPort: localPort,
            remotePort: remotePort
        }
    }
    
    function load(data) {
        localAddress = data.localAddress || ""
        localPort = data.localPort || 0
        remotePort = data.remotePort || 0
    }
    
    Component.onCompleted: objectSaver.init()
    
}
