import QtQuick 2.0
import Serialization 1.0
import Qt.labs.folderlistmodel 2.1
import QtQml 2.2
import "../../common/utils"

Object {
    id: root
    readonly property var names: folderModelInstantiator.object ? folderModelInstantiator.object.names : []
    readonly property string currentName: getFileNameByDate(currentDate)
    property date currentDate
    property var elements: []
    readonly property url folder: Serialization.applicationDataDirPath + "/operation_logs/"
    
    onCurrentNameChanged: {
        if(!currentName) {
            elements = []
            return
        }
        var fileUrl = getFileUrlByName(currentName)
        elements = Serialization.readFromJsonArrayFile(fileUrl)
    }

    // type: "info", "error"
    function addMessage(message, type) {
        if(!type) type = "info"
        var date = new Date()
        var name = getFileNameByDate(date)
        var fileUrl = getFileUrlByName(name)
        var element = {message: message, date: date, type: type}
        Serialization.appendJsonToFileAsync(fileUrl, element, null)
        if(name == currentName) {
            elements.push(element)
            elements = elements
        }
    }
    
    function getFileUrlByName(name) {
        return root.folder + name + ".ajson"
    }
    
    function getFileNameByDate(date) {
        return Qt.formatDate(date, "dd.MM.yyyy")
    }
        
    Instantiator {
        id: folderModelInstantiator
        active: false
        FolderListModel {
            sortField: FolderListModel.Time
            folder: root.folder
            showDotAndDotDot: true
            
            readonly property var names: {
                var result = []
                for(var i = 0; i < count; i++) {
                    var fullName = get(i, "fileName")
                    if(fullName.slice(-6) != ".ajson") continue
                    var name = fullName.slice(0, fullName.length - 6)
                    result.push(name)
                }
                return result
            }
        }
    }
    
    Component.onCompleted: {
        Serialization.createDirectory(folder)
        folderModelInstantiator.active = true
        currentDate = new Date()
    }
}
