import QtQuick 2.0
import Multimedia 1.0
import QtQuickUtils 1.0
import "../../common/utils"

Object {
    id: root
    property size requestedOutputSize
    readonly property size maxOutputSize: Qt.size(1920, 1080)
    readonly property size thumbnailSize: Qt.size(192, 108)
    readonly property real atlasHeight: thumbnailSize.height * 8
    property alias enabled: sender.enabled

    function calcuateSize(requestedSize, originalSize) {
        if(originalSize.width <= 0 || originalSize.height <= 0) return Qt.size(0, 0)
        if(requestedSize.width <= 0 || requestedSize.height <= 0) return Qt.size(0, 0)
        var scale = Math.min(requestedSize.width/originalSize.width, requestedSize.height/originalSize.height)
        if(scale > 1) scale = 1
        return Qt.size(originalSize.width * scale, originalSize.height * scale)
    }

    TextureSharingSender {
        id: sender
        name: "DonSouzTextureAtlas"
        backgroundColor: "transparent"
        frameSize: Qt.size(maxOutputSize.width, maxOutputSize.height + atlasHeight)
        sourceItem: Item {
            anchors.fill: parent
            OutputView {
                id: outputView
                readonly property size size: root.calcuateSize(requestedOutputSize, maxOutputSize)
                width: size.width
                height: size.height
                visible: width > 0 && height > 0
            }
            Flow {
                id: flow
                y: maxOutputSize.height
                width: sender.frameSize.width
                height: atlasHeight
                spacing: 0
                Repeater {
                    id: videoStreamRepeater
                    model: videoStreamCollection.streams
                    FillModeItem {
                        readonly property QtObject stream: modelData
                        width: root.thumbnailSize.width
                        height: root.thumbnailSize.height
                        aspect: stream.aspect
                        OffscreenItemView {
                            anchors.fill: parent
                            source: stream.visualContent
                        }
                    }
                }
            }
        }

        function getItemRect(item) {
            if(!item || item.width <= 0 || item.height <= 0) return Qt.rect(0, 0, 0, 0)
            return Qt.rect(item.x, item.y, item.width, item.height)
        }

        function getRects() {
            var result = {}
            for(var i = 0; i < videoStreamRepeater.count; i++) {
                var item = videoStreamRepeater.itemAt(i)
                if(!item) continue
                var rect = getItemRect(item)
                rect.y += flow.y
                result[item.stream.name] = rect
            }
            result["!@output"] = getItemRect(outputView)
            return result
        }

        attachedData: getRects()
    }
}
