import QtQuick 2.0
import "../../common/utils"

ObjectCollection {
    id: root
    property alias elements: root.objects
    objectComponent: RecordingSettingsElement {}
    signal validated(string error)

    function getVideoIndex(source) {
        for(var i in elements) {
            var element = elements[i]
            if(element.type != 0) continue
            if(element.source == source) return i
        }
        return -1
    }

    function getAudioIndex(source) {
        for(var i in elements) {
            var element = elements[i]
            if(element.type != 1) continue
            if(element.source == source) return i
        }
        return -1
    } 

    function validate() {
        if(elements.length == 0) {
            root.validated(qsTr("Can't start recording, no one source was selected."))
            return false
        }
        for(var i = 0; i < elements.length; i++) {
            for(var j = i + 1; j < elements.length; j++) {
                var e1 = elements[i]
                var e2 = elements[j]
                if(e1.type != e2.type) continue
                if(e1.source.toString() && e1.source == e2.source) {
                    root.validated(qsTr("Can't start recording, source was added multiple times."))
                    return false
                }
            }
        }
        for(var i = 0; i < elements.length; i++) {
            if(!elements[i].storage) {
                root.validated(qsTr("Can't start recording, storage is not selected."))
                return false
            }
        }
        root.validated("")
        return true
    }

    function save() {
        return {
            elements: saveObjects()
        }
    }

    function load(data) {
        loadObjects(data.elements)
    }
}
