import QtQuick 2.0

import Multimedia 1.0
import Multimedia.PostPlay 1.0
import Serialization 1.0

import "../../common/utils"

Object {
    id: root
    property int type: 0    //0 - video, 1 - audio
    property url source
    property var storageGuid: 0
    property int bitrate: 0
    property bool hardware: false

    readonly property string streamName: {
        var result = source.toString()
        if(type == 1) result += " (Audio)"
        return result
    }

    readonly property string typeString: {
        if(type == 0) return qsTr("Video")
        if(type == 1) return qsTr("Audio")
        return ""
    }
    
    property string sourceString: availableMedias.getNameByUrl(source)
    readonly property string storageName: storage ? storage.name : qsTr("<empty>")
    readonly property Storage storage: (PostPlayService.storages && storageGuid) ? PostPlayService.getStorageByGuid(storageGuid) : null

    Connections {
        target: storage
        onDestroyed: storageGuid = 0
        ignoreUnknownSignals: true
    }

    function save() {
        return {
            type: type,
            source: source,
            storage: storageGuid,
            bitrate: bitrate,
            hardware: hardware
        }
    }

    function load(data) {
        type = data.type || 0
        source = data.source || ""
        storageGuid = data.storage || 0
        bitrate = data.bitrate || 0
        hardware = data.hardware || false
    }
}
