import QtQuick 2.0
import "../../common/utils"

Object {
    id: root
    property string name
    property int gridX: 8
    property int gridY: 8
    property var elements: []
    readonly property var rects: {
        var result = []
        for (var i = 0; i < elements.length; i++) {
            result.push(elements[i].rect)
        }
        return result
    }
    
    Component {
        id: elementComponent
        TemplateElement {}
    }    
    
    function createElement() {
        return elementComponent.createObject(root)
    }
    
    function addElement() {
        var element = createElement()
        elements.push(element)
        elements = elements
        return element
    }    
    
    function clear() {
        var prevElements = elements
        elements = []
        for (var i = 0; i < prevElements.length; i++) {
            prevElements[i].destroy()
        }
    }
    
    function save() {
        var elementsData = []
        for (var i = 0; i < elements.length; i++) {
            elementsData.push(elements[i].save())
        }
        return {
            name: name,
            gridX: gridX,
            gridY: gridY,
            elements: elementsData
        }
    }

    function load(data) {
        clear()
        name = data.name || ""
        gridX = data.gridX || 8
        gridY = data.gridY || 8
        var elementsData = data.elements || []
        for(var i = 0; i < elementsData.length; i++) {
            var element = createElement()
            element.load(elementsData[i])
            elements.push(element)
        }
        elements = elements
    }       
    
}
