import QtQuick 2.0
import QtQuickUtils 1.0
import Multimedia 1.0
import "utils"
import "../../common/utils"

Object {
    id: root
    property string name
    property url source
    property string caption
    property string videoType
    property vector4d cropping
    readonly property bool archived: name && name.charAt(0)=='$'

    readonly property bool archivedActive: archiveActive && archived

    readonly property size resolution: Qt.size(visualContent.width, visualContent.height)
    readonly property real aspect: resolution.height > 0 ? resolution.width / resolution.height : 0
    readonly property bool hasData: visualContent.width > 0 && visualContent.height > 0
    readonly property alias media: mediaLoader.object
    readonly property var supportedMediaTypes: media ? media.supportedMediaTypes : []
    readonly property MediaSession mediaSession: media ? media.mediaSession : null
    property MediaSettings mediaSettings: media ? media.mediaSettings : null

    Connections {
        target: root.mediaSettings
        onVideoTypeChanged: {
            if (mediaSettings) videoType = mediaSettings.videoType
            if (mediaSession && sessionMediaType != mediaSettings.videoType) {
                mediaSession.mediaType = mediaSettings.videoType
            }
        }
    }

    onSessionMediaType: {
        if (!mediaSettings || undefined === sessionMediaType) return
        if (mediaSettings.videoType != sessionMediaType) {
            mediaSettings.videoType = sessionMediaType
        }
    }

    SharedObjectLoader {
        id: mediaLoader
        source: root.source
        Media { stream: root }
    }

    readonly property variant sessionMediaType: media ? media.mediaType : undefined

    readonly property real frameTime: visualContent.renderer.frameTime
    readonly property int frameNumber: Math.round(frameTime * 50 / 1000) // TODO: это как определять?

    property Item visualContent: OffscreenItem {
        id: visualContent
        mipmap: true
        smooth: true
        width: sourceRect.width
        height: sourceRect.height
        sourceRect: {
            var r = Qt.rect(0, 0, renderer.implicitWidth, renderer.implicitHeight)
            if(r.width <= 0 || r.height <= 0) return Qt.rect(0, 0, 0, 0)
            var x0 = r.x + cropping.x
            var x1 = Math.max(x0 + 1, r.x + r.width - cropping.y)
            var y0 = r.y + cropping.z
            var y1 = Math.max(y0 + 1, r.y + r.height - cropping.w)
            return Qt.rect(x0, y0, x1 - x0, y1 - y0)
        }

        readonly property Item renderer: archivedActive ? archivedRenderer: liveRenderer

        LiveVideoStreamRenderer {
            id: liveRenderer
            stream: root
            visible: !archivedActive
        }
        ArchivedVideoStreamRenderer {
            id: archivedRenderer
            source: root.source
            visible: archivedActive
        }
    }

    function findVideoType(v,arr) {
        for (var i in arr) if (v==arr[i]) return arr[i]; return arr[0];
    }

    function update() {
        if (!supportedMediaTypes.length) return
        var find = findVideoType(videoType,supportedMediaTypes)
        if (!find) return 
        videoType = find.toString()
        if (mediaSettings) mediaSettings.videoType = find
    }
    
    function save() {
        return {
            name: name,
            source: source,
            caption: caption,
            cropping: cropping,
            videoType: videoType
        }
    }

    function load(data) {
        name = data.name || ""
        source = data.source || ""
        caption = data.caption || ""
        cropping = data.cropping || Qt.vector4d(0, 0, 0, 0)
        videoType = data.videoType || ''
        update()
    }   
}
