import QtQuick 2.0
import QtQuick.Layouts 1.0
import "utils"
import "../../common/utils"

Object {
    id: root
    property bool enabled: true
    property var urls: []
    property var params: ({ monitor:0, quality:100 })
    property var streams: []
    property var rects: []
    property var names: []

    readonly property var quality: params && params.quality || 100
    readonly property var monitor: params && params.monitor || 0
    readonly property var shared : params && params.shared  || false
    property var mosaic: mosaicCollection.monitorMosaic(monitor)
    readonly property real scale: 1
    readonly property size maxOutputSize: Qt.size(Math.round(1280*scale), Math.round(720*scale))
    readonly property size thumbnailSize: Qt.size(160, 90)
    readonly property real atlasWidth: thumbnailSize.width*8

    Streamer {
        id: streamer
        enabled: root.enabled
        enabledAudio: quality>99
        audioStream: audioStreamCollection.originAudioStream
        urls: root.urls
        frameSize: Qt.size(maxOutputSize.width + atlasWidth, maxOutputSize.height)
        frameRate: Qt.size(30, 1)
        bitrate: quality*quality*quality*quality/14 // 10 ~ 5mbit/sec, 100 ~ 40 mbit/sec
        content: Row {
            spacing: 0
            MosaicView {
                id: mosaicView
                mosaic: root.mosaic 
                monitor: root.monitor
				border: 2
                captionFontSize: 20*Math.sqrt(root.scale)
                height: streamer.frameSize.height
                width: height * 16/9
            }
            
            Flow {
                id: flow
                width: atlasWidth
                height: streamer.frameSize.height
                spacing: 0
                Repeater {
                    id: repeater
                    model: root.urls.length ? root.streams.length:0

                    VideoStreamView {
                        implicitWidth: thumbnailSize.width
                        implicitHeight: thumbnailSize.height
                        stream: root.streams[model.index]
                    }
                }
                property var rects: {
                    var result = []
                    for(var i = 0; i < repeater.count; i++) {
                        var item = repeater.itemAt(i)
                        var rect = Qt.rect(flow.x + item.x + item.contentRect.x, item.y + item.contentRect.y, item.contentRect.width, item.contentRect.height)
                        result.push(rect)
                    }

                    return result
                }
                property var names: {
                    var result = []
                    for(var i = 0; i < repeater.count; i++) {
                        result.push(repeater.itemAt(i).stream.name)
                    }

                    return result
                }
                onRectsChanged: root.rects = rects
                onNamesChanged: root.names = names
            }
        }
    }
}
