import QtQuick 2.0
import Multimedia 1.0
import Multimedia.DeckLink 1.0
import "../../../common/utils"

Object {
    id: root
    property string deviceId
    property alias visualContent: sourceNode.item
    property alias audioStream: sourceNode.audioStream
    readonly property alias settings: settings
    property alias status: mediaSession.status
    
    readonly property QtObject device: {
        for(var i = 0; i < DeckLink.outputs.length; i++) {
            var output = DeckLink.outputs[i]
            if(output.name == deviceId) return output
        }
        return null
    }
    
    QtObject {
        id: settings
        readonly property string type: "dl"
        property int displayModeIndex: -1

        readonly property QtObject displayMode: {
            if (!device || displayModeIndex < 0) return null
            return device.displayModes[displayModeIndex]
        }
        
        readonly property var displayModeNames: {
            if(!device) return []
            var result = []
            for(var i = 0; i < device.displayModes.length; i++) {
                var mode = device.displayModes[i]
                result.push(mode.name)
            }
            return result
        }

        function getDisplayModeIndex(displayMode) {
            if(!device || !displayMode) return -1
            for(var i = 0; i < device.displayModes.length; i++) {
                var mode = device.displayModes[i]
                if(mode.size == displayMode.size && mode.frameRate == displayMode.frameRate && mode.fieldOrder == displayMode.fieldOrder) return i
            }
            return -1
        }

        function save() {
            if (!displayMode) return {}
            return {
                displayMode: {
                    "size": displayMode.size,
                    "frameRate": displayMode.frameRate,
                    "fieldOrder": displayMode.fieldOrder
                }
            }
        }
        
        function load(data) {
            displayModeIndex = getDisplayModeIndex(data.displayMode)            
        }
    }
    MediaSession {
        id: mediaSession 
        autoPlay: true
        QtQuickSource {
            id: sourceNode
        }
        DeckLinkSink {
            id: sink
            output: root.device
            displayMode: settings.displayMode
        }
        NodeConnection {
            from: sourceNode.videoOutput
            to: sink.videoInput
        }
        NodeConnection {
            from: sourceNode.audioOutput
            to: sink.audioInput
        }        
    }    
}

