import QtQuick 2.0
import Multimedia 1.0
import Multimedia.FDExt 1.0
import "../../../common/utils"

Object {
    id: root
    property string deviceId
    property alias visualContent: sourceNode.item
    property alias audioStream: sourceNode.audioStream
    readonly property alias settings: settings
    property alias status: mediaSession.status
    
    readonly property QtObject device: {
        for(var i = 0; i < FD.outputs.length; i++) {
            var output = FD.outputs[i]
            if(output.name == deviceId) return output
        }
        return null
    }
    
    signal error()
    
    QtObject {
        id: settings
        readonly property string type: "fd"
        property bool wssFlag: true
        property int audioChannel: -1 // TODO: не реализовано
        function save() {
            return {
                wssFlag: wssFlag,
                audioChannel: audioChannel
            }
        }
        function load(data) {
            wssFlag = data.wssFlag || false
            audioChannel = (data.audioChannel != undefined) ? data.audioChannel : -1
        }
    }
            
    MediaSession {
        id: mediaSession 
        autoPlay: true
        QtQuickSource {
            id: sourceNode
            frameAspect: sink.aspect
        }
        FDSink {
            id: sink
            output: root.device
            aspect: settings.wssFlag ? Qt.size(16, 9) : Qt.size(4, 3)
        }
        NodeConnection {
            from: sourceNode.videoOutput
            to: sink.videoInput
        }
        NodeConnection {
            from: sourceNode.audioOutput
            to: sink.audioInput
        }
    }     
}

