﻿import QtQuick 2.0
import Multimedia 1.0
import Multimedia.FDExt 1.0
import Multimedia.DeckLink 1.0
import MultipleScreens 1.0
import "../../../common/utils"

Object {
    id: root
    property string deviceId
    property Item visualContent
    property AudioStream audioStream
    readonly property alias device: loader.item
    readonly property QtObject settings: (device && device.settings) ? device.settings : null
    readonly property string what: 'screen'

    readonly property var availableDevices: {
        var result = []

        var force_monitors = getAppParam('--force_monitors')
        var force_number = parseInt(force_monitors)
        if (isNaN(force_number)) force_number = 0;

        if (what.indexOf('fd')!=-1)
        for(var i = 0; i < FD.outputs.length; i++) {
            var name = FD.outputs[i].name
            result.push({name: name, deviceId: "fd:///" + name})
        }
        if (what.indexOf('dl')!=-1)
        for(var i = 0; i < DeckLink.outputs.length; i++) {
            var name = DeckLink.outputs[i].name
            result.push({name: name, deviceId: "dl:///" + name})
        }
        if (what.indexOf('screen')!=-1)
        for(var i = force_monitors=='all' ? 0:1; i <(force_number ? force_number+1 : ScreenInfo.screens.length); i++) {
            var screen = ScreenInfo.screens[i<(force_number ? 1:ScreenInfo.screens.length) ? i:ScreenInfo.screens.length-1]
            var name = "Monitor %1 (%2)".arg(i + 1).arg(screen.model)
            result.push({name: name, deviceId: "screen:///" + (i), index:screen.index})
        }

        return result
    }
    
    Loader {
        id: loader
        function updateSource() {
            source = ""
            var deviceId = ""
            var qmlFile = ""
            if(root.deviceId.indexOf("fd:///") == 0) {
                deviceId = root.deviceId.replace("fd:///", "")
                qmlFile = "FdOutputDevice.qml"
            }
            else if(root.deviceId.indexOf("dl:///") == 0) {
                deviceId = root.deviceId.replace("dl:///", "")
                qmlFile = "DlOutputDevice.qml"
            }
            else if(root.deviceId.indexOf("screen:///") == 0) {
                deviceId = root.deviceId.replace("screen:///", "")
                qmlFile = "ScreenOutputDevice.qml"
            }
            setSource(qmlFile, {deviceId: deviceId, visualContent: visualContent, audioStream: root.audioStream})
        }
    }
    onDeviceIdChanged: loader.updateSource()
    onAudioStreamChanged: if(device) device.audioStream = audioStream
    onVisualContentChanged: if(device) device.visualContent = visualContent
    
    readonly property int mediaSessionStatus: device ? device.status : MediaSession.Loading
    onMediaSessionStatusChanged: {
        if(mediaSessionStatus == MediaSession.Error) {
            app.error(qsTr("Error in output device: ") + root.deviceId)
        }
    }
    
    Connections {
        target: device
        ignoreUnknownSignals: true
        onClosed: deviceId = ""
    }

    function save() {
        return {
            deviceId: deviceId,
            settings: settings ? settings.save() : {}
        }
    }
    
    function load(data) {
        deviceId = data.deviceId || ""
        if(settings) settings.load(data.settings || {})
    }
    
    function setDefault() {
        deviceId = ""
    }
    
}
