import QtQml 2.2
import Multimedia 1.0
import Multimedia.PostPlay 1.0
import Multimedia.DeckLink 1.0

MediaSession {
    id: root    
    property string deviceId
    property Stream stream
    property var settings
    
    property alias mediaType: sourceNode.videoType
    property alias supportedMediaTypes: sourceNode.supportedVideoTypes
    
    readonly property QtObject device: {
        for(var i = 0; i < DeckLink.inputs.length; i++) {
            var input = DeckLink.inputs[i]
            if(input.id == deviceId) return input
        }
        return null
    }

    recreateOnError: true
    autoPlay: false
    
    DeckLinkSource {
        id: sourceNode
        source: root.device
        videoType: root.settings.mediaType
    }
    VideoProcessor {
        id: videoProcessor
        pixelFormat: VideoProcessor.NV12
    }
    VideoEncoder {
        id: encoder
        codec: VideoEncoder.H264
        bitrate: root.settings ? (root.settings.bitrate * 1024) : 0
        hardware: (root.settings && root.settings.hardware) ? VideoEncoder.Nvidia : VideoEncoder.NoneHardware
    }
    PostPlaySink {
        id: videoSink
        stream: root.stream
    }
    
    NodeConnection {
        from: sourceNode.videoOutput
        to: videoProcessor.input
    }
    NodeConnection {
        from: videoProcessor.output
        to: encoder.input
    }        
    NodeConnection {
        from: encoder.output
        to: videoSink.input
    }
}
