pragma Singleton
import QtQml 2.2
import QmlUtils 1.0

QtObject {
    id: root
    property var instances: ({})
    
    property Component sharedObjectInstanceComponent: Component {
        QtObject {
            property SharedObject object
            property var loaders: []
            property Binding qualityBinding: Binding {
                target: object
                property: "quality"
                value: {
                    var bestQuality = 0
                    for(var i = 0; i < loaders.length; i++) {
                        if(loaders[i].reloading) return 0
                        var quality = loaders[i].quality
                        if(quality > bestQuality) bestQuality = quality
                    }
                    return bestQuality
                }
            }
        }
    }
    
    function addLoader(loader) {
        var instance = getOrCreateInstance(loader.source, loader.component)
        if(!instance) return null
        if(instance.loaders.indexOf(loader) > -1) return
        instance.loaders.push(loader)
        instance.loaders = instance.loaders
        return instance.object
    }
    
    function removeLoader(loader) {
        for(var source in instances) {
            var instance = instances[source]
            var i = instance.loaders.indexOf(loader)
            if (i > -1) {
                instance.loaders.splice(i, 1)
                if(instance.loaders.length == 0) {
                    delete instances[source]
                    QmlUtils.deleteObject(instance)
                    return
                }
                instance.loaders = instance.loaders
            }
        }
    }
    
    function getInstance(source) {
        return instances[source] || null
    }
    
    function getOrCreateInstance(source, component) {
        if(!source || !component) return null
        var instance = instances[source]
        if(instance) return instance
        instance = sharedObjectInstanceComponent.createObject(root)
        instance.object = component.createObject(instance, {source: source})
        instances[source] = instance
        return instance
    }
}
